@@ -1,6 +1,6 @@
-/* $Id: e2p_dircmp.c 1171 2008-08-29 10:30:36Z tpgww $
+/* $Id: e2p_dircmp.c 1991 2009-10-14 21:11:38Z tpgww $
 
-Portions copyright (C) 2006-2008 tooar <tooar@gmx.net>
+Portions copyright (C) 2006-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -24,7 +24,7 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 #include "emelfm2.h"
-#include "e2p_dircmp.h"
+#include <fcntl.h>
 #include "e2_plugins.h"
 #include "e2_fileview.h"
 #include "e2_filelist.h"
@@ -40,6 +40,10 @@ typedef struct _E2_CmpData
 	gchar *newroot;
 } E2_CmpData;
 
+/* Bytes in MD5 hash */
+#define HASH_LENGTH 16
+
+#ifndef USE_GLIB2_16
 /********* start of borrowed code *********/
 
 /*
@@ -49,6 +53,7 @@ copyright is claimed and that this code is in the public domain.
 
 Equivalent code is available from RSA Data Security, Inc.
 This code has been tested against that, and is functionally equivalent.
+(It is also functionally equivalent to the approach used by Glib >= 2.16.)
 
 To compute the message digest of a chunk of bytes, declare an
 MD5Context structure, pass it to MD5Init, call MD5Update as
@@ -56,26 +61,85 @@ needed on buffers full of bytes, and then call MD5Final, which
 will fill a supplied 16-byte array with the digest.
 */
 
-#ifndef HIGHFIRST
-#define byteReverse(buf, len)	/* Nothing */
-#else
-void byteReverse(unsigned char *buf, unsigned longs);
+struct MD5Context {
+	u_int32_t buf[4];
+	u_int32_t bits[2];
+	unsigned char in[64];
+};
 
-#ifndef ASM_MD5
+void MD5Init(struct MD5Context *context);
+void MD5Update(struct MD5Context *context, unsigned char const *buf, unsigned len);
+void MD5Final(unsigned char digest[16], struct MD5Context *context);
+void MD5Transform(u_int32_t buf[4], u_int32_t const in[16]);
+
+/* MD5 setup requires knowing if we're big- or little-endian */
+#if defined(__linux) || defined(__linux__)
+# include <endian.h>
+# if !defined(BYTE_ORDER) && defined(__BYTE_ORDER)
+#  define BYTE_ORDER __BYTE_ORDER
+# endif
+
+#elif defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__)
+# include <sys/endian.h>
+# if !defined(BYTE_ORDER) && defined(_BYTE_ORDER)
+#  define BYTE_ORDER _BYTE_ORDER
+# endif
+
+#elif defined(__LINUX) //CHECKME
+# ifndef __USE_BSD
+#  define __USE_BSD
+# endif
+# include <endian.h>
+# if !defined(BYTE_ORDER) && defined(__BYTE_ORDER)
+#  define BYTE_ORDER __BYTE_ORDER
+# endif
+
+#elif defined (__SOLARIS)
+
+# include <sys/isa_defs.h>
+# ifdef _BIG_ENDIAN
+#  define BYTE_ORDER 4321
+# else
+#  define BYTE_ORDER 1234
+# endif
+
+#elif defined (__APPLE__)
+# include <machine/endian.h>
+
+#endif
+
+#ifndef BIG_ENDIAN
+# define BIG_ENDIAN    4321
+#endif
+#ifndef LITTLE_ENDIAN
+# define LITTLE_ENDIAN 1234
+#endif
+
+#if BYTE_ORDER == BIG_ENDIAN
+# define HIGHFIRST
+// For Tiger
+# define BIG_ENDIAN_HOST
+#endif //BYTE_ORDER == BIG_ENDIAN
+
+#ifdef HIGHFIRST
+void byteReverse(unsigned char *buf, unsigned longs);
+# ifndef ASM_MD5
 /*
 Note: this code is harmless on little-endian machines.
 */
 void byteReverse(unsigned char *buf, unsigned longs)
 {
-    u_int32_t t;
-    do {
+	u_int32_t t;
+	do {
 	t = (u_int32_t) ((unsigned) buf[3] << 8 | buf[2]) << 16 |
-	    ((unsigned) buf[1] << 8 | buf[0]);
+		((unsigned) buf[1] << 8 | buf[0]);
 	*(u_int32_t *) buf = t;
 	buf += 4;
-    } while (--longs);
+	} while (--longs);
 }
-#endif
+# endif
+#else
+# define byteReverse(buf,len)	/* Nothing */
 #endif
 
 /*
@@ -84,85 +148,84 @@ initialization constants.
 */
 void MD5Init(struct MD5Context *ctx)
 {
-    ctx->buf[0] = 0x67452301;
-    ctx->buf[1] = 0xefcdab89;
-    ctx->buf[2] = 0x98badcfe;
-    ctx->buf[3] = 0x10325476;
+	ctx->buf[0] = 0x67452301;
+	ctx->buf[1] = 0xefcdab89;
+	ctx->buf[2] = 0x98badcfe;
+	ctx->buf[3] = 0x10325476;
 
-    ctx->bits[0] = 0;
-    ctx->bits[1] = 0;
+	ctx->bits[0] = 0;
+	ctx->bits[1] = 0;
 }
 
 /*
-Update context to reflect the concatenation of another buffer full
-of bytes.
+Update context to reflect the concatenation of another buffer full of bytes.
 */
 void MD5Update(struct MD5Context *ctx, unsigned char const *buf, unsigned len)
 {
-    u_int32_t t;
+	u_int32_t t;
 
-    /* Update bitcount */
+	/* Update bitcount */
 
-    t = ctx->bits[0];
-    if ((ctx->bits[0] = t + ((u_int32_t) len << 3)) < t)
+	t = ctx->bits[0];
+	if ((ctx->bits[0] = t + ((u_int32_t) len << 3)) < t)
 	ctx->bits[1]++;		/* Carry from low to high */
-    ctx->bits[1] += len >> 29;
+	ctx->bits[1] += len >> 29;
 
-    t = (t >> 3) & 0x3f;	/* Bytes already in shsInfo->data */
+	t = (t >> 3) & 0x3f;	/* Bytes already in shsInfo->data */
 
-    /* Handle any leading odd-sized chunks */
+	/* Handle any leading odd-sized chunks */
 
-    if (t) {
+	if (t) {
 	unsigned char *p = (unsigned char *) ctx->in + t;
 
 	t = 64 - t;
 	if (len < t) {
-	    memcpy(p, buf, len);
-	    return;
+		memcpy(p, buf, len);
+		return;
 	}
 	memcpy(p, buf, t);
 	byteReverse(ctx->in, 16);
 	MD5Transform(ctx->buf, (u_int32_t *) ctx->in);
 	buf += t;
 	len -= t;
-    }
-    /* Process data in 64-byte chunks */
+	}
+	/* Process data in 64-byte chunks */
 
-    while (len >= 64) {
+	while (len >= 64) {
 	memcpy(ctx->in, buf, 64);
 	byteReverse(ctx->in, 16);
 	MD5Transform(ctx->buf, (u_int32_t *) ctx->in);
 	buf += 64;
 	len -= 64;
-    }
+	}
 
-    /* Handle any remaining bytes of data. */
+	/* Handle any remaining bytes of data. */
 
-    memcpy(ctx->in, buf, len);
+	memcpy(ctx->in, buf, len);
 }
 
 /*
-Final wrapup - pad to 64-byte boundary with the bit pattern
-1 0* (64-bit count of bits processed, MSB-first)
+Final wrapup - pad to 64-byte boundary with the bit pattern 1 0*
+(64-bit count of bits processed, MSB-first)
 */
 void MD5Final(unsigned char digest[16], struct MD5Context *ctx)
 {
-    unsigned count;
-    unsigned char *p;
+	unsigned count;
+	unsigned char *p;
 
-    /* Compute number of bytes mod 64 */
-    count = (ctx->bits[0] >> 3) & 0x3F;
+	/* Compute number of bytes mod 64 */
+	count = (ctx->bits[0] >> 3) & 0x3F;
 
-    /* Set the first char of padding to 0x80.  This is safe since there is
-       always at least one byte free */
-    p = ctx->in + count;
-    *p++ = 0x80;
+	/* Set the first char of padding to 0x80.  This is safe since there is
+	   always at least one byte free */
+	p = ctx->in + count;
+	*p++ = 0x80;
 
-    /* Bytes of padding needed to make 64 bytes */
-    count = 64 - 1 - count;
+	/* Bytes of padding needed to make 64 bytes */
+	count = 64 - 1 - count;
 
-    /* Pad out to 56 mod 64 */
-    if (count < 8) {
+	/* Pad out to 56 mod 64 */
+	if (count < 8) {
 	/* Two lots of padding:  Pad the first block to 64 bytes */
 	memset(p, 0, count);
 	byteReverse(ctx->in, 16);
@@ -170,46 +233,38 @@ void MD5Final(unsigned char digest[16], struct MD5Context *ctx)
 
 	/* Now fill the next block with 56 bytes */
 	memset(ctx->in, 0, 56);
-    } else {
+	} else {
 	/* Pad block to 56 bytes */
 	memset(p, 0, count - 8);
-    }
-    byteReverse(ctx->in, 14);
+	}
+	byteReverse(ctx->in, 14);
 
-    /* Append length in bits and transform */
-    ((u_int32_t *) ctx->in)[14] = ctx->bits[0];
-    ((u_int32_t *) ctx->in)[15] = ctx->bits[1];
+	/* Append length in bits and transform */
+	((u_int32_t *) ctx->in)[14] = ctx->bits[0];
+	((u_int32_t *) ctx->in)[15] = ctx->bits[1];
 
-    MD5Transform(ctx->buf, (u_int32_t *) ctx->in);
-    byteReverse((unsigned char *) ctx->buf, 4);
-    memcpy(digest, ctx->buf, 16);
+	MD5Transform(ctx->buf, (u_int32_t *) ctx->in);
+	byteReverse((unsigned char *) ctx->buf, 4);
+	memcpy(digest, ctx->buf, 16);
 
-    memset(ctx, 0, sizeof(* ctx));	/* In case it's sensitive */
-    /* The original version of this code omitted the asterisk. In
-       effect, only the first part of ctx was wiped with zeros, not
-       the whole thing. Bug found by Derek Jones. Original line: */
-    // memset(ctx, 0, sizeof(ctx));	/* In case it's sensitive */
+	memset(ctx, 0, sizeof(* ctx));	/* In case it's sensitive */
+	/* The original version of this code omitted the asterisk. In
+	   effect, only the first part of ctx was wiped with zeros, not
+	   the whole thing. Bug found by Derek Jones. Original line: */
+	// memset(ctx, 0, sizeof(ctx));	/* In case it's sensitive */
 }
 
 #ifndef ASM_MD5
 
+#define MD5STEP(f, w, x, y, z, data, s) \
+	( w += f(x, y, z) + data,  w = w<<s | w>>(32-s),  w += x )
 /* The four core functions - F1 is optimized somewhat */
-
 /* #define F1(x, y, z) (x & y | ~x & z) */
 #define F1(x, y, z) (z ^ (x & (y ^ z)))
 #define F2(x, y, z) F1(z, x, y)
 #define F3(x, y, z) (x ^ y ^ z)
 #define F4(x, y, z) (y ^ (x | ~z))
 
-/* This is the central step in the MD5 algorithm. */
-#ifdef __PUREC__
-#define MD5STEP(f, w, x, y, z, data, s) \
-	( w += f /*(x, y, z)*/ + data,  w = w<<s | w>>(32-s),  w += x )
-#else
-#define MD5STEP(f, w, x, y, z, data, s) \
-	( w += f(x, y, z) + data,  w = w<<s | w>>(32-s),  w += x )
-#endif
-
 /*
 The core of the MD5 algorithm, this alters an existing MD5 hash to
 reflect the addition of 16 longwords of new data.  MD5Update blocks
@@ -217,173 +272,110 @@ the data and converts bytes into longwords for this routine.
 */
 void MD5Transform(u_int32_t buf[4], u_int32_t const in[16])
 {
-    register u_int32_t a, b, c, d;
+	register u_int32_t a, b, c, d;
 
-    a = buf[0];
-    b = buf[1];
-    c = buf[2];
-    d = buf[3];
+	a = buf[0];
+	b = buf[1];
+	c = buf[2];
+	d = buf[3];
 
-#ifdef __PUREC__	/* PureC Weirdness... (GG) */
-    MD5STEP(F1(b,c,d), a, b, c, d, in[0] + 0xd76aa478L, 7);
-    MD5STEP(F1(a,b,c), d, a, b, c, in[1] + 0xe8c7b756L, 12);
-    MD5STEP(F1(d,a,b), c, d, a, b, in[2] + 0x242070dbL, 17);
-    MD5STEP(F1(c,d,a), b, c, d, a, in[3] + 0xc1bdceeeL, 22);
-    MD5STEP(F1(b,c,d), a, b, c, d, in[4] + 0xf57c0fafL, 7);
-    MD5STEP(F1(a,b,c), d, a, b, c, in[5] + 0x4787c62aL, 12);
-    MD5STEP(F1(d,a,b), c, d, a, b, in[6] + 0xa8304613L, 17);
-    MD5STEP(F1(c,d,a), b, c, d, a, in[7] + 0xfd469501L, 22);
-    MD5STEP(F1(b,c,d), a, b, c, d, in[8] + 0x698098d8L, 7);
-    MD5STEP(F1(a,b,c), d, a, b, c, in[9] + 0x8b44f7afL, 12);
-    MD5STEP(F1(d,a,b), c, d, a, b, in[10] + 0xffff5bb1L, 17);
-    MD5STEP(F1(c,d,a), b, c, d, a, in[11] + 0x895cd7beL, 22);
-    MD5STEP(F1(b,c,d), a, b, c, d, in[12] + 0x6b901122L, 7);
-    MD5STEP(F1(a,b,c), d, a, b, c, in[13] + 0xfd987193L, 12);
-    MD5STEP(F1(d,a,b), c, d, a, b, in[14] + 0xa679438eL, 17);
-    MD5STEP(F1(c,d,a), b, c, d, a, in[15] + 0x49b40821L, 22);
+	MD5STEP(F1, a, b, c, d, in[0] + 0xd76aa478, 7);
+	MD5STEP(F1, d, a, b, c, in[1] + 0xe8c7b756, 12);
+	MD5STEP(F1, c, d, a, b, in[2] + 0x242070db, 17);
+	MD5STEP(F1, b, c, d, a, in[3] + 0xc1bdceee, 22);
+	MD5STEP(F1, a, b, c, d, in[4] + 0xf57c0faf, 7);
+	MD5STEP(F1, d, a, b, c, in[5] + 0x4787c62a, 12);
+	MD5STEP(F1, c, d, a, b, in[6] + 0xa8304613, 17);
+	MD5STEP(F1, b, c, d, a, in[7] + 0xfd469501, 22);
+	MD5STEP(F1, a, b, c, d, in[8] + 0x698098d8, 7);
+	MD5STEP(F1, d, a, b, c, in[9] + 0x8b44f7af, 12);
+	MD5STEP(F1, c, d, a, b, in[10] + 0xffff5bb1, 17);
+	MD5STEP(F1, b, c, d, a, in[11] + 0x895cd7be, 22);
+	MD5STEP(F1, a, b, c, d, in[12] + 0x6b901122, 7);
+	MD5STEP(F1, d, a, b, c, in[13] + 0xfd987193, 12);
+	MD5STEP(F1, c, d, a, b, in[14] + 0xa679438e, 17);
+	MD5STEP(F1, b, c, d, a, in[15] + 0x49b40821, 22);
 
-    MD5STEP(F2(b,c,d), a, b, c, d, in[1] + 0xf61e2562L, 5);
-    MD5STEP(F2(a,b,c), d, a, b, c, in[6] + 0xc040b340L, 9);
-    MD5STEP(F2(d,a,b), c, d, a, b, in[11] + 0x265e5a51L, 14);
-    MD5STEP(F2(c,d,a), b, c, d, a, in[0] + 0xe9b6c7aaL, 20);
-    MD5STEP(F2(b,c,d), a, b, c, d, in[5] + 0xd62f105dL, 5);
-    MD5STEP(F2(a,b,c), d, a, b, c, in[10] + 0x02441453L, 9);
-    MD5STEP(F2(d,a,b), c, d, a, b, in[15] + 0xd8a1e681L, 14);
-    MD5STEP(F2(c,d,a), b, c, d, a, in[4] + 0xe7d3fbc8L, 20);
-    MD5STEP(F2(b,c,d), a, b, c, d, in[9] + 0x21e1cde6L, 5);
-    MD5STEP(F2(a,b,c), d, a, b, c, in[14] + 0xc33707d6L, 9);
-    MD5STEP(F2(d,a,b), c, d, a, b, in[3] + 0xf4d50d87L, 14);
-    MD5STEP(F2(c,d,a), b, c, d, a, in[8] + 0x455a14edL, 20);
-    MD5STEP(F2(b,c,d), a, b, c, d, in[13] + 0xa9e3e905L, 5);
-    MD5STEP(F2(a,b,c), d, a, b, c, in[2] + 0xfcefa3f8L, 9);
-    MD5STEP(F2(d,a,b), c, d, a, b, in[7] + 0x676f02d9L, 14);
-    MD5STEP(F2(c,d,a), b, c, d, a, in[12] + 0x8d2a4c8aL, 20);
+	MD5STEP(F2, a, b, c, d, in[1] + 0xf61e2562, 5);
+	MD5STEP(F2, d, a, b, c, in[6] + 0xc040b340, 9);
+	MD5STEP(F2, c, d, a, b, in[11] + 0x265e5a51, 14);
+	MD5STEP(F2, b, c, d, a, in[0] + 0xe9b6c7aa, 20);
+	MD5STEP(F2, a, b, c, d, in[5] + 0xd62f105d, 5);
+	MD5STEP(F2, d, a, b, c, in[10] + 0x02441453, 9);
+	MD5STEP(F2, c, d, a, b, in[15] + 0xd8a1e681, 14);
+	MD5STEP(F2, b, c, d, a, in[4] + 0xe7d3fbc8, 20);
+	MD5STEP(F2, a, b, c, d, in[9] + 0x21e1cde6, 5);
+	MD5STEP(F2, d, a, b, c, in[14] + 0xc33707d6, 9);
+	MD5STEP(F2, c, d, a, b, in[3] + 0xf4d50d87, 14);
+	MD5STEP(F2, b, c, d, a, in[8] + 0x455a14ed, 20);
+	MD5STEP(F2, a, b, c, d, in[13] + 0xa9e3e905, 5);
+	MD5STEP(F2, d, a, b, c, in[2] + 0xfcefa3f8, 9);
+	MD5STEP(F2, c, d, a, b, in[7] + 0x676f02d9, 14);
+	MD5STEP(F2, b, c, d, a, in[12] + 0x8d2a4c8a, 20);
 
-    MD5STEP(F3(b,c,d), a, b, c, d, in[5] + 0xfffa3942L, 4);
-    MD5STEP(F3(a,b,c), d, a, b, c, in[8] + 0x8771f681L, 11);
-    MD5STEP(F3(d,a,b), c, d, a, b, in[11] + 0x6d9d6122L, 16);
-    MD5STEP(F3(c,d,a), b, c, d, a, in[14] + 0xfde5380cL, 23);
-    MD5STEP(F3(b,c,d), a, b, c, d, in[1] + 0xa4beea44L, 4);
-    MD5STEP(F3(a,b,c), d, a, b, c, in[4] + 0x4bdecfa9L, 11);
-    MD5STEP(F3(d,a,b), c, d, a, b, in[7] + 0xf6bb4b60L, 16);
-    MD5STEP(F3(c,d,a), b, c, d, a, in[10] + 0xbebfbc70L, 23);
-    MD5STEP(F3(b,c,d), a, b, c, d, in[13] + 0x289b7ec6L, 4);
-    MD5STEP(F3(a,b,c), d, a, b, c, in[0] + 0xeaa127faL, 11);
-    MD5STEP(F3(d,a,b), c, d, a, b, in[3] + 0xd4ef3085L, 16);
-    MD5STEP(F3(c,d,a), b, c, d, a, in[6] + 0x04881d05L, 23);
-    MD5STEP(F3(b,c,d), a, b, c, d, in[9] + 0xd9d4d039L, 4);
-    MD5STEP(F3(a,b,c), d, a, b, c, in[12] + 0xe6db99e5L, 11);
-    MD5STEP(F3(d,a,b), c, d, a, b, in[15] + 0x1fa27cf8L, 16);
-    MD5STEP(F3(c,d,a), b, c, d, a, in[2] + 0xc4ac5665L, 23);
+	MD5STEP(F3, a, b, c, d, in[5] + 0xfffa3942, 4);
+	MD5STEP(F3, d, a, b, c, in[8] + 0x8771f681, 11);
+	MD5STEP(F3, c, d, a, b, in[11] + 0x6d9d6122, 16);
+	MD5STEP(F3, b, c, d, a, in[14] + 0xfde5380c, 23);
+	MD5STEP(F3, a, b, c, d, in[1] + 0xa4beea44, 4);
+	MD5STEP(F3, d, a, b, c, in[4] + 0x4bdecfa9, 11);
+	MD5STEP(F3, c, d, a, b, in[7] + 0xf6bb4b60, 16);
+	MD5STEP(F3, b, c, d, a, in[10] + 0xbebfbc70, 23);
+	MD5STEP(F3, a, b, c, d, in[13] + 0x289b7ec6, 4);
+	MD5STEP(F3, d, a, b, c, in[0] + 0xeaa127fa, 11);
+	MD5STEP(F3, c, d, a, b, in[3] + 0xd4ef3085, 16);
+	MD5STEP(F3, b, c, d, a, in[6] + 0x04881d05, 23);
+	MD5STEP(F3, a, b, c, d, in[9] + 0xd9d4d039, 4);
+	MD5STEP(F3, d, a, b, c, in[12] + 0xe6db99e5, 11);
+	MD5STEP(F3, c, d, a, b, in[15] + 0x1fa27cf8, 16);
+	MD5STEP(F3, b, c, d, a, in[2] + 0xc4ac5665, 23);
 
-    MD5STEP(F4(b,c,d), a, b, c, d, in[0] + 0xf4292244L, 6);
-    MD5STEP(F4(a,b,c), d, a, b, c, in[7] + 0x432aff97L, 10);
-    MD5STEP(F4(d,a,b), c, d, a, b, in[14] + 0xab9423a7L, 15);
-    MD5STEP(F4(c,d,a), b, c, d, a, in[5] + 0xfc93a039L, 21);
-    MD5STEP(F4(b,c,d), a, b, c, d, in[12] + 0x655b59c3L, 6);
-    MD5STEP(F4(a,b,c), d, a, b, c, in[3] + 0x8f0ccc92L, 10);
-    MD5STEP(F4(d,a,b), c, d, a, b, in[10] + 0xffeff47dL, 15);
-    MD5STEP(F4(c,d,a), b, c, d, a, in[1] + 0x85845dd1L, 21);
-    MD5STEP(F4(b,c,d), a, b, c, d, in[8] + 0x6fa87e4fL, 6);
-    MD5STEP(F4(a,b,c), d, a, b, c, in[15] + 0xfe2ce6e0L, 10);
-    MD5STEP(F4(d,a,b), c, d, a, b, in[6] + 0xa3014314L, 15);
-    MD5STEP(F4(c,d,a), b, c, d, a, in[13] + 0x4e0811a1L, 21);
-    MD5STEP(F4(b,c,d), a, b, c, d, in[4] + 0xf7537e82L, 6);
-    MD5STEP(F4(a,b,c), d, a, b, c, in[11] + 0xbd3af235L, 10);
-    MD5STEP(F4(d,a,b), c, d, a, b, in[2] + 0x2ad7d2bbL, 15);
-    MD5STEP(F4(c,d,a), b, c, d, a, in[9] + 0xeb86d391L, 21);
-#else
-    MD5STEP(F1, a, b, c, d, in[0] + 0xd76aa478, 7);
-    MD5STEP(F1, d, a, b, c, in[1] + 0xe8c7b756, 12);
-    MD5STEP(F1, c, d, a, b, in[2] + 0x242070db, 17);
-    MD5STEP(F1, b, c, d, a, in[3] + 0xc1bdceee, 22);
-    MD5STEP(F1, a, b, c, d, in[4] + 0xf57c0faf, 7);
-    MD5STEP(F1, d, a, b, c, in[5] + 0x4787c62a, 12);
-    MD5STEP(F1, c, d, a, b, in[6] + 0xa8304613, 17);
-    MD5STEP(F1, b, c, d, a, in[7] + 0xfd469501, 22);
-    MD5STEP(F1, a, b, c, d, in[8] + 0x698098d8, 7);
-    MD5STEP(F1, d, a, b, c, in[9] + 0x8b44f7af, 12);
-    MD5STEP(F1, c, d, a, b, in[10] + 0xffff5bb1, 17);
-    MD5STEP(F1, b, c, d, a, in[11] + 0x895cd7be, 22);
-    MD5STEP(F1, a, b, c, d, in[12] + 0x6b901122, 7);
-    MD5STEP(F1, d, a, b, c, in[13] + 0xfd987193, 12);
-    MD5STEP(F1, c, d, a, b, in[14] + 0xa679438e, 17);
-    MD5STEP(F1, b, c, d, a, in[15] + 0x49b40821, 22);
+	MD5STEP(F4, a, b, c, d, in[0] + 0xf4292244, 6);
+	MD5STEP(F4, d, a, b, c, in[7] + 0x432aff97, 10);
+	MD5STEP(F4, c, d, a, b, in[14] + 0xab9423a7, 15);
+	MD5STEP(F4, b, c, d, a, in[5] + 0xfc93a039, 21);
+	MD5STEP(F4, a, b, c, d, in[12] + 0x655b59c3, 6);
+	MD5STEP(F4, d, a, b, c, in[3] + 0x8f0ccc92, 10);
+	MD5STEP(F4, c, d, a, b, in[10] + 0xffeff47d, 15);
+	MD5STEP(F4, b, c, d, a, in[1] + 0x85845dd1, 21);
+	MD5STEP(F4, a, b, c, d, in[8] + 0x6fa87e4f, 6);
+	MD5STEP(F4, d, a, b, c, in[15] + 0xfe2ce6e0, 10);
+	MD5STEP(F4, c, d, a, b, in[6] + 0xa3014314, 15);
+	MD5STEP(F4, b, c, d, a, in[13] + 0x4e0811a1, 21);
+	MD5STEP(F4, a, b, c, d, in[4] + 0xf7537e82, 6);
+	MD5STEP(F4, d, a, b, c, in[11] + 0xbd3af235, 10);
+	MD5STEP(F4, c, d, a, b, in[2] + 0x2ad7d2bb, 15);
+	MD5STEP(F4, b, c, d, a, in[9] + 0xeb86d391, 21);
 
-    MD5STEP(F2, a, b, c, d, in[1] + 0xf61e2562, 5);
-    MD5STEP(F2, d, a, b, c, in[6] + 0xc040b340, 9);
-    MD5STEP(F2, c, d, a, b, in[11] + 0x265e5a51, 14);
-    MD5STEP(F2, b, c, d, a, in[0] + 0xe9b6c7aa, 20);
-    MD5STEP(F2, a, b, c, d, in[5] + 0xd62f105d, 5);
-    MD5STEP(F2, d, a, b, c, in[10] + 0x02441453, 9);
-    MD5STEP(F2, c, d, a, b, in[15] + 0xd8a1e681, 14);
-    MD5STEP(F2, b, c, d, a, in[4] + 0xe7d3fbc8, 20);
-    MD5STEP(F2, a, b, c, d, in[9] + 0x21e1cde6, 5);
-    MD5STEP(F2, d, a, b, c, in[14] + 0xc33707d6, 9);
-    MD5STEP(F2, c, d, a, b, in[3] + 0xf4d50d87, 14);
-    MD5STEP(F2, b, c, d, a, in[8] + 0x455a14ed, 20);
-    MD5STEP(F2, a, b, c, d, in[13] + 0xa9e3e905, 5);
-    MD5STEP(F2, d, a, b, c, in[2] + 0xfcefa3f8, 9);
-    MD5STEP(F2, c, d, a, b, in[7] + 0x676f02d9, 14);
-    MD5STEP(F2, b, c, d, a, in[12] + 0x8d2a4c8a, 20);
-
-    MD5STEP(F3, a, b, c, d, in[5] + 0xfffa3942, 4);
-    MD5STEP(F3, d, a, b, c, in[8] + 0x8771f681, 11);
-    MD5STEP(F3, c, d, a, b, in[11] + 0x6d9d6122, 16);
-    MD5STEP(F3, b, c, d, a, in[14] + 0xfde5380c, 23);
-    MD5STEP(F3, a, b, c, d, in[1] + 0xa4beea44, 4);
-    MD5STEP(F3, d, a, b, c, in[4] + 0x4bdecfa9, 11);
-    MD5STEP(F3, c, d, a, b, in[7] + 0xf6bb4b60, 16);
-    MD5STEP(F3, b, c, d, a, in[10] + 0xbebfbc70, 23);
-    MD5STEP(F3, a, b, c, d, in[13] + 0x289b7ec6, 4);
-    MD5STEP(F3, d, a, b, c, in[0] + 0xeaa127fa, 11);
-    MD5STEP(F3, c, d, a, b, in[3] + 0xd4ef3085, 16);
-    MD5STEP(F3, b, c, d, a, in[6] + 0x04881d05, 23);
-    MD5STEP(F3, a, b, c, d, in[9] + 0xd9d4d039, 4);
-    MD5STEP(F3, d, a, b, c, in[12] + 0xe6db99e5, 11);
-    MD5STEP(F3, c, d, a, b, in[15] + 0x1fa27cf8, 16);
-    MD5STEP(F3, b, c, d, a, in[2] + 0xc4ac5665, 23);
-
-    MD5STEP(F4, a, b, c, d, in[0] + 0xf4292244, 6);
-    MD5STEP(F4, d, a, b, c, in[7] + 0x432aff97, 10);
-    MD5STEP(F4, c, d, a, b, in[14] + 0xab9423a7, 15);
-    MD5STEP(F4, b, c, d, a, in[5] + 0xfc93a039, 21);
-    MD5STEP(F4, a, b, c, d, in[12] + 0x655b59c3, 6);
-    MD5STEP(F4, d, a, b, c, in[3] + 0x8f0ccc92, 10);
-    MD5STEP(F4, c, d, a, b, in[10] + 0xffeff47d, 15);
-    MD5STEP(F4, b, c, d, a, in[1] + 0x85845dd1, 21);
-    MD5STEP(F4, a, b, c, d, in[8] + 0x6fa87e4f, 6);
-    MD5STEP(F4, d, a, b, c, in[15] + 0xfe2ce6e0, 10);
-    MD5STEP(F4, c, d, a, b, in[6] + 0xa3014314, 15);
-    MD5STEP(F4, b, c, d, a, in[13] + 0x4e0811a1, 21);
-    MD5STEP(F4, a, b, c, d, in[4] + 0xf7537e82, 6);
-    MD5STEP(F4, d, a, b, c, in[11] + 0xbd3af235, 10);
-    MD5STEP(F4, c, d, a, b, in[2] + 0x2ad7d2bb, 15);
-    MD5STEP(F4, b, c, d, a, in[9] + 0xeb86d391, 21);
-#endif
-
-    buf[0] += a;
-    buf[1] += b;
-    buf[2] += c;
-    buf[3] += d;
+	buf[0] += a;
+	buf[1] += b;
+	buf[2] += c;
+	buf[3] += d;
 }
 
 #endif
 
 /*************** end of borrowed code ***************/
+#endif //ndef USE_GLIB2_16
 
 /**
 @brief blockwise read and hash the regular file @a filepath
 Uses file descriptors, which means files must be real, local
 Any error message here expects BGL to be open
+Returned 'string' is actually a 16-byte buffer which may contain '\0'
+before its end, followed by trailing '\0'
 @param filepath localised string, absolute path of item to read
-@return pointer to newly-allocated string containing the computed hash, or NULL on error
+@return pointer to newly-allocated 'string' containing the computed hash, or NULL on error
 */
 static guchar *_e2p_diff_dohash (VPATH *filepath)
 {
-	HASH_CONTEXT md;
+#ifdef USE_GLIB2_16
+	GChecksum *chk;
+#else
+	struct MD5Context md;
+#endif
 	guchar buffer [BUFSIZ];
-	guchar sum [HASH_LENGTH+1]; //computed hash + trailing \0
+	guint8 sum [HASH_LENGTH+1]; //computed hash + trailing \0
 	gint fdesc;
 	E2_ERR_DECLARE
 
@@ -412,7 +404,11 @@ static guchar *_e2p_diff_dohash (VPATH *filepath)
 
 	guint64 bytes_read = 0;	//used for error recovery
 
-	HASH_Init (&md);
+#ifdef USE_GLIB2_16
+	chk = g_checksum_new (G_CHECKSUM_MD5);
+#else
+	MD5Init (&md);
+#endif
 
 	while (TRUE)
 	{
@@ -421,7 +417,11 @@ static guchar *_e2p_diff_dohash (VPATH *filepath)
 		ssize_t n_read = e2_fs_read (fdesc, buffer, BUFSIZ E2_ERR_PTR());
 		if (n_read == BUFSIZ)
 		{
-			HASH_Update (&md, buffer, BUFSIZ);
+#ifdef USE_GLIB2_16
+			g_checksum_update (chk, buffer, BUFSIZ);
+#else
+			MD5Update (&md, buffer, BUFSIZ);
+#endif
 			bytes_read += n_read;
 		}
 		else if (n_read == 0)
@@ -443,7 +443,11 @@ static guchar *_e2p_diff_dohash (VPATH *filepath)
 			}
 			//non-fatal, we can continue
 			E2_ERR_CLEAR
-			HASH_Update(&md, buffer, BUFSIZ);
+#ifdef USE_GLIB2_16
+			g_checksum_update (chk, buffer, BUFSIZ);
+#else
+			MD5Update(&md, buffer, BUFSIZ);
+#endif
 			bytes_read += BUFSIZ;
 		 /* but the file pointer's now undefined. So manually advance it to the
 			next read position */
@@ -453,15 +457,26 @@ static guchar *_e2p_diff_dohash (VPATH *filepath)
 		{
 		  //end of the file, but not a full buffer
 		  //update the hash accordingly
-			HASH_Update (&md, buffer, n_read);
+#ifdef USE_GLIB2_16
+			g_checksum_update (chk, buffer, n_read);
+#else
+			MD5Update (&md, buffer, n_read);
+#endif
 			break;
 		}
 	}
 
-	HASH_Final (sum, &md);
+#ifdef USE_GLIB2_16
+	gsize len = sizeof (sum);
+	g_checksum_get_digest (chk, sum, &len);
+	g_checksum_free (chk);
+#else
+	MD5Final (sum, &md);
+#endif
 
 /* if we want to return result in human-readable form
-	static gchar hex [] = "0123456789abcdef";
+//	static
+	gchar hex [] = "0123456789abcdef";
 //	static
 	gchar result [2 * HASH_LENGTH + 1];
 	gint i;
@@ -486,7 +501,7 @@ else */
 		gdk_threads_enter ();
 		e2_output_print_error (msg, TRUE);
 		gdk_threads_leave ();
-		F_FREE (utf);
+		F_FREE (utf, filename);
 	} */
 
 	TEMP_FAILURE_RETRY (close (fdesc));	//FIXME vfs
@@ -500,7 +515,7 @@ BGL is expected to be open/off
 @param statptr pointer to info from lstat() for @a localpath
 @param localotherpath absolute path of item to be compared, localised string
 
-@return TRUE if the items are equal, (also FALSE after memory shortage)
+@return TRUE if the items are equal, (also FALSE after memory shortage or path error)
 */
 static gboolean _e2p_diff1 (VPATH *localpath, const struct stat *statptr,
 	VPATH *localotherpath)
@@ -529,7 +544,7 @@ static gboolean _e2p_diff1 (VPATH *localpath, const struct stat *statptr,
 			}
 			else
 			{
-				retval = ! strcmp ((gchar *) hash1, (gchar *) hash2);
+				retval = !memcmp (hash1, hash2, HASH_LENGTH);
 				g_free (hash1);
 				g_free (hash2);
 			}
@@ -547,9 +562,16 @@ static gboolean _e2p_diff1 (VPATH *localpath, const struct stat *statptr,
 		CHECKALLOCATEDWARNT (target1, );
 		if (target1 != NULL)
 		{
-			gint len = e2_fs_readlink (localpath, target1, PATH_MAX E2_ERR_NONE());	//FIXME vfs
-			if (len < 0) len = 0;	//hack to deal with error
-			*(target1 + len) = '\0';
+			gint len = e2_fs_readlink (localpath, target1, PATH_MAX E2_ERR_NONE());
+			if (len < 1) //error or empty
+			{
+#ifdef USE_GLIB2_10
+				g_slice_free1 (PATH_MAX, target1);
+#else
+				g_free (target1);
+#endif
+				return FALSE;
+			}
 #ifdef USE_GLIB2_10
 			target2 = g_slice_alloc (PATH_MAX);
 #else
@@ -558,9 +580,8 @@ static gboolean _e2p_diff1 (VPATH *localpath, const struct stat *statptr,
 			CHECKALLOCATEDWARNT (target2, );
 			if (target2 != NULL)
 			{
-				len = e2_fs_readlink (localotherpath, target2, PATH_MAX E2_ERR_NONE());	//FIXME vfs
-				if (len < 0) len = 0;
-				*(target2 + len) = '\0';
+				len = e2_fs_readlink (localotherpath, target2, PATH_MAX E2_ERR_NONE());
+				if (len < 0) len = 0;  //CHECKME ok to ignore error ?
 				retval = !strcmp (target1, target2);
 /*
 //the targets actually match ?
@@ -579,7 +600,7 @@ static gboolean _e2p_diff1 (VPATH *localpath, const struct stat *statptr,
 							return FALSE;
 						}
 						gchar *target2 = g_try_malloc (PATH_MAX);
-						CHECKALLOCATEDWARNT (target2, FIXMEg_free (target1); return FALSE;)
+						CHECKALLOCATEDWARNT (target2, FIXME;)
 						if (target2 != NULL)
 						{
 							g_strlcpy (target2, localotherpath, PATH_MAX);
@@ -676,7 +697,7 @@ static E2_TwResult _e2p_diff_twcb (VPATH *localpath,
 			localotherpath = e2_utils_strcat (twdata->newroot,
 				(gchar *)localpath + twdata->oldroot_len);
 #ifdef E2_VFS
-			ddata.localpath = localotherpath;
+			ddata.path = localotherpath;
 			ddata.spacedata = twdata->otherspace;
 			if (_e2p_diff1 (localpath, statptr, &ddata))
 #else
@@ -692,7 +713,7 @@ static E2_TwResult _e2p_diff_twcb (VPATH *localpath,
 			localotherpath = e2_utils_strcat (twdata->newroot,
 				(gchar *)localpath + twdata->oldroot_len);
 #ifdef E2_VFS
-			ddata.localpath = localotherpath;
+			ddata.path = localotherpath;
 			ddata.spacedata = twdata->otherspace;
 			if (e2_fs_stat (&ddata, &othrstatbuf E2_ERR_NONE())
 #else
@@ -748,17 +769,8 @@ static gpointer _e2p_diff_all (gpointer thread_data)
 		gdk_threads_enter (); //prevent thread-related hang!
 		e2_window_set_cursor (GDK_WATCH);
 		gdk_threads_leave ();
-		while (TRUE)
-		{
-			LISTS_LOCK
-			//CHECKME abort if cd detected ?
-			gboolean busy = curr_view->listcontrols.refresh_working
-						 || curr_view->listcontrols.cd_working;
-			LISTS_UNLOCK
-			if (!busy)
-				break;
-			usleep (100000);
-		}
+
+		WAIT_FOR_REFRESH(curr_view)
 
 		gchar *curr_local, *other_local, *currpath;
 		FileInfo *info;
@@ -772,8 +784,8 @@ static gpointer _e2p_diff_all (gpointer thread_data)
 		sdata.spacedata = curr_view->spacedata;
 		ddata.spacedata = other_view->spacedata;
 #endif
-		curr_local = F_FILENAME_TO_LOCALE (curr_view->dir);
-		other_local = F_FILENAME_TO_LOCALE (other_view->dir);
+		curr_local = D_FILENAME_TO_LOCALE (curr_view->dir); //always dup, to avoid (unlikely) dirchange race
+		other_local = D_FILENAME_TO_LOCALE (other_view->dir);
 		gboolean matched;
 		GtkTreeSelection *sel = curr_view->selection;
 		gdk_threads_enter ();
@@ -787,7 +799,7 @@ static gpointer _e2p_diff_all (gpointer thread_data)
 			gtk_tree_model_get (model, &iter, FINFO, &info, -1);
 			data.newroot = e2_utils_strcat (other_local, info->filename);
 #ifdef E2_VFS
-			ddata.localpath = data.newroot;
+			ddata.path = data.newroot;
 			if (e2_fs_lstat (&ddata, &othersb E2_ERR_NONE()))
 #else
 			if (e2_fs_lstat (data.newroot, &othersb E2_ERR_NONE()))
@@ -800,7 +812,7 @@ static gpointer _e2p_diff_all (gpointer thread_data)
 				{
 					data.oldroot_len = strlen (currpath);
 #ifdef E2_VFS
-					sdata.localpath = currpath;
+					sdata.path = currpath;
 					matched = e2_fs_tw (&sdata, _e2p_diff_twcb, &data, -1,
 #else
 					matched = e2_fs_tw (currpath, _e2p_diff_twcb, &data, -1,
@@ -810,7 +822,7 @@ static gpointer _e2p_diff_all (gpointer thread_data)
 				else if (!S_ISDIR (info->statbuf.st_mode) && !S_ISDIR (othersb.st_mode))
 #ifdef E2_VFS
 				{
-					sdata.localpath = currpath;
+					sdata.path = currpath;
 					matched = _e2p_diff1 (&sdata, &info->statbuf, &ddata);
 				}
 #else
@@ -832,8 +844,8 @@ static gpointer _e2p_diff_all (gpointer thread_data)
 			g_free (data.newroot);
 		} while (gtk_tree_model_iter_next (model, &iter));
 
-		F_FREE (curr_local);
-		F_FREE (other_local);
+		g_free (curr_local);
+		g_free (other_local);
 
 #ifdef E2_VFS
 		//cleanup data.operr	//or something ...
@@ -881,12 +893,14 @@ gboolean init_plugin (Plugin *p)
   p->icon = "plugin_"ANAME E2ICONTB;  //prepend path if appropriate
 
   if (p->action == NULL)
- {
-	//no need to free this
-	gchar *action_name = g_strconcat (_A(14),".",aname,NULL);
-	p->action = e2_plugins_action_register
-	 (action_name, E2_ACTION_TYPE_ITEM, _e2p_diff, NULL, FALSE, 0, NULL);
-	  return TRUE;
+  {
+	//don't free name string here
+	E2_Action plugact =
+	{g_strconcat (_A(14),".",aname,NULL),_e2p_diff,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+	p->action = e2_plugins_action_register (&plugact);
+	if (p->action != NULL)
+		return TRUE;
+	g_free (plugact.name);
   }
   return FALSE;
 }
