@@ -1,6 +1,6 @@
-/* $Id: e2p_rename.c 1412 2009-02-01 21:51:22Z tpgww $
+/* $Id: e2p_rename.c 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2005-2009 tooar <tooar@gmx.net>
+Copyright (C) 2005-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -586,15 +586,15 @@ static E2_TwResult _e2p_ren_twcb (VPATH *localpath,
 	{
 		utf = F_DISPLAYNAME_FROM_LOCALE (dirpath);
 		msg = g_strdup_printf (_("Cannot open directory %s"), utf);
+		F_FREE (utf, dirpath);
 		e2_output_print_error (msg, TRUE);
-		F_FREE (utf);
 		return;
 	}
 
 	while (!e2_fs_dir_read (dp, &entrybuf, &entry E2_ERR_NONE()) && entry != NULL)	//FIXME vfs
 	{
 		if (ITEM_ISHIDDEN(entry->d_name)
-			&& (entry->d_name[1] == '\0' || g_str_equal (entry->d_name, ".."))
+			&& (entry->d_name[1] == '\0' || !strcmp (entry->d_name, ".."))
 			)
 			continue;
 		if (_e2p_ren_match_name (entry->d_name, data))
@@ -624,7 +624,13 @@ static void _e2p_ren_rename (E2_RenDialogRuntime *rt)
 	const gchar *old, *new;
 	if (!_e2p_ren_get_flag (OLD_SEL_P))	//, rt))
 	{
-		old = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (rt->pattern)->child));
+		old = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+			GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->pattern)))
+#else
+			GTK_ENTRY (GTK_BIN (rt->pattern)->child)
+#endif
+			);
 		if (*old == '\0')
 		{
 			e2_output_print_error (_("No current name pattern is specified"), FALSE);
@@ -637,7 +643,13 @@ static void _e2p_ren_rename (E2_RenDialogRuntime *rt)
 
 	if (_e2p_ren_get_flag (NEW_THIS_P))	//, rt))
 	{
-		new = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (rt->newpattern)->child));
+		new = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+			GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->newpattern)))
+#else
+			GTK_ENTRY (GTK_BIN (rt->newpattern)->child)
+#endif
+			);
 		if (*new == '\0')
 		{
 			e2_output_print_error (_("No replacement name pattern is specified"), FALSE);
@@ -736,7 +748,13 @@ static void _e2p_ren_rename (E2_RenDialogRuntime *rt)
 # endif
 			sdata.spacedata = curr_view->spacedata;
 #endif
-			startdir = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (rt->directory)->child));
+			startdir = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+				GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->directory)))
+#else
+				GTK_ENTRY (GTK_BIN (rt->directory)->child)
+#endif
+			);
 			startdir = e2_utils_pass_whitespace ((gchar *)startdir);
 			if (startdir == NULL)
 			{
@@ -747,9 +765,21 @@ static void _e2p_ren_rename (E2_RenDialogRuntime *rt)
 				if (*startdir != G_DIR_SEPARATOR)
 				{
 					tmp2 = e2_utils_strcat (curr_view->dir, startdir);
-					gtk_entry_set_text (GTK_ENTRY (GTK_BIN (rt->directory)->child), tmp2);
+					gtk_entry_set_text (
+#ifdef USE_GTK2_14
+						GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->directory))),
+#else
+						GTK_ENTRY (GTK_BIN (rt->directory)->child),
+#endif
+						tmp2);
 					g_free (tmp2);
-					startdir = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (rt->directory)->child));
+					startdir = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+						GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->directory)))
+#else
+						GTK_ENTRY (GTK_BIN (rt->directory)->child)
+#endif
+						);
 				}
 				e2_list_update_history (startdir, &dir_history, NULL, 20, FALSE);
 			}
@@ -791,7 +821,7 @@ static void _e2p_ren_rename (E2_RenDialogRuntime *rt)
 					g_free (tmp2);
 				}
 #ifdef E2_VFS
-				sdata.localpath = localroot;
+				sdata.path = localroot;
 #endif
 				if (_e2p_ren_get_flag (SEARCH_SUBDIRS_P))	//, rt))
 				{
@@ -823,7 +853,12 @@ static void _e2p_ren_rename (E2_RenDialogRuntime *rt)
 				//restore real flag state
 				if (_e2p_ren_get_flag (SEARCH_ALL_P))	//, rt))
 				{
-					gboolean oldstate = GTK_TOGGLE_BUTTON (rt->recurse_button)->active;
+					gboolean oldstate =
+#ifdef USE_GTK2_14
+						gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->recurse_button));
+#else
+						GTK_TOGGLE_BUTTON (rt->recurse_button)->active;
+#endif
 					_e2p_ren_set_flag (SEARCH_SUBDIRS_P, oldstate);	//, rt);
 				}
 
@@ -843,6 +878,9 @@ static void _e2p_ren_rename (E2_RenDialogRuntime *rt)
 		gtk_widget_set_sensitive (rt->help_button, TRUE);
 		gtk_widget_set_sensitive (rt->start_button, TRUE);
 		gtk_widget_set_sensitive (rt->stop_button, FALSE);
+//#ifndef USE_GLIB2_22
+		g_ptr_array_foreach (data.candidates, (GFunc)g_free, NULL);
+//#endif
 		g_ptr_array_free (data.candidates, TRUE);
 		WAIT_FOR_EVENTS
 		*rt->status = E2_TASK_PAUSED;
@@ -967,11 +1005,13 @@ static void _e2p_ren_rename (E2_RenDialogRuntime *rt)
 			GtkWidget *label = g_object_get_data (G_OBJECT (dialog), "e2-dialog-label");
 			gtk_label_set_markup (GTK_LABEL (label), prompt);
 			g_free (prompt);
-			gtk_widget_show (dialog);
+
+			e2_dialog_run (dialog, NULL, 0);
+
 			*rt->status = E2_TASK_PAUSED;
 			choice = e2_dialog_wait (dialog, TRUE, TRUE, multi, TRUE);
 			*rt->status = E2_TASK_RUNNING;
-			gtk_widget_hide (dialog);
+			gtk_widget_hide (dialog); //CHECKME if closed by user
 			gtk_widget_grab_focus (rt->dialog);
 		}
 		else	//no confirmation
@@ -997,13 +1037,13 @@ static void _e2p_ren_rename (E2_RenDialogRuntime *rt)
 				//maybe new name exists, or maybe it's just the old name with some different case
 				gchar *old_name_lc = g_utf8_casefold (*thisone, -1);
 				gchar *new_name_lc = g_utf8_casefold (newpath, -1);
-				if (!g_str_equal (old_name_lc, new_name_lc))
+				if (strcmp (old_name_lc, new_name_lc))
 				{	//we may have just a case-difference that looks the same to the kernel
 					//do interim name change to check, & also to ensure case-change happens
 					slocal = F_FILENAME_TO_LOCALE (*thisone);
 					tempname = e2_utils_get_tempname (slocal);
 #ifdef E2_VFS
-					tdata.localpath = tempname;
+					tdata.path = tempname;
 #endif
 					gdk_threads_leave ();	//downstream errors invoke local mutex locking
 #ifdef E2_VFS
@@ -1045,7 +1085,7 @@ static void _e2p_ren_rename (E2_RenDialogRuntime *rt)
 					{
 						gdk_threads_leave ();	//downstream errors invoke local mutex locking
 #ifdef E2_VFS
-						tdata.localpath = *thisone;
+						tdata.path = *thisone;
 						e2_fs_error_simple (_("Cannot rename %s"), &tdata);
 #else
 						e2_fs_error_simple (_("Cannot rename %s"), *thisone);
@@ -1053,14 +1093,23 @@ static void _e2p_ren_rename (E2_RenDialogRuntime *rt)
 						gdk_threads_enter ();
 						choice = CANCEL;
 					}
-					F_FREE (slocal);
+					F_FREE (slocal, *thisone);
 					g_free (tempname);
 				}
-		/*			else	//we have a real difference
+/*				else	//we have a real difference
 				{	//FIXME duplication here
-					e2_filelist_enable_refresh ();  //allow updates while we wait
-					doit = e2_dialog_ow_check (*thisone, dlocal, NOALL);
-					e2_filelist_disable_refresh ();
+					gdk_threads_leave ();
+					doit = e2_dialog_ow_check (
+#ifdef E2_VFS
+# ifdef E2_VFSTMP
+						&tdata??,
+# endif
+						&ddata,
+#else
+						*thisone, dlocal,
+#endif
+						NOALL);
+					gdk_threads_enter ();
 				} */
 				g_free (old_name_lc);
 				g_free (new_name_lc);
@@ -1069,7 +1118,7 @@ static void _e2p_ren_rename (E2_RenDialogRuntime *rt)
 			{
 				gdk_threads_leave ();	//downstream errors invoke local mutex locking
 #ifdef E2_VFS
-				tdata.localpath = *thisone;
+				tdata.path = *thisone;
 				success = e2_task_backend_rename (&tdata, &ddata);
 #else
 				success = e2_task_backend_rename (*thisone, dlocal);
@@ -1084,7 +1133,7 @@ static void _e2p_ren_rename (E2_RenDialogRuntime *rt)
 				}
 			}
 			g_free (newpath);
-			F_FREE (dlocal);
+			F_FREE (dlocal, newpath);
 		}
 
 		g_free (base);
@@ -1102,7 +1151,7 @@ static void _e2p_ren_rename (E2_RenDialogRuntime *rt)
 		thisone++;
 	}	//end of items loop
 
-	if (dialog != NULL)
+	if (GTK_IS_DIALOG(dialog))	//not explicitly closed by the user
 		gtk_widget_destroy (dialog);
 	*rt->status = E2_TASK_PAUSED;
 	//we don't know what really needs refreshing ...
@@ -1134,7 +1183,14 @@ static void _e2p_ren_rename (E2_RenDialogRuntime *rt)
 	gtk_widget_set_sensitive (rt->stop_button, FALSE);
 
 	gtk_window_present (GTK_WINDOW (rt->dialog));
-//	gtk_widget_grab_focus (GTK_BIN (rt->pattern)->child);
+/*	gtk_widget_grab_focus (
+#ifdef USE_GTK2_14
+		gtk_bin_get_child (GTK_BIN (rt->pattern))
+#else
+		GTK_BIN (rt->pattern)->child
+#endif
+	);
+*/
 }
 
   /*********************/
@@ -1161,8 +1217,14 @@ static void _e2p_ren_choose_directory_cb (GtkFileChooser *chooser, E2_RenDialogR
 			if (*dirpath != '\0')
 			{
 				gchar *opendir = F_FILENAME_FROM_LOCALE (dirpath);
-				gtk_entry_set_text (GTK_ENTRY (GTK_BIN (rt->directory)->child), opendir);
-				F_FREE (opendir);
+				gtk_entry_set_text (
+#ifdef USE_GTK2_14
+					GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->directory))),
+#else
+					GTK_ENTRY (GTK_BIN (rt->directory)->child),
+#endif
+					opendir);
+				F_FREE (opendir, dirpath);
 			}
 			g_free (dirpath);
 		}
@@ -1184,12 +1246,7 @@ static gboolean _e2p_ren_add_chooser (E2_RenDialogRuntime *rt)
 	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (rt->chooser_button), rt->chooser_startdir);
 	g_free (rt->chooser_startdir);
 	rt->chooser_startdir = NULL;
-# ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-# else
-	e2_widget_set_tooltip (NULL,
-# endif
-		rt->chooser_button, message);
+	e2_widget_set_safetip (rt->chooser_button, message);
 	g_signal_connect (G_OBJECT (rt->chooser_button), "current-folder-changed",
 		G_CALLBACK (_e2p_ren_choose_directory_cb), rt);
 
@@ -1205,17 +1262,21 @@ static gboolean _e2p_ren_add_chooser (E2_RenDialogRuntime *rt)
 /**
 @brief toggle specified option flag
 
-@param button clicked button widget
+@param button toggled button widget
 @param flagnum pointerized number of the flag to be toggled
 
 @return
 */
-static void _e2p_ren_toggle_cb (GtkWidget *button, gpointer flagnum)
+static void _e2p_ren_toggle_cb (GtkToggleButton *button, gpointer flagnum)
 {
 	E2_RenDialogRuntime *rt = g_object_get_data (G_OBJECT (button), "e2-runtime");
 	//if this if this is during setup, before a widget is created ...
+#ifdef USE_GTK2_99
+	if (!gtk_widget_get_mapped (rt->dialog))
+#else
 	if (!GTK_WIDGET_MAPPED (rt->dialog))
-	  return;
+#endif
+		return;
 
 	renflag_t flg = (renflag_t) flagnum;
 	gboolean newflag = ! _e2p_ren_get_flag (flg);	//, rt);
@@ -1239,7 +1300,13 @@ static void _e2p_ren_toggle_cb (GtkWidget *button, gpointer flagnum)
 		  if (newflag)
 		  {
 			gtk_widget_set_sensitive (rt->pattern, TRUE);
-			gtk_widget_grab_focus (GTK_BIN (rt->pattern)->child);
+			gtk_widget_grab_focus (
+#ifdef USE_GTK2_14
+				gtk_bin_get_child (GTK_BIN (rt->pattern))
+#else
+				GTK_BIN (rt->pattern)->child
+#endif
+			);
 		  }
 		  break;
 		case SEARCH_ALL_P:
@@ -1258,13 +1325,25 @@ static void _e2p_ren_toggle_cb (GtkWidget *button, gpointer flagnum)
 		  {
 			if (_e2p_ren_get_flag (OLD_SEL_P))
 			  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rt->wild_button), TRUE);
-			gtk_widget_grab_focus (GTK_BIN (rt->directory)->child);
+			gtk_widget_grab_focus (
+#ifdef USE_GTK2_14
+				gtk_bin_get_child (GTK_BIN (rt->directory))
+#else
+				GTK_BIN (rt->directory)->child
+#endif
+			);
 		  }
 		  break;
 		case NEW_THIS_P:
 		  gtk_widget_set_sensitive (rt->newpattern, newflag);
 		  if (newflag)
-			gtk_widget_grab_focus (GTK_BIN (rt->newpattern)->child);
+			gtk_widget_grab_focus (
+#ifdef USE_GTK2_14
+				gtk_bin_get_child (GTK_BIN (rt->newpattern))
+#else
+				GTK_BIN (rt->newpattern)->child
+#endif
+		  );
 		  break;
 		default:
 		  break;
@@ -1273,12 +1352,12 @@ static void _e2p_ren_toggle_cb (GtkWidget *button, gpointer flagnum)
 /**
 @brief toggle specified option flag
 
-@param button clicked button widget
+@param button toggled button widget
 @param flagnum pointerized number of the flag to be toggled
 
 @return
 */
-static void _e2p_ren_grouptoggle_cb (GtkWidget *button, gpointer flagnum)
+static void _e2p_ren_grouptoggle_cb (GtkToggleButton *button, gpointer flagnum)
 {
 	renflag_t flg = (renflag_t) flagnum;
 //	E2_RenDialogRuntime *rt = g_object_get_data (G_OBJECT (button), "e2-runtime");
@@ -1286,13 +1365,15 @@ static void _e2p_ren_grouptoggle_cb (GtkWidget *button, gpointer flagnum)
 	_e2p_ren_set_flag (flg, newflag);	//, rt);
 	if (newflag)
 	{	//clear all other members of the group
-		GtkWidget *tmp = g_object_get_data (G_OBJECT (button), "group_leader");
-		GSList *members = g_object_get_data (G_OBJECT (tmp), "group_members");
-		for (;members != NULL; members = members->next)
+		GtkToggleButton *tmp = (GtkToggleButton *)g_object_get_data
+			(G_OBJECT (button), "group_leader");
+		GSList *members;
+		for (members = g_object_get_data (G_OBJECT (tmp), "group_members");
+			 members != NULL; members = members->next)
 		{
-			tmp = (GtkWidget *)members->data;
+			tmp = (GtkToggleButton *)members->data;
 			if (tmp != button)
-			  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tmp), FALSE);
+				gtk_toggle_button_set_active (tmp, FALSE);
 		}
 	}
 }
@@ -1385,18 +1466,18 @@ static void _e2p_ren_response_cb (GtkDialog *dialog, gint response,
 @param box the widget into which the button is to be placed
 @param label  translated string for the button label
 @param state T/F initial state of the toggle
-@param callback_fn the button's callback function
+@param callback the button's "toggled" signal callback function
 @param f enumerated value of flag to be associated with the button
 @param rt ptr to dialog data struct
 
 @return the button widget (UNUSED, now)
 */
 static GtkWidget *__e2p_ren_create_toggle_button (GtkWidget *box,
-	gchar *label, gboolean state, gpointer callback_fn,
+	gchar *label, gboolean state, void (*callback)(GtkToggleButton*,gpointer),
 	renflag_t f, E2_RenDialogRuntime *rt)
 {
   GtkWidget *button = e2_button_add_toggle (box, TRUE, state, label,
-	NULL, FALSE, E2_PADDING_XSMALL, callback_fn, (gpointer) f);
+	NULL, FALSE, E2_PADDING_XSMALL, callback, (gpointer) f);
   g_object_set_data (G_OBJECT (button), "e2-runtime", rt);
   //cached flags default to FALSE
 //  if (state)
@@ -1517,7 +1598,6 @@ static GtkWidget *_e2p_ren_create_filesel (gchar *title, void (*ok)(GtkWidget *,
 		     "clicked", G_CALLBACK (ok), rt);
   g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (wid)->cancel_button),
 		     "clicked", G_CALLBACK (cancel), rt);
-  gtk_widget_show (wid);
   return wid;
 } */
 /**
@@ -1544,7 +1624,7 @@ static gboolean _e2p_renameQ (E2_ActionTaskData *qed)
 	*qed->status = E2_TASK_PAUSED;
 	//create dialog
 	rt.dialog = e2_dialog_create (NULL, NULL, _("rename items"),
-		_e2p_ren_response_cb, &rt);
+		(ResponseFunc)_e2p_ren_response_cb, &rt);
 
 	//populate it with widgets
 	GtkWidget *dialog_vbox, *hbox;
@@ -1587,32 +1667,29 @@ static gboolean _e2p_renameQ (E2_ActionTaskData *qed)
 	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (rt.chooser_button), local);
 	if (s > local && *s == '\0')
 		*s = G_DIR_SEPARATOR;
-# ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-# else
-	e2_widget_set_tooltip (NULL,
-# endif
-		rt.chooser_button, message);
+	e2_widget_set_safetip (rt.chooser_button, message);
 	g_signal_connect (G_OBJECT (rt.chooser_button), "current-folder-changed",
 		G_CALLBACK (_e2p_rename_choose_directory_cb), &rt);
 
 	gtk_box_pack_end (GTK_BOX (hbox), rt.chooser_button, FALSE, FALSE, 0);
 	gboolean state = _e2p_ren_get_flag (SEARCH_THIS_P);
 	gtk_widget_set_sensitive (rt.chooser_button, state);
-	gtk_widget_show (rt.chooser_button);
 #endif
 */
-	gdk_threads_enter ();	//reduce delay when keying in data (dunno why)
 	rt.directory = e2_combobox_add (dialog_vbox, FALSE, E2_PADDING_XSMALL,
-		_e2p_ren_activation_cb, &rt, &dir_history,
+		(ActivateFunc)_e2p_ren_activation_cb, &rt, &dir_history,
 		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_FOCUS_ON_CHANGE);
 //#ifndef E2_RENCHOOSER
 	gboolean state = _e2p_ren_get_flag (SEARCH_THIS_P);
 //#endif
 	gtk_widget_set_sensitive (rt.directory, state);	//later toggled if corresponsing radio is selected
-	gdk_threads_leave ();
 	//handle path completion
-	g_signal_connect (G_OBJECT (GTK_BIN (rt.directory)->child),
+	g_signal_connect (G_OBJECT
+#ifdef USE_GTK2_14
+		(gtk_bin_get_child (GTK_BIN (rt.directory))),
+#else
+		(GTK_BIN (rt.directory)->child),
+#endif
 		"key-press-event", G_CALLBACK (_e2p_ren_key_press2_cb), NULL);
 
 #ifdef E2_RENCHOOSER
@@ -1644,15 +1721,19 @@ static gboolean _e2p_renameQ (E2_ActionTaskData *qed)
 	e2_widget_add_mid_label (hbox, _("Current name is like this:"), 0.0, FALSE, E2_PADDING);
 	GtkSizeGroup *same = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 	gtk_size_group_add_widget (same, wid);
-	gdk_threads_enter ();
 
 	rt.pattern = e2_combobox_add (hbox, TRUE, E2_PADDING_XSMALL,
-		_e2p_ren_activation_cb, &rt, &pattern_history,
+		(ActivateFunc)_e2p_ren_activation_cb, &rt, &pattern_history,
 		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_FOCUS_ON_CHANGE);
-	gtk_entry_set_text (GTK_ENTRY (GTK_BIN (rt.pattern)->child), "(.*)");
+	gtk_entry_set_text (
+#ifdef USE_GTK2_14
+		GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt.pattern))),
+#else
+		GTK_ENTRY (GTK_BIN (rt.pattern)->child),
+#endif
+		"(.*)");
 	state = _e2p_ren_get_flag (OLD_SEL_P);
 	gtk_widget_set_sensitive (rt.pattern, !state);	//later toggled if corresponsing radio is selected
-	gdk_threads_leave ();
 
 	e2_widget_add_separator (dialog_vbox, TRUE, 0);
 
@@ -1675,7 +1756,7 @@ static gboolean _e2p_renameQ (E2_ActionTaskData *qed)
 	gtk_size_group_add_widget (same, wid);
 	g_object_unref (G_OBJECT (same));
 	rt.newpattern = e2_combobox_add (hbox, TRUE, E2_PADDING_XSMALL,
-		_e2p_ren_activation_cb, &rt, &newpattern_history,
+		(ActivateFunc)_e2p_ren_activation_cb, &rt, &newpattern_history,
 		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_FOCUS_ON_CHANGE);
 	state = _e2p_ren_get_flag (NEW_THIS_P);
 	gtk_widget_set_sensitive (rt.newpattern, state);	//later toggled if corresponsing radio is selected
@@ -1705,14 +1786,21 @@ static gboolean _e2p_renameQ (E2_ActionTaskData *qed)
 	e2_dialog_set_negative_response (rt.dialog, E2_RESPONSE_NOTOALL);
 
 	if (!_e2p_ren_get_flag (OLD_SEL_P))
-		gtk_widget_grab_focus (GTK_BIN (rt.pattern)->child);
+		gtk_widget_grab_focus (
+#ifdef USE_GTK2_14
+			gtk_bin_get_child (GTK_BIN (rt.pattern))
+#else
+			GTK_BIN (rt.pattern)->child
+#endif
+		);
 
+	gdk_threads_enter ();
 	e2_dialog_setup (rt.dialog, app.main_window);
 	//block actions Q until user closes
-	e2_dialog_run (rt.dialog, NULL,
-		E2_DIALOG_BLOCKED | E2_DIALOG_CLOSELOCK | E2_DIALOG_FREE);
+	e2_dialog_run (rt.dialog, NULL,	E2_DIALOG_BLOCKED | E2_DIALOG_FREE);
 
 //	gtk_widget_grab_focus (curr_view->treeview);
+	gdk_threads_leave ();
 
 	return TRUE;
 }
@@ -1738,6 +1826,15 @@ gboolean init_plugin (Plugin *p)
 
 	if (p->action == NULL)
 	{
+		//don't free name string here
+		E2_Action plugact =
+		{g_strconcat (_A(1),".",aname,NULL),_e2p_rename_dialog_create,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+		p->action = e2_plugins_action_register (&plugact);
+		if G_UNLIKELY((p->action == NULL))
+		{
+			g_free (plugact.name);
+			return FALSE;
+		}
 		dir_history = (GList *)g_new0 (gpointer, 1);
 		pattern_history = (GList *)g_new0 (gpointer, 1);
 		newpattern_history = (GList *)g_new0 (gpointer, 1);
@@ -1754,10 +1851,6 @@ gboolean init_plugin (Plugin *p)
 		e2_cache_list_register ("rename-dir-history", &dir_history);
 		e2_cache_list_register ("rename-oldpattern-history", &pattern_history);
 		e2_cache_list_register ("rename-newpattern-history", &newpattern_history);
-		//no need to free this
-		gchar *action_name = g_strconcat (_A(1),".",aname,NULL);
-		p->action = e2_plugins_action_register
-		  (action_name, E2_ACTION_TYPE_ITEM, _e2p_rename_dialog_create, NULL, FALSE, 0, NULL);
 		return TRUE;
 	}
 	return FALSE;
