@@ -1,6 +1,6 @@
-/* $Id: e2p_unpack.c 1367 2008-12-31 09:18:19Z tpgww $
+/* $Id: e2p_unpack.c 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -32,12 +32,6 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 //same enum as in pack plugin, though it need not be so
 enum { TAR_GZ, TAR_BZ2, TAR_LZMA, TAR, /*DEB, RPM, */ZIP, Z7Z, RAR, ARJ, ZOO, MAXTYPES };
 
-//tag E2_BADQUOTES
-#define UNPACKPATH "%s"
-//#else
-//# define UNPACKPATH "\"%s\""
-//#endif
-
 typedef struct _E2P_Unpackdata
 {
 	gchar *package;	//absolute path of source archive, UTF-8 string
@@ -264,7 +258,7 @@ static gboolean _e2p_unpack_delete_dir (E2P_Unpackdata *data)
 #endif
 
 	e2_filelist_enable_refresh ();
-	F_FREE (local);
+	F_FREE (local, data->workdir);
 	_e2p_unpack_cleanup (data);
 	//FIXME different refresh approach with E2_ASYNC
 	//in case a pane shows parent of temp dir
@@ -289,17 +283,20 @@ static void _e2p_unpack_clear (E2P_Unpackdata *data)
 	data->pack_id = g_idle_add_full (G_PRIORITY_LOW,
 		(GSourceFunc)_e2p_unpack_delete_dir, data, NULL);
 }
-/**
+/* *
 @brief timer callback to periodically check whether a repack is completed
-
+Needed if repack command does not include cleanup
 @param data pointer to plugin data struct
 
 @return TRUE if the repack process is still running
 */
-static gboolean _e2p_unpack_clean_dir (E2P_Unpackdata *data)
+/*static gboolean _e2p_unpack_clean_dir (E2P_Unpackdata *data)
 {
 	if (e2_command_find_process ((guint)data->thispid))
 		return TRUE;	//wait some more
+	//FIXME only delete dir if repack command succeeded, or else don't delete
+	//original archive until repack succeeded
+
 	//now we're ready for cleanup
 //	g_source_remove (data->pack_id);
 //	if (_e2p_unpack_delete_dir (data))
@@ -308,6 +305,7 @@ static gboolean _e2p_unpack_clean_dir (E2P_Unpackdata *data)
 			(GSourceFunc)_e2p_unpack_delete_dir, data, NULL);
 	return FALSE;
 }
+*/
 /**
 @brief repack the temp dir
 This is executed inside a callback with BGL closed
@@ -320,27 +318,38 @@ static void _e2p_unpack_repack (E2P_Unpackdata *data, gpointer from)
 {
 #ifdef E2_VFSTMP
 	if (curr_view->spacedata != NULL)	//CHECKME fuse ok?
-	  return;
+	{
+		_e2p_unpack_cleanup (data);
+		return;
+	}
 #endif
-	static gchar *cmd_str [MAXTYPES] =
+	const gchar *pack_str [MAXTYPES] =
 	{	//these command strings are in same order as enum
-		">tar cf - . | gzip - > "UNPACKPATH,
-		">tar cf - . | bzip2 - > "UNPACKPATH,	//default block size -9 is not significantly better, maybe use -3 instead ?
-		">tar cf - . | lzma - > "UNPACKPATH,	//default compresssion -7, maybe use -2 instead ?
-		"tar cf "UNPACKPATH" .",
+		//they are all designed to be executed from the unpack temp dir, and
+		//on all its contents (recursive if appropriate)
+		//CHECKME "." always remains correct during repack execution ?
+		">tar cf - . | gzip - > %s",
+		">tar cf - . | bzip2 - > %s",	//default block size -9 is not significantly better, maybe use -3 instead ?
+		">tar cf - . | lzma - > %s",	//default compresssion -7, maybe use -2 instead ?
+		"tar cf %s .",
 //		deb 'pack' command ?
 //		rpm 'pack' command does not exist
-		"zip -r "UNPACKPATH" .",
-		"7za a -t7z "UNPACKPATH" .",	//CHECKME
-		"rar u -ol "UNPACKPATH" .",
-		"arj u -al "UNPACKPATH" .",
-		"zoo ahP "UNPACKPATH" ."	//FIXME replace, not add ?
+		">zip -r - . > %s",
+		">7za a -t7z -so . > %s",	//CHECKME
+		//updates of archives probably don't capture renames,deletes
+		"rar u -as -ol -tl -r %s .", //OK
+
+		"arj a -u -r -s -a -2s %s .", //CHECKME
+		"zoo unP %s ."	//CHECKME replace, not update ? deletions ?
 	};
 /*
 compress ANSI files: 7za a -tzip archive.zip file1 file2 ... fileN
 compress ANSI dir:   7za a -tzip archive.zip dirnametocompress\
 compress UNICODE files: 7za a -t7z archive.7z file1 file2 ... fileN
 compress UNICODE dir:   7za a -t7z archive.7z dirnametocompress\
+To backup directories you must use tar :
+   - to backup a directory : tar cf - directory | 7za a -si directory.tar.7z
+   - to restore your backup : 7za x -so directory.tar.7z | tar xf -
 
 ar -x package-file.deb
 a deb top-level package contains a small file with a version number,
@@ -350,45 +359,93 @@ install/uninstall scripts, the other the files to be installed.
 	gchar *package = data->package;
 #ifdef E2_VFSTMP
 	VPATH ddata;
-	ddata.locapath = F_FILENAME_TO_LOCALE (package);
+	ddata.path = F_FILENAME_TO_LOCALE (package);
 	ddata.spacedata = curr_view->spacedata; //FIXME
 	gint index = _e2p_unpack_match_type (&ddata);
-	F_FREE (ddata.locapath);
 #else
 	gchar *local = F_FILENAME_TO_LOCALE (package);
 	gint index = _e2p_unpack_match_type ((VPATH*)local);
-	F_FREE (local);
 #endif
-	if (index == -1)
-		return;	//should never happen
-
+	gchar *fmt, *qp;
 	g_free (data->command);
+
+	switch (index)
+	{
+		case TAR_GZ:
+		case TAR_BZ2:
+		case TAR_LZMA:
+		case TAR:
+		case ZIP:
+		case Z7Z:
+			//these types of archive are overwritten
+			fmt = g_strconcat (pack_str [index], " && mv -f %s %s && rm -rfd %s", NULL);
+			qp = e2_utils_quote_string (package);
+			gchar *tmp = e2_utils_get_tempname
+#ifdef E2_VFSTMP
+				(ddata.path);
+#else
+				(local);
+#endif
+			gchar *utftmp = F_FILENAME_FROM_LOCALE (tmp);
 //tag E2_BADQUOTES
-	gchar *qp = e2_utils_quote_string (package);
-	data->command = g_strdup_printf (cmd_str [index], qp);
+			gchar *qpt = e2_utils_quote_string (utftmp);
+			g_free (tmp);
+			F_FREE (utftmp, tmp);
+			data->command = g_strdup_printf (fmt, qpt, qpt, qp, data->workdir);
+			g_free (qpt);
+			break;
+		case RAR:
+		case ARJ:
+		case ZOO:
+			//these types of archive are just updated
+			fmt = g_strconcat (pack_str [index], " && rm -rfd %s", NULL);
+			qp = e2_utils_quote_string (package);
+			data->command = g_strdup_printf (fmt, qp, data->workdir, NULL);
+//			printd (DEBUG, "repack command %s", data->command);
+			break;
+		default:
+		//case -1:	//should never happen
+			fmt = qp = NULL;	//warning prevention
+			_e2p_unpack_cleanup (data);
+			return;
+	}
+
+	g_free (fmt);
 	g_free (qp);
-	//command strings are all designed to be executed from the
-	//unpack temp dir, and on all its contents
+
+#ifdef E2_VFSTMP
+	F_FREE (ddata.path, package);
+#else
+	F_FREE (local, package);
+#endif
+
 	gint res = e2_command_run_at (data->command, data->workdir,
 		E2_COMMAND_RANGE_DEFAULT, from
 #ifdef E2_COMMANDQ
 		, T/F ?
 #endif
 	);
-	//FIXME race here if something else is run at a bad time, so find the pid
-	//with matching command string instead
 	if (res == 0)
 	{
+/* use this if command doesn't include cleanups i.e. rm workdir
+	//FIXME race here if something else is run at a bad time, so find the pid
+	//with matching command-string instead
 		E2_TaskRuntime *td = e2_task_find_last_running_child (TRUE);
 		data->thispid = (td != NULL) ? td->pid : 0;
+		//periodically check whether re-build finished, when so, cleanup the temp dir
+		//CHECKME make this timer cancellable at session end
+		data->pack_id = g_timeout_add (500, (GSourceFunc) _e2p_unpack_clean_dir, data);
+		//CHECKME refreshing etc may move CWD away from the temp dir while the repack is underway
+*/
+		_e2p_unpack_cleanup (data); //or else, just cleanup data
 	}
 	else
-		data->thispid = 0;
-	//CHECKME refreshing may move CWD away from the temp dir while the repack is underway
-
-	//periodically check whether re-build finished, when so, cleanup the temp dir
-	//CHECKME make this timer cancellable at session end
-	data->pack_id = g_timeout_add (500, (GSourceFunc) _e2p_unpack_clean_dir, data);
+	{
+//		data->thispid = 0;
+		printd (WARN, "repack command cannot be run");
+		//WARN user ?
+		_e2p_unpack_cleanup (data);
+	}
 }
 /**
 @brief callback for "what-to-do" dialog's "response" signal
@@ -404,10 +461,10 @@ static void _e2p_unpack_response_decode_cb (GtkDialog *dialog, gint response,
 {
 	gtk_widget_destroy (GTK_WIDGET(dialog));
 	//do this outside of current hook func, as we need to check both panes and data
-	e2_hook_unregister (&app.pane1.hook_change_dir, _e2p_unpack_change_dir_hook,
-		data, TRUE);
-	e2_hook_unregister (&app.pane2.hook_change_dir, _e2p_unpack_change_dir_hook,
-		data, TRUE);
+	e2_hook_unregister (&app.pane1.hook_change_dir,
+		(HookFunc)_e2p_unpack_change_dir_hook, data, TRUE);
+	e2_hook_unregister (&app.pane2.hook_change_dir,
+		(HookFunc)_e2p_unpack_change_dir_hook, data, TRUE);
 
 	switch (response)
 	{
@@ -461,7 +518,7 @@ static gboolean _e2p_unpack_change_dir_hook (gchar *path, E2P_Unpackdata *data)
 	//ask user what to do with the unpacked items
 	GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
 		_("What do you want to do with the unpacked item(s) ?"), NULL,
-		_e2p_unpack_response_decode_cb, data);
+		(ResponseFunc)_e2p_unpack_response_decode_cb, data);
 
 	e2_dialog_add_simple_button (dialog, GTK_STOCK_CLEAR,
 		_("Re_pack"), E2_RESPONSE_USER1);
@@ -470,8 +527,8 @@ static gboolean _e2p_unpack_change_dir_hook (gchar *path, E2P_Unpackdata *data)
 	GtkWidget *button = e2_dialog_add_simple_button (dialog, GTK_STOCK_DELETE,
 		_("_Delete"), E2_RESPONSE_REMOVE);
 
-	e2_dialog_setup (dialog, app.main_window);
 	gdk_threads_enter ();
+	e2_dialog_setup (dialog, app.main_window);
 	e2_dialog_run (dialog, NULL, 0);
 	gtk_widget_grab_focus (button);
 	gtk_window_present (GTK_WINDOW (dialog));
@@ -482,7 +539,7 @@ static gboolean _e2p_unpack_change_dir_hook (gchar *path, E2P_Unpackdata *data)
 }
 /**
 @brief unpack plugin action : unpack a supported archive into a temp dir
-
+Best if refreshing is disabled when this action is initiated
 @param from the button, menu item etc which was activated
 @param art action runtime data
 
@@ -500,43 +557,47 @@ static gboolean _e2p_unpack (gpointer from, E2_ActionRuntime *art)
 #if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(darwin)
 		//tar decompression options -z, -j, --lzma are GNU extensions
 		//ditto for --overwrite, and for -C, if that's wanted
-		">gzip -cd "UNPACKPATH" | tar -xpf -",
-		">bzip2 -cd "UNPACKPATH" | tar -xpf -",
-		">lzma -cd "UNPACKPATH" | tar -xpf -",
-		"tar -xpf "UNPACKPATH,
+		">gzip -cd %s | tar -xpf -",
+		">bzip2 -cd %s | tar -xpf -",
+		">lzma -cd %s | tar -xpf -",
+		"tar -xpf %s",
 #else
 		//note: an --atime-preserve in these tar commands prevents the
 		//file-monitoring process from noticing anything in the temp dir
 		//--overwrite not needed
-		"tar -xpzf "UNPACKPATH,
-		"tar -xpjf "UNPACKPATH,
-		"tar --lzma -xpf "UNPACKPATH,
-		"tar -xpf "UNPACKPATH,
+		"tar -xpzf %s",
+		"tar -xpjf %s",
+		"tar --lzma -xpf %s",
+		"tar -xpf %s",
 #endif
 		//.deb unpack command by Martin Zelaia (has GNU-specific tar options)
-		//">>mkdir ./DEBIAN ./CONTENTS;ar -x "UNPACKPATH" | tar -xfz control.tar.gz -C ./DEBIAN | tar -xfz data.tar.gz -C ./CONTENTS; rm control.tar.gz data.tar.gz;cp ./DEBIAN/control ./INFO;rm ./debian-binary";
-		//">rpm2cpio "UNPACKPATH" | cpio -id"  this is ok, but no spec data from a non-src file, and there's no re-pack command available
-		"unzip -o "UNPACKPATH,  //or "unzip -o -d "UNPACKPATH" "UNPACKPATH
+		//">>mkdir ./DEBIAN ./CONTENTS;ar -x %s | tar -xfz control.tar.gz -C ./DEBIAN | tar -xfz data.tar.gz -C ./CONTENTS; rm control.tar.gz data.tar.gz;cp ./DEBIAN/control ./INFO;rm ./debian-binary";
+		//">rpm2cpio %s | cpio -id"  this is ok, but no spec data from a non-src file, and there's no re-pack command available
+		"unzip -o %s",  //or "unzip -o -d %s %s"
 /*
 decompress ANSI:    7za x archive.zip -odirname -aoa
 decompress UNICODE: 7za x archive.7z -odirname -aoa
 */
-		"7za x "UNPACKPATH" -aoa", //or ??
-		"rar x -o+ "UNPACKPATH, //rar will only extract to current dir
-		"arj x -y "UNPACKPATH, //or "arj x -y "UNPACKPATH" "UNPACKPATH, //NOTE swapped order of archive & path
-		"zoo xO "UNPACKPATH	//zoo will only extract to current dir
+		"7za x %s -aoa", //or ??
+		"rar x -o+ %s", //rar will only extract to current dir
+		"arj x -y %s", //or "arj x -y "UNPACKPATH" "UNPACKPATH, //NOTE swapped order of archive & path
+		"zoo xO %s"	//zoo will only extract to current dir
 	};
 
+	//CHECKME 	e2_filelist_disable_refresh ();
 	FileInfo *info = e2_fileview_get_selected_first_local (curr_view, FALSE);
 	if (info == NULL)
+	{
+//		e2_filelist_enable_refresh ();
 		return FALSE;	//nothing selected
+	}
 
 #ifdef E2_VFSTMP
 	VPATH ddata;
-	ddata.localpath = e2_utils_dircat (curr_view, info->filename, TRUE);
+	ddata.path = e2_utils_dircat (curr_view, info->filename, TRUE);
 	ddata.spacedata = curr_view->spacedata;
 	gint index = _e2p_unpack_match_type (&ddata);
-	g_free (ddata.localpath);
+	g_free (ddata.path);
 #else
 	gchar *local = e2_utils_dircat (curr_view, info->filename, TRUE);
 	gint index = _e2p_unpack_match_type ((VPATH*)local);
@@ -545,6 +606,7 @@ decompress UNICODE: 7za x archive.7z -odirname -aoa
 	if (index == -1)
 	{
 		e2_output_print_error (_("Selected item is not a supported archive"), FALSE);
+//		e2_filelist_enable_refresh ();
 		return FALSE;
 	}
 
@@ -561,6 +623,7 @@ decompress UNICODE: 7za x archive.7z -odirname -aoa
 #endif
 	{
 		e2_output_print_error (_("Recursive unpack is not supported"), FALSE);
+//		e2_filelist_enable_refresh ();
 		return FALSE;
 	}
 
@@ -569,7 +632,7 @@ decompress UNICODE: 7za x archive.7z -odirname -aoa
 #ifdef E2_VFS
 	VPATH ddata = { workdir, NULL };	//local unpacking only
 #endif
-	F_FREE (converted);
+	F_FREE (converted, unpack_tmp);
 	//(re)make it
 #ifdef E2_VFS
 	if (e2_fs_recurse_mkdir (&ddata, 0777 E2_ERR_NONE()))
@@ -581,8 +644,9 @@ decompress UNICODE: 7za x archive.7z -odirname -aoa
 		gchar *msg = g_strdup_printf ("Could not create working directory '%s'",
 			converted);
 		e2_output_print_error (msg, TRUE);
-		F_FREE (converted);
+		F_FREE (converted, workdir);
 		g_free (workdir);
+//		e2_filelist_enable_refresh ();
 		return FALSE;
 	}
 
@@ -591,17 +655,17 @@ decompress UNICODE: 7za x archive.7z -odirname -aoa
 	data->workdir = D_FILENAME_FROM_LOCALE (workdir);
 	g_free (workdir);
 
-	converted = F_FILENAME_FROM_LOCALE (info->filename);
+	converted = F_FILENAME_FROM_LOCALE (info->filename); //not much race chance, don't D_...
 #ifdef E2_VFSTMP
 	FIXME dir when not mounted local
 #else
 	data->package = e2_utils_strcat (curr_view->dir, converted);  //dir has trailing /
-	F_FREE (converted);
+	F_FREE (converted, info->filename);
 //tag E2_BADQUOTES
 	gchar *qp = e2_utils_quote_string (data->package);
 #endif
 	//need no conversion of command encoding
-	data->command = g_strdup_printf (cmd_str [index], qp);
+	data->command = g_strdup_printf (cmd_str [index], qp); //CHECKME append 1>/dev/null
 	g_free (qp);
 	e2_window_set_cursor (GDK_WATCH);
 	//unpack the archive into the temp directory
@@ -616,21 +680,25 @@ decompress UNICODE: 7za x archive.7z -odirname -aoa
 	{
 		workdir = F_FILENAME_TO_LOCALE (data->workdir);
 #ifdef E2_VFS
-		ddata.localpath = workdir;
+		ddata.path = workdir;
 		e2_task_backend_delete (&ddata);
 #else
 		e2_task_backend_delete (workdir);
 #endif
-		F_FREE (workdir);
+		F_FREE (workdir, data->workdir);
 		_e2p_unpack_cleanup (data);
+//		e2_filelist_enable_refresh ();
 		return FALSE;
 	}
 	//now go see it
 	e2_pane_change_dir (NULL, data->workdir);
 	//setup to clean when temp dir not open anymore
-	e2_hook_register (&app.pane1.hook_change_dir, _e2p_unpack_change_dir_hook, data);
-	e2_hook_register (&app.pane2.hook_change_dir, _e2p_unpack_change_dir_hook, data);
+	e2_hook_register (&app.pane1.hook_change_dir,
+		(HookFunc)_e2p_unpack_change_dir_hook, data);
+	e2_hook_register (&app.pane2.hook_change_dir,
+		(HookFunc)_e2p_unpack_change_dir_hook, data);
 
+//	e2_filelist_enable_refresh ();
 	return TRUE;
 }
 
@@ -655,7 +723,7 @@ gboolean init_plugin (Plugin *p)
 #ifdef E2_VFS
 	aname = _("unpack_with_plugin");
 #else
-	aname = _A(103);
+	aname = _A(104);
 #endif
 	p->signature = ANAME VERSION;
 	p->menu_name = _("_Unpack");
@@ -664,11 +732,13 @@ gboolean init_plugin (Plugin *p)
 
 	if (p->action == NULL)
 	{
-		//no need to free this
-		gchar *action_name = g_strconcat (_A(6),".",aname,NULL);
-		p->action = e2_plugins_action_register
-			(action_name, E2_ACTION_TYPE_ITEM, _e2p_unpack, NULL, FALSE, 0, NULL);
+		//don't free name string here
+		E2_Action plugact =
+		{g_strconcat (_A(6),".",aname,NULL),_e2p_unpack,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+		p->action = e2_plugins_action_register (&plugact);
+		if G_LIKELY((p->action != NULL))
 			return TRUE;
+		g_free (plugact.name);
 	}
 	return FALSE;
 }
@@ -683,10 +753,10 @@ gboolean clean_plugin (Plugin *p)
 {
 	//FIXME prevent unload when an unpack is underway
 	//clear any current hook(s)
-	while (e2_hook_unregister (&app.pane1.hook_change_dir, _e2p_unpack_change_dir_hook,
-		NULL, FALSE)) {}
-	while (e2_hook_unregister (&app.pane2.hook_change_dir, _e2p_unpack_change_dir_hook,
-		NULL, FALSE)) {}
+	while (e2_hook_unregister (&app.pane1.hook_change_dir,
+		(HookFunc)_e2p_unpack_change_dir_hook, NULL, FALSE)) {}
+	while (e2_hook_unregister (&app.pane2.hook_change_dir,
+		(HookFunc)_e2p_unpack_change_dir_hook, NULL, FALSE)) {}
 
 	gchar *action_name = g_strconcat (_A(6),".",aname,NULL);
 	gboolean ret = e2_plugins_action_unregister (action_name);
