@@ -1,6 +1,6 @@
-/* $Id: e2p_upgrade.c 1420 2009-02-09 22:21:04Z tpgww $
+/* $Id: e2p_upgrade.c 2019 2009-11-20 11:06:20Z tpgww $
 
-Copyright (C) 2005-2009 tooar <tooar@gmx.net>
+Copyright (C) 2005-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -64,7 +64,7 @@ static void _e2p_upgrade_reconfig (void)
 	{
 		set = *walker;
 		if (set->type == E2_OPTION_TYPE_TREE)
-			set->ex.tree.def = NULL;
+			set->ex.tree.def.tree_strings = NULL;	//CHECKME leak ?
 	}
 	//clear current option values
 	e2_option_clear_data ();
@@ -75,9 +75,9 @@ static void _e2p_upgrade_reconfig (void)
 //CHECKME assumes BGL is closed, native file only
 static void _e2p_upgrade_backup (gchar *file)
 {
-	gchar *config_file = g_strdup_printf ("%s"G_DIR_SEPARATOR_S"%s",
+	gchar *cfg_file = g_strdup_printf ("%s"G_DIR_SEPARATOR_S"%s",
 		e2_cl_options.config_dir, file);
-	gchar *local = F_FILENAME_TO_LOCALE (config_file);
+	gchar *local = F_FILENAME_TO_LOCALE (cfg_file);
 #ifdef E2_VFS
 	VPATH ddata = { local, NULL };
 	if (e2_fs_access (&ddata, F_OK E2_ERR_NONE()) == 0)	//traverse link, if any
@@ -96,8 +96,23 @@ static void _e2p_upgrade_backup (gchar *file)
 		gdk_threads_enter ();
 		g_free (saved_file);
 	}
-	g_free (config_file);
-	F_FREE (local);
+	g_free (cfg_file);
+	F_FREE (local, cfg_file);
+}
+
+static gboolean _e2p_upgrade_run (const gchar *command, const gchar *cfg_path)
+{
+	if (system (command) == 0)
+		cancelled = FALSE;
+	else
+	{
+		cancelled = TRUE;
+		printd (DEBUG, "failed command '%s'", command);
+		gchar *revert = g_strconcat ("mv -f ", cfg_path, ".save ", cfg_path, NULL);
+		system (revert);
+		g_free (revert);
+	}
+	return !cancelled;
 }
 
 static gint _e2p_upgrade_dialog (gchar *msg)
@@ -132,8 +147,9 @@ static gint _e2p_upgrade_dialog (gchar *msg)
 		msg, _("update information"), DEFAULT_RESPONSE_CB, NULL);
 	e2_dialog_show (dialog, NULL, 0,
 		&yes_btn, &no_btn, NULL);
-//	gint choice = gtk_dialog_run (GTK_DIALOG (dialog));	//not a local loop as options not yet available
-	gint choice = e2_dialog_run_simple (dialog, NULL);
+//	gint choice = e2_dialog_run_simple (dialog, NULL);
+	//not a local loop, as main-window (for fake event) & options not yet available
+	gint choice = gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 	cancelled = (choice != GTK_RESPONSE_YES);
 	return choice;
@@ -165,15 +181,65 @@ static void _e2p_upgrade_numbers (void)
 			sed, " -e '1s/", app.cfgfile_version, "/"VERSION RELEASE"/'",
 				 " -e '2,$s/0\\.[0-9]\\.[0-9]/"VERSION"/'",
 				 " ",localtmp,".save >",local,NULL);
-		system (command);
+		if (!_e2p_upgrade_run (command, localtmp))
+			printd (WARN, "failed to execute command to upgrade config version no's");
 		g_free (sed);
 		g_free (cfg_file);
-		F_FREE (local);
+		F_FREE (local, cfg_file);
 		g_free (localtmp);
 		g_free (command);
 	}
+	else
+		cancelled = TRUE;
 }
 #endif
+
+#ifdef E2_POLKIT
+/**
+@brief upgrade su-related strings
+@return TRUE if the upgrade was done
+*/
+static gboolean _e2p_upgrade_su (void)
+{
+	gchar *sed = _e2p_upgrade_get_sed ();
+	if (sed != NULL)
+	{
+		const gchar *prompt1 = _("Enter command:");
+		const gchar *prompt2 = _("Done. Press enter ");
+		//commandbar item
+		gchar *oldstr1 = g_strconcat("\\|xterm\\|-e 'su -c \\\"%\\{\\(root-commands\\)@",prompt1,"\\}\\\";echo -n \\\"",prompt2,"\\\";read'",NULL);
+		gchar *newstr1 = g_strconcat("|pkexec|%{(root-commands)@",prompt1,"}",NULL);
+		//alias
+		//2nd (.*) around "\2" so this can work (and be inserted in the replacement string)
+		gchar *oldstr2 = g_strconcat("\\|xterm -e sh -c 'su -c \\\"(.*)\\\";echo -n \\\"",prompt2,"\\\";read'",NULL);
+		const gchar *newstr2 = "|pkexec \\2";
+
+		gchar *cfg_file = g_build_filename (e2_cl_options.config_dir,
+			default_config_file, NULL);
+		gchar *local = F_FILENAME_TO_LOCALE (cfg_file);
+		gchar *command = g_strconcat ("cp -f ", local, " ", local, ".save;",
+		sed,
+			" -r"	//support back-referencing
+			" -e \"s~(.*)",oldstr1,"~\\1",newstr1,"~\"",
+			" -e \"s~(.*)",oldstr2,"~\\1",newstr2,"~\"",
+			" ",local,".save >",local,NULL);
+		gboolean success = _e2p_upgrade_run (command, local);
+		g_free (oldstr1);
+		g_free (newstr1);
+		g_free (oldstr2);
+		g_free (cfg_file);
+		F_FREE (local, cfg_file);
+		g_free (command);
+		g_free (sed);
+		if (success)
+			return TRUE;
+		printd (WARN, "failed to execute command to do su-upgrade");
+	}
+	cancelled = TRUE;
+	return FALSE;
+}
+#endif
+
 static void _e2p_upgrade_pre_0_1 (void)
 {
 	gchar *msg = g_strdup_printf (gettext (default_msg),
@@ -222,7 +288,7 @@ static void _e2p_upgrade_0_5_1 (void)
 	{
 		//keybinding to add
 		gchar *oldstr1 = g_strconcat("\t\t\t|<Control>i|false|",_A(7),".",_A(57),"|",NULL);
-		gchar *newstr1 = g_strconcat("\t\t\t|<Control>d|false|",_A(7),".",_A(126),"|",NULL);
+		gchar *newstr1 = g_strconcat("\t\t\t|<Control>d|false|",_A(7),".",_A(80),"|",NULL);
 
 		gchar *cfg_file = g_build_filename (e2_cl_options.config_dir,
 			default_config_file, NULL);
@@ -230,16 +296,16 @@ static void _e2p_upgrade_0_5_1 (void)
 		gchar *command = g_strconcat ("cp -f ", local, " ", local, ".save;",
 		sed,
 #ifndef E2_VERSIONDOCS
-			 " -e '1s/", app.cfgfile_version, "/", VERSION RELEASE, "/'",
+			" -e '1s/", app.cfgfile_version, "/", VERSION RELEASE, "/'",
 #endif
 			" -e '/",oldstr1,"$/a\\\n",newstr1,"'",
-			 " ",local,".save >",local,NULL);
-		system (command);
-
+			" ",local,".save >",local,NULL);
+		if (!_e2p_upgrade_run (command, local))
+			printd (WARN, "failed to execute command to do upgrade 0.5.1");
 		g_free (oldstr1);
 		g_free (newstr1);
 		g_free (cfg_file);
-		F_FREE (local);
+		F_FREE (local, cfg_file);
 		g_free (command);
 		g_free (sed);
 	}
@@ -247,6 +313,66 @@ static void _e2p_upgrade_0_5_1 (void)
 		_e2p_upgrade_0_4_5 ();	//suggest default
 }
 
+static void _e2p_upgrade_0_5_1_1 (void)
+{
+	gchar *sed = _e2p_upgrade_get_sed ();
+	if (sed != NULL)
+	{
+		//keybindings to change
+		gchar *oldstr1 = g_strconcat(_A(10),".",_A(31),"|*,1",NULL);
+		gchar *newstr1 = g_strconcat("!",_A(10),".",_A(31),"|1,*",NULL);
+		gchar *oldstr2 = g_strconcat(_A(10),".",_A(31),"|*,0",NULL);
+		gchar *newstr2 = g_strconcat("!",_A(10),".",_A(31),"|0,*",NULL);
+		gchar *oldstr3 = g_strconcat(_A(14),".",_A(31),"|*,1",NULL);
+		gchar *newstr3 = g_strconcat("!",_A(14),".",_A(31),"|0,*",NULL);
+		gchar *oldstr4 = g_strconcat(_A(14),".",_A(31),"|*,0",NULL);
+		gchar *newstr4 = g_strconcat("!",_A(14),".",_A(31),"|1,*",NULL);
+
+		gchar *cfg_file = g_build_filename (e2_cl_options.config_dir,
+			default_config_file, NULL);
+		gchar *local = F_FILENAME_TO_LOCALE (cfg_file);
+		gchar *command = g_strconcat ("cp -f ", local, " ", local, ".save;",
+		sed,
+#ifndef E2_VERSIONDOCS
+			" -e '1s/", app.cfgfile_version, "/", VERSION RELEASE, "/'",
+#endif
+			" -e 's/",oldstr1,"/",newstr1,"/'",
+			" -e 's/",oldstr2,"/",newstr2,"/'",
+			" -e 's/",oldstr3,"/",newstr3,"/'",
+			" -e 's/",oldstr4,"/",newstr4,"/'",
+			" ",local,".save >",local,NULL);
+		if (!_e2p_upgrade_run (command, local))
+			printd (WARN, "failed to execute command to do upgrade 0.5.1.1");
+		g_free (oldstr1);
+		g_free (newstr1);
+		g_free (oldstr2);
+		g_free (newstr2);
+		g_free (oldstr3);
+		g_free (newstr3);
+		g_free (oldstr4);
+		g_free (newstr4);
+		g_free (cfg_file);
+		F_FREE (local, cfg_file);
+		g_free (command);
+		g_free (sed);
+	}
+	else
+		_e2p_upgrade_0_4_5 ();	//suggest default
+}
+
+static void _e2p_upgrade_0_6_0 (void)
+{
+	//update key-shortcut translations
+	E2_OptionSet *set = e2_option_get ("keybindings");
+	if (set->ex.tree.synced) //this set's data processed already
+	{
+		GtkTreeIter iter;
+		if (gtk_tree_model_get_iter_first (set->ex.tree.model, &iter))
+			e2_keybinding_localise (set->ex.tree.model, &iter);
+	}
+	//else FIXME
+}
+
 gboolean init_plugin (Plugin *p)
 {
 #define ANAME "uprade"
@@ -268,14 +394,32 @@ gboolean init_plugin (Plugin *p)
 		{
 #ifdef E2_VERSIONDOCS
 			_e2p_upgrade_numbers ();	//always update docs version etc
+#endif
+#ifdef E2_POLKIT
+			//this #define can be changed at any time
+			cancelled = !_e2p_upgrade_su ();
+#else
+			cancelled = TRUE;
 #endif
 			if (strcmp (app.cfgfile_version,"0.5.1") < 0)
+			{
+				cancelled = FALSE; //may be changed downstream
 				_e2p_upgrade_0_5_1 ();
-			else
-				cancelled = TRUE;
+			}
+			if (strcmp (app.cfgfile_version,"0.5.1.1") < 0)
+			{
+				cancelled = FALSE;
+				_e2p_upgrade_0_5_1_1 ();
+			}
 		}
 		if (!cancelled)
 			_e2p_upgrade_reconfig ();	//implement all updates
+
+		//after any config-file-based updates, work on stored data ...
+
+		if (strcmp (app.cfgfile_version,"0.6.0") < 0)
+			_e2p_upgrade_0_6_0 ();
+
 	}
 	return TRUE;
 }
