@@ -1,6 +1,6 @@
-/* $Id: e2_action.c 1412 2009-02-01 21:51:22Z tpgww $
+/* $Id: e2_action.c 2008 2009-11-12 01:25:40Z tpgww $
 
-Copyright (C) 2003-2009 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -39,6 +39,7 @@ ToDo
 #include "e2_action.h"
 #include "e2_action_option.h"
 #include "e2_task.h"
+#include "e2_filelist.h"
 #include "e2_plugins.h"
 #include "e2_filetype.h"
 #include "e2_dialog.h"
@@ -71,8 +72,8 @@ static gboolean _e2_action_do_toggle (gpointer from, E2_ActionRuntime *art)
 	gint i;
 	for (i = 0; i < E2_TOGGLE_COUNT; i++)
 	{
-		if (g_str_equal (toggles_array [i], ex->true_action)
-			&& g_str_equal (ex->true_action, ex->false_action))
+		if (!strcmp (toggles_array [i], ex->true_action)
+			&& !strcmp (ex->true_action, ex->false_action))
 				break;
 	}
 	if (i < E2_TOGGLE_COUNT)
@@ -123,6 +124,36 @@ static gboolean _e2_action_custom_command (gpointer from, E2_ActionRuntime *art)
 	g_free (command);
 	return (res == 0);
 }
+/**
+@brief initiate configuration change
+
+@param from the button, menu item etc which was activated
+@param art action runtime data
+
+@return TRUE
+*/
+static gboolean _e2_action_configure (gpointer from, E2_ActionRuntime *art)
+{  //put this in config dialog file ?
+//	gtk_widget_set_sensitive (app.main_window, FALSE);
+	gchar *page = (gchar *)art->data;
+	e2_config_dialog_create (page);
+//	gtk_widget_set_sensitive (app.main_window, TRUE);
+	return TRUE;
+}
+/**
+@brief revert all configuration settings to defailt values
+
+@param from the button, menu item etc which was activated
+@param art action runtime data
+
+@return TRUE
+*/
+static gboolean _e2_action_configure_default (gpointer from, E2_ActionRuntime *art)
+{
+	//dump old data, don't re-read the config file, recreate screen
+	e2_option_refresh (FALSE, TRUE);
+	return TRUE;
+}
 #ifdef E2_VFS
 /**
 @brief dummy action for plugins which do not really belong on a plugins menu
@@ -212,10 +243,164 @@ static gboolean _e2_action_visible_cb (GtkTreeModel *model,
 	return retval;
 }
 
+static gboolean _e2_action_enter_menu_cb (GtkWidget *widget, GdkEventCrossing *event,
+		E2HoverData *data)
+{
+	if (data->timer_id > 0)
+	{
+		g_source_remove (data->timer_id);
+		data->timer_id = 0;
+	}
+	return FALSE;
+}
+
+/**
+@brief timer callback to implement a hover callback
+@brief data pointer to hover data
+@return FALSE to cancel the timer
+*/
+static gboolean _e2_action_do_hover_cb (E2HoverData *data)
+{
+	//get rid of this ASAP in case of pending departure from hover-widget
+	if (data->timer_id > 0)
+	{
+		g_source_remove (data->timer_id);
+		data->timer_id = 0;
+	}
+
+	gdk_threads_enter ();
+	(*data->callback) (data->hovered, data);
+
+	if (GTK_IS_MENU (data->popup) &&
+#ifdef USE_GTK2_18
+		!gtk_widget_get_visible (data->popup)
+#else
+		!GTK_WIDGET_VISIBLE (data->popup)
+#endif
+		)
+	{
+		GtkWidget *popwin = gtk_widget_get_ancestor (data->popup, GTK_TYPE_WINDOW); //would be parent, actually
+		gtk_window_set_transient_for (GTK_WINDOW (popwin), GTK_WINDOW (app.main_window));
+		g_signal_connect (G_OBJECT (popwin), "enter-notify-event",
+			G_CALLBACK (_e2_action_enter_menu_cb), data);
+		g_signal_connect (G_OBJECT (popwin), "leave-notify-event",
+		G_CALLBACK (e2_action_end_hover_cb), data);
+
+		gtk_widget_show (data->popup); //must do this here, not later - dunno why
+
+		GtkRequisition requisition;
+		gtk_widget_size_request (data->popup, &requisition);
+		gint x, y;
+		gboolean push = TRUE;
+		e2_toolbar_set_menu_position (GTK_MENU (data->popup), &x, &y, &push, data->hovered);
+		gtk_widget_size_request (popwin, &requisition);
+		gtk_widget_set_size_request (popwin, requisition.width, requisition.height);
+		gtk_window_move (GTK_WINDOW (popwin), x, y); //this might be ignored by WM
+		gtk_widget_show_all (popwin); //doesn't work, dunno why
+		gtk_window_move (GTK_WINDOW (popwin), x, y); //fallback
+	}
+	gdk_threads_leave ();
+
+	return FALSE;
+}
+/**
+@brief timer callback to cleanup after a hover
+@brief data pointer to hover data
+@return FALSE to cancel the timer
+*/
+static gboolean _e2_action_clear_hover_cb (E2HoverData *data)
+{
+	if (GTK_IS_WIDGET (data->popup))
+	{
+		gdk_threads_enter ();
+		gtk_widget_destroy (data->popup);
+		gdk_threads_leave ();
+	}
+	data->popup = NULL;
+
+	return FALSE;
+}
+
+static void _e2_action_abort_hover_cb (GtkButton *button, E2HoverData *data)
+{
+	printd (DEBUG, "in function _e2_action_abort_hover_cb");
+	if (data->timer_id > 0)
+	{
+		g_source_remove (data->timer_id);
+		data->timer_id = 0;
+	}
+	if (GTK_IS_WIDGET (data->popup))
+		gtk_widget_destroy (data->popup);
+	data->popup = NULL;
+}
+
   /******************/
  /***** public *****/
 /******************/
 
+/**
+@brief setup for a potential hover process
+@param widget the entered widget (for now at least, only a toolbar button)
+@param event pointer to event data
+@para data pointer to hover data struct
+
+This is a "enter-notify-event" signal callback for all toolbar buttons based on
+a E2_ACTION_TYPE_HOVER action.
+
+@return	FALSE always so the event is propagated
+*/
+gboolean e2_action_start_hover_cb (GtkWidget *widget, GdkEventCrossing *event,
+		E2HoverData *data)
+{
+	printd (DEBUG, "in function e2_action_start_hover_cb");
+	if (data->timer_id > 0)
+	{
+		//we're re-entering the buttton from a menu
+		g_source_remove (data->timer_id);
+		data->timer_id = 0;
+	}
+	if (data->popup == NULL)
+	{
+		//no popup if button is clicked
+		g_signal_connect (G_OBJECT (widget), "clicked", //not "button_press-event", that's blocked
+			G_CALLBACK (_e2_action_abort_hover_cb), data);
+		//FIXME relate interval to gtk's tooltip popup time
+		data->timer_id = g_timeout_add (1000, (GSourceFunc)_e2_action_do_hover_cb, data);
+	}
+
+	return FALSE;
+}
+/**
+@brief abort an actual or potential hover process
+@param widget the departed widget
+@param event pointer to event data
+@para data pointer to hover data struct
+
+This is a "leave-notify-event" signal callback for all toolbar buttons based on
+a E2_ACTION_TYPE_HOVER action, and for a popped-up menu widget.
+
+@return	FALSE always so the event is propagated
+*/
+gboolean e2_action_end_hover_cb (GtkWidget *widget, GdkEventCrossing *event,
+		E2HoverData *data)
+{
+	printd (DEBUG, "in function e2_action_end_hover_cb");
+	if (data->timer_id > 0)
+	{
+		g_source_remove (data->timer_id);
+		data->timer_id = 0;
+	}
+	if (GTK_IS_WIDGET (data->popup))
+		//allow short interval to move pointer from button into the menu or vice-versa, before killing it
+		data->timer_id = g_timeout_add (100, (GSourceFunc)_e2_action_clear_hover_cb, data);
+	else
+		data->popup = NULL;
+
+	g_signal_handlers_disconnect_by_func ((gpointer)widget, _e2_action_abort_hover_cb, data);
+
+	return FALSE;
+}
+
 /**
 @brief option helper for action columns
 
@@ -235,45 +420,21 @@ GtkTreeModel *e2_action_filter_store (gpointer ex)
 		(GtkTreeModelFilterVisibleFunc) _e2_action_visible_cb, ex, NULL);
 	return filter;
 }
-/** @brief short-form register action function
-
-See e2_action_register function description. This one has no exclude flags or data2
-
-@return registered action E2_Action
-*/
-E2_Action *e2_action_register_simple (gchar *name, E2_ACTION_TYPE type,
-	void *func, gpointer data, gboolean has_arg)
-{
-	return e2_action_register (name, type, func, data, has_arg, 0, NULL);
-}
 
 gint entry_level = 0;
 
 /**
-@brief register action names
-
-This function registers 'public' action names, and corresponding parameters, for later use as needed.
+@brief register 'public' action names, and corresponding parameters, for later use as needed.
 It replaces any previously-registered action with the same name.
 Also adds the action name etc to the options store, as a convenience for the option system
 Action names need to be freed (= action removed from the hash) when the session ends, if not before.
 Potentially re-entrant.
-@a has_arg is stored here, but never used elsehwhere !
 As another convenience, the non-constant name string used to register the action is freed at exit.
-
-@param name freeable string with possible '.' separating 'parent/ and 'child' parts of the action
-@param type flags for one of: item (?) submenu, file_actions, custom_command, plugins, separator,
-	tear_off_menu, toggle, command_line, bookmarks, dummy
-@param func the function to be called when the action is run
-@param data pointer to data to be supplied to @a func, or NULL
-@param has_arg boolean flag signalling whether the action-string uses context-specific argument(s)
-@param exclude flags indicating whether to exclude this action from toolbars, menus &| keybindings
-@param data2 additional data needed by the action, or NULL
+@param newaction pointer to action data to be copied and heaped
 
 @return registered action E2_Action
 */
-E2_Action *e2_action_register (gchar *name, E2_ACTION_TYPE type,
-	void *func, gpointer data, gboolean has_arg, E2_ACTION_EXCLUDE exclude,
-	gpointer data2)
+E2_Action *e2_action_register (const E2_Action *newaction)
 {
 	//unregister any previously registered actions with that name
 	//(for convenience)
@@ -289,15 +450,9 @@ E2_Action *e2_action_register (gchar *name, E2_ACTION_TYPE type,
 	E2_Action *action = ALLOCATE (E2_Action);
 #endif
 	CHECKALLOCATEDFATAL (action);
-	//freed if the action is removed from the actions_hash table
+	//action name is freed if the action is removed from the actions_hash table
 	//because the names are used as keys
-	action->name = name;
-	action->type = type;
-	action->func = func;
-	action->has_arg = has_arg;
-	action->data = data;
-	action->exclude = exclude;
-	action->data2 = data2;
+	*action = *newaction;
 	//add the action to the static hash table and pointer array register
 	g_hash_table_replace (actions_hash, action->name, action);
 //	g_ptr_array_add (actions_array, action);
@@ -305,19 +460,18 @@ E2_Action *e2_action_register (gchar *name, E2_ACTION_TYPE type,
 	//to get structured dropdown lists, find and register the 'parent' action, if any
 	gboolean registered_parent = FALSE;
 	E2_Action *parent = NULL;
-	gchar *search = g_strdup (name);
+	gchar *search = g_strdup (newaction->name);
 	gchar *child = strrchr (search, '.');	//always ascii '.', don't need g_utf8_strrchr()
-//	if (child == NULL)
-//		child = strrchr (search, ':'); //ditto
 	if (child != NULL)
 	{ //also register a new parent, if need be
 		*child = '\0';
 		parent = e2_action_get (search);
 		if (parent == NULL)
 		{
+			E2_Action found =
+			{search, NULL, FALSE, E2_ACTION_TYPE_DUMMY, 0, NULL, NULL};
 			//re-entry !!
-			parent = e2_action_register_simple (search, E2_ACTION_TYPE_DUMMY,
-				NULL, NULL, FALSE);
+			parent = e2_action_register (&found);
 			registered_parent = TRUE;
 		}
 	}
@@ -528,11 +682,11 @@ gboolean e2_action_run (gpointer from, E2_ActionRuntime *rt)
 	rt->state = e2_action_get_current_state ();
 	if (from == NULL)
 		from = app.main_window;
-	gboolean (*fun) (gpointer, E2_ActionRuntime *) = rt->action->func;
+	gboolean (*func) (gpointer, E2_ActionRuntime *) = rt->action->func;
 #ifndef DEBUG_MESSAGES
-	return fun (from, rt);
+	return func (from, rt);
 #else
-	gboolean res = fun (from, rt);
+	gboolean res = func (from, rt);
 	printd (DEBUG,"e2_action_run () ends");
 	return res;
 #endif
@@ -543,14 +697,14 @@ gboolean e2_action_run (gpointer from, E2_ActionRuntime *rt)
 This function creates an action runtime object and initializes it with
 supplied user data.
 
-@param action  the action to create the runtime object for, or command string for a "custom command"
-@param data  action user data
-@param data_free  function to free data when the runtime object is freed
+@param action the action to create the runtime object for, or command string for a "custom command"
+@param data action user data
+@param data_free function to free data when the runtime object is freed
 
 @return newly allocated action runtime object that should be freed
 */
 E2_ActionRuntime *e2_action_pack_runtime (E2_Action *action, gpointer data,
-	gpointer data_free)
+	void(*data_free)(gpointer))
 {
 //	printd (DEBUG, "action pack runtime for %s", action->name);
 	E2_ActionRuntime *rt = ALLOCATE (E2_ActionRuntime);
@@ -575,15 +729,13 @@ calling this function.
 void e2_action_free_runtime (E2_ActionRuntime *rt)
 {
 //	printd (DEBUG, "e2_action_free_runtime");
-	if (rt == NULL)
-		return;
-
-	if (rt->data_free != NULL)
+	if (rt != NULL)
 	{
-		void (*fun) (gpointer) = rt->data_free;
-		fun (rt->data);
+		if (rt->data != NULL && rt->data_free != NULL)
+			(*rt->data_free) (rt->data);
+
+		DEALLOCATE (E2_ActionRuntime, rt);
 	}
-	DEALLOCATE (E2_ActionRuntime, rt);
 }
 /**
 @brief determine state flags for an action about to run
@@ -648,7 +800,11 @@ static void _e2_action_free_toggle (E2_ToggleData *ex)
 */
 static void _e2_action_clean1 (E2_Action *action)
 {
-	//action name is the hash key, cleared by g_free
+	//action->name is the actions-hash key, therefore cleared by g_free
+	if (G_UNLIKELY(action->type == E2_ACTION_TYPE_HOVER))
+		DEMALLOCATE (E2_Callback, action->data);
+//	if (action->data != NULL && action->cleaner != (GDestroyNotify)NULL)
+//		(*action->cleaner) (data);
 	GtkTreeIter iter;
 	if (e2_tree_ref_to_iter (actions_store, action->ref, &iter))
 		gtk_tree_store_remove (actions_store, &iter);
@@ -715,70 +871,48 @@ void e2_actions_init (void)
 
 	toggles_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
 		(GDestroyNotify) _e2_action_free_toggle);
-	toggles_array [E2_TOGGLE_PANE1FULL] = g_strconcat (_A(11),".",_A(112),NULL);
-	toggles_array [E2_TOGGLE_PANE2FULL] = g_strconcat (_A(12),".",_A(112),NULL);
-	toggles_array [E2_TOGGLE_PANE1HIDDEN] = g_strconcat (_A(11),".",_A(83),NULL);
-	toggles_array [E2_TOGGLE_PANE2HIDDEN] = g_strconcat (_A(12),".",_A(83),NULL);
+	toggles_array [E2_TOGGLE_PANE1FULL] = g_strconcat (_A(11),".",_A(113),NULL);
+	toggles_array [E2_TOGGLE_PANE2FULL] = g_strconcat (_A(12),".",_A(113),NULL);
+	toggles_array [E2_TOGGLE_PANE1HIDDEN] = g_strconcat (_A(11),".",_A(84),NULL);
+	toggles_array [E2_TOGGLE_PANE2HIDDEN] = g_strconcat (_A(12),".",_A(84),NULL);
 	toggles_array [E2_TOGGLE_PANE1FILTERS] = g_strconcat (_A(11),".",_A(25),NULL);
 	toggles_array [E2_TOGGLE_PANE2FILTERS] = g_strconcat (_A(12),".",_A(25),NULL);
-	toggles_array [E2_TOGGLE_OUTPUTFULL] = g_strconcat (_A(10),".",_A(112),NULL);
-	toggles_array [E2_TOGGLE_OUTPUTSHADE] = g_strconcat (_A(10),".",_A(82),NULL);
+	toggles_array [E2_TOGGLE_OUTPUTFULL] = g_strconcat (_A(10),".",_A(113),NULL);
+	toggles_array [E2_TOGGLE_OUTPUTSHADE] = g_strconcat (_A(10),".",_A(83),NULL);
 #ifdef E2_VFS
-	toggles_array [E2_TOGGLE_PANE1SPACE] = g_strconcat (_A(11),".",_A(119),NULL);
-	toggles_array [E2_TOGGLE_PANE2SPACE] = g_strconcat (_A(12),".",_A(119),NULL);
+	toggles_array [E2_TOGGLE_PANE1SPACE] = g_strconcat (_A(11),".",_A(120),NULL);
+	toggles_array [E2_TOGGLE_PANE2SPACE] = g_strconcat (_A(12),".",_A(120),NULL);
 #endif
 	//register most actions
 	//note translated action labels needed here, i.e. before config loaded
+	//all action names must be freeable
 	E2_Action homeless_actions[] =
 	{	//here we don't bother explicitly setting 3 trailing NULL parameters
-		{ NULL, NULL, FALSE, E2_ACTION_TYPE_FILE_ACTIONS,
-			E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_TOOLBAR | E2_ACTION_EXCLUDE_LAYOUT },
-		{ NULL, NULL, FALSE, E2_ACTION_TYPE_PLUGINS,
-			E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_TOOLBAR },
-		{ NULL, NULL, FALSE, E2_ACTION_TYPE_SEPARATOR,
-			E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_LAYOUT },
-		{ NULL, NULL, FALSE, E2_ACTION_TYPE_SUBMENU,
-			E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_LAYOUT },
-		{ NULL, e2_main_user_shutdown, E2_ACTION_TYPE_ITEM, FALSE, 0 },
+		{ g_strconcat(_A(6),".",_A(23),NULL), (gboolean (*)(gpointer,E2_ActionRuntime*))NULL, FALSE, E2_ACTION_TYPE_FILE_ACTIONS,
+			E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_TOOLBAR | E2_ACTION_EXCLUDE_LAYOUT, },
+		{ g_strconcat(_A(16),".",_A(27),NULL), (gboolean (*)(gpointer,E2_ActionRuntime*))NULL, FALSE, E2_ACTION_TYPE_PLUGINS,
+			E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_TOOLBAR, },
+		{ g_strdup(_A(21)), (gboolean (*)(gpointer,E2_ActionRuntime*))NULL, FALSE, E2_ACTION_TYPE_SEPARATOR,
+			E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_LAYOUT, },
+		{ g_strdup(_A(22)), (gboolean (*)(gpointer,E2_ActionRuntime*))NULL, FALSE, E2_ACTION_TYPE_SUBMENU,
+			E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_LAYOUT, },
+		{ g_strconcat(_A(1),".",_A(72),NULL), e2_main_user_shutdown, E2_ACTION_TYPE_ITEM, FALSE, 0, },
 
-		{ NULL, e2_task_configure, TRUE, E2_ACTION_TYPE_ITEM, 0 },
-		{ NULL, e2_task_configure_default, FALSE, E2_ACTION_TYPE_ITEM, 0 },
-		{ NULL, e2_plugins_configure, FALSE, E2_ACTION_TYPE_ITEM, 0 },
-		{ NULL, _e2_action_custom_command, TRUE, E2_ACTION_TYPE_ITEM,
-			E2_ACTION_EXCLUDE_GENERAL },
-		{ NULL, _e2_action_do_toggle, TRUE, E2_ACTION_TYPE_TOGGLE,
-			E2_ACTION_EXCLUDE_TOGGLE },
-		{ NULL, _e2_action_do_toggle, TRUE, E2_ACTION_TYPE_TOGGLE,
-			E2_ACTION_EXCLUDE_TOGGLE },
+		{ g_strconcat(_A(3),".",_C(34),NULL), e2_plugins_configure, FALSE, E2_ACTION_TYPE_ITEM, 0, },
+		{ g_strdup(_A(20)), _e2_action_custom_command, TRUE, E2_ACTION_TYPE_ITEM,
+			E2_ACTION_EXCLUDE_GENERAL, },
+		{ g_strconcat(_A(17),".",_A(115),NULL), _e2_action_do_toggle, TRUE, E2_ACTION_TYPE_TOGGLE,
+			E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_TOGGLE, },
+		{ g_strconcat(_A(17),".",_A(116),NULL), _e2_action_do_toggle, TRUE, E2_ACTION_TYPE_TOGGLE,
+			E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_TOGGLE, },
+
+		{ g_strconcat(_A(3),".",_A(32),NULL), _e2_action_configure, FALSE, E2_ACTION_TYPE_ITEM, 0, },
+		{ g_strconcat(_A(3),".",_A(40),NULL), _e2_action_configure_default, FALSE, E2_ACTION_TYPE_ITEM, 0, },
 	};
-	//all action names must be freeable
-	homeless_actions[0].name = g_strconcat(_A(6),".",_A(23),NULL);
-	homeless_actions[1].name = g_strconcat(_A(16),".",_A(27),NULL);
-	homeless_actions[2].name = g_strdup(_A(21));
-	homeless_actions[3].name = g_strdup(_A(22));
 
-	homeless_actions[4].name = g_strconcat(_A(1),".",_A(72),NULL);
-	//configuration
-	homeless_actions[5].name = g_strconcat(_A(3),".",_A(32),NULL);
-	homeless_actions[6].name = g_strconcat(_A(3),".",_A(40),NULL);
-	homeless_actions[7].name = g_strconcat(_A(3),".",_C(34),NULL);
-	homeless_actions[8].name = g_strdup(_A(20));
-	homeless_actions[9].name = g_strconcat(_A(17),".",_A(114),NULL);
-	homeless_actions[10].name = g_strconcat(_A(17),".",_A(115),NULL);
-
-	gint count = sizeof (homeless_actions) / sizeof (E2_Action);
-	gint i;
+	guint i, count = sizeof (homeless_actions) / sizeof (E2_Action);
 	for (i = 0; i < count; i++)
-	{
-		e2_action_register (
-			homeless_actions[i].name,
-			homeless_actions[i].type,
-			homeless_actions[i].func,
-			NULL,
-			homeless_actions[i].has_arg,
-			homeless_actions[i].exclude,
-			NULL);
-	}
+		e2_action_register (&homeless_actions[i]);
 	//register most other actions
 	// (some others are in pane create fn)
 	e2_task_actions_register ();
@@ -847,7 +981,6 @@ void e2_action_setup_labels (void)
 	action_labels[20] = _("<custom command>");
 	action_labels[21] = _("<separator>");
 	action_labels[22] = _("<submenu>");
-
 	//'child' names
 	action_labels[23] = _("<actions>");
 	action_labels[24] = _("<bookmarks>");
@@ -905,66 +1038,68 @@ void e2_action_setup_labels (void)
 	action_labels[77] = _("scroll_down");
 	action_labels[78] = _("scroll_up");
 	action_labels[79] = _("search");  //= _C(37)
-	action_labels[80] = _("send");
-	action_labels[81] = _("set");
-	action_labels[82] = _("show");
-	action_labels[83] = _("show_hidden");
-	action_labels[84] = _("show_menu");
-	action_labels[85] = _("sortaccesssed");
-	action_labels[86] = _("sortchanged");
-	action_labels[87] = _("sortext");
-	action_labels[88] = _("sortgroup");
-	action_labels[89] = _("sortmodified");
-	action_labels[90] = _("sortname");
-	action_labels[91] = _("sortpermission");
-	action_labels[92] = _("sortsize");
-	action_labels[93] = _("sortuser");
-	action_labels[94] = _("switch");
-	action_labels[95] = _("symlink");
-	action_labels[96] = _("symlink_as");
-	action_labels[97] = _("sync");
-	action_labels[98] = _("toggle_direction");
-	action_labels[99] = _("toggle_focus");
-	action_labels[100] = _("toggle_select_all");
-	action_labels[101] = _("toggle_selected");
+	action_labels[80] = _("select_type");
+	action_labels[81] = _("send");
+	action_labels[82] = _("set");
+	action_labels[83] = _("show");
+	action_labels[84] = _("show_hidden");
+	action_labels[85] = _("show_menu");
+	action_labels[86] = _("sortaccesssed");
+	action_labels[87] = _("sortchanged");
+	action_labels[88] = _("sortext");
+	action_labels[89] = _("sortgroup");
+	action_labels[90] = _("sortmodified");
+	action_labels[91] = _("sortname");
+	action_labels[92] = _("sortpermission");
+	action_labels[93] = _("sortsize");
+	action_labels[94] = _("sortuser");
+	action_labels[95] = _("switch");
+	action_labels[96] = _("symlink");
+	action_labels[97] = _("symlink_as");
+	action_labels[98] = _("sync");
+	action_labels[99] = _("toggle_direction");
+	action_labels[100] = _("toggle_focus");
+	action_labels[101] = _("toggle_select_all");
+	action_labels[102] = _("toggle_selected");
 #ifdef E2_TREEDIALOG
-	action_labels[102] = _("tree");
+	action_labels[103] = _("tree");
 #endif
-	action_labels[103] = _("unpack");
-//	action_labels[104] = _("untrash"); see e2_task_actions_register()
-	action_labels[105] = _("view");  //= _C(42)
-	action_labels[106] = _("view_again");
-	action_labels[107] = _("view_at");
+	action_labels[104] = _("unpack");
+#ifdef WITH_UNTRASH
+	action_labels[105] = _("untrash");
+#endif
+	action_labels[106] = _("view");  //= _C(42)
+	action_labels[107] = _("view_again");
+	action_labels[108] = _("view_at");
 	//these are action _parameter_ strings
-	action_labels[108] = _("child");
-	action_labels[109] = _("ctrl");
-	action_labels[110] = _("dirs");
-	action_labels[111] = _("escape");
-	action_labels[112] = _("expand");
-	action_labels[113] = _("files");
-	action_labels[114] = _("off");
-	action_labels[115] = _("on");
-	action_labels[116] = _("quote");
-	action_labels[117] = _("shift");
-	action_labels[118] = _("top");
+	action_labels[109] = _("child");
+	action_labels[110] = _("ctrl");
+	action_labels[111] = _("dirs");
+	action_labels[112] = _("escape");
+	action_labels[113] = _("expand");
+	action_labels[114] = _("files");
+	action_labels[115] = _("off");
+	action_labels[116] = _("on");
+	action_labels[117] = _("quote");
+	action_labels[118] = _("shift");
+	action_labels[119] = _("top");
 	//late entries, out of order
 	//maybe more space here, see array definition in header file
 #ifdef E2_VFS
-	action_labels[119] = _("<spacemenu>");
-	action_labels[120] = _("dummy");
-	action_labels[121] = _("namespace");
-	action_labels[122] = _("unpack_in_other");
+	action_labels[120] = _("<spacemenu>");
+	action_labels[121] = _("dummy");
+	action_labels[122] = _("namespace");
+	action_labels[123] = _("unpack_in_other");
 #endif
 #ifdef WITH_KEYFAKE
-	action_labels[123] = _("key");
+	action_labels[124] = _("key");
 #endif
 #if defined(WITH_KEYFAKE) || defined(WITH_BUTTONFAKE)
-	action_labels[124] = _("fake");
+	action_labels[125] = _("fake");
 #endif
 #ifdef WITH_BUTTONFAKE
-	action_labels[125] = _("button");
+	action_labels[126] = _("button");
 #endif
-	action_labels[126] = _("select_type");
 }
 #define LAST_NAME_INDEX 126
 #if LAST_NAME_INDEX > ALLOCATED_NAMES - 1
