@@ -1,6 +1,6 @@
-/* $Id: e2_action_option.c 1277 2008-09-27 01:10:48Z tpgww $
+/* $Id: e2_action_option.c 1840 2009-09-17 01:15:11Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -39,7 +39,7 @@ static gboolean _e2_action_option_set (gpointer from, E2_ActionRuntime *art)
 
 void e2_action_option_actions_register ()
 {
-	gchar *action_name = g_strconcat (_A(9),".",_A(81),NULL);  //_(option.set")
-	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
-		_e2_action_option_set, NULL, FALSE);
+	E2_Action action =
+	{g_strconcat(_A(9),".",_A(82),NULL),_e2_action_option_set,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+	e2_action_register (&action);
 }
