@@ -1,6 +1,6 @@
-/* $Id: e2_alias.c 1357 2008-12-05 03:00:55Z tpgww $
+/* $Id: e2_alias.c 1922 2009-09-28 05:22:51Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -302,7 +302,8 @@ This function is called only if the default is missing from the config file
 */
 static void _e2_alias_tree_defaults (E2_OptionSet *set)
 {
-	gchar *utf = F_FILENAME_FROM_LOCALE (DOC_DIR G_DIR_SEPARATOR_S MAIN_HELP);
+	gchar *local = DOC_DIR G_DIR_SEPARATOR_S MAIN_HELP;
+	gchar *utf = F_FILENAME_FROM_LOCALE (local);
 	e2_option_tree_setup_defaults (set,
 	g_strdup("command-aliases=<"),  //internal name
 // NOW IN COMMAND INTERPRETER
@@ -316,24 +317,29 @@ static void _e2_alias_tree_defaults (E2_OptionSet *set)
 //	g_strconcat(_("su"),"|true|$[command-xterm] -e sh -c 'su -c \\2;echo -n \"",_("Done. Press enter "),"\";read'",NULL),
 	g_strconcat(_("x"),"|true|xterm -e sh -c '\\2'",NULL),
 	g_strconcat(_("xx"),"|true|xterm -e sh -c '\\2;echo -n \"",_("Done. Press enter "),"\";read'",NULL),
+#ifdef E2_POLKIT
+	//see also _e2p_upgrade_su()
+	g_strdup("su|true|pkexec \\2"),
+#else
 	g_strconcat("su|true|xterm -e sh -c 'su -c \"\\2\";echo -n \"",_("Done. Press enter "),"\";read'",NULL),
+#endif
 	g_strdup("cp|true|cp -i \\2"),
 	g_strdup("rm|true|rm -i \\2"),
 	g_strconcat(_("clear"),"|true|",_A(10),".",_A(33),NULL),	//<
 	g_strconcat(_("quit"),"|true|",_A(1),".",_A(72),NULL),	//command.quit 	//<
-	g_strconcat(_("help"),"|true|",_A(6),".",_A(107)," ",utf," [commands]",NULL),  //file.view_at _I( NOTE helpfile titles are not (yet?) translated
+	g_strconcat(_("help"),"|true|",_A(6),".",_A(108)," ",utf," [commands]",NULL),  //file.view_at _I( NOTE helpfile titles are not (yet?) translated
 	g_strconcat(_("keys"),"|true|",_A(10),".",_A(54)," ",_("keys")," \\2",NULL),
 #ifdef E2_MOUSECUSTOM
 	g_strconcat(_("buttons"),"|true|",_A(10),".",_A(54)," ",_("buttons")," \\2",NULL),
 #endif
 	g_strconcat(_("e2ps"),"|true|",_A(2),".",_A(58),NULL),
 #ifdef E2_VFS
-	g_strconcat(_("cns"),"|true|",_A(13),".",_A(121),NULL),	//pane.namespace
+	g_strconcat(_("cns"),"|true|",_A(13),".",_A(122),NULL),	//pane.namespace
 #endif
 	g_strdup("wget|true|\\1 --progress=bar:force \\2"),
 	g_strdup(">"),
 	NULL);
-	F_FREE (utf);
+	F_FREE (utf, local);
 }
 /**
 @brief initialize alias-related options, and init some alias flags
