@@ -1,6 +1,6 @@
-/* $Id: e2_option.c 1244 2008-09-14 02:01:16Z tpgww $
+/* $Id: e2_option.c 1952 2009-10-02 01:43:23Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -54,16 +54,16 @@ static void _e2_option_clean1 (E2_OptionSet *set);
 
 /**
 @brief Initialize the config directory pointer
-It's intended that config dirs be native, not virtual
-The config directory pointer (e2_cl_options.config_dir) is set
-from command line or default dir. If the dir doesn't exist,
-create it if possible.
+It's intended that config dirs be native, not virtual.
+The config directory pointer (e2_cl_options.config_dir) is set from command line
+or default dir. If the dir doesn't exist, create it if possible.
 Any problem results in an appropriate error message.
 
 @return TRUE if dir exists or is successfully created, else FALSE
 */
 gboolean e2_option_set_config_dir (void)
 {
+	gboolean retval = TRUE;
 	//check if the config dir is really there
 	gchar *local = F_FILENAME_TO_LOCALE (e2_cl_options.config_dir);
 #ifdef E2_VFS
@@ -82,8 +82,7 @@ gboolean e2_option_set_config_dir (void)
 		{
 			printd (WARN, "could not create config directory '%s'",
 				e2_cl_options.config_dir);
-			F_FREE (local);
-			return FALSE;
+			retval = FALSE;
 		}
 	}
 	//it's there, but can we use it ?
@@ -95,11 +94,10 @@ gboolean e2_option_set_config_dir (void)
 	{	//nope
 		printd (WARN, "cannot access config directory '%s'",
 			e2_cl_options.config_dir);
-		F_FREE (local);
-		return FALSE;
+		retval = FALSE;
 	}
-	F_FREE (local);
-	return TRUE;
+	F_FREE (local, e2_cl_options.config_dir);
+	return retval;
 }
 /**
 @brief Initialize the default trash directory and a pointer to it
@@ -128,7 +126,7 @@ void e2_option_set_trash_dir (void)
 		gchar *subdir = g_build_filename (local, "files", NULL);
 		printd (INFO, "creating trash directory '%s'", subdir);
 #ifdef E2_VFS
-		ddata.localpath = subdir;
+		ddata.path = subdir;
 		if (e2_fs_recurse_mkdir (&ddata, 0755 E2_ERR_NONE()))
 #else
 		if (e2_fs_recurse_mkdir (local, 0755 E2_ERR_NONE()))
@@ -145,7 +143,7 @@ void e2_option_set_trash_dir (void)
 		g_free (subdir);
 		subdir = g_build_filename (local, "info", NULL);
 #ifdef E2_VFS
-		ddata.localpath = subdir;
+		ddata.path = subdir;
 		if (e2_fs_recurse_mkdir (&ddata, 0755 E2_ERR_NONE()))
 #else
 		if (e2_fs_recurse_mkdir (local, 0755 E2_ERR_NONE()))
@@ -165,12 +163,12 @@ void e2_option_set_trash_dir (void)
 	{	//nope
 		printd (WARN, "cannot access trash directory '%s'",
 			e2_cl_options.trash_dir);
-		F_FREE (local);
+		F_FREE (local, e2_cl_options.trash_dir);
 		g_free (e2_cl_options.trash_dir);
 		e2_cl_options.trash_dir = NULL;
 		return;
 	}
-	F_FREE (local);
+	F_FREE (local, e2_cl_options.trash_dir);
 	//cleanup, add trailer
 	if (e2_cl_options.trash_dir != NULL)
 		e2_cl_options.trash_dir = e2_utils_path_clean (e2_cl_options.trash_dir);
@@ -179,8 +177,10 @@ void e2_option_set_trash_dir (void)
 @brief dump all current configuration data and recreate it, with window content updates as appropriate
 This may be called from within a timer function, or manually
 It must not be run when there is an open config dialog (closing that causes crash)
+Probably needs BGL closed upon arrival here.
 @param reload TRUE to reload the config file before recreating config data
-@param recreate TRUE to recreate main window in accord with updated data
+@param recreate TRUE to recreate main window in accord with updated data,
+  FALSE to just update a few things
 
 @return
 */
@@ -190,6 +190,9 @@ void e2_option_refresh (gboolean reload, gboolean recreate)
 	//this is the one option that we want to keep ...
 	gboolean advanced = e2_option_bool_get ("advanced-config");
 	e2_keybinding_clean ();
+#ifdef E2_MOUSECUSTOM
+	e2_mousebinding_clean ();
+#endif
 	g_hash_table_destroy (app.filetypes);
 	e2_plugins_unload_all (FALSE);	//do this before data are cleared
 	//clear relevant current information
@@ -211,15 +214,17 @@ void e2_option_refresh (gboolean reload, gboolean recreate)
 	e2_pane_create_option_data (&app.pane1);
 	e2_pane_create_option_data (&app.pane2);
 
-	e2_toolbar_initialise (E2_BAR_PANE1);
-	e2_toolbar_initialise (E2_BAR_PANE2);
-	e2_toolbar_initialise (E2_BAR_TASK);
-	e2_toolbar_initialise (E2_BAR_COMMAND);
-
 	if (recreate)
-		e2_window_recreate (&app.window); //this also recreates key bindings
+		e2_window_recreate (&app.window); //this also recreates bindings
 	else
 	{
+		e2_toolbar_create (&app.pane1.toolbar);
+		e2_toolbar_rebadge (&app.pane1.toolbar);
+		e2_toolbar_create (&app.pane2.toolbar);
+		e2_toolbar_rebadge (&app.pane2.toolbar);
+		e2_toolbar_create (&app.toolbar);
+		e2_toolbar_create (&app.commandbar);
+
 		e2_keybinding_register_all ();
 #ifdef E2_MOUSECUSTOM
 		e2_mousebinding_register_all ();
@@ -277,13 +282,11 @@ gboolean cfgdirty;
 /**
 @brief check for and respond to changed config file
 
-This is the fn called by the check-config-files timer
-With FAM use, update is normally detected and flagged as part
-of the filepane monitoring, as all change-data arrives via the
-same stream
-Otherwise, here we check config file mtim.tvsec against a stored
-value.
-If update is indicated, recreate window after re-reading config data
+This is the fn called by the check-config-files timer.
+With E2_FAM, update is normally detected and flagged as part of the filepane
+monitoring, as all change-data arrives via the same stream.
+Otherwise, here we check config file mtime against a cached value.
+If update is indicated, re-create window after re-reading config data.
 @param user_data UNUSED pointer specified when the timer was initiated
 @return TRUE, always, so timer continues
 */
@@ -294,42 +297,44 @@ gboolean e2_option_check_config_files (gpointer user_data)
 	{
 #endif
 		struct stat stat_buf;
-		gchar *filename = g_build_filename (e2_cl_options.config_dir, default_config_file, NULL);
-		gchar *local = F_FILENAME_TO_LOCALE (filename);
+		gchar *cfg_file = g_build_filename (e2_cl_options.config_dir, default_config_file, NULL);
+		gchar *local = F_FILENAME_TO_LOCALE (cfg_file);
 #ifdef E2_VFS
 		VPATH ddata = { local, NULL }; //config data always local
-		cfgdirty = (!e2_fs_stat (&ddata, &stat_buf E2_ERR_NONE())
+		if (!e2_fs_stat (&ddata, &stat_buf E2_ERR_NONE())
 #else
-		cfgdirty = (!e2_fs_stat (local, &stat_buf E2_ERR_NONE())
+		if (!e2_fs_stat (local, &stat_buf E2_ERR_NONE())
 #endif
-				&& stat_buf.st_mtime != app.config_mtime);
+			&& stat_buf.st_size >= 20)	//first line of config file should be "# emelFM2 (v x.y.z)"
+				cfgdirty = (stat_buf.st_mtime != app.config_mtime);
+		else
+			cfgdirty = FALSE; //no point in reloading if the file is gone
 		if (cfgdirty && app.rebuild_enabled)
 			app.config_mtime = stat_buf.st_mtime;
-		F_FREE (local);
-		g_free (filename);
+		g_free (cfg_file);
+		F_FREE (local, cfg_file);
 #ifdef E2_FAM
 	}
 #endif
 	if (cfgdirty && app.rebuild_enabled)
 	{
-		//we're in a timer fn, so gtk's callback lock doesn't apply
-		//explicit lock - or else the filelists won't rebuild FIXME why is that
+		//CHECKME e2_option_refresh() probably needs BGL closed
 		gdk_threads_enter ();
 		e2_option_refresh (TRUE, TRUE);
-		gdk_threads_leave ();
 		e2_output_print (&app.tab, _("Configuration data re-loaded"), NULL, TRUE, NULL);
+		gdk_threads_leave ();
 		cfgdirty = FALSE;
 	}
 	return TRUE;
 }
 /**
 @brief register a config option
-
+This does not return if there's an allocation failure
 @param type flag for the type of set that it is
-@param name name of the option, a constant string
-@param group group the option belongs to, used in config dialog, a r-t string  FREEME
-@param desc textual description of the option used in config dialog, a r-t _() string FREEME ?
-@param tip tooltip used when displaying the config dialog, a _() string, or NULL
+@param name name of the option
+@param group group the option belongs to, used in config dialog
+@param desc textual description of the option used in config dialog
+@param tip tooltip used when displaying the config dialog, or NULL
 @param depends name of another option this one depends on, or NULL
 @param flags bitflags determining how the option data is handled
 
@@ -347,9 +352,12 @@ E2_OptionSet *e2_option_register (E2_OptionType type, gchar *name, gchar *group,
 #else
 		set = ALLOCATE (E2_OptionSet);
 #endif
-		CHECKALLOCATEDWARN (set, )
+#if (CHECKALLOCATEDFATAL)
+		CHECKALLOCATEDFATAL (set)
+#else
 		if (set != NULL)
 		{
+#endif
 			g_ptr_array_add (options_array, set);
 			g_hash_table_insert (options_hash, name, set);	//replace not valid
 			//non-freeable parameters set only once
@@ -357,7 +365,9 @@ E2_OptionSet *e2_option_register (E2_OptionType type, gchar *name, gchar *group,
 			set->flags = flags;
 			set->hook_freezed = FALSE;
 			set->widget = NULL;
+#if !(CHECKALLOCATEDFATAL)
 		}
+#endif
 	}
 	//CHECKME handle re-registration of a different type ?
 	//string parameters set each registration, so clear any old versions elsewhere
@@ -377,14 +387,14 @@ E2_OptionSet *e2_option_register (E2_OptionType type, gchar *name, gchar *group,
 	}
 	return set;
 }
-/**
+/* *
 @brief unregister option named @a name
 This is for plugin options, essentially
 @param name option name string
 
-@return TRUE if the option was registered
+@return TRUE if the option was unregistered
 */
-gboolean e2_option_unregister (gchar *name)
+/*gboolean e2_option_unregister (gchar *name)
 {
 	E2_OptionSet *set = g_hash_table_lookup (options_hash, name);
 	if (set == NULL)
@@ -394,6 +404,55 @@ gboolean e2_option_unregister (gchar *name)
 	g_hash_table_remove (options_hash, name);
 	return TRUE;
 }
+*/
+/**
+@brief unregister and backup option named @a name
+This is for plugin options, essentially
+FIXME ATM is only for non-tree options
+@param name option name string
+@return TRUE if @a option exists
+*/
+gboolean e2_option_backup (gchar *name)
+{
+	E2_OptionSet *set = g_hash_table_lookup (options_hash, name);
+	if (set != NULL)
+	{
+		gchar *saveme;
+		//reinstate the form of the config line(s)
+		switch (set->type)
+		{
+			case E2_OPTION_TYPE_INT:
+				saveme = g_strdup_printf ("%s=%d", name, set->ival);
+				break;
+			case E2_OPTION_TYPE_BOOL:
+			case E2_OPTION_TYPE_STR:
+			case E2_OPTION_TYPE_FONT:
+			case E2_OPTION_TYPE_COLOR:
+			case E2_OPTION_TYPE_SEL:
+				saveme = g_strconcat (name, "=", set->sval, NULL);
+				break;
+/*			case E2_OPTION_TYPE_TREE:
+				saveme = %s=<, option
+				GtkTreeIter iter;
+				if (gtk_tree_model_get_iter_first (set->ex.tree.model, &iter))
+					e2_option_tree_write_to_file (f, set, &iter, 0);	//error handling ?
+				if (e2_fs_file_write (f, ">\n") == 0)
+					goto error_handler;
+				break;
+*/
+			default:
+				printd (WARN, "don't know how to backup option '%s'", name);
+				return FALSE;
+		}
+		//log the freeable strings
+		e2_option_unknown_record (g_strdup (name), saveme);
+
+		g_ptr_array_remove (options_array, set);
+		g_hash_table_remove (options_hash, name);
+		return TRUE;
+	}
+	return FALSE;
+}
 /**
 @brief get E2_OptionSet with warning
 
@@ -401,11 +460,13 @@ gboolean e2_option_unregister (gchar *name)
 
 @return option or NULL if it cannot be found
 */
-E2_OptionSet *e2_option_get (gchar *option)
+E2_OptionSet *e2_option_get (const gchar *option)
 {
 	E2_OptionSet *set = (E2_OptionSet *) g_hash_table_lookup (options_hash, option);
+#ifdef DEBUG_MESSAGES
 	if (set == NULL)
 		printd (WARN, "trying to get option '%s' which doesn't exist", option);
+#endif
 	return set;
 }
 /**
@@ -415,7 +476,7 @@ E2_OptionSet *e2_option_get (gchar *option)
 
 @return option or NULL if the option cannot be found
 */
-E2_OptionSet *e2_option_get_simple (gchar *option)
+E2_OptionSet *e2_option_get_simple (const gchar *option)
 {
 	return (E2_OptionSet *) g_hash_table_lookup (options_hash, option);
 }
@@ -426,7 +487,7 @@ void *e2_option_void_get (E2_OptionSet *set, gchar *option)
 	switch (set->type)
 	{
 		case E2_OPTION_TYPE_BOOL:
-			if (g_str_equal (option, "true")) return (void *) TRUE;
+			if (!strcmp (option, "true")) return (void *) TRUE;
 			else return (void *) FALSE;
 			break;
 		case E2_OPTION_TYPE_STR:
@@ -443,7 +504,7 @@ void *e2_option_void_get (E2_OptionSet *set, gchar *option)
 */
 
 E2_OptionSet *e2_option_attach_value_changed (gchar *option, GtkWidget *widget,
-	gpointer func, gpointer data)
+	gboolean (*func)(gpointer,gpointer), gpointer data)
 {
 	E2_OptionSet *set = e2_option_get (option);
 	if (set != NULL)
@@ -452,7 +513,7 @@ E2_OptionSet *e2_option_attach_value_changed (gchar *option, GtkWidget *widget,
 }
 
 void e2_option_attach_value_changed_simple (E2_OptionSet *set, GtkWidget *widget,
-	gpointer func, gpointer data)
+	gboolean (*func)(gpointer,gpointer), gpointer data)
 {
 	GHook *hook = e2_hook_register (&set->hook_value_changed, func, data);
 	E2_Duo *duo = MALLOCATE (E2_Duo);	//too small for slice
@@ -473,56 +534,15 @@ void e2_option_connect (GtkWidget *controller, gboolean active)
 */
 void e2_option_clear_data (void)
 {
-/*FIXME if we only clear relevant data, when we re-walk the array later,
- it has some items with crap content
- more-or-less same code as _e2_option_clean1()
-	guint i;
-	gpointer *walker;
-	for (i = 0, walker = options_array->pdata; i < options_array->len; i++, walker++)
-	{
-		E2_OptionSet *set = *walker;
-		E2_OptionFlags flags = set->flags;
-		//several strings have to be re-installed when re-registering a set,
-		//so get rid of existing ones
- 		if (flags & E2_OPTION_FLAG_FREENAME)
-			g_free (set->name);
-		if (flags & E2_OPTION_FLAG_FREEGROUP)
-			g_free (set->group);
-		if (flags & E2_OPTION_FLAG_FREEDESC)
-			g_free (set->desc);
-		if (flags & E2_OPTION_FLAG_FREETIP)
-			g_free (set->tip);
-		if (flags & E2_OPTION_FLAG_FREEDEPENDS)
-			g_free (set->depends);
-
-		E2_OptionType type = set->type;
-		if (type &
-			( E2_OPTION_TYPE_INT | E2_OPTION_TYPE_STR
-			| E2_OPTION_TYPE_FONT | E2_OPTION_TYPE_COLOR ))
-				g_free (set->sval);
-		else if (type & E2_OPTION_TYPE_SEL)
-			g_strfreev (set->ex.sel.def);
-		else if (type & E2_OPTION_TYPE_TREE)
-		{
-			if (set->ex.tree.def != NULL)
-			{
-				g_strfreev (set->ex.tree.def);
-				set->ex.tree.def = NULL;
-			}
-			e2_list_free_with_data (&set->ex.tree.columns);
-			set->ex.tree.columns_num = 0;	//insurance, not really needed
-			g_object_unref (G_OBJECT (set->ex.tree.model));
-		}
-		//set-related hooks are un-registered when config dialog is destroyed
-		//anyway, we may want to run the hook after this change
-	}
-*/
-	g_ptr_array_free (options_array, TRUE);
-	options_array = g_ptr_array_new ();
-
+	g_ptr_array_set_size (options_array, 0); //no need for data clearing
+#ifdef USE_GLIB2_12
+	g_hash_table_remove_all (options_hash);	//data cleared
+#else
+//	g_hash_table_foreach_remove (options_hash, (GHRFunc) <return TRUE>, NULL);
 	g_hash_table_destroy (options_hash);
 	options_hash = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
 		(GDestroyNotify) _e2_option_clean1);
+#endif
 }
 
 /*void e2_option_tree_stores_clear (void)
@@ -538,9 +558,9 @@ void e2_option_clear_data (void)
 	}
 } */
 /* this was used only at session end, so don't bother to cleanup
-void destroy_config ()
+void e2_option_destroy_config ()
 {
-	g_ptr_array_free (options_array, FALSE);
+	g_ptr_array_free (options_array, TRUE);
 	g_hash_table_destroy (options_hash);
 	g_hash_table_destroy (options_queue);
 } */
@@ -566,16 +586,16 @@ void e2_option_file_write (const gchar *utfpath)
 	VPATH tdata;
 #endif
 	gboolean freepath = (utfpath == NULL);
-	gchar *usepath = (freepath) ?
+	gchar *cfg_file = (freepath) ?
 		g_build_filename (e2_cl_options.config_dir, default_config_file, NULL):
 		(gchar *)utfpath;
-	gchar *local = F_FILENAME_TO_LOCALE (usepath);
+	gchar *local = F_FILENAME_TO_LOCALE (cfg_file);
 	gchar *tempname = e2_utils_get_tempname (local);
 
 	E2_FILE *f = e2_fs_open_writestream (tempname E2_ERR_NONE());
 	if (f != NULL)
 	{
-		printd (DEBUG, "write config file: %s", usepath);
+		printd (DEBUG, "write config file: %s", cfg_file);
 		if (e2_fs_file_write (f,
 			//the 1st line is language-independent, for version verification
 			"# "PROGNAME" (v "VERSION RELEASE")\n\n" ) == 0)
@@ -605,17 +625,9 @@ void e2_option_file_write (const gchar *utfpath)
 							goto error_handler;
 					break;
 				case E2_OPTION_TYPE_BOOL:
-//					if (e2_fs_file_write (f, "# %s\n%s=%s\n",
-//						set->desc, set->name, set->sval) == 0)
-//							goto error_handler;
-//					break;
 				case E2_OPTION_TYPE_STR:
 				case E2_OPTION_TYPE_FONT:
 				case E2_OPTION_TYPE_COLOR:
-//					if (e2_fs_file_write (f, "# %s\n%s=%s\n",
-//						set->desc, set->name, set->sval) == 0)
-//							goto error_handler;
-//					break;
 				case E2_OPTION_TYPE_SEL:
 					if (e2_fs_file_write (f, "# %s\n%s=%s\n",
 						set->desc, set->name, set->sval) == 0)
@@ -645,11 +657,19 @@ void e2_option_file_write (const gchar *utfpath)
 		if (write_error)
 			goto error_handler;
 
+		//fs rename operation may be not reliably atomic (ext4)
+		gint fdesc = fileno (f);
+		if (fdesc == -1)
+			goto error_handler;
+		if (e2_fs_writeflush (fdesc) != 0)
+			goto error_handler;
+
 		e2_fs_close_stream (f);
+
 #ifdef E2_VFS
-		tdata.localpath = tempname;
+		tdata.path = tempname;
 		tdata.spacedata = NULL;
-		ddata.localpath = local;
+		ddata.path = local;
 		ddata.spacedata = NULL;
 #endif
 		gdk_threads_leave ();	//downstream errors invoke local mutex locking
@@ -676,13 +696,13 @@ error_handler:
 		gdk_threads_enter ();
 	}
 	gchar *msg = g_strdup_printf (_("Cannot write config file %s - %s"),
-		usepath, g_strerror (errno));	//ok for native-only config file
+		cfg_file, g_strerror (errno));	//ok for native-only config file
 	e2_output_print_error (msg, TRUE);
 	sleep (1);
 cleanup:
 	if (freepath)
-		g_free (usepath);
-	F_FREE (local);
+		g_free (cfg_file);
+	F_FREE (local, cfg_file);
 	g_free (tempname);
 }
 /**
@@ -729,7 +749,7 @@ gboolean e2_option_set_value_from_string (E2_OptionSet *set, gchar *str)
 	{
 		case E2_OPTION_TYPE_BOOL:
 			set->hook_freezed = TRUE;
-			e2_option_bool_set_direct (set, g_str_equal (str, "true") ? TRUE : FALSE);
+			e2_option_bool_set_direct (set, !strcmp (str, "true") ? TRUE : FALSE);
 			set->hook_freezed = FALSE;
 			break;
 		case E2_OPTION_TYPE_INT:
@@ -755,7 +775,7 @@ gboolean e2_option_set_value_from_string (E2_OptionSet *set, gchar *str)
 			set->ival = -1;
 			while ((val = set->ex.sel.def[i]) != NULL)
 			{
-				if (g_str_equal (str, val))
+				if (!strcmp (str, val))
 				{
 					set->ival = i;
 					set->sval = (gchar *) val;
@@ -804,7 +824,7 @@ void e2_option_read_array (gchar *f[])
 		split = g_strsplit (line, "=", 2);
 		if (split[1] != NULL)
 		{
-			if (!g_str_equal (split[1], "<")) //not a tree set
+			if (strcmp (split[1], "<")) //not a tree set
 			{
 			 	if (!e2_option_set_from_string (split[0], split[1]))
 					printd (WARN, "could not set option '%s'", split[0]);
@@ -827,8 +847,8 @@ If the file is read successfully, log the post-read config dir timestamp
 static gboolean _e2_option_config_file_read (gpointer *contents)
 {
 	//find absolute path to config file
-	gchar *filepath = g_build_filename (e2_cl_options.config_dir, default_config_file, NULL);
-	gchar *local = F_FILENAME_TO_LOCALE (filepath);
+	gchar *cfg_file = g_build_filename (e2_cl_options.config_dir, default_config_file, NULL);
+	gchar *local = F_FILENAME_TO_LOCALE (cfg_file);
 #ifdef E2_VFS
 	VPATH ddata = { local, NULL };	//only local config data
 	if (e2_fs_access (&ddata, R_OK E2_ERR_NONE()))
@@ -836,19 +856,19 @@ static gboolean _e2_option_config_file_read (gpointer *contents)
 	if (e2_fs_access (local, R_OK E2_ERR_NONE()))
 #endif
 	{	//try for the old-style name
-		g_free (filepath);
-		F_FREE (local);
-		filepath = g_build_filename (e2_cl_options.config_dir, "config", NULL);
-		local = F_FILENAME_TO_LOCALE (filepath);
+		g_free (cfg_file);
+		F_FREE (local, cfg_file);
+		cfg_file = g_build_filename (e2_cl_options.config_dir, "config", NULL);
+		local = F_FILENAME_TO_LOCALE (cfg_file);
 #ifdef E2_VFS
-		ddata.localpath = local;
+		ddata.path = local;
 		if (e2_fs_access (&ddata, R_OK E2_ERR_NONE()))
 #else
 		if (e2_fs_access (local, R_OK E2_ERR_NONE()))
 #endif
 		{
-			g_free (filepath);
-			F_FREE (local);
+			g_free (cfg_file);
+			F_FREE (local, cfg_file);
 			return FALSE;
 		}
 	}
@@ -871,16 +891,16 @@ static gboolean _e2_option_config_file_read (gpointer *contents)
 #endif
 		app.config_mtime = stat_buf.st_mtime;
 //#endif
-		printd (DEBUG, "config file '%s' read", filepath);
+		printd (DEBUG, "config file '%s' read", cfg_file);
 		retval = TRUE;
 	}
 	else
 	{
-		printd (WARN, "could not read config file '%s'", filepath);
+		printd (WARN, "could not read config file '%s'", cfg_file);
 		retval = FALSE;
 	}
-	g_free (filepath);
-	F_FREE (local);
+	g_free (cfg_file);
+	F_FREE (local, cfg_file);
 	return retval;
 }
 /**
@@ -956,8 +976,11 @@ static void _e2_option_clean1 (E2_OptionSet *set)
 		g_strfreev (set->ex.sel.def);
 	else if (type & E2_OPTION_TYPE_TREE)
 	{
-		if (set->ex.tree.def != NULL)
-			g_strfreev (set->ex.tree.def);
+		if (set->ex.tree.def.tree_strings != NULL)
+		{
+			g_strfreev (set->ex.tree.def.tree_strings);
+			set->ex.tree.def.tree_strings = NULL;
+		}
 		e2_list_free_with_data (&set->ex.tree.columns);
 		g_object_unref (G_OBJECT (set->ex.tree.model));
 	}
@@ -1042,13 +1065,13 @@ void e2_option_setup_labels (void)
 	config_labels[1]=_("bookmarks");
 	config_labels[2]=_("colors");
 	config_labels[3]=_("columns");
-	config_labels[4]=_("command bar");
+	config_labels[4]=_("command toolbar");
 	config_labels[5]=_("command line");
 	config_labels[6]=_("commands");
 	config_labels[7]=_("confirmation");
 	config_labels[8]=_("context menu");
 	config_labels[9]=_("custom menus");
-	config_labels[10]=_("delays");
+//	config_labels[10]=_("delays"); UNUSED
 	config_labels[11]=_("dialogs");
 	config_labels[12]=_("directory lines");
 	config_labels[13]=_("extensions");
@@ -1063,14 +1086,14 @@ void e2_option_setup_labels (void)
 	config_labels[22]=_("key bindings");
 	config_labels[23]=_("main"); //for bindings
 	config_labels[24]=_("make directory");
-	config_labels[25]=_("menus");
+//	config_labels[25]=_("menus"); UNUSED
 	config_labels[26]=_("miscellaneous");
 	config_labels[27]=_("options");
 	config_labels[28]=_("output");
 	config_labels[29]=_("pane 1");
-	config_labels[30]=_("pane1 bar");
+	config_labels[30]=_("pane1 toolbar");
 	config_labels[31]=_("pane 2");
-	config_labels[32]=_("pane2 bar");
+	config_labels[32]=_("pane2 toolbar");
 	config_labels[33]=_("panes");
 	config_labels[34]=_("plugins");
 #ifdef E2_MOUSECUSTOM
@@ -1081,7 +1104,7 @@ void e2_option_setup_labels (void)
 	config_labels[38]=_("startup");
 	config_labels[39]=_("style");
 	config_labels[40]=_("tab completion");
-	config_labels[41]=_("task bar");
+	config_labels[41]=_("task toolbar");
 	config_labels[42]=_("view");
 //maybe more space here, see array definition in header file
 }
