@@ -1,6 +1,6 @@
-/* $Id: e2_option__default.c 1327 2008-10-21 20:55:24Z tpgww $
+/* $Id: e2_option__default.c 1902 2009-09-25 05:07:11Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -61,13 +61,13 @@ initialisation tasks which are not strictly option-related.
 void e2_option_default_register (void)
 {
 	printd (DEBUG, "registering all default (non-tree) options");
+//NOTE: each group with a non-constant group_name must include one and only one
+//option with E2_OPTION_FLAG_FREEGROUP
 
 	  /*******************/
 	 /***** general *****/
 	/*******************/
 
-//NOTE: the option after each 'constructed' group_name string must include E2_OPTION_FLAG_FREEGROUP
-
 	gchar *group_name = g_strconcat(_C(17),":",_C(26),NULL); //_("general:miscellaneous"
 	e2_option_bool_register ("advanced-config", group_name,
 		_("show all options in config dialogs"), NULL,
@@ -82,21 +82,23 @@ void e2_option_default_register (void)
 		_("Directories with more than this many entries will be displayed in an interim state, until loading and sorting are completed"),
 		NULL, 100, 0, 1<<(sizeof (gint)*4,
 		E2_OPTION_FLAG_ADVANCED);
-); */
-	gchar *utf = F_FILENAME_FROM_LOCALE (DOC_DIR G_DIR_SEPARATOR_S MAIN_HELP);
+*/
+	gchar *local = DOC_DIR G_DIR_SEPARATOR_S MAIN_HELP;
+	gchar *utf = F_FILENAME_FROM_LOCALE (local);
 	e2_option_str_register ("usage-help-doc", group_name,
 		_("document containing usage advice"),
 		_("This document is opened from the help dialog usage page"),
 		NULL, utf,
 		E2_OPTION_FLAG_ADVANCED);	//no rebuild
-	F_FREE (utf);
-	utf = F_FILENAME_FROM_LOCALE (DOC_DIR G_DIR_SEPARATOR_S CFG_HELP);
+	F_FREE (utf, local);
+	local = DOC_DIR G_DIR_SEPARATOR_S CFG_HELP;
+	utf = F_FILENAME_FROM_LOCALE (local);
 	e2_option_str_register ("config-help-doc", group_name,
 		_("document containing configuration advice"),
 		_("This document is opened from the help dialog configuration page"),
 		NULL, utf,
 		E2_OPTION_FLAG_ADVANCED);	//no rebuild
-	F_FREE (utf);
+	F_FREE (utf, local);
 	e2_option_bool_register ("session-end-warning", group_name,
 		_("warn about running processes when shutting down"),
 		_("This enables a reminder about incomplete commands and actions"),
@@ -109,7 +111,7 @@ void e2_option_default_register (void)
 
 	e2_bookmark_options_register ();
 	e2_command_options_register ();
-	e2_filetype_options_register();
+	e2_filetype_options_register ();
 	e2_plugins_options_register ();
 
 	  /*********************/
@@ -118,7 +120,6 @@ void e2_option_default_register (void)
 
 	group_name = g_strconcat(_C(20) ,":",_C(26),NULL);  //_("interface:miscellaneous"
 #ifdef E2_COMPOSIT
-//	UNCOMMENT THIS
 	e2_option_int_register ("window-opacity", group_name, _("opacity"),
 		_("Window translucence, 30 (faint) to 100 (opaque)"),
 		NULL, 100, 30, 100,
@@ -137,13 +138,17 @@ void e2_option_default_register (void)
 		_("If activated, clicking on a free area will not clear the current selection"),
 		"windows-right-click", FALSE,
 		E2_OPTION_FLAG_ADVANCED); //no rebuild
-	e2_option_bool_register ("dialog-button-icons", group_name, _("show icons in dialog buttons"),
-		_("If activated, dialog buttons will show an icon as well as a label"),
-		NULL, TRUE,
-		E2_OPTION_FLAG_BASIC); //no rebuild
 
 	e2_keybinding_options_register ();	//here to make nicer order in tab
 
+	e2_option_int_register ("submenu-up-delay", group_name, _("menu popup delay (ms)"),
+		_("The delay (in milliseconds) after the mouse pointer arrives at a menu item, before any submenu will pop up"),
+		NULL, 50, 0, 10000,
+		E2_OPTION_FLAG_ADVANCED); //no rebuild
+	e2_option_int_register ("submenu-down-delay", group_name, _("menu popdown delay (ms)"),
+		_("The delay (in milliseconds) after the mouse pointer leaves a menu item, before popping down an activated submenu"),
+		NULL, 400, 0, 10000,
+		E2_OPTION_FLAG_ADVANCED); //no rebuild
 	const gchar *opt_scrollbar_position[] =
 		{_("bottom-right"), _("top-right"), _("bottom-left"), _("top-left"), NULL};
 	e2_option_sel_register ("scrollbar-position", group_name, _("scrollbar position"),
@@ -160,7 +165,23 @@ void e2_option_default_register (void)
 		_("The directory from which icon files will be retrieved"),
 			"use-icon-dir", "/YOURHOME/."BINNAME, E2_OPTION_FLAG_BUILDICONS |
 			E2_OPTION_FLAG_BUILDBARS | E2_OPTION_FLAG_ADVANCED);
-
+	const gchar *opt_icons_show[] =
+		{_("theme"),_("yes"), _("no"),NULL};
+	e2_option_sel_register ("dialog-button-icons", group_name, _("show icons in dialog buttons"),
+		_("Show an icon as well as a label"),
+		NULL, 1, opt_icons_show,
+		E2_OPTION_FLAG_ADVANCED); //no rebuild
+	e2_option_sel_register ("menu-show-icons", group_name, _("show icons in menus"),
+		_("Show an icon as well as a label"),
+		NULL, 1, opt_icons_show,
+		E2_OPTION_FLAG_ADVANCED);
+/*	const gchar *opt_menu_isize[] =
+		{_("menu"), _("toolbar small"), _("toolbar large"), _("button"), _("dnd"), _("dialog"), NULL};
+	e2_option_sel_register ("menu-isize", group_name, _("menu icon size"),
+		_("This sets the icon size for ALL menus"),
+		NULL/ *"menu-show-icons"* /, 0, opt_menu_isize,
+		E2_OPTION_FLAG_ADVANCED); //no rebuild
+*/
 #ifdef E2_MOUSECUSTOM
 	e2_mousebinding_options_register ();
 # ifdef E2_PTRGESTURES
@@ -169,37 +190,11 @@ void e2_option_default_register (void)
 #endif
 	e2_menu_custom_option_register ();
 
-	  /***************************/
-	 /***** interface.menus *****/
-	/***************************/
-
-	group_name = g_strconcat(_C(20),".",_C(25),":",_C(39),NULL); //_("interface.menus:style"
-	e2_option_bool_register ("menu-show-icons", group_name, _("show icons in menus"),
-		_("Some people think that icons help you recognize items more quickly"),
-		NULL, TRUE,
-		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_FREEGROUP); //no rebuild
-	const gchar *opt_menu_isize[] =
-		{_("menu"), _("toolbar small"), _("toolbar large"), _("button"), _("dnd"), _("dialog"), NULL};
-	e2_option_sel_register ("menu-isize", group_name, _("menu icon size"),
-		_("This sets the icon size for ALL menus"),
-		"menu-show-icons", 0, opt_menu_isize,
-		E2_OPTION_FLAG_ADVANCED); //no rebuild
-
-	group_name = g_strconcat(_C(20),".",_C(25),":",_C(10),NULL);  //_("interface.menus:delays"
-	e2_option_int_register ("submenu-up-delay", group_name, _("menu popup delay (ms)"),
-		_("The delay (in milliseconds) after the mouse pointer arrives at a menu item, before any submenu will pop up"),
-		NULL, 50, 0, 10000,
-		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_FREEGROUP); //no rebuild
-	e2_option_int_register ("submenu-down-delay", group_name, _("menu popdown delay (ms)"),
-		_("The delay (in milliseconds) after the mouse pointer leaves a menu item, before popping down an activated submenu"),
-		NULL, 400, 0, 10000,
-		E2_OPTION_FLAG_ADVANCED); //no rebuild
-
 	  /*****************/
 	 /***** panes *****/
 	/*****************/
 
-	group_name = g_strconcat(_C(33) ,":",_C(26),NULL);  //_("panes:miscellaneous"   //FREEME group_name not here !
+	group_name = g_strconcat(_C(33) ,":",_C(26),NULL);  //_("panes:miscellaneous"
 	e2_option_bool_register ("auto-refresh", group_name, _("auto refresh"),
 		_("This enables automatic checking for changes to the content of displayed directories. Otherwise you need to refresh manually"), NULL, TRUE,
 		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_FREEGROUP ); //no rebuild
@@ -222,7 +217,7 @@ void e2_option_default_register (void)
 #else
 		FALSE,
 #endif
-		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_FREEGROUP | E2_OPTION_FLAG_BUILDALL);
+		E2_OPTION_FLAG_BASIC  | E2_OPTION_FLAG_BUILDALL | E2_OPTION_FLAG_FREEGROUP);
 	const gchar *opt_menu_activeflag[] =
 		{_("colored headers"), _("bold name header"),_("sensitivity"),NULL};
 	e2_option_sel_register ("active-pane-signal", group_name, _("type of indicator for active pane"),
@@ -260,22 +255,14 @@ void e2_option_default_register (void)
 	e2_option_bool_register ("show-updir-entry", group_name, _("show parent directory entry '..' in file lists"),
 		_("This slows status-line updates"), NULL, FALSE,
 		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_BUILDLISTS);
+	e2_option_bool_register ("sort-dirs-first", group_name, _("directories first"),
+		_("This places directories ahead of other items in file lists"),
+		NULL, TRUE,
+		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_BUILDLISTS);
 	e2_option_bool_register ("namesort-case-sensitive", group_name, _("filename sort is case sensitive"),
 		_("This places all the capitalised file/directory names ahead of the others, if your LANG_C suports that"),
 		NULL, FALSE,
 		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_BUILDLISTS);
-	e2_option_bool_register ("pane2-uses-other", group_name, _("both panes are like pane 1"),
-		g_strdup_printf (
-		_("This makes pane %d options (other than toolbar placement and content) apply to pane %d"),
-		1,2),
-		"!pane1-uses-other", FALSE,
-		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_FREETIP);
-	e2_option_bool_register ("pane1-uses-other", group_name, _("both panes are like pane 2"),
-		g_strdup_printf (
-		_("This makes pane %d options (other than toolbar placement and content) apply to pane %d"),
-		2,1),
-		"!pane2-uses-other", FALSE,
-		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_FREETIP);
 
 	  /******************/
 	 /***** colors *****/
@@ -284,7 +271,7 @@ void e2_option_default_register (void)
 	group_name = g_strconcat(_C(33),".",_C(2),":",_C(21),NULL); //_("panes.colors:filetypes"
 	e2_option_color_register ("color-ft-exec", group_name, _("executable"),
 		_("Executable file names are listed in this color"), NULL, "forest green",
-		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_FREEGROUP | E2_OPTION_FLAG_BUILDLISTS);
+		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_BUILDLISTS | E2_OPTION_FLAG_FREEGROUP);
 	e2_option_color_register ("color-ft-dir", group_name, _("directory"),
 		_("Directory names are listed in this color"), NULL, "blue",
 		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_BUILDLISTS);
@@ -311,7 +298,7 @@ void e2_option_default_register (void)
 #endif
 	e2_option_color_register ("color-active-pane", group_name, _("active pane header color"),
 		_("This color is used for the background of column headers in the active pane"), NULL, "dark grey",
-		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_FREEGROUP | E2_OPTION_FLAG_BUILDPANES);
+		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_BUILDPANES | E2_OPTION_FLAG_FREEGROUP);
 	e2_option_color_register ("color-highlight", group_name, _("highlight color"),
 		_("This color is used for the background of highlighed item-names and other text"), NULL, "wheat",
 		E2_OPTION_FLAG_ADVANCED); //no rebuild
@@ -339,17 +326,17 @@ tags not supported now
 		opt_toolbar_style[0], opt_toolbar_style[1]);
 	e2_option_sel_register ("allbars-style", group_name, _("toolbars button style"), tip, NULL,
 		1, opt_toolbar_style,
-		E2_OPTION_FLAG_BASICONLY | E2_OPTION_FLAG_FREEGROUP
-		| E2_OPTION_FLAG_FREETIP | E2_OPTION_FLAG_BUILDBARS
-		| E2_OPTION_FLAG_BUILDSAMEBARS);
+		E2_OPTION_FLAG_BASICONLY | E2_OPTION_FLAG_BUILDBARS
+		| E2_OPTION_FLAG_BUILDSAMEBARS | E2_OPTION_FLAG_FREEGROUP
+		| E2_OPTION_FLAG_FREETIP);
 	const gchar *opt_toolbar_isize[] =
 		{_("theme"),_("menu"), _("toolbar small"), _("toolbar large"), _("button"), _("dnd"), _("dialog"),NULL};
 	tip = g_strdup_printf (_("'%s' uses the Gtk default, '%s' is smallest, '%s' is largest"),
 		opt_toolbar_isize[0], opt_toolbar_isize[1], opt_toolbar_isize[6]);
-		e2_option_sel_register ("allbars-isize", group_name, _("toolbars icon size"), tip,
+	e2_option_sel_register ("allbars-isize", group_name, _("toolbars icon size"), tip,
 		NULL, 2, opt_toolbar_isize,
-		E2_OPTION_FLAG_BASICONLY | E2_OPTION_FLAG_FREETIP
-		| E2_OPTION_FLAG_BUILDBARS | E2_OPTION_FLAG_BUILDSAMEBARS);
+		E2_OPTION_FLAG_BASICONLY | E2_OPTION_FLAG_BUILDBARS
+		| E2_OPTION_FLAG_BUILDSAMEBARS | E2_OPTION_FLAG_FREETIP);
 
 	e2_toolbar_toolbaroptions_register ();
 
@@ -384,5 +371,4 @@ tags not supported now
 // keybindings have a separate data structure too
 //above	e2_keybinding_init(); //this also does some other stuff
 //above	e2_filetype_options_register();
-
 }
