@@ -1,6 +1,6 @@
-/* $Id: e2_option_tree.c 1280 2008-09-27 23:36:53Z tpgww $
+/* $Id: e2_option_tree.c 1949 2009-10-01 13:26:52Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -32,6 +32,7 @@ Functions to handle tree options
 #ifdef E2_MOUSECUSTOM
 # include "e2_mousebinding.h"
 #endif
+#include "e2_keybinding.h"
 
 //data struct for a deferred tree-option overwrite dialog
 typedef struct _E2_OptionTreeCheck
@@ -85,6 +86,7 @@ static void _e2_option_tree_key_edit_start_cb (GtkCellRenderer *renderer,
 static void _e2_option_tree_btn_edit_start_cb (GtkCellRenderer *renderer,
 	GtkCellEditable *editable, gchar *path_string, E2_OptionSet *set);
 #endif
+static void _e2_option_tree_unblock_mnemonics (void);
 
 extern GtkTreeStore *actions_store;
 
@@ -111,9 +113,9 @@ void e2_option_tree_flag_change (E2_OptionSet *set)
   /**************************/
  /***** data functions *****/
 /**************************/
-/*These functions set some properties of a treeview cell instead of using
-the straight mapping between the cell and the treemodel. They are called
-whenever the cell is rendered, and at the end of (not during) cell-editing */
+/*These functions set some properties of a treeview cell instead of using the
+straight mapping between the cell and the treemodel. They are called whenever
+the cell is rendered, and at the end of (not during) cell-editing */
 static void _e2_option_tree_bool_data_func (GtkTreeViewColumn *tree_column,
 	GtkCellRenderer *cell, GtkTreeModel *model, GtkTreeIter *iter,
 	E2_OptionTreeColumn *col)
@@ -123,14 +125,17 @@ static void _e2_option_tree_bool_data_func (GtkTreeViewColumn *tree_column,
 	gboolean b;
 	gtk_tree_model_get (model, iter, column, &b, -1);
 
-	gboolean visible = TRUE;
+	gboolean visible;
 	if (col->visible_check_func != NULL)
 	{
 		gboolean (*fun) (GtkTreeModel *, GtkTreeIter *, GtkCellRenderer *,
 			gpointer) = col->visible_check_func;
 		visible = fun (model, iter, cell,  col->visible_check_data);
 	}
-	g_object_set (G_OBJECT (cell), "visible", visible, "active", b, NULL);
+	else
+		visible = TRUE;
+	//in spite of gtk API doc, setting "visible" property to a bool value spits an error message
+	g_object_set (G_OBJECT (cell), "visible", (visible)?1:0, "active", b, NULL);
 }
 
 static void _e2_option_tree_str_data_func (GtkTreeViewColumn *tree_column,
@@ -142,14 +147,17 @@ static void _e2_option_tree_str_data_func (GtkTreeViewColumn *tree_column,
 	gchar *str;
 	gtk_tree_model_get (model, iter, column, &str, -1);
 
-	gboolean visible = TRUE;
+	gboolean visible;
 	if (col->visible_check_func != NULL)
 	{
 		gboolean (*fun) (GtkTreeModel *, GtkTreeIter *, GtkCellRenderer *,
 			gpointer) = col->visible_check_func;
 		visible = fun (model, iter, cell,  col->visible_check_data);
 	}
-	g_object_set (G_OBJECT (cell), "visible", visible, "text", str, NULL);
+	else
+		visible = TRUE;
+	//in spite of gtk API doc, setting "visible" property to a bool value spits an error message
+	g_object_set (G_OBJECT (cell), "visible", (visible)?1:0, "text", str, NULL);
 	g_free (str);
 }
 
@@ -178,9 +186,13 @@ static void _e2_option_tree_string_edited_cb (GtkCellRendererText *cell,
 	if (new_text == NULL)	//this probably can't happen
 		return;
 	printd (DEBUG, "tree-option edited cb, new text is %s", new_text);
-	if (g_str_equal (set->name, "keybindings")
-		&& g_str_equal (new_text, _("Press key")))
+	if (!strcmp (set->name, "keybindings")
+		&& !strcmp (new_text, _("Press key")))
+	{
+		e2_keybinding_unblock (config_dialog, NULL);
+		_e2_option_tree_unblock_mnemonics ();
 		return;
+	}
 
 	GtkTreeIter iter;
 	if (gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (set->ex.tree.model),
@@ -326,8 +338,8 @@ static gboolean _e2_option_tree_btnchange_check (E2_BtnChangeData *data)
 			GtkTreeModel *mdl = data->set->ex.tree.model;
 			gchar *new_seq, *new_btnname = data->newtext;
 			gboolean new_dbl, new_trpl;
-			gboolean button = g_str_equal (data->set->name, "mousebuttons");
-			gboolean stroke = g_str_equal (data->set->name, "mousedrags");
+			gboolean button = !strcmp (data->set->name, "mousebuttons");
+			gboolean stroke = !strcmp (data->set->name, "mousedrags");
 
 			if (button)
 			{
@@ -355,7 +367,7 @@ static gboolean _e2_option_tree_btnchange_check (E2_BtnChangeData *data)
 			{
 				gchar *storedbtn;
 				gtk_tree_model_get (mdl, &srch_iter, MCOL_BTN, &storedbtn, -1);
-				if (g_str_equal (storedbtn, new_btnname)
+				if (!strcmp (storedbtn, new_btnname)
 					&& (srch_iter.stamp != iter.stamp
 					 || srch_iter.user_data != iter.user_data
 					 || srch_iter.user_data2 != iter.user_data2
@@ -369,7 +381,7 @@ static gboolean _e2_option_tree_btnchange_check (E2_BtnChangeData *data)
 						gchar *sequence;
 						gtk_tree_model_get (mdl, &srch_iter,
 							3, &sequence, 4, &action, 5, &arg, -1);
-						matched = g_str_equal (new_seq, sequence);
+						matched = !strcmp (new_seq, sequence);
 						g_free (sequence);
 					}
 					else
@@ -511,7 +523,7 @@ static gboolean _e2_option_tree_btnchange_cb (GtkEntry *entry,
 	else if (event->type == GDK_2BUTTON_PRESS || event->type == GDK_2BUTTON_PRESS)
 	{
 		//FIXME this changes focus and therefore ends the edit
-		if (g_str_equal (data->set->name, "mousebuttons"))
+		if (!strcmp (data->set->name, "mousebuttons"))
 		{
 			GtkTreeIter iter;
 			GtkTreeModel *mdl = data->set->ex.tree.model;
@@ -540,7 +552,7 @@ static gboolean _e2_option_tree_btnchange_cb (GtkEntry *entry,
 			//FIXME storing new data ends the edit, so prevents double-clicks
 			_e2_option_tree_btnchange_check (data); //verify before storing
 			//FIXME also unblock bindings if the user cancels or changes focus etc
-//			if (g_str_equal (data->set->name, "mousebuttons"))
+//			if (!strcmp (data->set->name, "mousebuttons"))
 				e2_mousebinding_unblock (config_dialog, NULL);
 //			else //"mousedrags"
 //				e2_mousegesture_unblock (config_dialog, NULL); insurance ?
@@ -599,7 +611,7 @@ static void _e2_option_tree_btn_edit_start_cb (GtkCellRenderer *renderer,
 		api->editing_done = _e2_option_tree_fake_edit_done;
 
 		//prevent operation of any aberrant current binding
-//		if (g_str_equal (set->name, "mousebuttons"))
+//		if (!strcmp (set->name, "mousebuttons"))
 			e2_mousebinding_block (config_dialog, NULL);
 //		else //"mousedrags"
 //			e2_mousegesture_block (config_dialog, NULL); insurance ?
@@ -639,7 +651,7 @@ static gboolean _e2_option_tree_keychange_check (E2_KeyChangeData *data)
 			gchar *storedkey, *action, *arg;
 			gtk_tree_model_get (mdl, &srch_iter, 1, &storedkey,
 				3, &action, 4, &arg, -1);
-			if (g_str_equal (storedkey, keyname)
+			if (!strcmp (storedkey, keyname)
 				&& (srch_iter.stamp != iter.stamp
 				 || srch_iter.user_data != iter.user_data
 				 || srch_iter.user_data2 != iter.user_data2
@@ -680,10 +692,6 @@ static gboolean _e2_option_tree_keychange_check (E2_KeyChangeData *data)
 				_e2_option_tree_key_edit_start_cb, data->set);
 		}
 	}
-
-	g_free (data->path_string);
-	g_free (data->newtext);
-	DEALLOCATE (KeyChangeData, data);
 	return FALSE;
 }
 /**
@@ -762,10 +770,117 @@ static gboolean _e2_option_tree_keychange_cb (GtkEntry *entry,
 
 	data->newtext = keyname;
 	_e2_option_tree_keychange_check (data);
-	//FIXME also unblock bindings if the user cancels, changes focus etc
+	//CHECKME also unblock bindings if the user cancels, changes focus etc
 	e2_keybinding_unblock (config_dialog, NULL);
+	_e2_option_tree_unblock_mnemonics ();
 	return TRUE;
 }
+
+/* block mnemonic activation of dialog buttons while doing a key-binding */
+
+typedef struct _E2_KeyBlock
+{
+	gboolean blocked;
+	E2_KeyChangeData *data;
+} E2_KeyBlock;
+//static data used cuz only one config dialog can exist
+//Otherwise need to send variable data to callback via each relevant button
+E2_KeyBlock blocknow = {FALSE, NULL};
+
+#define KEYCHANGEKEY "_e2-binding-key_"
+/**
+@brief "mnemonic-activate" signal callback for some config dialog buttons
+
+Block further handling of the button-press, during a keybindings configuration.
+In that case, determine and process the mnemonic keyval for @a button here, as
+there will be no button-press-cb later
+
+@param button the activated button
+@param group_cycling UNUSED TRUE if > 1 widget has the mnemonic being processed now
+@param bdata pointer to data to use in the callback
+
+@return FALSE when further (i.e. normal) processing of the mnemonic keypress is ok
+*/
+static gboolean _e2_option_tree_mnemonic_cb (GtkWidget *button, gboolean group_cycling,
+	E2_KeyBlock *bdata)
+{
+	printd (DEBUG, "_e2_option_tree_mnemonic_cb");
+//	E2_KeyBlock *bdata = g_object_get_data (G_OBJECT(button), KEYCHANGEKEY);
+	if (bdata != NULL && bdata->blocked && bdata->data != NULL)
+	{
+		GList *child, *all = gtk_widget_list_mnemonic_labels (button);
+		for (child = all; child != NULL; child = child->next)
+		{
+			GtkWidget *wid = child->data;
+			if (GTK_IS_LABEL(wid))
+			{
+				guint code = gtk_label_get_mnemonic_keyval (GTK_LABEL(wid));
+				gchar *keyname = gtk_accelerator_name (code, GDK_MOD1_MASK); //CHECKME get actual modifiers ?
+				//replicate _e2_option_tree_keychange_cb()
+				bdata->data->newtext = keyname;
+				_e2_option_tree_keychange_check (bdata->data);
+				//CHECKME also unblock bindings if the user cancels, changes focus etc
+				e2_keybinding_unblock (config_dialog, NULL);
+				//unblock all button-mnemonics
+				bdata->blocked = FALSE;
+				bdata->data = NULL; //will have been cleared downstream
+				break;
+			}
+		}
+		g_list_free (all);
+		return TRUE;
+	}
+	else if (bdata != NULL)
+		return bdata->blocked;
+	else
+		return FALSE;
+}
+
+void e2_option_tree_connect_mnemonics (GtkWidget *button_box)
+{
+//	printd (DEBUG, "e2_option_tree_connect_mnemonics");
+	GList *btn, *all = gtk_container_get_children (GTK_CONTAINER(button_box));
+	for (btn = all; btn != NULL; btn = btn->next)
+	{
+		g_signal_connect (G_OBJECT(btn->data), "mnemonic-activate",
+			G_CALLBACK(_e2_option_tree_mnemonic_cb), &blocknow);
+	}
+	g_list_free (all);
+}
+/*
+static void _e2_option_tree_set_mnemonics_data (GtkWidget **buttons, guint count,
+	E2_KeyChangeData *data)
+{
+	guint i;
+	printd (DEBUG, "_e2_option_tree_set_mnemonics_data");
+
+	blocknow.data = data;
+
+	for (i = 0; i < count; i++, buttons++)
+	{
+		if (*buttons != NULL)
+			g_object_set_data_full (G_OBJECT(*buttons), KEYCHANGEKEY, &blocknow, NULL);
+	}
+}
+*/
+/**
+@brief re-start button mnemonics after a keybinding config
+
+@return
+*/
+static void _e2_option_tree_unblock_mnemonics (void)
+{
+//	printd (DEBUG, "_e2_option_tree_unblock_mnemonics");
+	blocknow.blocked = FALSE;
+	if (blocknow.data != NULL)
+	{
+		g_free (blocknow.data->path_string);
+		g_free (blocknow.data->newtext);
+		DEALLOCATE(E2_KeyChangeData, blocknow.data);
+		blocknow.data = NULL;
+	}
+}
+
 /**
 @brief setup to amend the key name in a treeview cell
 
@@ -797,6 +912,17 @@ static void _e2_option_tree_key_edit_start_cb (GtkCellRenderer *renderer,
 
 		//prevent operation of any aberrant current binding
 		e2_keybinding_block (config_dialog, NULL);
+		//or of any dialog-button mnemonic
+		blocknow.blocked = TRUE;
+		blocknow.data = data;
+/* if static data not viable
+		//for main dialog buttons
+		E2_Sextet *st = ?
+		_e2_option_tree_set_mnemonics_data ((GtkWidget**)st, 6, &blocknow-replacement);
+		//for current page buttons
+		st = g_object_get_data (G_OBJECT(set->widget), "e2-config-buttons");
+		_e2_option_tree_set_mnemonics_data ((GtkWidget**)st, 6, &blocknow-replacement);
+*/
 		g_signal_connect (G_OBJECT (entry), "key-press-event",
 			G_CALLBACK (_e2_option_tree_keychange_cb), data);
     }
@@ -834,9 +960,12 @@ static void _e2_option_tree_cell_edit_start_cb (GtkCellRenderer *renderer,
 static void _e2_option_tree_cell_edit_stop_cb (GtkCellRenderer *renderer,
 	gpointer user_data)
 {
-//	printd (DEBUG, "stop cell edit cb");
+	printd (DEBUG, "stop cell edit cb");
 	g_signal_handlers_unblock_by_func (G_OBJECT (config_dialog),
 		e2_dialog_key_neg_cb, config_dialog);
+	//CHECKME cancel other key blocking
+	//e2_keybinding_unblock (config_dialog, NULL);
+	//_e2_option_tree_unblock_mnemonics ();
 }
 
 static void _e2_option_tree_page_opened_cb (GtkWidget *widget, E2_OptionSet *set)
@@ -889,9 +1018,8 @@ static gboolean _e2_option_tree_drag_delete_cb (GtkTreeDragSource *drag_source,
 	else
 		return FALSE;
 }
-
-static gboolean _e2_option_tree_move_up_cb (GtkWidget *widget,
-	E2_OptionSet *set)
+/* Shared callback for button "clicked" and menu-item "activate" */
+static gboolean _e2_option_tree_move_up_cb (GtkWidget *widget, E2_OptionSet *set)
 {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
@@ -909,9 +1037,8 @@ static gboolean _e2_option_tree_move_up_cb (GtkWidget *widget,
 	}
 	return TRUE;
 }
-
-static gboolean _e2_option_tree_move_down_cb (GtkWidget *widgget,
-	E2_OptionSet *set)
+/* Shared callback for button "clicked" and menu-item "activate" */
+static gboolean _e2_option_tree_move_down_cb (GtkWidget *widget, E2_OptionSet *set)
 {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
@@ -1086,7 +1213,7 @@ static gboolean _e2_option_tree_button_press_cb (GtkWidget *treeview,
 	return FALSE;
 }
 
-static gboolean _e2_option_tree_help_cb (GtkWidget *button, E2_OptionSet *set)
+static gboolean _e2_option_tree_help_cb (GtkButton *button, E2_OptionSet *set)
 {
 	//get the config dialog page label, from set->group
 	// (after a parent separator '.' if any)
@@ -1096,7 +1223,7 @@ static gboolean _e2_option_tree_help_cb (GtkWidget *button, E2_OptionSet *set)
 		page = realpage + sizeof (gchar);
 
 	//relevant section label is "[page]"
-	gchar *command = g_strconcat(_A(6),".",_A(107)," ",
+	gchar *command = g_strconcat(_A(6),".",_A(108)," ",
 		e2_option_str_get ("config-help-doc")," [",page,"]",NULL);	//file.view_at;
 	e2_command_run (command, E2_COMMAND_RANGE_DEFAULT, button
 #ifdef E2_COMMANDQ
@@ -1106,8 +1233,175 @@ static gboolean _e2_option_tree_help_cb (GtkWidget *button, E2_OptionSet *set)
 	g_free (command);
 	return TRUE;
 }
+/**
+@brief translate all current keybindings
 
-static gboolean e2_option_tree_del_direct_cb (GtkWidget *button, E2_OptionSet *set)
+@param button activated widget, UNUSED
+@param set ptr to keybindings option data struct
+
+@return
+*/
+static void _e2_option_tree_translate_cb (GtkWidget *button, E2_OptionSet *set)
+{
+	GtkTreeIter iter;
+	if (gtk_tree_model_get_iter_first (set->ex.tree.model, &iter))
+	{
+		e2_keybinding_localise (set->ex.tree.model, &iter);
+		e2_option_tree_flag_change (set);
+	}
+}
+
+/**
+@brief recursively replace sub-string(s) in toolbar item argument string(s) and maybe swap 'go' buttons
+@param model treemodel for the store being copied
+@param iter pointer to 'parent' treeiter in @a model, NULL for root
+@param oldarg1 one of the substrings to be replaced
+@param oldarg2 one of the substrings to be replaced
+@param newarg1 one of the replacement substrings
+@param newarg2 one of the replacement substrings
+@param actforwd name of action to swap
+@param actback name of action to swap
+
+@return
+*/
+static void _e2_option_tree_update_bar_descendants (
+	GtkTreeModel *model, GtkTreeIter *iter,
+	const gchar *oldarg1, const gchar *oldarg2,
+	const gchar *newarg1, const gchar *newarg2,
+	const gchar *actforwd, const gchar *actback)
+{
+	GtkTreeIter child;
+	if (gtk_tree_model_iter_children (model, &child, iter))
+	{
+		GtkTreePath *gopath = NULL;
+		do
+		{
+			gchar *tmp1, *tmp2 ,*actstr, *argstr;
+			gboolean swap = FALSE;
+
+			gtk_tree_model_get (model, &child, 3, &actstr, 4, &argstr, -1);
+			if (actstr)
+			{
+				if (actforwd && actback
+					&& (!strcmp (actstr, actforwd) || !strcmp (actstr, actback)))
+					swap = TRUE;
+				tmp1 = e2_utils_str_replace (actstr, oldarg1, newarg1);
+				gtk_tree_store_set (GTK_TREE_STORE(model), &child, 3, tmp1, -1);
+				g_free (tmp1);
+				g_free (actstr);
+			}
+			if (argstr)
+			{
+				tmp1 = e2_utils_str_replace (argstr, oldarg1, newarg1);
+				tmp2 = e2_utils_str_replace (tmp1, oldarg2, newarg2);
+				g_free (tmp1);
+				gtk_tree_store_set (GTK_TREE_STORE(model), &child, 4, tmp2, -1);
+				g_free (tmp2);
+				g_free (argstr);
+			}
+			if (gtk_tree_model_iter_has_child (model, &child))
+				_e2_option_tree_update_bar_descendants (model, &child,
+					oldarg1, oldarg2, newarg1, newarg2, actforwd, actback);
+			if (swap)
+			{
+				if (gopath == NULL)
+					//crudely log the first of a potential pair
+					gopath = gtk_tree_model_get_path (model, &child);
+				else
+				{
+					GtkTreeIter goiter;
+					gtk_tree_model_get_iter (model, &goiter, gopath);
+					gtk_tree_store_swap (GTK_TREE_STORE (model), &goiter, &child);
+					child = goiter;
+					gtk_tree_path_free (gopath);
+					gopath = NULL;
+				}
+			}
+		} while (gtk_tree_model_iter_next (model, &child));
+
+		if (gopath != NULL)
+			gtk_tree_path_free (gopath);
+	}
+}
+/**
+@brief replicate all bar data
+
+@param button UNUSED clicked widget
+@param set ptr to keybindings option data struct for the bar to be changed
+
+@return
+*/
+static void _e2_option_tree_barcopy_cb (GtkButton *button, E2_OptionSet *set)
+{
+	gint srcp;
+	//get the 'source' set
+	E2_OptionSet *srcset = e2_option_get_simple ("panebar2");
+	if (srcset == set)
+	{
+		srcp = 1;
+		srcset = e2_option_get_simple ("panebar1");
+	}
+	else
+		srcp = 2;
+
+	//replicate all data
+	gpointer newstore;
+	e2_tree_store_copy (srcset->ex.tree.model, TRUE, &newstore);
+	//update relevant data
+	GtkTreeIter iter;
+	if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL(newstore), &iter))
+	{
+		const gchar *oldarg1, *oldarg2, *newarg1, *newarg2;
+		gchar *actforwd, *actback;
+		gboolean hori = !e2_option_bool_get ("panes-horizontal");
+
+		if (hori)
+		{
+			//names of actions to be swapped when filelists are side-by-side
+			actforwd = g_strconcat (_A(13), ".", _A(50), NULL);
+			actback = g_strconcat (_A(13), ".", _A(49), NULL);
+		}
+		else
+		{
+			actforwd = actback = NULL;
+		}
+
+		if (srcp == 1)
+		{
+			oldarg1 = _A(11); //_("pane1")
+			newarg1 = _A(12); //_("pane2");
+			oldarg2 = "1";
+			newarg2 = "2";
+		}
+		else
+		{
+			oldarg1 = _A(12);
+			newarg1 = _A(11);
+			oldarg2 = "2";
+			newarg2 = "1";
+		}
+
+		_e2_option_tree_update_bar_descendants (GTK_TREE_MODEL(newstore), NULL,
+					oldarg1, oldarg2, newarg1, newarg2, actforwd, actback);
+
+		if (hori)
+		{
+			g_free (actforwd);
+			g_free (actback);
+		}
+	}
+
+	gtk_tree_view_set_model (GTK_TREE_VIEW (set->widget), GTK_TREE_MODEL(newstore));
+//	g_object_unref (G_OBJECT (newstore)); problem when dialog is closed
+	g_object_unref (G_OBJECT (GTK_TREE_STORE (set->ex.tree.model)));
+	set->ex.tree.model = (GTK_TREE_MODEL (newstore));
+
+	gtk_tree_view_expand_all (GTK_TREE_VIEW (set->widget));
+
+	e2_option_tree_flag_change (set);
+}
+
+static void e2_option_tree_del_direct_cb (GtkButton *button, E2_OptionSet *set)
 {
 	GtkTreeIter iter;
 	if (gtk_tree_selection_get_selected (gtk_tree_view_get_selection
@@ -1164,9 +1458,8 @@ static gboolean e2_option_tree_del_direct_cb (GtkWidget *button, E2_OptionSet *s
 			}
 		}
 	}
-	return TRUE;
 }
-
+/* Shared callback for button "clicked" and menu-item "activate" */
 static gboolean _e2_option_tree_add_child_cb (GtkWidget *wid, E2_OptionSet *set)
 {
 	GtkTreeIter iter;
@@ -1185,7 +1478,7 @@ static gboolean _e2_option_tree_add_child_cb (GtkWidget *wid, E2_OptionSet *set)
 
 	return TRUE;
 }
-
+/* Shared callback for button "clicked" and menu-item "activate" */
 static gboolean _e2_option_tree_add_below_cb (GtkWidget *wid, E2_OptionSet *set)
 {
 	GtkTreeIter iter;
@@ -1219,17 +1512,18 @@ static gboolean _e2_option_tree_add_below_cb (GtkWidget *wid, E2_OptionSet *set)
 */
 void e2_option_tree_adjust_buttons (GtkTreeView *view, gboolean value)
 {
-	E2_Sextet *q = g_object_get_data (G_OBJECT (view), "e2-config-buttons");
-	if (q == NULL)
-		return;
-	if (q->b != NULL)
-		gtk_widget_set_sensitive (q->b, value);
-	if (q->c != NULL)
-		gtk_widget_set_sensitive (q->c, value);
-	if (q->d != NULL)
-		gtk_widget_set_sensitive (q->d, value);
-	if (q->e != NULL)
-		gtk_widget_set_sensitive (q->e, value);
+	E2_Sextet *st = g_object_get_data (G_OBJECT (view), "e2-config-buttons");
+	if (st != NULL)
+	{
+		if (st->b != NULL)
+			gtk_widget_set_sensitive (st->b, value);
+		if (st->c != NULL)
+			gtk_widget_set_sensitive (st->c, value);
+		if (st->d != NULL)
+			gtk_widget_set_sensitive (st->d, value);
+		if (st->f != NULL)
+			gtk_widget_set_sensitive (st->f, value);
+	}
 }
 /**
 @brief create, and add to @a box, a config-data treeview for tree-option @a set
@@ -1253,6 +1547,7 @@ void e2_option_tree_add_widget (GtkWidget *parent, gboolean single, GtkWidget *b
 	//setup treeview
 	GtkTreeView *treeview = GTK_TREE_VIEW (gtk_tree_view_new_with_model
 		(set->ex.tree.model));
+	//don't unref set->ex.tree.model here !
 	gtk_container_add (GTK_CONTAINER (sw), GTK_WIDGET (treeview));
 	gtk_tree_view_set_reorderable (treeview, TRUE);
 //	gtk_tree_view_set_headers_clickable (treeview, FALSE);  =default
@@ -1302,13 +1597,11 @@ void e2_option_tree_add_widget (GtkWidget *parent, gboolean single, GtkWidget *b
 		{
 			case E2_OPTION_TREE_TYPE_BOOL:
 				renderer = gtk_cell_renderer_toggle_new ();
-				g_object_set (G_OBJECT (renderer), "activatable", TRUE, NULL);
 				g_signal_connect (G_OBJECT (renderer), "toggled",
 					G_CALLBACK (_e2_option_tree_toggle_cb), set);
 				column = gtk_tree_view_column_new_with_attributes
 					(opt->label, renderer, "active", j, NULL);
-				g_object_set (G_OBJECT (column), "alignment", 0.5, NULL);
-				gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN (column), TRUE);
+//				gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN (column), TRUE);
 				gtk_tree_view_column_set_cell_data_func (column, renderer,
 					(GtkTreeCellDataFunc) _e2_option_tree_bool_data_func, opt, NULL);
 				gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
@@ -1325,7 +1618,7 @@ void e2_option_tree_add_widget (GtkWidget *parent, gboolean single, GtkWidget *b
 				g_object_set (G_OBJECT (renderer), "editable", editable, NULL);
 
 //				g_signal_connect (G_OBJECT (?), "key-press-event",
-//					G_CALLBACK (_e2_option_tree_cell_keypress_cb), renderer);
+//					G_CALLBACK (_e2_option_tree_cell_keypress_cb), renderer); ?translation
 				g_signal_connect (G_OBJECT (renderer), "editing-started",
 					G_CALLBACK (_e2_option_tree_cell_edit_start_cb), NULL);
 				if (opt->type == E2_OPTION_TREE_TYPE_KEY)
@@ -1365,9 +1658,9 @@ void e2_option_tree_add_widget (GtkWidget *parent, gboolean single, GtkWidget *b
 #else
 					_e2_option_tree_cell_pixbuf_data_func, NULL, NULL);
 #endif
-				gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN (column), TRUE);
-				gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column),
-					GTK_TREE_VIEW_COLUMN_GROW_ONLY);
+//				gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN (column), TRUE);
+//				gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column),
+//					GTK_TREE_VIEW_COLUMN_GROW_ONLY);
 				gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 				break;
 			case E2_OPTION_TREE_TYPE_SEL:
@@ -1432,66 +1725,97 @@ void e2_option_tree_add_widget (GtkWidget *parent, gboolean single, GtkWidget *b
 //	gtk_tree_view_set_hover_expand (treeview, TRUE); //annoying ?
 
 	//add control buttons
-	//single-page dialog doesn't need higher buttons
-	GtkPolicyType h_policy = (single) ? GTK_POLICY_NEVER : GTK_POLICY_AUTOMATIC;
-	sw = e2_widget_get_sw_plain (h_policy, GTK_POLICY_NEVER);
-	gtk_box_pack_start (GTK_BOX (vbox), sw, FALSE, FALSE, 0);
-	GtkWidget *bbox = gtk_hbox_new (FALSE, E2_PADDING);
-	e2_widget_sw_add_with_viewport (sw, bbox);
-	E2_Sextet *q = (E2_Sextet *) e2_utils_sextet_new ();
-	g_object_set_data_full (G_OBJECT (treeview), "e2-config-buttons", q,
-		(GDestroyNotify) e2_utils_sextet_destroy);
-	if (q != NULL)
+	E2_Sextet *st = (E2_Sextet *) e2_utils_sextet_new ();
+	if (st != NULL)
 	{
-		q->a = e2_button_add (bbox, FALSE, 0, _("_Help"), GTK_STOCK_HELP,
-			_("Get help on this option"),  _e2_option_tree_help_cb, set);
-	//insert extra button on specific pages
-		if (e2_option_get_simple ("plugins") == set)
-			e2_button_add (bbox, FALSE, 0, _("_Select"), GTK_STOCK_INDEX,
-			_("Show plugin-selection dialog"), e2_confdlg_choose_plugin_cb, set);
-#ifdef E2_RAINBOW
-		else if (e2_option_get_simple ("filetypes") == set)
-			e2_button_add (bbox, FALSE, 0, _("Co_lor"), GTK_STOCK_SELECT_COLOR,
-			_("Show color-selection dialog"), e2_confdlg_extcolorpick_cb, set);
-#endif
-		if (set->ex.tree.flags & E2_OPTION_TREE_UP_DOWN)
-		{
-			q->b = e2_button_add (bbox, FALSE, 0, _("_Up"), GTK_STOCK_GO_UP,
-				_("Move the selected row one place up"),
-				_e2_option_tree_move_up_cb, set);
-			q->c = e2_button_add (bbox, FALSE, 0, _("_Down"), GTK_STOCK_GO_DOWN,
-				_("Move the selected row one place down"),
-				_e2_option_tree_move_down_cb, set);
-		}
+		g_object_set_data_full (G_OBJECT (treeview), "e2-config-buttons", st,
+			(GDestroyNotify) e2_utils_sextet_destroy);
+
+		GtkWidget *bbox = gtk_hbox_new (FALSE, E2_PADDING);
+
 		if (set->ex.tree.flags & E2_OPTION_TREE_ADD_DEL)
 		{
-			q->e = e2_button_add (bbox, FALSE, 0, _("_Add"), GTK_STOCK_ADD,
-				_("Add a row after the currently selected row in the tree"),
-				_e2_option_tree_add_below_cb, set);
-			if (! (set->ex.tree.flags & E2_OPTION_TREE_LIST))
-				q->f = e2_button_add (bbox, FALSE, 0, _("_Child"), GTK_STOCK_INDENT,
-					_("Add a child row to the currently selected one"),
-					_e2_option_tree_add_child_cb, set);
-			q->d = e2_button_add (bbox, FALSE, 0, _("_Remove"), GTK_STOCK_REMOVE,
+			st->f = e2_button_add_end (bbox, FALSE, 0, _("_Remove"), GTK_STOCK_REMOVE,
 				_("Remove the selected row"),  e2_option_tree_del_direct_cb, set);
+			if (! (set->ex.tree.flags & E2_OPTION_TREE_LIST))
+				st->e = e2_button_add_end (bbox, FALSE, 0, _("_Child"), GTK_STOCK_INDENT,
+					_("Add a child row to the currently selected one"),
+					(void(*)())_e2_option_tree_add_child_cb, set);
+			st->d = e2_button_add_end (bbox, FALSE, 0, _("_Add"), GTK_STOCK_ADD,
+				_("Add a row after the currently selected row in the tree"),
+				(void(*)())_e2_option_tree_add_below_cb, set);
+		}
+		if (set->ex.tree.flags & E2_OPTION_TREE_UP_DOWN)
+		{
+			st->c = e2_button_add_end (bbox, FALSE, 0, _("_Down"), GTK_STOCK_GO_DOWN,
+				_("Move the selected row one place down"),
+				(void(*)())_e2_option_tree_move_down_cb, set);
+			st->b = e2_button_add_end (bbox, FALSE, 0, _("_Up"), GTK_STOCK_GO_UP,
+				_("Move the selected row one place up"),
+				(void(*)())_e2_option_tree_move_up_cb, set);
+		}
+		//insert an extra button on some pages
+		if (e2_option_get_simple ("plugins") == set)
+			e2_button_add_end (bbox, FALSE, 0, _("_Select"), GTK_STOCK_INDEX,
+			_("Show plugin-selection dialog"), e2_confdlg_choose_plugin_cb, set);
+		else if (e2_option_get_simple ("keybindings") == set)
+			e2_button_add_end (bbox, FALSE, 0, _("Tra_nslate"), GTK_STOCK_CONVERT,
+			_("Set all single-letter bindings to current locale"),
+			_e2_option_tree_translate_cb, set);
+		else if (e2_option_get_simple ("panebar1") == set ||
+				 e2_option_get_simple ("panebar2") == set)
+			e2_button_add_end (bbox, FALSE, 0, _("Re_plicate"), GTK_STOCK_COPY,
+			_("Import data of the other toolbar"),
+			_e2_option_tree_barcopy_cb, set);
+#ifdef E2_RAINBOW
+		else if (e2_option_get_simple ("filetypes") == set)
+			e2_button_add_end (bbox, FALSE, 0, _("Co_lor"), GTK_STOCK_SELECT_COLOR,
+			_("Show color-selection dialog"), e2_confdlg_extcolorpick_cb, set);
+#endif
+		st->a = e2_button_add_end (bbox, FALSE, 0, _("_Help"), GTK_STOCK_HELP,
+			_("Get help on this option"), (void(*)())_e2_option_tree_help_cb, set);
+		//setup to process button-mnemonic keyvals for keybindings
+		if (e2_option_get_simple ("keybindings") == set)
+			e2_option_tree_connect_mnemonics (bbox);
+
+		if (single)
+			//single-page dialog doesn't need extra scrolling
+			gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 0);
+		else
+		{
+			//can't find a way to reduce sw height when no bar needed !
+			//so AUTOMATIC horz. policy resizes buttons, or leaves gap ...
+#if 0
+			sw = e2_widget_get_sw_plain (GTK_POLICY_AUTOMATIC, GTK_POLICY_NEVER);
+			e2_widget_sw_add_with_viewport (sw, bbox);
+#else
+			GtkWidget *vbox2 = gtk_vbox_new (FALSE, 0);
+			gtk_box_pack_start (GTK_BOX (vbox2), bbox, FALSE, FALSE, 0);
+//			sw = e2_widget_get_sw_plain (GTK_POLICY_ALWAYS, GTK_POLICY_NEVER);
+			sw = e2_widget_get_sw_plain (GTK_POLICY_AUTOMATIC, GTK_POLICY_NEVER);
+			e2_widget_sw_add_with_viewport (sw, vbox2);
+#endif
+			gtk_box_pack_start (GTK_BOX (vbox), sw, FALSE, FALSE, 0);
 		}
 	}
-	//add context-menu
+	//add persistent context-menu
 	E2_TreeContextMenuFlags context_flags = E2_TREE_CONTEXT_DEFAULT;
 	if (!((gboolean) set->ex.tree.flags & E2_OPTION_TREE_LIST))
 		context_flags |= E2_TREE_CONTEXT_EXP_COL;
 	GtkWidget *context_menu = gtk_menu_new ();
-	_e2_option_tree_menu_items_add (treeview, context_menu, context_flags, set);
+	_e2_option_tree_populate_menu (context_menu, treeview, context_flags, set);
 
-		//connect callbacks
+	//connect callbacks
 	//no longer needed it seems
 //	g_signal_connect (G_OBJECT (treeview), "drag-drop", G_CALLBACK (drop_cb), set);
 //	g_signal_connect (G_OBJECT (treeview), "drag-end", G_CALLBACK (drag_end_cb), set);
-
+	//cleanup menu with the treeview
+	g_object_set_data_full (G_OBJECT(treeview), "destroy-menu", context_menu,
+		(GDestroyNotify)gtk_widget_destroy);
 	//arrange for data backup if/when the page is opened
 	g_signal_connect (G_OBJECT (treeview), "realize",
 		G_CALLBACK (_e2_option_tree_page_opened_cb), set);
-	g_signal_connect (G_OBJECT (treeview), "popup_menu",
+	g_signal_connect (G_OBJECT (treeview), "popup-menu",
 		G_CALLBACK (_e2_option_tree_popup_menu_cb), context_menu);
 /* the dialog's "negative response" handler takes care of this
 	g_signal_connect_after (G_OBJECT (treeview), "key-press-event",
@@ -1539,7 +1863,7 @@ E2_OptionSet *e2_option_tree_register (gchar *name, gchar *group, gchar *desc,
 	set->ival = -1;
 	set->sval = NULL;
 	set->ex.tree.synced = FALSE;
-	set->ex.tree.def = NULL;
+	set->ex.tree.def.func = NULL;
 //	set->ex.tree.columns_num = 0;	//set when all cols processed
 	set->ex.tree.columns = NULL;
 	set->ex.tree.flags = flags;
@@ -1781,7 +2105,7 @@ This fn does nothing if there is already backup data for the set
 */
 void e2_option_tree_backup (E2_OptionSet *set)
 {
-	if (set->ex.tree.def == NULL)
+	if (set->ex.tree.def.tree_strings == NULL)
 	{	//backup data does not exist now, so ok to create it
 		GPtrArray *lines = g_ptr_array_sized_new (21);
 		g_ptr_array_add (lines, (gpointer) g_strdup_printf ("%s=<", set->name));
@@ -1791,7 +2115,7 @@ void e2_option_tree_backup (E2_OptionSet *set)
 		g_ptr_array_add (lines, (gpointer) g_strdup (">"));
 //		set->ex.tree.def_num = (gint) lines->len;
 		g_ptr_array_add (lines, NULL);
-		set->ex.tree.def = lines->pdata;
+		set->ex.tree.def.tree_strings = (gchar **)lines->pdata;
 		g_ptr_array_free (lines, FALSE);
 	}
 }
@@ -1805,14 +2129,15 @@ void e2_option_tree_backup (E2_OptionSet *set)
 */
 static void _e2_option_tree_revert (E2_OptionSet *set, gboolean empty_check)
 {
-	if ((!set->ex.tree.synced || !empty_check) && set->ex.tree.def != NULL) //CHECKME this test
+	if ((!set->ex.tree.synced || !empty_check)
+		&& set->ex.tree.def.tree_strings != NULL) //CHECKME this test
 	{
 		g_object_unref (set->ex.tree.model);
 		e2_option_tree_create_store (set);
-		//option needs default configuration which is stored in
-		//array at ex.tree.def of the OptionSet
-		e2_option_tree_set_from_array (set->name,
-			(gchar **) set->ex.tree.def, NULL, NULL);
+		//option needs default configuration which is stored in array at
+		//set->ex.tree.def.tree_strings
+		e2_option_tree_set_from_array (set->name, set->ex.tree.def.tree_strings,
+			NULL, NULL);
 		//make sure the set is recognised as clean
 		set->ex.tree.flags &= ~E2_OPTION_TREE_SET_EDITED;
 	}
@@ -1839,13 +2164,13 @@ void e2_option_tree_revert (gchar *option_name)
 */
 void e2_option_tree_unbackup (E2_OptionSet *set, gboolean revert)
 {
-	if (set->ex.tree.def != NULL)
+	if (set->ex.tree.def.tree_strings != NULL)
 	{	//backup data exists
 		if (revert)
 			_e2_option_tree_revert (set, FALSE);
 		//cleanup
-		g_strfreev ((gchar **)set->ex.tree.def);
-		set->ex.tree.def = NULL;
+		g_strfreev (set->ex.tree.def.tree_strings);
+		set->ex.tree.def.tree_strings = NULL;
 	}
 }
 /**
@@ -1868,24 +2193,23 @@ void e2_option_tree_restore_all (void)
 /**
 @brief log a function to be called to setup tree-option defaults if needed
 
-@a func needs to be of the form void (*func) (E2_Optionset *) and to call
-e2_option_tree_setup_defaults ()
+@a func needs to iclude a call to e2_option_tree_setup_defaults ()
 
 @param set the set data structure
 @param func pointer to function which will install default tree data
 
 @return
 */
-void e2_option_tree_prepare_defaults (E2_OptionSet *set, gpointer func)
+void e2_option_tree_prepare_defaults (E2_OptionSet *set, void (*func)(E2_OptionSet*))
 {
-	set->ex.tree.def = func;
+	set->ex.tree.def.func = func;
 }
 /**
 @brief setup default config tree contents, ready for installation
 
-This creates a NULL-terminated vector of tree-row data strings, which
-in aggregate (are expected to) have the same format as the config file
-data for the set being processed.
+This creates a NULL-terminated vector of tree-row data strings, which in
+aggregate (are expected to) have the same format as the config file data for
+the set being processed.
 
 @param set the set data structure
 @param first start of a null-terminated series of arguments,
@@ -1895,32 +2219,33 @@ data for the set being processed.
 */
 void e2_option_tree_setup_defaults (E2_OptionSet *set, gchar *first, ...)
 {
-	if (first == NULL) return;
-//	printd (DEBUG, "preparing default option tree for %s>", first);
-	va_list args;
-	va_start (args, first);
-	gchar *cur = first;
-	GPtrArray *lines = g_ptr_array_sized_new (32);
-	while (cur)
+	if (first != NULL)
 	{
-//		cur = g_strdup (cur);
-		//store ptr to constant string (do not free, later)
-		g_ptr_array_add (lines, (gpointer) cur);
-		cur = va_arg (args, gchar *);
+	//	printd (DEBUG, "preparing default option tree for %s>", first);
+		va_list args;
+		va_start (args, first);
+		gchar *cur = first;
+		GPtrArray *lines = g_ptr_array_sized_new (32);
+		while (cur)
+		{
+	//		cur = g_strdup (cur);
+			//store ptr to constant string (do not free, later)
+			g_ptr_array_add (lines, (gpointer) cur);
+			cur = va_arg (args, gchar *);
+		}
+		va_end (args);
+	//	set->ex.tree.def_num = (gint) lines->len; UNUSED
+		g_ptr_array_add (lines, NULL);  //null signals end, to parser
+		set->ex.tree.def.tree_strings = (gchar **)lines->pdata;
+		g_ptr_array_free (lines, FALSE);
 	}
-	va_end (args);
-//	set->ex.tree.def_num = (gint) lines->len; UNUSED
-	g_ptr_array_add (lines, NULL);  //null signals end, to parser
-	set->ex.tree.def = lines->pdata;
-	g_ptr_array_free (lines, FALSE);
 }
 /**
 @brief install default config tree data where needed
 
-after loading the config file (if any) at session start, check if any
-tree option needs a default config, (stored in arrays)
-and install it if so
-then free the default tree 'branches'
+after loading the config file (if any) at session start, check if any tree
+option needs a default config, (stored in arrays), install it if so, then
+free the default tree 'branches'
 
 @return
 */
@@ -1935,22 +2260,20 @@ void e2_option_tree_install_defaults (void)
 		if (set->type == E2_OPTION_TYPE_TREE)
 		{
 			if (!set->ex.tree.synced //option needs default data
-				&& set->ex.tree.def != NULL)	//setup func has been logged
+				&& set->ex.tree.def.func != NULL)	//setup func has been logged
 			{
-//				printd (DEBUG, "installing default option tree %s>",
-//				VPSTR(set->ex.tree.def));
-				void (*install_func) (E2_OptionSet *) = set->ex.tree.def;
+//				printd (DEBUG, "installing default option tree %s>", set->name);
 				//call the defaults-install function, which in turn calls
 				// e2_option_tree_setup_defaults () which sets up a
-				// pointer array at set->ex.tree.def again
-				(*install_func) (set);
+				// pointer array at set->ex.tree.def.tree_strings again
+				(*set->ex.tree.def.func) (set);
 				//parse the data
 				e2_option_tree_set_from_array (set->name,
-					(gchar **) set->ex.tree.def, NULL, NULL);
+					set->ex.tree.def.tree_strings, NULL, NULL);
 				//cleanup
-				g_strfreev ((gchar **) set->ex.tree.def);
+				g_strfreev (set->ex.tree.def.tree_strings);
 			}
-			set->ex.tree.def = NULL;	//always zap the function/vector pointer
+			set->ex.tree.def.tree_strings = NULL;	//no more use for function, always zap the vector pointer
 		}
 	}
 }
@@ -2169,11 +2492,15 @@ gboolean e2_option_tree_set_from_array (gchar *name, gchar *f[], gint *index,
 //		printd (DEBUG, "found config tree for unknown option %s", name);
 		//accumulate the lines
 		GPtrArray *unknown_lines = g_ptr_array_sized_new (11);
+		gboolean freefirst;
 		if (strstr (f[idx], "=<") == NULL)
 		{	//incomplete tree, from transient store-updates
 			line = g_strconcat (name, "-part=<", NULL);
 			g_ptr_array_add (unknown_lines, line);
+			freefirst = TRUE;
 		}
+		else
+			freefirst = FALSE;
 		while ((line = f[idx]) != NULL)
 		{
 			idx++;	//now we know array's not finished
@@ -2187,6 +2514,8 @@ gboolean e2_option_tree_set_from_array (gchar *name, gchar *f[], gint *index,
 				break;
 			}
 		}
+		if (freefirst)
+			g_free (g_ptr_array_index (unknown_lines, 0));
 		g_ptr_array_free (unknown_lines, TRUE);
 		if (index != NULL)
 			*index = idx;
@@ -2242,8 +2571,8 @@ gboolean e2_option_tree_set_from_array (gchar *name, gchar *f[], gint *index,
 		GList *columns;
 		//unescape helper for concatting, later on
 		GList *freecats = NULL;
-		//transform splitted values to void pointers and
-		//add default values in case of a missing value/field
+		//transform splitted values to void pointers and add default values
+		//in case of a missing value/field
 		for (columns = set->ex.tree.columns; columns != NULL; columns = columns->next)
 		{
 			if (split[i] == NULL)
@@ -2317,10 +2646,10 @@ gboolean e2_option_tree_set_from_array (gchar *name, gchar *f[], gint *index,
 
 	if (line != NULL && tree_mode != E2_TREE_ABORTED)
 	{
-		//found config for this tree option, so don't init it
-		//with the default config later on
+		//found config for this tree option, so don't init it with the default
+		//config later on
 		set->ex.tree.synced = TRUE;
-//		set->ex.tree.def = NULL;
+//		set->ex.tree.def.tree_strings = NULL;
 		//make sure the set is recognised as clean
 		set->ex.tree.flags &= ~E2_OPTION_TREE_SET_EDITED;
 	}
@@ -2358,7 +2687,7 @@ void *_e2_option_tree_get_void_simple (E2_OptionTreeColumn *col, gchar *option)
 	switch (col->type)
 	{
 		case E2_OPTION_TREE_TYPE_BOOL:
-			if (g_str_equal (option, "true"))
+			if (!strcmp (option, "true"))
 				return (void *) TRUE;
 			else
 				return (void *) FALSE;
