@@ -1,6 +1,6 @@
-/* $Id: e2_config_dialog.c 1256 2008-09-20 10:59:13Z tpgww $
+/* $Id: e2_config_dialog.c 2018 2009-11-19 20:50:44Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark
 
 This file is part of emelFM2.
@@ -130,16 +130,19 @@ static void _e2_cd1_response_cb (GtkDialog *dialog, gint response,
 	switch (response)
 	{
 		case E2_RESPONSE_APPLY:
-			//perform any post-dialog data processing, via caller-specified 'apply' action
-			(*rt->apply_function) ();
+		{
+			void (*apply_new) () = rt->apply_function; //keep this thru the cleanup
 			//clean up without restoring tree data
 			_e2_cd1_cancel (rt, FALSE);
+			//perform post-dialog data processing, via caller-specified 'apply'
+			apply_new ();
 			e2_option_disable_config_checks ();
 			//update stored config data
 			e2_option_file_write (NULL);
 			//update stamp, to prevent auto-refresh
 			e2_fs_touch_config_dir ();
 			e2_option_enable_config_checks ();
+		}
 			break;
 		case E2_RESPONSE_MORE:
 			(*rt->apply_function) ();
@@ -161,22 +164,22 @@ dialog. It applies only to tree-type option sets.
 Clicking 'apply' or 'ok' performs a caller-specified fn (if any) and updates the config file
 
 @param set_name string, non-translated 'internal' name of the tree optionset to be worked on
-@param function ptr to caller's <void fn(void)> 'apply changes' function, or NULL if nothing to be done
+@param apply_function ptr to caller's 'apply changes' function, or NULL if nothing to be done
 @param showit boolean TRUE to show the dialog immediately, FALSE if not
 
 @return E2_SpecificConfDialogRuntime created, with dialog data, or NULL if error occurred
 */
 E2_SpecificConfDialogRuntime *e2_config_dialog_single (gchar *set_name,
-	gpointer function, gboolean showit)
+	void (*apply_function)(void), gboolean showit)
 {
 	E2_SpecificConfDialogRuntime *rt = ALLOCATE (E2_SpecificConfDialogRuntime);
 	CHECKALLOCATEDWARN (rt, return NULL;)
 	rt->set = e2_option_get_simple (set_name);
-	rt->apply_function = (function != NULL) ? function : _e2_cd1_no_application;
+	rt->apply_function = (apply_function != NULL) ? apply_function : _e2_cd1_no_application;
   //set config_dialog too, so that list-cell-renderer can access the dialog
 	config_dialog =
 	rt->dialog = e2_dialog_create (NULL, NULL, _("configuration"),
-		_e2_cd1_response_cb, rt);
+		(ResponseFunc)_e2_cd1_response_cb, rt);
 	gtk_widget_set_size_request (rt->dialog, 600, 400);
 	GtkWidget *dialog_vbox =
 #ifdef USE_GTK2_14
@@ -244,15 +247,19 @@ void e2_confdlg_menu_set_position (GtkMenu *menu,
 	gint *x, gint *y, gboolean *push_in, GtkWidget *treeview)
 {
 	GtkWidget *window = gtk_widget_get_toplevel (treeview);
+#ifdef USE_GTK2_18
+	if (gtk_widget_is_toplevel (window))
+#else
 	if (GTK_WIDGET_TOPLEVEL (window))
+#endif
 	{
 		gint left, top;
 		gtk_window_get_position (GTK_WINDOW (window), &left, &top);
-		GtkAllocation alloc =
-#ifdef USE_GTK2_99
-			gtk_widget_get_allocation (treeview);
+		GtkAllocation alloc;
+#ifdef USE_GTK2_18
+		gtk_widget_get_allocation (treeview, &alloc);
 #else
-			treeview->allocation;
+		alloc = treeview->allocation;
 #endif
 		*x = left + alloc.x + alloc.width/3;
 		*y = top + alloc.y + alloc.height/3;
@@ -267,7 +274,7 @@ void e2_confdlg_menu_set_position (GtkMenu *menu,
 	}
 }
 /**
-@brief construct and show categories treeview context menu
+@brief construct and pop up destroyable context-menu for categories treeview
 @param treeview the widget to which the menu belongs
 @param event_button which mouse button was clicked (0 for a menu key)
 @param event_time time that the event happened (0 for a menu key)@param time
@@ -284,7 +291,7 @@ static void _e2_confdlg_show_context_menu (GtkWidget *treeview, guint event_butt
 	e2_menu_add (menu, _("C_ollapse"), GTK_STOCK_ZOOM_OUT,
 		_("Collapse all rows"), e2_tree_collapse_all_cb, treeview);
 	g_signal_connect (G_OBJECT (menu), "selection-done",
-		G_CALLBACK (e2_menu_destroy_cb), NULL);
+		G_CALLBACK (e2_menu_selection_done_cb), NULL);
 	if (event_button == 0)
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
 			(GtkMenuPositionFunc) e2_confdlg_menu_set_position,
@@ -353,7 +360,11 @@ static gboolean _e2_confdlg_negdepends_cb (GtkWidget *button,
 	GtkWidget *dependant)
 {
 	gtk_widget_set_sensitive (dependant,
+#ifdef USE_GTK2_14
+		!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)));
+#else
 		!GTK_TOGGLE_BUTTON (button)->active);
+#endif
 	return TRUE;
 }
 /**
@@ -368,7 +379,11 @@ static gboolean _e2_confdlg_posdepends_cb_ (GtkWidget *button,
 	GtkWidget *dependant)
 {
 	gtk_widget_set_sensitive (dependant,
+#ifdef USE_GTK2_14
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)));
+#else
 		GTK_TOGGLE_BUTTON (button)->active);
+#endif
 	return TRUE;
 }
 /**
@@ -381,11 +396,10 @@ static void _e2_confdlg_clean (void)
 	if (config_dialog != NULL)
 	{
 		printd (DEBUG, "config dialog widget destroy");
-		app.cfg_alloc =
-#ifdef USE_GTK2_99
-		gtk_widget_get_allocation (config_dialog);
+#ifdef USE_GTK2_18
+		gtk_widget_get_allocation (config_dialog, &app.cfg_alloc);
 #else
-		config_dialog->allocation;
+		app.cfg_alloc = config_dialog->allocation;
 #endif
 		gtk_widget_destroy (config_dialog);
 		config_dialog = NULL;
@@ -394,27 +408,29 @@ static void _e2_confdlg_clean (void)
 	g_hash_table_destroy (tree_view_buffer_hash);
 }
 /**
-@brief process apply or more button click
+@brief process commit or apply button-click
 
-@param apply TRUE if 'more' button was pressed, FALSE if ordinary 'apply'
+@param close TRUE if 'commit' button was pressed, FALSE if 'apply'
 
 @return
 */
-static gboolean _e2_confdlg_apply (gboolean stay)
+static gboolean _e2_confdlg_apply (gboolean close)
 {
-	printd (DEBUG, "config dialog callback: %s", (stay) ? "more" : "apply");
+	gboolean bchoice;
+	gint ichoice;
+	guint i;
+	E2_OptionFlags buildflags;
+	gchar *cchoice, *ccurrent;
+	gpointer *walker;
+	E2_OptionSet *set;
+
+	printd (DEBUG, "config dialog callback: %s", (close) ? "commit" : "apply");
 
 	e2_filelist_disable_refresh ();
 	e2_option_disable_config_checks ();
 
-	gboolean bchoice;
-	gint ichoice;
-	gchar *cchoice, *ccurrent;
+	buildflags = 0;
 
-	E2_OptionFlags buildflags = 0;
-	guint i;
-	gpointer *walker;
-	E2_OptionSet *set;
 	//update all options that were included in the dialog
 	for (i = 0, walker = options_array->pdata; i < options_array->len; i++, walker++)
 	{
@@ -444,7 +460,7 @@ static gboolean _e2_confdlg_apply (gboolean stay)
 				case E2_OPTION_TYPE_COLOR:
 					cchoice = gtk_editable_get_chars (GTK_EDITABLE (set->widget), 0, -1);
 					ccurrent = e2_option_str_get_direct (set);
-					if (! g_str_equal (cchoice, ccurrent))
+					if (strcmp (cchoice, ccurrent))
 					{
 						if (set->type == E2_OPTION_TYPE_COLOR)
 							e2_option_color_set_str_direct (set, cchoice);
@@ -472,7 +488,7 @@ static gboolean _e2_confdlg_apply (gboolean stay)
 					e2_option_tree_unbackup (set, FALSE);
 					//if the config dialog is not finished, get ready for next ok/apply
 					//FIXME this not needed if backup triggered by dirty tree
-					if (stay)
+					if (!close)
 						e2_option_tree_backup (set);
 					break;
 				default:
@@ -481,8 +497,8 @@ static gboolean _e2_confdlg_apply (gboolean stay)
 		}
 	}
 
-	//if this is an 'apply' button response, close the dialog etc
-	if (!stay)
+	//if this is a 'commit' button response, close the dialog etc
+	if (close)
 		_e2_confdlg_clean ();
 
 	e2_utils_update_gtk_settings ();
@@ -628,6 +644,9 @@ static gboolean _e2_confdlg_apply (gboolean stay)
 		if (buildflags & E2_OPTION_FLAG_BUILDBARS)
 			e2_toolbar_recreate_all ();
 
+		if (buildflags & E2_OPTION_FLAG_BUILDOUT)
+			e2_output_update_style ();
+
 		if (buildflags & E2_OPTION_FLAG_BUILDKEYS)
 //CHECKME ok to re-register command-line key bindings, if bars re-created ?
 //CHECKME maybe fileview bindings should be registered before the general
@@ -655,14 +674,12 @@ static gboolean _e2_confdlg_apply (gboolean stay)
 	if (buildflags & E2_OPTION_FLAG_BUILDFILES)
 		e2_filetype_apply_allnew ();
 
-	if (!stay)	//if this is an 'apply' button response
+	if (close)	//if this is a 'commit' button response
 	{
-		e2_option_disable_config_checks ();
 		//update the stored config data
 		e2_option_file_write (NULL);
-		//update stamp, to prevent auto-refresh
+		//update stamp, to prevent auto-refresh here
 		e2_fs_touch_config_dir ();
-		e2_option_enable_config_checks ();
 		//may need to show the revised config file data
 /*	FIXME this may cause crash
 		if (!(buildflags & (E2_OPTION_FLAG_BUILDALL | E2_OPTION_FLAG_BUILDPANES)))
@@ -739,23 +756,21 @@ static void _e2_confdlg_response_cb (GtkDialog *dialog, gint response,
 {
 	switch (response)
 	{
-		case E2_RESPONSE_MORE:
-			_e2_confdlg_apply (TRUE);
-			break;
 		case E2_RESPONSE_APPLY:
-			_e2_confdlg_apply (FALSE);	//does some cleanup
+			_e2_confdlg_apply (TRUE);	//does some cleanup
 			DEALLOCATE (E2_ConfigDialogRuntime, rt);//CHECKME more cleanup ?
 			break;
+		case E2_RESPONSE_MORE:
+			_e2_confdlg_apply (FALSE);
+			break;
 		case E2_RESPONSE_USER1:	//revert to default options
 		{
 			DialogButtons choice = e2_dialog_warning (
 			_("Reverting to default configuration cannot be undone"), NULL);
 			if (choice == OK)
 			{
-				//dump old data, don't re-read the config file, recreate screen
-			//	_e2_confdlg_cancel_cb ();	//must kill dialog before clearing option data
-//				WAIT_FOR_EVENTS	//visually better to repaint screen
-				_e2_confdlg_clean (); //MUST kill dialog before clearing option data
+				_e2_confdlg_clean ();	//MUST kill dialog before clearing option data
+//				app.keytrans = TRUE;	//translate the re-created keybindings, later on
 				e2_option_refresh (FALSE, TRUE);
 				DEALLOCATE (E2_ConfigDialogRuntime, rt);
 				WAIT_FOR_EVENTS
@@ -790,7 +805,10 @@ static void _e2_confdlg_response_cb (GtkDialog *dialog, gint response,
 gboolean e2_confdlg_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 	gpointer data)
 {
-//	if (event->keyval == GDK_Return)  return activates any focused button
+//	if (event->keyval == GDK_Return
+//	 || event->keyval == GDK_KP_Enter
+//	 || event->keyval == GDK_ISO_Enter
+//	 || event->keyval == GDK_3270_Enter)  return activates any focused button
 //		return e2_confdlg_ok_cb (FALSE);
 	if (event->keyval == GDK_Escape)
 	{
@@ -820,7 +838,11 @@ static gboolean _e2_confdlg_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 			e2_dialog_key_neg_cb, config_dialog);
 		cancel_blocked = TRUE;
 	}
-	else if (event->keyval == GDK_Escape || event->keyval == GDK_Return)
+	else if (event->keyval == GDK_Escape
+		  || event->keyval == GDK_Return
+		  || event->keyval == GDK_KP_Enter
+		  || event->keyval == GDK_ISO_Enter
+		  || event->keyval == GDK_3270_Enter)
 	{
 		if (event->keyval == GDK_Escape)
 		{
@@ -906,7 +928,7 @@ static gboolean _e2p_confdlg_popup_menu_cb (GtkWidget *treeview, gpointer data)
 static void _e2_confdlg_plugpick_response_cb (GtkDialog *dialog, gint response,
 	E2_TreeCfgData *data)
 {
-	gchar *plocal, *local, *utf, *utfp;
+	gchar *plocal;
 	switch (response)
 	{
 		case E2_RESPONSE_USER1:	//toggle hidden items display
@@ -920,37 +942,12 @@ static void _e2_confdlg_plugpick_response_cb (GtkDialog *dialog, gint response,
 				Plugin *p = e2_plugins_open1 (plocal);
 				if (p != NULL)
 				{
-					//get plugin UI data only
-					p->action = GINT_TO_POINTER (1); //non-NULL prevents full initialisation
-					p->plugin_init (p);
-					const gchar *menu_name = (p->menu_name != NULL) ? p->menu_name : "";
-					const gchar *icon = (p->icon != NULL) ? p->icon : "";
-					const gchar *description = (p->description != NULL) ? p->description : NULL;
-					local = strrchr (plocal, G_DIR_SEPARATOR);	//should never fail
-					*local = '\0';	//truncate plocal at trailing /
-					if (g_str_equal (plocal, PLUGINS_DIR))	//from Makefile this is localised, no trailer
-						utfp = "";
-					else
-						utfp = F_FILENAME_FROM_LOCALE (plocal);
-					utf = F_FILENAME_FROM_LOCALE (++local);
-
 					GtkTreeIter iter;
 					gtk_tree_model_get_iter (data->set->ex.tree.model, &iter, data->tpath);
-					gtk_tree_store_set (GTK_TREE_STORE (data->set->ex.tree.model),
-						&iter, 1, TRUE, 2, menu_name, 3, icon, 4, description, 5, utf, 6, utfp, -1);
-					p->show_in_menu = TRUE;	//for child-rows data
-					if (p->child_list != NULL)
-					{
-						//process child UI data into config store and cleanup
-						e2_plugins_store_child_data (data->set->ex.tree.model, &iter, p);
-						gtk_tree_view_expand_to_path (GTK_TREE_VIEW (data->set->widget), data->tpath);
-					}
+					//assume the user wants to use the selected plugin, so TRUE menu etc flags
+					e2_plugins_store_data (data->set->ex.tree.model, &iter, p, TRUE, plocal);
+					gtk_tree_view_expand_row (GTK_TREE_VIEW (data->set->widget), data->tpath, FALSE);
 					e2_option_tree_flag_change (data->set);
-
-					F_FREE (utf);
-					if (*utfp != '\0')
-						F_FREE (utfp);
-
 					DEALLOCATE (Plugin, p);	//no leaks provided all embedded strings are constant
 				}
 				g_free (plocal);
@@ -972,12 +969,12 @@ static void _e2_confdlg_plugpick_response_cb (GtkDialog *dialog, gint response,
 /**
 @brief bring up a system find-file window to choose a plugin
 
-@param button activated widget, UNUSED
+@param button UNUSED clicked widget
 @param set ptr to plugins option data struct
 
 @return
 */
-void e2_confdlg_choose_plugin_cb (GtkWidget *button, E2_OptionSet *set)
+void e2_confdlg_choose_plugin_cb (GtkButton *button, E2_OptionSet *set)
 {
 	GtkTreeView *tvw = GTK_TREE_VIEW (set->widget);
 	GtkTreePath *tpath;
@@ -1112,12 +1109,12 @@ static void _e2_confdlg_font_select_cancel_cb (GtkWidget *button, GtkWidget *dia
 /**
 @brief create font-selection dialog
 
-@param button the activated button widget
+@param button the clicked button
 @param set pointer to data struct for the option to be altered
 
 @return
 */
-static void _e2_confdlg_font_select_cb (GtkWidget *button, E2_OptionSet *set)
+static void _e2_confdlg_font_select_cb (GtkButton *button, E2_OptionSet *set)
 {
 	gchar *title = g_strdup_printf (_("Choose font: %s"), set->desc);
 	GtkFontSelectionDialog *dialog = g_object_get_data (G_OBJECT (config_dialog), title);
@@ -1127,20 +1124,45 @@ static void _e2_confdlg_font_select_cb (GtkWidget *button, E2_OptionSet *set)
 	{
 		//CHECKME the normal dialog properties like close on Esc press ?
 		dialog = GTK_FONT_SELECTION_DIALOG (gtk_font_selection_dialog_new (title));
-		gtk_widget_destroy (dialog->apply_button);
-		g_object_set_data (G_OBJECT (dialog->ok_button), "dialog_widget", GTK_WIDGET (dialog));
+		gtk_widget_destroy (
+#ifdef USE_GTK2_14
+			gtk_font_selection_dialog_get_apply_button (dialog)
+#else
+			dialog->apply_button
+#endif
+		);
+		g_object_set_data (
+#ifdef USE_GTK2_14
+			G_OBJECT (gtk_font_selection_dialog_get_ok_button (dialog)),
+#else
+			G_OBJECT (dialog->ok_button),
+#endif
+			"dialog_widget", GTK_WIDGET (dialog));
 		gtk_font_selection_dialog_set_font_name (dialog,
 			gtk_entry_get_text (GTK_ENTRY (set->widget)));
 #ifdef E2_COMPOSIT
 		e2_window_set_opacity (GTK_WIDGET (dialog), DIALOG_OPACITY_LEVEL);	//constant opacity for dialogs
 #endif
-		g_signal_connect (G_OBJECT (dialog->ok_button), "clicked",
+		g_signal_connect (
+#ifdef USE_GTK2_14
+			G_OBJECT (gtk_font_selection_dialog_get_ok_button (dialog)),
+#else
+			G_OBJECT (dialog->ok_button),
+#endif
+			"clicked",
 			G_CALLBACK (_e2_confdlg_font_select_ok_cb), set->widget);
-		g_signal_connect (G_OBJECT (dialog->cancel_button), "clicked",
+		g_signal_connect (
+#ifdef USE_GTK2_14
+			G_OBJECT (gtk_font_selection_dialog_get_cancel_button (dialog)),
+#else
+			G_OBJECT (dialog->cancel_button),
+#endif
+			"clicked",
 			G_CALLBACK (_e2_confdlg_font_select_cancel_cb), dialog);
 		g_signal_connect (G_OBJECT (dialog), "delete-event",
 			G_CALLBACK (_e2_confdlg_font_select_cancel_cb), NULL);
-		g_object_set_data_full (G_OBJECT (config_dialog), title, dialog, (void *) gtk_widget_destroy);
+		g_object_set_data_full (G_OBJECT (config_dialog), title, dialog,
+			(gpointer) gtk_widget_destroy);
 		e2_dialog_show (GTK_WIDGET (dialog), config_dialog, 0, NULL);
 	}
 	g_free (title);
@@ -1214,12 +1236,12 @@ static void _e2_confdlg_colorpick_response_cb (GtkDialog *dialog, gint response,
 @brief callback for color-selection button in filetypes page
 Color strings are stored in col 3 of rows with extensions title
 (path depth 2) or a row with a specific extension (depth 3)
-@param button UNUSED the color button widget
+@param button UNUSED the clicked color button
 @param set pointer to filetypes option data
 
 @return
 */
-void e2_confdlg_extcolorpick_cb (GtkWidget *button, E2_OptionSet *set)
+void e2_confdlg_extcolorpick_cb (GtkButton *button, E2_OptionSet *set)
 {
 	GtkTreeView *tvw = GTK_TREE_VIEW (set->widget);
 	GtkTreePath *path;
@@ -1237,7 +1259,7 @@ void e2_confdlg_extcolorpick_cb (GtkWidget *button, E2_OptionSet *set)
 			gtk_tree_model_get_iter (set->ex.tree.model, &iter, path);
 			gtk_tree_model_get (set->ex.tree.model, &iter, 1, &current, -1);
 			//extensions node is ok
-			if (g_str_equal (current, _C(13)))
+			if (!strcmp (current, _C(13)))
 				valid = TRUE;
 			g_free (current);
 			break;
@@ -1248,7 +1270,7 @@ void e2_confdlg_extcolorpick_cb (GtkWidget *button, E2_OptionSet *set)
 			gtk_tree_model_get_iter (set->ex.tree.model, &iter, nodepath);
 			gtk_tree_path_free (nodepath);
 			gtk_tree_model_get (set->ex.tree.model, &iter, 1, &current, -1);
-			if (g_str_equal (current, _C(13)))
+			if (!strcmp (current, _C(13)))
 				valid = TRUE;
 			g_free (current);
 		}
@@ -1295,12 +1317,26 @@ void e2_confdlg_extcolorpick_cb (GtkWidget *button, E2_OptionSet *set)
 #ifdef E2_COMPOSIT
 	e2_window_set_opacity (dialog, DIALOG_OPACITY_LEVEL);	//constant opacity for dialogs
 #endif
-	e2_dialog_add_defined_button (dialog, &E2_BUTTON_APPLY);
+	//substitute our own button
+	E2_Button yes_btn;
+	e2_button_derive (&yes_btn, &E2_BUTTON_APPLY, BTN_YES_CONTINUE);
+	GtkWidget *bbox =
+#ifdef USE_GTK2_14
+		gtk_dialog_get_action_area (GTK_DIALOG (dialog));
+#else
+		GTK_DIALOG (dialog)->action_area;
+#endif
+
+	GList *btns = gtk_container_get_children (GTK_CONTAINER (bbox));
+	gtk_container_remove (GTK_CONTAINER (bbox), btns->next->data);
+	g_list_free (btns);
+	GtkWidget *btn = e2_dialog_add_defined_button (dialog, &yes_btn);
 
 	g_signal_connect (G_OBJECT (dialog), "response",
 		G_CALLBACK (_e2_confdlg_colorpick_response_cb), data);
 
 	e2_dialog_setup (dialog, config_dialog);
+	gtk_widget_show_all (btn);
 	gtk_widget_show (dialog);
 }
 #endif
@@ -1345,12 +1381,12 @@ static void _e2_confdlg_color_select_cancel_cb (GtkWidget *button, GtkWidget *di
 /**
 @brief callback for color-selection button in non-filetypes page
 
-@param button the clicked color-button widget
+@param button the clicked color-button
 @param set pointer to data for the relevant config option
 
 @return
 */
-static void _e2_confdlg_color_select_cb (GtkWidget *button, E2_OptionSet *set)
+static void _e2_confdlg_color_select_cb (GtkButton *button, E2_OptionSet *set)
 {
 	gchar *title = g_strdup_printf (_("Choose color: %s"), set->desc);
 	GtkColorSelectionDialog *dialog = g_object_get_data (G_OBJECT (config_dialog), title);
@@ -1360,7 +1396,13 @@ static void _e2_confdlg_color_select_cb (GtkWidget *button, E2_OptionSet *set)
 	{
 		//CHECKME close on Esc press ?
 		dialog = GTK_COLOR_SELECTION_DIALOG (gtk_color_selection_dialog_new (title));
-		gtk_widget_destroy (dialog->help_button);
+		gtk_widget_destroy (
+#ifdef USE_GTK2_99
+			gtk_color_selection_dialog_get_help_button (dialog)
+#else
+			dialog->help_button
+#endif
+		);
 
 		GdkColor color;
 		gchar *color_str = gtk_editable_get_chars (GTK_EDITABLE (set->widget), 0, -1);
@@ -1373,9 +1415,21 @@ static void _e2_confdlg_color_select_cb (GtkWidget *button, E2_OptionSet *set)
 #endif
 			&color);
 		g_free (color_str);
-		g_signal_connect (G_OBJECT (dialog->ok_button), "clicked",
+		g_signal_connect (
+#ifdef USE_GTK2_99
+			G_OBJECT (gtk_color_selection_dialog_get_ok_button (dialog)),
+#else
+			G_OBJECT (dialog->ok_button),
+#endif
+			"clicked",
 			G_CALLBACK (_e2_confdlg_color_select_ok_cb), set->widget);
-		g_signal_connect (G_OBJECT (dialog->cancel_button), "clicked",
+		g_signal_connect (
+#ifdef USE_GTK2_99
+			G_OBJECT (gtk_color_selection_dialog_get_cancel_button (dialog)),
+#else
+			G_OBJECT (dialog->cancel_button),
+#endif
+			"clicked",
 			G_CALLBACK (_e2_confdlg_color_select_cancel_cb), dialog);
 		g_signal_connect (G_OBJECT (dialog),
 			"delete-event", G_CALLBACK (_e2_confdlg_color_select_cancel_cb), dialog);
@@ -1447,12 +1501,7 @@ static gboolean _e2_confdlg_category_selected_cb
 static void _e2_confdlg_set_entry (GtkWidget *box, E2_OptionSet *set)
 {
 	set->widget = e2_widget_add_entry (box, set->sval, TRUE, FALSE);
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		set->widget, set->tip);
+	e2_widget_set_safetip (set->widget, set->tip);
 	//arrange localised handling of <Esc> key presses
 	g_signal_connect (G_OBJECT (GTK_ENTRY (set->widget)), "key-press-event",
 		G_CALLBACK (_e2_confdlg_key_press_cb), set);
@@ -1488,7 +1537,7 @@ static GtkWidget *_e2_confdlg_add_page (GtkTreeIter *parent_iter,
 #endif
 		 0, label_text, 1, page_num, -1);
 	//is this one the one nominated to open at startup?
-	if (g_str_equal (page_last_name, label_text))
+	if (!strcmp (page_last_name, label_text))
 	{
 		rt->openpath = gtk_tree_model_get_path
 			(GTK_TREE_MODEL (rt->store), &child_iter);
@@ -1601,7 +1650,7 @@ static GtkWidget *_e2_confdlg_get_page (E2_OptionSet *set, E2_ConfigDialogRuntim
 }
 /**
 @brief create and show configuration dialog
-
+Expects BGL closed upon arrival here
 @param page name of page to show when dialog is started, or "" or NULL to use the last-used page
 
 @return
@@ -1624,7 +1673,7 @@ void e2_config_dialog_create (gchar *page)
 	rt->openpath = NULL;
 
 	config_dialog = e2_dialog_create (NULL, NULL, _("configuration"),
-		_e2_confdlg_response_cb, rt);
+		(ResponseFunc)_e2_confdlg_response_cb, rt);
 
 	e2_option_connect (config_dialog, FALSE);
 
@@ -1698,6 +1747,7 @@ void e2_config_dialog_create (gchar *page)
 
 	rt->opthash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
 		(GDestroyNotify)_e2_hashfree);
+
 	guint i;
 	gpointer *walker;
 	E2_OptionSet *set;
@@ -1751,7 +1801,7 @@ void e2_config_dialog_create (gchar *page)
 				gtk_size_group_add_widget (size_group, label);
 				_e2_confdlg_set_entry (hbox, set);
 #ifdef E2_ASSISTED
-				e2_widget_set_label_relations (GTK_LABEL (label), set->widget);
+				e2_widget_set_label_relations (label, set->widget);
 #endif
 				break;
 			case E2_OPTION_TYPE_INT:
@@ -1772,7 +1822,7 @@ void e2_config_dialog_create (gchar *page)
 				//font entry for font name
 				_e2_confdlg_set_entry (hbox, set);
 #ifdef E2_ASSISTED
-				e2_widget_set_label_relations (GTK_LABEL (label), set->widget);
+				e2_widget_set_label_relations (label, set->widget);
 #endif
 				gtk_entry_set_text (GTK_ENTRY (set->widget), set->sval);
 				//button for font select dialog
@@ -1799,7 +1849,7 @@ void e2_config_dialog_create (gchar *page)
 				//color entry for color names/color hex values
 				_e2_confdlg_set_entry (hbox, set);
 #ifdef E2_ASSISTED
-				e2_widget_set_label_relations (GTK_LABEL (label), set->widget);
+				e2_widget_set_label_relations (label, set->widget);
 #endif
 				gtk_entry_set_text (GTK_ENTRY (set->widget), set->sval);
 				//button for color select dialog
@@ -1921,17 +1971,32 @@ void e2_config_dialog_create (gchar *page)
 		_("Revert all options to their default settings"),
 		NULL, NULL);
 	if (e2_option_bool_get ("advanced-config"))
+	{
 		e2_dialog_add_custom_button_full (config_dialog, FALSE,
 			E2_RESPONSE_USER2, _("_Basic"), GTK_STOCK_REDO,
 			_("Display only the basic configuration options"),
 			NULL, NULL);
+	}
 	else
+	{
 		e2_dialog_add_custom_button_full (config_dialog, FALSE,
 			E2_RESPONSE_USER2, _("Ad_vanced"), GTK_STOCK_REDO,
 			_("Display all configuration options"),
 			NULL, NULL);
+	}
 
 	E2_BUTTON_APPLY.showflags &= ~E2_BTN_DEFAULT;	//CHECKME modify local copy ?
 	e2_dialog_show (config_dialog, app.main_window, 0,
 		&E2_BUTTON_MORE, &E2_BUTTON_DISCARD, &E2_BUTTON_APPLY, NULL);
+
+	//prepare all the buttons for mnemonic-blocking during any keybinding config
+	e2_option_tree_connect_mnemonics (
+#ifdef USE_GTK2_14
+		gtk_dialog_get_action_area (GTK_DIALOG(config_dialog))
+#else
+		GTK_DIALOG(config_dialog)->action_area
+#endif
+	);
+
+//	app.keytrans = FALSE;	//cancel any keys-translation-flag from earlier in this session
 }
