@@ -1,6 +1,6 @@
-/* $Id: e2_dialog.h 1345 2008-11-05 01:54:23Z tpgww $
+/* $Id: e2_dialog.h 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -50,6 +50,7 @@ typedef enum
 	E2_DIALOG_DONT_SHOW_ALL=1<<4,
 	E2_DIALOG_CLOSELOCK    =1<<5,	//threads_enter/leave around showing the dialog
 	E2_DIALOG_MULTI        =1<<6,	//dialog relates to multiple selected items
+	E2_DIALOG_POS          =1<<7	//remember horz and vert position for reuse next time
 } E2_DialogFlags;
 
 typedef enum
@@ -70,13 +71,14 @@ typedef enum
 	NONE   = 0,
 	YESALL = 1,
 	NOALL  = 1 << 1,
-	BOTHALL= 1 << 2
+	BOTHALL= 1 << 2,
+	PERMITTED = 1 << 6 //add "no permission" label to dialog and adjust buttons accordingly
 } OW_ButtonFlags;
 
 enum { E2_TYPE_CANCELLED, E2_TYPE_HANDLED, E2_TYPE_NOTHANDLED };
 
-#define DEFAULT_RESPONSE_CB NULL
-#define DUMMY_RESPONSE_CB GINT_TO_POINTER(1)	//non-NULL
+#define DEFAULT_RESPONSE_CB (ResponseFunc)NULL
+#define DUMMY_RESPONSE_CB (ResponseFunc)1	//non-NULL
 
 typedef struct _E2_Wait
 {
@@ -103,25 +105,32 @@ DialogButtons e2_dialog_password_input (gchar* window_title, gchar *prompt,
 DialogButtons e2_dialog_line_input (gchar* window_title, gchar *prompt,
 	gchar *suggestion, OW_ButtonFlags extras, gboolean select_text,
 	gchar **input);
+DialogButtons e2_dialog_positioned_input (gchar* window_title, gchar *prompt,
+	gchar *suggestion, OW_ButtonFlags extras, gboolean select_text, gboolean permitted,
+	gint *horz, gint *vert, gchar **input);
 DialogButtons e2_dialog_combo_input (gchar* window_title, gchar *prompt,
 	gchar *suggestion, OW_ButtonFlags extras, GList **history_list, gchar **input);
+//delete confirmation dialog
+DialogButtons e2_dialog_delete_check (VPATH *localpath, gboolean multi,
+	gboolean permitted,	gint *horz, gint *vert);
 //overwrite check dialog
-DialogButtons e2_dialog_ow_check (gchar *slocal, gchar *dlocal, OW_ButtonFlags extras);
+DialogButtons e2_dialog_ow_check (VPATH *slocal, VPATH *dlocal, OW_ButtonFlags extras);
 DialogButtons e2_dialog_warning (gchar *prompt, const gchar *yes_label);
 GtkWidget *e2_dialog_slow (gchar *prompt_type, gchar *tip_type,
-	gpointer response_func, gpointer data);
+	void(*response_func) (GtkDialog*,gint,gpointer), gpointer data);
 void e2_dialog_setup_chooser (GtkWidget *dialog, const gchar *title,
 	const gchar *startpath, GtkFileChooserAction action, gboolean showhidden,
 	gboolean multi, gint defresponse);
 
 //extern gboolean e2_dialog_key_cb (GtkWidget *widget, GdkEventKey *event, gpointer data);
+void e2_dialog_setup_auth (GtkWidget *box);
 void e2_dialog_setup (GtkWidget *dialog, GtkWidget *parent);
 gint e2_dialog_show (GtkWidget *dialog, GtkWidget *parent, E2_DialogFlags flags,
 	E2_Button *button, ...);
 gint e2_dialog_run (GtkWidget *dialog, GtkWidget *parent, E2_DialogFlags flags);
 //GtkWidget *get_dialog (GtkWidget *parent, GtkMessageType type, gchar *label_text);
 GtkWidget *e2_dialog_create (const gchar *stock, const gchar *label_text,
-	const gchar *title, gpointer callback, gpointer data);
+	const gchar *title,	void (*callback) (GtkDialog*,gint,gpointer), gpointer data);
 GtkWidget *e2_dialog_add_sw (GtkWidget *dialog);
 GtkWidget *e2_dialog_add_defined_button (GtkWidget *dialog, E2_Button *button);
 GtkWidget *e2_dialog_add_simple_button (GtkWidget *dialog, const gchar *stock,
@@ -131,25 +140,25 @@ GtkWidget *e2_dialog_add_custom_button (GtkWidget *dialog, E2_Button *button,
 GtkWidget *e2_dialog_add_custom_button_full
 	(GtkWidget *dialog, gboolean is_default, guint response,
 	  const gchar *label, const gchar *stock, const gchar *tip,
-	  gpointer callback, gpointer data);
+	  void (*callback)(/*GtkButton*,gpointer*/), gpointer data);
 GtkWidget *e2_dialog_add_toggle_button (GtkWidget *dialog, gboolean value,
 	gchar *label, gchar *tip, gint response);
 GtkWidget *e2_dialog_add_check_button (GtkWidget *dialog, gboolean value,
 	gchar *label, gchar *tip, gint response);
-void e2_dialog_attach_menu (GtkWidget *dialog, GtkWidget *menu);
 //void e2_dialog_set_positive_response (GtkWidget *dialog, gint response);
 void e2_dialog_set_negative_response (GtkWidget *dialog, gint response);
 //void e2_dialog_set_responses (GtkWidget *dialog, gint pos, gint neg);
 void e2_dialog_resize (GtkWidget *dialog, gfloat factor);
-void e2_dialog_resize_with_sw (GtkWidget *dialog, GtkWidget *scrolled);
 void e2_dialog_set_cursor (GtkWidget *dialog, GdkCursorType type);
 void e2_dialog_options_register (void);
 
 //dialog setup functions from other files
-void e2_name_filter_dialog_create_cb (GtkWidget *item, ViewInfo *view);
-void e2_size_filter_dialog_create_cb (GtkWidget *item, ViewInfo *view);
-void e2_date_filter_dialog_create_cb (GtkWidget *item, ViewInfo *view);
-void e2_select_items_byname_dialog_create (GtkWidget *item);
+void e2_name_filter_dialog_create_cb (GtkCheckMenuItem *item, ViewInfo *view);
+void e2_size_filter_dialog_create_cb (GtkCheckMenuItem *item, ViewInfo *view);
+void e2_date_filter_dialog_create_cb (GtkCheckMenuItem *item, ViewInfo *view);
+time_t e2_date_filter_dialog_get_time (const gchar *datestring, gchar *buf,
+	gint bufsize, gint date_format_index);
+void e2_select_items_byname_dialog_create (GtkMenuItem *item);
 gint e2_filetype_dialog_create (VPATH *localpath, gboolean text,
 	gboolean ambig, gboolean newtype);
 DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi);
