@@ -1,6 +1,6 @@
-/* $Id: e2_mkdir_dialog.c 1363 2008-12-31 08:58:05Z tpgww $
+/* $Id: e2_mkdir_dialog.c 2031 2009-12-21 02:35:14Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -46,9 +46,9 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed);
 /**
 @brief determine a suggested directory name, by appending or incrementing a number
 
-@param dir last combo box entry, ie utf8, maybe NULL if no entry
-@param parent_dir is rt->path, utf8
-@return newly-allocated utf8 string
+@param dir last combo box entry, ie UTF-8, maybe NULL if no entry
+@param parent_dir is rt->path, UTF-8
+@return newly-allocated UTF-8 string
 */
 static gchar *_e2_mkdirdlg_find_dir (const gchar *dir, const gchar *parent_dir)
 {
@@ -65,7 +65,7 @@ static gchar *_e2_mkdirdlg_find_dir (const gchar *dir, const gchar *parent_dir)
 	gchar *part1 = (g_path_is_absolute (dir)) ? "" : (gchar *)parent_dir;
 	//part of the combobox string that's a relative path like "bar/" from "bar/foo"
 	gchar *part2 = g_path_get_dirname (dir);
-	if (g_str_equal (part2, "."))
+	if (!strcmp (part2, "."))
 	{	//no path entered
 		g_free (part2);
 		part2 = g_strdup ("");
@@ -112,7 +112,7 @@ static gchar *_e2_mkdirdlg_find_dir (const gchar *dir, const gchar *parent_dir)
 					digit_end = j;
 				}
 //				digit_start = j;
-				if (j == 0)	//no chance of a preceeding non-digit
+				if (j == 0)	//no chance of a preceding non-digit
 					digit_start = 0;
 			}
 			else if (last_was_digit)
@@ -162,18 +162,18 @@ static gchar *_e2_mkdirdlg_find_dir (const gchar *dir, const gchar *parent_dir)
 		while (!e2_fs_access (local, F_OK E2_ERR_NONE()))
 #endif
 		{
-			i++;
 			g_free (path);
+			F_FREE (local, path);
+			i++;
 			if (digit_start != -1)
 				path = g_strdup_printf (format, part1, part2, part3, i, part4);
 			else
 				path = g_strdup_printf (format, part1, part2, part3, i);
-			F_FREE (local);
 			local = F_FILENAME_TO_LOCALE (path);
 		}
 		g_free (format);
 		g_free (path);
-		F_FREE (local);
+		F_FREE (local, path);
 
 		//now build the string for the entry
 		//from the the string the user really entered
@@ -203,10 +203,10 @@ static gchar *_e2_mkdirdlg_find_dir (const gchar *dir, const gchar *parent_dir)
 /**
 @brief walk up the path of @a parent until an existing directory is found
 
-@param parent path of dir to check, freeable utf8 string
+@param parent path of dir to check, freeable UTF-8 string
 @param exists store for flag indicating that directory @a parent was not found
 @param i store for no. of path segments scanned
-@return utf8 string, @a parent or newly-allocated substitute
+@return UTF-8 string, @a parent or newly-allocated substitute
 */
 static gchar *_e2_mkdirdlg_real_parent (gchar *parent, gboolean *exists, gint *i)
 {
@@ -223,75 +223,73 @@ static gchar *_e2_mkdirdlg_real_parent (gchar *parent, gboolean *exists, gint *i
 	{
 		(*i)++;
 		*exists = FALSE;
-		if ((parent[0] == '/') && (parent[1] == '\0'))
+		if ((parent[0] == G_DIR_SEPARATOR) && (parent[1] == '\0'))
 			break;
+		F_FREE (local, parent);
 		gchar *freeme = parent;
 		parent = g_path_get_dirname (parent);
-		F_FREE (local);
 		local = F_FILENAME_TO_LOCALE (parent);
+#ifdef E2_VFS
+		ddata.path = local;
+#endif
 		g_free (freeme);
 	}
-	F_FREE (local);
+	F_FREE (local, parent);
 	return parent;
 }
 /**
-@brief clean up after a mkdir dialog
-This is called only with BGL open
-@param rt pointer to dialog data struct
-@return
-*/
-static void _e2_mkdirdlg_dialog_destroy (E2_MkdirDialogRuntime *rt)
-{
-//#ifdef RACE_CHECK
-	printd (DEBUG, "mkdir dialog destroy");
-//#endif
-	gdk_threads_enter ();
-	gtk_widget_destroy (rt->dialog);
-	gdk_threads_leave ();
-	e2_list_free_with_data (&rt->history);
-	e2_hook_unregister (&app.pane1.hook_change_dir, _e2_mkdirdlg_change_dir_hook, rt, TRUE);
-	e2_hook_unregister (&app.pane2.hook_change_dir, _e2_mkdirdlg_change_dir_hook, rt, TRUE);
-	e2_hook_unregister (&app.hook_pane_focus_changed, _e2_mkdirdlg_change_focus_hook, rt, TRUE);
-//#ifdef RACE_CHECK
-//	printd (DEBUG, "mkdir dialog unhooks completed");
-//#endif
-	g_free (rt->path);
-	if (rt->idle_id != 0)
-#ifdef RACE_CHECK
-	{
-		printd (DEBUG, "mkdir dialog");
-		gboolean debug =
-#endif
-		g_source_remove (rt->idle_id);
-#ifdef RACE_CHECK
-		if (debug)
-			printd (DEBUG, " idle removal completed");
-		else
-			printd (DEBUG, " idle removal failed");
-	}
-//	else
-//		printd (DEBUG, "mkdir dialog NO idle source to remove");
-#endif
-	DEALLOCATE (E2_MkdirDialogRuntime, rt);
-//#ifdef RACE_CHECK
-//	printd (DEBUG, "mkdir dialog destroy COMPLETED");
-//#endif
-}
-/**
 @brief change size of a mkdir dialog window
-This is called from several placces, all inside BGL
+This is called from several places, all with BGL closed
 @param rt pointer to dialog data struct
 @return
 */
 static void _e2_mkdirdlg_update_dialog_size (E2_MkdirDialogRuntime *rt)
 {
-//	printd (DEBUG, "mkdir dialog resize pending");
-	e2_dialog_resize_with_sw (rt->dialog, rt->scrolled);	//this works at idle-time
+	WAIT_FOR_EVENTS
+	GtkAllocation alloc;
+#ifdef USE_GTK2_18
+	gtk_widget_get_allocation (rt->dialog, &alloc);
+#else
+	alloc = rt->dialog->allocation;
+#endif
+#if 1
+	GtkRequisition req;
+	gtk_widget_size_request (rt->dialog, &req);
+	if (alloc.width != req.width || alloc.height != req.height)
+#else
+	//this way is not so reliable
+	if ((alloc.width != rt->dialog->requisition.width) ||
+		(alloc.height != rt->dialog->requisition.height))
+#endif
+	{
+		//suspend scrollers so that resizing works as intended
+		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (rt->scrolled),
+			GTK_POLICY_NEVER, GTK_POLICY_NEVER);
+//		WAIT_FOR_EVENTS
+#if 1
+		gtk_widget_size_request (rt->dialog, &req);//CHECKME maybe different now ?
+		//+20 wide seems needed for vertical scroller
+		if (req.width > 0 && req.height > 0)
+			gtk_window_resize (GTK_WINDOW (rt->dialog), req.width + 20, req.height);
+		else
+			printd (DEBUG, "Bad dialog size %d X %d", req.width, req.height);
+#else
+		if (rt->dialog->requisition.width > 0 && rt->dialog->requisition.height > 0)
+			gtk_window_resize (GTK_WINDOW (rt->dialog),
+				rt->dialog->requisition.width + 20, rt->dialog->requisition.height);
+		else
+			printd (DEBUG, "Bad dialog size %d X %d",
+				rt->dialog->requisition.width, rt->dialog->requisition.height);
+#endif
+//		WAIT_FOR_EVENTS
+		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (rt->scrolled),
+			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	}
 }
 /**
 @brief update dialog label which describes the parent path
 This is called from _e2_mkdirdlg_update_status() (inside BGL)
-@param path parent directory path, utf8 string
+@param path parent directory path, UTF-8 string
 @param rt pointer to dialog data struct
 @return
 */
@@ -313,16 +311,17 @@ static void _e2_mkdirdlg_update_parent_label (gchar *path, E2_MkdirDialogRuntime
 /**
 @brief update dialog label which describes creation, and also buttons' sensitivity
 This is called from _e2_mkdirdlg_update_status() (inside BGL)
-@param reason explanation string
+@param reason explanation string, or NULL
 @param rt pointer to dialog data struct
 @return
 */
 static void _e2_mkdirdlg_update_creation_widgets (gchar *reason, E2_MkdirDialogRuntime *rt)
 {
-	printd (DEBUG, "mkdir dialog update creation widgets");
 	gchar *color;
 	gchar *constant;
+	gchar *label;
 
+	printd (DEBUG, "mkdir dialog update creation widgets");
 	if (rt->creation_possible)
 	{
 		color = e2_option_str_get ("color-positive");
@@ -333,7 +332,7 @@ static void _e2_mkdirdlg_update_creation_widgets (gchar *reason, E2_MkdirDialogR
 		color = e2_option_str_get ("color-negative");
 		constant = _("no");
 	}
-	gchar *label = g_strconcat ("<span weight=\"bold\" size=\"small\" foreground=\"",
+	label = g_strconcat ("<span weight=\"bold\" size=\"small\" foreground=\"",
 			color, "\">", constant, " </span>", reason == NULL ? NULL : "<small>(",
 			reason, ")</small>", NULL);
 	gtk_label_set_markup (GTK_LABEL (rt->info_label2), label);
@@ -344,7 +343,6 @@ static void _e2_mkdirdlg_update_creation_widgets (gchar *reason, E2_MkdirDialogR
 	//sensitivity is ever changed either way
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (rt->dialog), E2_RESPONSE_MORE, rt->creation_possible);
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (rt->dialog), GTK_RESPONSE_YES, rt->creation_possible);
-
 }
 /**
 @brief update the two info labels in the info frame, and buttons' sensitivity
@@ -358,43 +356,50 @@ static void _e2_mkdirdlg_update_status (E2_MkdirDialogRuntime *rt)
 {
 	printd (DEBUG, "mkdir dialog update status");
 	gboolean absolute;
+	gint len;
 	gchar *path, *parent;
 	E2_ERR_DECLARE
 
 	rt->creation_possible = FALSE;
 
-	const gchar *entry_text = gtk_entry_get_text
-		(GTK_ENTRY (GTK_BIN (rt->combo)->child));
-	//if the entry is empty, we'll only check if the parent directory
-	//is writable, so, set path to the currently active directory
-	if (*entry_text == '\0')
+	const gchar *dir = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+		GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->combo)))
+#else
+		GTK_ENTRY (GTK_BIN (rt->combo)->child)
+#endif
+	);
+	//if the entry is empty, we'll only check if the parent directory is
+	//writable, so, set path to the currently active directory
+	if (*dir == '\0')
 	{
+		absolute = TRUE;
 		path = g_strdup (rt->path);
-		absolute = TRUE;	//CHECKME
-		parent = g_strdup (path);
+		len = strlen (path);
+		if (len > 1 && *(path + len - 1) == G_DIR_SEPARATOR)
+			*(path + len - 1) = '\0';
+		parent = g_strdup (path);	//CHECKME
 	}
-	else //otherwise we have to create an absolute path from
-			//the entry text and the active directory
+	else //otherwise we have to create an absolute path from the active
+		 //directory and the entry text
 	{
-		path = (gchar *) entry_text;
-		absolute = g_path_is_absolute (path);
-		if (absolute)
-			path = g_strdup (entry_text); //make sure this is always freeable
-		else
-			//the directory entered is not absolute, prepend ...
-			path = g_strconcat (rt->path, path, NULL);  //CHECKME extra separator ?
+		absolute = g_path_is_absolute ((gchar *) dir);
+		path = e2_utils_translate_relative_path (rt->path, (gchar *) dir);
+		len = strlen (path);
+		if (len > 1 && *(path + len - 1) == G_DIR_SEPARATOR)
+			*(path + len - 1) = '\0';
 		parent = g_path_get_dirname (path);
 	}
 
 	gboolean parent_exists = TRUE;
 	//number of "ancestor" dirs that need to be created
-	gint num = g_str_has_suffix (entry_text, G_DIR_SEPARATOR_S) ? 0 : 1;
+//	gint num = g_str_has_suffix (path, G_DIR_SEPARATOR_S) ? 0 : 1;
+	gint num = 1;
 	parent = _e2_mkdirdlg_real_parent (parent, &parent_exists, &num);
-
 	_e2_mkdirdlg_update_parent_label (parent, rt);
 
 	gchar *reason = NULL;
-	if (*entry_text == '\0')
+	if (*dir == '\0')
 	{
 		_e2_mkdirdlg_update_creation_widgets (reason, rt);
 		g_free (path);
@@ -438,13 +443,13 @@ static void _e2_mkdirdlg_update_status (E2_MkdirDialogRuntime *rt)
 		E2_ERR_CLEAR
 		g_free (path);
 		g_free (parent);
-		F_FREE (local);
+		F_FREE (local, parent);
 		return;
 	}
-	F_FREE (local);
+	F_FREE (local, parent);
 	local = F_FILENAME_TO_LOCALE (path);
 #ifdef E2_VFS
-	ddata.localpath = local;
+	ddata.path = local;
 	if (!e2_fs_access (&ddata, F_OK E2_ERR_NONE()))
 #else
 	if (!e2_fs_access (local, F_OK E2_ERR_NONE()))
@@ -461,10 +466,10 @@ static void _e2_mkdirdlg_update_status (E2_MkdirDialogRuntime *rt)
 		_e2_mkdirdlg_update_creation_widgets (reason, rt);
 		g_free (path);
 		g_free (parent);
-		F_FREE (local);
+		F_FREE (local, path);
 		return;
 	}
-	F_FREE (local);
+	F_FREE (local, path);
 
 	if (num > 1)
 		reason = g_strdup_printf ("%d %s",num,_("directories will be created"));
@@ -474,15 +479,20 @@ static void _e2_mkdirdlg_update_status (E2_MkdirDialogRuntime *rt)
 	g_free (path);
 }
 /**
-@brief update suggested name for the next mkdir, and consequent UI changes
+@brief update suggested name for the next mkdir
 This is called from several places, all with BGL closed
 @param rt pointer to dialog data struct
 @return
 */
-static void _e2_mkdirdlg_update_name_and_status (E2_MkdirDialogRuntime *rt)
+static void _e2_mkdirdlg_update_name (E2_MkdirDialogRuntime *rt)
 {
-	printd (DEBUG, "mkdir dialog update name and status");
-	GtkWidget *entry = GTK_BIN (rt->combo)->child;
+	printd (DEBUG, "mkdir dialog update name");
+	GtkWidget *entry =
+#ifdef USE_GTK2_14
+		gtk_bin_get_child (GTK_BIN (rt->combo));
+#else
+		GTK_BIN (rt->combo)->child;
+#endif
 	if (rt->opt_suggest_dir || rt->opt_show_last)
 	{
 		gchar *text = e2_combobox_first_text (GTK_COMBO_BOX (rt->combo));
@@ -503,17 +513,15 @@ static void _e2_mkdirdlg_update_name_and_status (E2_MkdirDialogRuntime *rt)
 	else
 		gtk_entry_set_text (GTK_ENTRY (entry), "");
 
-	_e2_mkdirdlg_update_status (rt);
-
 #ifdef RACE_CHECK
 	printd (DEBUG, "mkdir update combo completed");
 #endif
 }
 /**
 @brief set or clear mechanism for following changes of active directory
-This is called out of BGL
-If following is turned on, the default path is updated and status info etc
-updated accordingly
+If @a follow is TRUE, the default path is updated and status info etc updated
+accordingly.
+This is called with BGL closed.
 @param rt pointer to dialog data struct
 @param follow TRUE to turn on following, FALSE to turn it off
 @return
@@ -525,27 +533,25 @@ static void _e2_mkdirdlg_update_follow_dir (E2_MkdirDialogRuntime *rt, gboolean
 	{
 		g_free (rt->path);
 		rt->path = g_strdup (curr_pane->path);
-		gdk_threads_enter ();
 		_e2_mkdirdlg_update_status (rt);
-		gdk_threads_leave ();
 #ifdef E2_VFSTMP
 	//CHECKME hooks for non-mounted dirs
 #endif
-		e2_hook_register (&app.pane1.hook_change_dir, _e2_mkdirdlg_change_dir_hook, rt);
-		e2_hook_register (&app.pane2.hook_change_dir, _e2_mkdirdlg_change_dir_hook, rt);
-		e2_hook_register (&app.hook_pane_focus_changed, _e2_mkdirdlg_change_focus_hook, rt);
+		e2_hook_register (&app.pane1.hook_change_dir, (HookFunc)_e2_mkdirdlg_change_dir_hook, rt);
+		e2_hook_register (&app.pane2.hook_change_dir, (HookFunc)_e2_mkdirdlg_change_dir_hook, rt);
+		e2_hook_register (&app.hook_pane_focus_changed, (HookFunc)_e2_mkdirdlg_change_focus_hook, rt);
 	}
 	else
 	{
-		e2_hook_unregister (&app.pane1.hook_change_dir, _e2_mkdirdlg_change_dir_hook, rt, TRUE);
-		e2_hook_unregister (&app.pane2.hook_change_dir, _e2_mkdirdlg_change_dir_hook, rt, TRUE);
-		e2_hook_unregister (&app.hook_pane_focus_changed, _e2_mkdirdlg_change_focus_hook, rt, TRUE);
+		e2_hook_unregister (&app.pane1.hook_change_dir, (HookFunc)_e2_mkdirdlg_change_dir_hook, rt, TRUE);
+		e2_hook_unregister (&app.pane2.hook_change_dir, (HookFunc)_e2_mkdirdlg_change_dir_hook, rt, TRUE);
+		e2_hook_unregister (&app.hook_pane_focus_changed, (HookFunc)_e2_mkdirdlg_change_focus_hook, rt, TRUE);
 	}
 }
 /**
 @brief hook function for app.paneX.hook_change_dir
-This is called with BGL off/open
-@param path path of an opened directory, utf-8 string
+This is initiated from cd thread, with BGL off/open
+@param path path of an opened directory, UTF-8 string
 @param rt pointer to dialog's data struct
 @return TRUE
 */
@@ -584,9 +590,9 @@ static gboolean _e2_mkdirdlg_change_focus_hook (E2_PaneRuntime *pane_rt, E2_Mkdi
 }
 /**
 @brief create a new directory, along with any missing "ancestor(s)", in the active-pane dir
-The string provided may be an absolute or relative path, the latter
-including just a name. If relative, the last-used dir?? will be prepended
-This is called only from _e2_mkdirdlg_response_cb () with BGL closed
+The string provided may be an absolute or relative path, the latter including
+just a name. If relative, the last-used CWD (rt->path) will be prepended.
+This is called only from _e2_mkdirdlg_response_cb() with BGL closed
 @param entry the entry widget for the name-entry combo
 @param rt pointer to dialog's data struct
 @param close TRUE if the dialog will be closed when finished here
@@ -604,23 +610,22 @@ static void _e2_mkdirdlg_create_dir (GtkWidget *entry, E2_MkdirDialogRuntime *rt
 	if (*dir == '\0')
 		return;
 
-	gchar *path, *parent, *local, *freeme = NULL;
-	if (g_path_is_absolute (dir))
-		path = g_strdup (dir);
-	else
-		path = g_strconcat (rt->path, dir, NULL);
+	gchar *path, *parent1, *parent, *local, *freeme;
+	path = e2_utils_translate_relative_path (rt->path, (gchar *) dir);
 
 	//find closest existing "ancestor" of the dir to be created, and any
 	//intervening ancestor dir(s) that need to be created
-	GList *tmp, *missing = NULL;
-	parent = g_path_get_dirname (path);
+	parent1 = g_path_get_dirname (path);
 	if (g_str_has_suffix (path, G_DIR_SEPARATOR_S))
 	{
-		freeme = parent;
-		parent = g_path_get_dirname (parent);
+		freeme = parent1;
+		parent1 = g_path_get_dirname (parent1);
 		g_free (freeme);
 	}
-	freeme = local = F_FILENAME_TO_LOCALE (parent);
+	freeme = local = F_FILENAME_TO_LOCALE (parent1);
+	parent = parent1;
+
+	GList *missing = NULL;
 #ifdef E2_VFS
 	VPATH ddata = { local, curr_view->spacedata };
 	while (e2_fs_access (&ddata, F_OK E2_ERR_NONE()) || !e2_fs_is_dir3 (&ddata E2_ERR_NONE()))
@@ -628,27 +633,36 @@ static void _e2_mkdirdlg_create_dir (GtkWidget *entry, E2_MkdirDialogRuntime *rt
 	while (e2_fs_access (local, F_OK E2_ERR_NONE()) || !e2_fs_is_dir3 (local E2_ERR_NONE()))
 #endif
 	{
+		if ((parent[0] == G_DIR_SEPARATOR) && (parent[1] == '\0'))
+		{
+			if (parent != parent1)
+				g_free (parent);
+			break;
+		}
+		missing = g_list_prepend (missing, g_strdup(local));
 #ifdef E2_VFSTMP
 	//FIXME parent path for virtual dirs
 #endif
-		if ((parent[0] == G_DIR_SEPARATOR) && (parent[1] == '\0'))
-			break;
-		missing = g_list_prepend (missing, parent);
-		parent = g_path_get_dirname (parent);
-		F_FREE (local);
-		local = F_FILENAME_TO_LOCALE (parent);
+		local = g_path_get_dirname (parent);
+		if (parent != parent1)
+			g_free (parent);
+		parent = local;
+#ifdef E2_VFS
+		ddata.path = local;
+#endif
 	}
-	g_free (parent);
-	F_FREE (freeme);
+	g_free (parent1);
+	F_FREE (freeme, parent1);
 
 	E2_ERR_DECLARE
+	GList *tmp;
 	//create any missing ancestor(s)
 	for (tmp = missing; tmp != NULL; tmp = g_list_next (tmp))
 	{
-		local = F_FILENAME_TO_LOCALE ((gchar *)tmp->data);
+		local = (gchar *)tmp->data;
 //		E2_ERR_NAME = NULL;
 #ifdef E2_VFS
-		ddata.localpath = local;
+		ddata.path = local;
 		if (e2_fs_mkdir (&ddata, 0777 E2_ERR_PTR()))	//FIXME vfs
 #else
 		if (e2_fs_mkdir (local, 0777 E2_ERR_PTR()))	//FIXME vfs
@@ -671,25 +685,26 @@ static void _e2_mkdirdlg_create_dir (GtkWidget *entry, E2_MkdirDialogRuntime *rt
 #else
 			e2_filelist_request_refresh (rt->path, TRUE);
 #endif
-			F_FREE (local);
 			e2_list_free_with_data (&missing);
 			return;
 		}
-		F_FREE (local);
 	}
 	e2_list_free_with_data (&missing);
 
-	//now create the chosen dir itself
+	//now create the lowest-level dir
 	local = F_FILENAME_TO_LOCALE (path);
 #ifdef E2_VFS
-	ddata.localpath = local;
+	ddata.path = local;
 	if (!e2_fs_mkdir (&ddata, 0777 E2_ERR_PTR()))
 #else
 	if (!e2_fs_mkdir (local, 0777 E2_ERR_PTR()))
 #endif
 	{	//succeeded
 		if (!close)
-			_e2_mkdirdlg_update_name_and_status (rt);
+		{
+			_e2_mkdirdlg_update_name (rt);
+			_e2_mkdirdlg_update_status (rt);
+		}
 #ifdef E2_VFSTMP
 	//FIXME path for non-mounted dirs
 #else
@@ -713,7 +728,7 @@ static void _e2_mkdirdlg_create_dir (GtkWidget *entry, E2_MkdirDialogRuntime *rt
 			_e2_mkdirdlg_update_status (rt);
 	}
 	g_free (path);
-	F_FREE (local);
+	F_FREE (local, path);
 }
 
   /*********************/
@@ -736,41 +751,66 @@ static void _e2_mkdirdlg_response_cb (GtkDialog *dialog, gint response,
 	switch (response)
 	{
 		case GTK_RESPONSE_YES:
+			if (rt->idle_id > 0)
+				g_source_remove (rt->idle_id);	//no more UI-update callbacks when we're finishing
 		case E2_RESPONSE_MORE:
-			entry = GTK_BIN (rt->combo)->child;
+			entry =
+#ifdef USE_GTK2_14
+				gtk_bin_get_child (GTK_BIN (rt->combo));
+#else
+				GTK_BIN (rt->combo)->child;
+#endif
 			const gchar *dir = gtk_entry_get_text (GTK_ENTRY (entry));
-			//quick exit
-			if (*dir == '\0')
-				break;
-			if (strchr (dir, '%') != NULL)
+			if (*dir != '\0')
 			{
-				gchar *freeme = e2_utils_expand_macros ((gchar *)dir, NULL);
-				if (freeme != NULL && freeme != GINT_TO_POINTER (1))
+				if (strchr (dir, '%') != NULL)
 				{
-					gtk_entry_set_text (GTK_ENTRY (entry), freeme);
-					g_free (freeme);
+					gchar *freeme = e2_utils_expand_macros ((gchar *)dir, NULL);
+					if (freeme != NULL && freeme != GINT_TO_POINTER (1))
+					{
+						gtk_entry_set_text (GTK_ENTRY (entry), freeme);
+						g_free (freeme);
+					}
 				}
+//				*rt->status = E2_TASK_RUNNING;
+				if (response == E2_RESPONSE_MORE)
+				{
+					e2_combobox_activated_cb (entry, GINT_TO_POINTER (FALSE));
+					_e2_mkdirdlg_create_dir (entry, rt, FALSE);
+//					*rt->status = E2_TASK_PAUSED;
+					gtk_widget_grab_focus (entry);
+					//prevent the dialog being closed by the other cb
+					g_signal_stop_emission_by_name ((gpointer)dialog, "response");
+				}
+				else
+					_e2_mkdirdlg_create_dir (entry, rt, TRUE);
 			}
-			e2_combobox_activated_cb (entry, GINT_TO_POINTER (FALSE));
-			*rt->status = E2_TASK_RUNNING;
-			if (response == E2_RESPONSE_MORE)
-			{
-				_e2_mkdirdlg_create_dir (entry, rt, FALSE);
-				*rt->status = E2_TASK_PAUSED;
-				gtk_widget_grab_focus (entry);
-				//prevent the dialog being closed by the other cb
-				g_signal_stop_emission_by_name (G_OBJECT (dialog), "response");
-			}
-			else
-				_e2_mkdirdlg_create_dir (entry, rt, TRUE);
 		default:
 			break;
 	}
 }
 /**
-@brief combobox active-item "changed" callback
-This is also called every time a letter is added to or removed from the
-dialog's name-entry
+@brief idle callback to update dialog UI after any change of the name-entry contents
+@param rt pointer to dialog's data struct
+@return FALSE after update done
+*/
+static gboolean _e2_mkdirdlg_deferred_change_cb (E2_MkdirDialogRuntime *rt)
+{
+	if (GTK_IS_DIALOG (rt->dialog))
+	{
+		gdk_threads_enter ();
+		_e2_mkdirdlg_update_status (rt);
+		_e2_mkdirdlg_update_dialog_size (rt);
+		gdk_threads_leave ();
+
+		rt->idle_id = 0;
+	}
+	return FALSE;
+}
+/**
+@brief directory-name combobox "changed" signal callback, initiates a GUI update
+This is also called every time a letter is added to or removed from the dialog's
+name-entry. So GUI update is deferred to idle time, to help eliminate keypress latency
 @param combo UNUSED the affected combobox
 @param rt pointer to dialog's data struct
 @return
@@ -778,8 +818,29 @@ dialog's name-entry
 static void _e2_mkdirdlg_changed_cb (GtkComboBox *combo, E2_MkdirDialogRuntime *rt)
 {
 	printd (DEBUG, "mkdir dialog changed cb");
-	_e2_mkdirdlg_update_status (rt);
-	_e2_mkdirdlg_update_dialog_size (rt);
+	if (rt->idle_id == 0)	//don't want re-entrance
+		rt->idle_id = g_idle_add ((GSourceFunc)_e2_mkdirdlg_deferred_change_cb, rt);
+}
+/**
+@brief dialog "button-press-event" signal callback, to pop up a context menu
+@param dialog UNUSED the widget which received the mouse button press
+@param event pointer to event data struct
+@param menu pointer to context menu for the dialog
+@return TRUE if the event was a right-button click
+*/
+static gboolean _e2_mkdirdlg_button_press_cb (GtkWidget *dialog,
+	GdkEventButton *event, GtkWidget *menu)
+{
+	if (event->button == 3 && event->type == GDK_BUTTON_PRESS
+#ifdef E2_MOUSECUSTOM
+		&& (event->state & E2_MODIFIER_MASK) == 0
+#endif
+		)
+	{
+		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
+		return TRUE;
+	}
+	return FALSE;
 }
 /**
 @brief combobox entry "activate" callback
@@ -804,10 +865,7 @@ static void _e2_mkdirdlg_toggled1_cb (GtkWidget *menu_item, E2_MkdirDialogRuntim
 	printd (DEBUG, "mkdir dialog follow pane toggle");
 	gboolean state = gtk_check_menu_item_get_active
 		(GTK_CHECK_MENU_ITEM (menu_item));
-	//open BGL to allow downstream mutex management
-	gdk_threads_leave ();
 	_e2_mkdirdlg_update_follow_dir (rt, state);
-	gdk_threads_enter ();
 	gpointer p = g_object_get_data (G_OBJECT (rt->dialog), "e2-controller-blocked");
 	if (!GPOINTER_TO_INT (p))
 		e2_option_bool_set_direct (follow_pane, state);
@@ -823,7 +881,8 @@ static void _e2_mkdirdlg_toggled2_cb (GtkWidget *menu_item, E2_MkdirDialogRuntim
 	printd (DEBUG, "mkdir dialog show last entry toggle");
 	rt->opt_show_last = gtk_check_menu_item_get_active
 		(GTK_CHECK_MENU_ITEM (menu_item));
-	_e2_mkdirdlg_update_name_and_status (rt);
+	_e2_mkdirdlg_update_name (rt);
+	_e2_mkdirdlg_update_status (rt);
 	gpointer p = g_object_get_data (G_OBJECT (rt->dialog), "e2-controller-blocked");
 	if (!GPOINTER_TO_INT (p))
 		e2_option_bool_set_direct (show_last, rt->opt_show_last);
@@ -839,7 +898,8 @@ static void _e2_mkdirdlg_toggled3_cb (GtkWidget *menu_item, E2_MkdirDialogRuntim
 	printd (DEBUG, "mkdir dialog suggest dir toggle");
 	rt->opt_suggest_dir = gtk_check_menu_item_get_active
 		(GTK_CHECK_MENU_ITEM (menu_item));
-	_e2_mkdirdlg_update_name_and_status (rt);
+	_e2_mkdirdlg_update_name (rt);
+	_e2_mkdirdlg_update_status (rt);
 	gpointer p = g_object_get_data (G_OBJECT (rt->dialog), "e2-controller-blocked");
 	if (!GPOINTER_TO_INT (p))
 		e2_option_bool_set_direct (suggest_dir, rt->opt_suggest_dir);
@@ -855,21 +915,60 @@ static void _e2_mkdirdlg_expander_toggled_cb (GtkWidget *expander,
 	GParamSpec *spec, E2_MkdirDialogRuntime *rt)
 {
 //	printd (DEBUG, "mkdir dialog expander toggle");
+	gboolean value = gtk_expander_get_expanded (GTK_EXPANDER (expander));
+	if (value)
+	{
+#ifdef USE_GTK2_18
+		if (!gtk_widget_get_visible (rt->info_box))
+#else
+		if (!GTK_WIDGET_VISIBLE (rt->info_box))
+#endif
+			gtk_widget_show (rt->info_box);
+	}
+	else
+#ifdef USE_GTK2_18
+		if (gtk_widget_get_visible (rt->info_box))
+#else
+		if (GTK_WIDGET_VISIBLE (rt->info_box))
+#endif
+		gtk_widget_hide (rt->info_box);
+
 	_e2_mkdirdlg_update_dialog_size (rt);
 }
-/**
-@brief callback for dialog window shown
-@param dialog the shown widget
-@param rt pointer to dialog's data struct
-@return
-*/
-static void _e2_mkdirdlg_show_cb (GtkWidget *dialog, E2_MkdirDialogRuntime *rt)
+static void _e2_mkdirdlg_expander_toggled_cb2 (GtkWidget *expander,
+	GParamSpec *spec, E2_OptionSet *boolset)
 {
-	printd (DEBUG, "mkdir dialog show cb");
-	//convenient to set name here, inside BGL
-	_e2_mkdirdlg_update_name_and_status (rt);
-	gtk_expander_set_expanded (GTK_EXPANDER (rt->info_expander),
-		e2_option_bool_get ("dialog-mkdir-show-info"));
+//	printd (DEBUG, "mkdir dialog expander toggle 2");
+	GtkWidget *controller = g_object_get_data (G_OBJECT (expander),
+			"e2-controller-widget");
+	if (!GPOINTER_TO_INT (g_object_get_data (G_OBJECT (controller),
+			"e2-controller-blocked")))
+	{
+		gboolean value = gtk_expander_get_expanded (GTK_EXPANDER (expander));
+		e2_option_bool_set_direct (boolset, value);
+	}
+}
+/**
+@brief set @a expander state if it's not blocked and not already at the desired state
+This is a hook-function callack upon change of set data associated with @a expander
+@param state pointerised TRUE/FALSE, the value to apply to @a expander
+@param expander the widget to change
+
+@return TRUE always
+*/
+static gboolean _e2_mkdirdlg_set_expander_hook (gpointer state, GtkWidget *expander)
+{
+	GtkWidget *controller = g_object_get_data (G_OBJECT (expander),
+		"e2-controller-widget");
+	if (!GPOINTER_TO_INT (g_object_get_data (G_OBJECT (controller),
+		"e2-controller-blocked")))
+	{
+		gboolean value = GPOINTER_TO_INT (state);
+		gboolean current = gtk_expander_get_expanded (GTK_EXPANDER (expander));
+		if (value != current)
+			gtk_expander_set_expanded (GTK_EXPANDER (expander), value);
+	}
+	return TRUE;
 }
 
   /******************/
@@ -886,57 +985,67 @@ static gboolean _e2_mkdir_dialog_create (gpointer from, E2_ActionRuntime *art)
 }
 static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 {
-	//init runtime object
-	E2_MkdirDialogRuntime *rt = ALLOCATE (E2_MkdirDialogRuntime);
-	CHECKALLOCATEDWARN (rt, return FALSE;)
-	rt->history = e2_list_copy_with_data (mkdir_history);
+	E2_MkdirDialogRuntime rt;
+
+	rt.history = e2_list_copy_with_data (mkdir_history);
 #ifdef E2_VFSTMP
 	//FIXME path for non-mounted dirs
 #else
-	rt->path = D_FILENAME_FROM_LOCALE (qed->currdir);
+	rt.path = D_FILENAME_FROM_LOCALE (qed->currdir);
 #endif
-	rt->status = qed->status;	//enable on-the-fly status changes
-	rt->creation_possible = TRUE;
-	rt->idle_id = 0;
+	rt.status = qed->status;	//enable on-the-fly status changes
+	rt.creation_possible = TRUE;
+
+//tag DEBUGfreeze;
+	//maybe this helps gtk 2.16 crashes and/or keypress latency ?
+	//gdk_threads_enter ();
 
 	//create dialog
-	rt->dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
+	rt.dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
 		_("What is the new directory's name?"), _("create directory"),
-		_e2_mkdirdlg_response_cb, rt);
+		(ResponseFunc)_e2_mkdirdlg_response_cb, &rt);
 
-	GtkWidget *vbox = e2_dialog_add_sw (rt->dialog);
-	rt->scrolled = gtk_widget_get_ancestor (vbox, GTK_TYPE_SCROLLED_WINDOW);
-//		vbox->parent->parent;	//bad !!
+	GtkWidget *vbox = e2_dialog_add_sw (rt.dialog);
+	rt.scrolled = gtk_widget_get_ancestor (vbox, GTK_TYPE_SCROLLED_WINDOW);
 
-	//CHECKME this is needed to workaround gtk weirdness,
-	//= bad delay when any key is pressed during a cursor-visible interval
-	gdk_threads_enter ();
-	//add combo
-	rt->combo = e2_combobox_add (vbox, FALSE, E2_PADDING,
-		_e2_mkdirdlg_activated_cb, rt, &mkdir_history,
+	rt.combo = e2_combobox_add (vbox, FALSE, E2_PADDING,
+		(ActivateFunc)_e2_mkdirdlg_activated_cb, &rt, &mkdir_history,
 		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_FOCUS_ON_CHANGE);
-	gdk_threads_leave ();
-	//add expander with info box as its child
-	GtkWidget *info_box = e2_widget_get_box (TRUE, FALSE, 1);
-	rt->info_expander = e2_widget_add_expander (vbox, _("info"),
-		e2_option_get ("dialog-mkdir-show-info"), rt->dialog, info_box);
 
-	g_signal_connect (G_OBJECT (rt->info_expander), "notify::expanded",
-		G_CALLBACK (_e2_mkdirdlg_expander_toggled_cb), rt);
+	rt.info_expander = gtk_expander_new (_("info"));
+	gtk_box_pack_start (GTK_BOX (vbox), rt.info_expander, FALSE, FALSE, 0);
 
-	GtkSizeGroup *group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-	GtkWidget *hbox = e2_widget_add_box (info_box, FALSE, 0, FALSE, FALSE, 0);
+	E2_OptionSet *infoshow = e2_option_get ("dialog-mkdir-show-info");
+	if (e2_option_bool_get_direct (infoshow))
+		gtk_expander_set_expanded (GTK_EXPANDER(rt.info_expander), TRUE);
+
+	g_object_set_data (G_OBJECT (rt.info_expander), "e2-controller-widget", rt.dialog);
+	e2_option_attach_value_changed_simple (infoshow, rt.info_expander,
+		(HookFunc)_e2_mkdirdlg_set_expander_hook, rt.info_expander);
+
+	g_signal_connect (G_OBJECT (rt.info_expander), "notify::expanded",
+		G_CALLBACK (_e2_mkdirdlg_expander_toggled_cb), &rt);
+	g_signal_connect (G_OBJECT (rt.info_expander), "notify::expanded",
+		G_CALLBACK (_e2_mkdirdlg_expander_toggled_cb2), infoshow);
+
+	//info box is packed into vbox, not the expander, so that dialog-resizing works
+	rt.info_box = e2_widget_get_box (TRUE, FALSE, 1);
+	gtk_container_set_border_width (GTK_CONTAINER (rt.info_box), E2_PADDING_SMALL);
+	gtk_box_pack_start (GTK_BOX (vbox), rt.info_box, FALSE, FALSE, 0);
+
+	GtkWidget *hbox = e2_widget_add_box (rt.info_box, FALSE, 0, FALSE, FALSE, 0);
 	GtkWidget *label = e2_widget_add_label (hbox,
 		_("<small>parent directory:</small>"), 0.0, 0.0, FALSE, E2_PADDING_SMALL);
+	GtkSizeGroup *group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 	gtk_size_group_add_widget (group, label);
+	rt.info_label = e2_widget_add_mid_label (hbox, "", 0.0, TRUE, E2_PADDING_SMALL);
 
-	rt->info_label = e2_widget_add_mid_label (hbox, "", 0.0, TRUE, E2_PADDING_SMALL);
-	hbox = e2_widget_add_box (info_box, FALSE, 0, FALSE, FALSE, 0);
+	hbox = e2_widget_add_box (rt.info_box, FALSE, 0, FALSE, FALSE, 0);
 	label = e2_widget_add_label (hbox,
 		_("<small>creation possible:</small>"), 0.0, 0.0, FALSE, E2_PADDING_SMALL);
 	gtk_size_group_add_widget (group, label);
-	rt->info_label2 = e2_widget_add_label (hbox, "", 0.0, 0.0, TRUE, E2_PADDING_SMALL);
 	g_object_unref (G_OBJECT (group));
+	rt.info_label2 = e2_widget_add_label (hbox, "", 0.0, 0.0, TRUE, E2_PADDING_SMALL);
 
 	if (follow_pane == NULL)
 		follow_pane = e2_option_get ("dialog-mkdir-follow-pane");
@@ -947,52 +1056,90 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 	if (connected == NULL)
 		connected = e2_option_get ("dialog-mkdir-connected");
 
-	rt->menu = e2_menu_create_options_menu (rt->dialog, NULL,
-		follow_pane, _e2_mkdirdlg_toggled1_cb, rt,
-		suggest_dir, _e2_mkdirdlg_toggled3_cb, rt,
-		show_last, _e2_mkdirdlg_toggled2_cb, rt,
-		connected, e2_menu_control_cb, rt->dialog,
+	rt.menu = e2_menu_create_options_menu (rt.dialog, NULL,
+		follow_pane, _e2_mkdirdlg_toggled1_cb, &rt,
+		suggest_dir, _e2_mkdirdlg_toggled3_cb, &rt,
+		show_last, _e2_mkdirdlg_toggled2_cb, &rt,
+		connected, e2_menu_control_cb, rt.dialog,
 		NULL);
-	e2_dialog_attach_menu (rt->dialog, rt->menu);
 
-//do this after create button is created
-	_e2_mkdirdlg_update_follow_dir (rt, e2_option_bool_get_direct (follow_pane));
-	e2_option_connect (rt->dialog, e2_option_bool_get_direct (connected));
-	rt->opt_show_last = e2_option_bool_get_direct (show_last);
-	rt->opt_suggest_dir = e2_option_bool_get_direct (suggest_dir);
+	//arrange menu cleanup
+	g_object_set_data_full (G_OBJECT (rt.dialog), "menu", rt.menu,
+		(GDestroyNotify) e2_menu_selection_done_cb);
+	//and popup
+	g_signal_connect (G_OBJECT (rt.dialog), "button-press-event",
+		G_CALLBACK (_e2_mkdirdlg_button_press_cb), rt.menu);
 
-	g_signal_connect (G_OBJECT (rt->dialog), "show",
-		G_CALLBACK (_e2_mkdirdlg_show_cb), rt);
+//do this after create button is created ?
+//tag DEBUGfreeze
+	gdk_threads_enter ();
+	//this updates status but not the suggested dir name
+	_e2_mkdirdlg_update_follow_dir (&rt, e2_option_bool_get_direct (follow_pane));
+//tag DEBUGfreeze
+	gdk_threads_leave ();
+	e2_option_connect (rt.dialog, e2_option_bool_get_direct (connected));
+	rt.opt_show_last = e2_option_bool_get_direct (show_last);
+	rt.opt_suggest_dir = e2_option_bool_get_direct (suggest_dir);
 
-	E2_Button more_btn;
-	more_btn = E2_BUTTON_MORE;
-	more_btn.tip = _("Create and continue");
-	more_btn.showflags |= E2_BTN_TIPPED;
+//tag DEBUGfreeze
+	gdk_threads_enter ();
+	_e2_mkdirdlg_update_name (&rt);
+//tag DEBUGfreeze
+	gdk_threads_leave ();
 
-	E2_Button no_btn;
-	e2_button_derive (&no_btn, &E2_BUTTON_NO, BTN_NO_CANCEL);
-	no_btn.tip = NULL;
-	no_btn.showflags |= E2_BTN_DEFAULT;
+	E2_Button local_btn;
+	local_btn = E2_BUTTON_MORE;
+	local_btn.tip = _("Create and continue");
+	local_btn.showflags |= E2_BTN_TIPPED;
+	e2_dialog_add_defined_button (rt.dialog, &local_btn);
 
-	E2_Button yes_btn;
-	yes_btn.label = _("C_reate");	//this appears with "_Cancel"
-	yes_btn.stock = GTK_STOCK_DIRECTORY;
-	yes_btn.showflags = 0;
-	yes_btn.response = GTK_RESPONSE_YES;
+	e2_button_derive (&local_btn, &E2_BUTTON_NO, BTN_NO_CANCEL);
+	local_btn.tip = NULL;
+	local_btn.showflags |= E2_BTN_DEFAULT;
+	e2_dialog_add_defined_button (rt.dialog, &local_btn);
 
+	local_btn.label = _("C_reate");	//this appears with "_Cancel"
+	local_btn.stock = GTK_STOCK_DIRECTORY;
+	local_btn.showflags = 0;
+	local_btn.response = GTK_RESPONSE_YES;
+	e2_dialog_add_defined_button (rt.dialog, &local_btn);
 
-	e2_dialog_show (rt->dialog, app.main_window, E2_DIALOG_CLOSELOCK | E2_DIALOG_DONT_SHOW_ALL,
-		&more_btn, &no_btn, &yes_btn, NULL);
+	vbox =
+#ifdef USE_GTK2_14
+		gtk_dialog_get_content_area (GTK_DIALOG (rt.dialog));
+#else
+		GTK_DIALOG(rt.dialog)->vbox;
+#endif
+	gtk_widget_show_all (vbox);
+	if (!e2_option_bool_get_direct (infoshow))
+		gtk_widget_hide (rt.info_box);
+	rt.idle_id = g_idle_add ((GSourceFunc)_e2_mkdirdlg_deferred_change_cb, &rt);
 
-	//do this after showing, to reduce repitition
-	g_signal_connect (G_OBJECT (rt->combo), "changed",
-		G_CALLBACK (_e2_mkdirdlg_changed_cb), rt);
+	gdk_threads_enter ();
+	e2_dialog_setup (rt.dialog, app.main_window);
+	e2_dialog_run (rt.dialog, NULL, E2_DIALOG_DONT_SHOW_ALL);
+	gdk_threads_leave ();
+
+	//do this after showing, to reduce repetition
+	g_signal_connect (G_OBJECT (rt.combo), "changed",
+		G_CALLBACK (_e2_mkdirdlg_changed_cb), &rt);
 
 	*qed->status = E2_TASK_PAUSED;
-	e2_dialog_wait (rt->dialog, FALSE, FALSE, FALSE, FALSE);
+	e2_dialog_wait (rt.dialog, FALSE, TRUE, FALSE, FALSE);  //CHECKME TRUE maincontext ?
 	*qed->status = E2_TASK_RUNNING;
 
-	_e2_mkdirdlg_dialog_destroy (rt);
+	if (GTK_IS_DIALOG(rt.dialog))
+	{	//not explicitly closed by the user
+		gdk_threads_enter ();
+		gtk_widget_destroy (rt.dialog);
+		gdk_threads_leave ();
+	}
+
+	g_free (rt.path);
+	e2_list_free_with_data (&rt.history);
+	e2_hook_unregister (&app.pane1.hook_change_dir, (HookFunc)_e2_mkdirdlg_change_dir_hook, &rt, TRUE);
+	e2_hook_unregister (&app.pane2.hook_change_dir, (HookFunc)_e2_mkdirdlg_change_dir_hook, &rt, TRUE);
+	e2_hook_unregister (&app.hook_pane_focus_changed, (HookFunc)_e2_mkdirdlg_change_focus_hook, &rt, TRUE);
 
 	return TRUE;
 }
@@ -1003,11 +1150,11 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 
 void e2_mkdir_dialog_actions_register ()
 {
+	E2_Action action =
+	{g_strconcat(_A(1),".",_A(60),NULL),_e2_mkdir_dialog_create,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+	e2_action_register (&action);
+	//while we're at it, init some session parameters
 	e2_cache_list_register ("mkdir-history", &mkdir_history);
-
-	gchar *action_name = g_strconcat(_A(1),".",_A(60),NULL);
-	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
-		_e2_mkdir_dialog_create, NULL, FALSE);
 }
 
 void e2_mkdir_dialog_options_register ()
