@@ -1,6 +1,6 @@
-/* $Id: e2_select_image_dialog.c 1359 2008-12-05 03:04:09Z tpgww $
+/* $Id: e2_select_image_dialog.c 1987 2009-10-14 00:41:33Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -204,7 +204,7 @@ static void _e2_sidlg_dir_change_cb (GtkFileChooser *chooser, E2_SID_Runtime *rt
 			gtk_list_store_clear (GTK_LIST_STORE (rt->custommodel));
 #ifdef E2_VFSTMP
 			VPATH data;
-			data.localpath = dirpath;
+			data.path = dirpath;
 			data.spacedata = NULL;	//CHECKME ok?
 			_e2_sidlg_fill_custom_store (&data, rt);
 #else
@@ -353,7 +353,7 @@ static GtkListStore *_e2_sidlg_create_custom_store (E2_SID_Runtime *rt)
 	gchar *path = _e2_sidlg_get_icon_dir (rt);
 #ifdef E2_VFS
 	VPATH localpath;
-	localpath.localpath = path;
+	localpath.path = path;
 	localpath.spacedata = NULL;	//local icon files
 	_e2_sidlg_fill_custom_store (&localpath, rt);
 #else
@@ -617,7 +617,12 @@ GtkWidget *e2_sid_create (GtkWidget *parent, const gchar *name, gchar *icon)
 	//for every parent widget
 	E2_SID_Runtime *rt = g_object_get_data (G_OBJECT (parent), name);
 	//the user may have double-clicked on an icon, or ...
-	if (rt != NULL && rt->dialog != NULL && GTK_WIDGET_VISIBLE (rt->dialog))
+	if (rt != NULL && rt->dialog != NULL &&
+#ifdef USE_GTK2_18
+		gtk_widget_get_visible (rt->dialog))
+#else
+		GTK_WIDGET_VISIBLE (rt->dialog))
+#endif
 	{	//the dialog already exists
 		printd (NOTICE, "select image dialog already exists");
 		if (rt->icon != NULL)
@@ -645,7 +650,8 @@ GtkWidget *e2_sid_create (GtkWidget *parent, const gchar *name, gchar *icon)
 
 	//create dialog widgets
 
-	rt->dialog = e2_dialog_create (NULL, NULL, rt->name, _e2_sidlg_response_cb, rt);
+	rt->dialog = e2_dialog_create (NULL, NULL, rt->name,
+		(ResponseFunc)_e2_sidlg_response_cb, rt);
 	GtkWidget *dialog_vbox =
 #ifdef USE_GTK2_14
 		gtk_dialog_get_content_area (GTK_DIALOG (rt->dialog));
@@ -654,7 +660,8 @@ GtkWidget *e2_sid_create (GtkWidget *parent, const gchar *name, gchar *icon)
 #endif
 //	gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), E2_PADDING);
 	//when the dialog is destroyed, free the runtime object
-	g_signal_connect (rt->dialog, "destroy", G_CALLBACK (_e2_sidlg_destroy_cb), rt);
+	g_signal_connect (GTK_OBJECT(rt->dialog), "destroy",
+		G_CALLBACK (_e2_sidlg_destroy_cb), rt);
 	//ensure this dialog is destroyed with the parent widget
 	if (parent != NULL)
 		g_object_set_data_full (G_OBJECT (parent), rt->name, rt,
@@ -662,7 +669,7 @@ GtkWidget *e2_sid_create (GtkWidget *parent, const gchar *name, gchar *icon)
 
 	if (
 #ifdef E2_VFSTMP
-		initial_dir.localpath != NULL
+		initial_dir.path != NULL
 #else
 		initial_dir != NULL
 #endif
@@ -670,16 +677,16 @@ GtkWidget *e2_sid_create (GtkWidget *parent, const gchar *name, gchar *icon)
 	{
 		//<Ctrl> press always opens default icons dir
 #ifdef E2_VFSTMP
-		g_free (initial_dir.localpath);
-		initial_dir.localpath = NULL;
+		g_free (initial_dir.path);
+		initial_dir.path = NULL;
 #else
 		g_free (initial_dir);
 		initial_dir = NULL;
 #endif
 	}
 #ifdef E2_VFSTMP
-	if (initial_dir.localpath == NULL)
-		initial_dir.localpath = _e2_sidlg_get_icon_dir (rt);
+	if (initial_dir.path == NULL)
+		initial_dir.path = _e2_sidlg_get_icon_dir (rt);
 //	CHECKME NULL initial_dir.spacedata is ok
 #else
 	if (initial_dir == NULL)
@@ -695,13 +702,7 @@ GtkWidget *e2_sid_create (GtkWidget *parent, const gchar *name, gchar *icon)
 	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (rt->dir_chooser), initial_dir);
 	g_signal_connect (G_OBJECT (rt->dir_chooser), "current-folder-changed",
 		G_CALLBACK (_e2_sidlg_dir_change_cb), rt);
-	gtk_widget_show (rt->dir_chooser);
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		rt->dir_chooser, message);
+	e2_widget_set_safetip (rt->dir_chooser, message);
 
 	//a notebook is used to change between the icon views
 	rt->notebook = GTK_NOTEBOOK (e2_widget_add_notebook
@@ -751,12 +752,7 @@ GtkWidget *e2_sid_create (GtkWidget *parent, const gchar *name, gchar *icon)
 	gtk_box_reorder_child (GTK_BOX (dialog_vbox), bbox, 0);
 
 	rt->rem_btn = e2_dialog_add_defined_button (rt->dialog, &E2_BUTTON_REMOVE);
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		rt->rem_btn, _("Remove the current icon"));
+	e2_widget_set_safetip (rt->rem_btn, _("Remove the current icon"));
 	if (*icon == '\0')	//nothing to remove at present
 		gtk_widget_set_sensitive (rt->rem_btn, FALSE);
 
