@@ -1,6 +1,6 @@
-/* $Id: e2_bookmark.c 1277 2008-09-27 01:10:48Z tpgww $
+/* $Id: e2_bookmark.c 1966 2009-10-08 02:28:27Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -30,7 +30,7 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include "e2_task.h"
 #include "e2_option_tree.h"
 
-static E2_OptionSet *set;
+static E2_OptionSet *bookmarks_set;
 
   /*****************/
  /***** utils *****/
@@ -38,6 +38,8 @@ static E2_OptionSet *set;
 
 /**
 @brief re-create all toolbars which have bookmarks
+Way too comlicated to create mechanism to replace only specific
+buttons and menus
 @return
 */
 static void _e2_bookmark_recreate_toolbars (void)
@@ -56,7 +58,7 @@ static void _e2_bookmark_recreate_toolbars (void)
 */
 static void _e2_bookmark_delete_mark (GtkTreeIter *iter)
 {
-	e2_option_tree_del_direct (set, iter);
+	e2_option_tree_del_direct (bookmarks_set, iter);
 	_e2_bookmark_recreate_toolbars ();
 	if (GTK_IS_MENU (app.context_menu))
 		gtk_menu_popdown (GTK_MENU (app.context_menu));
@@ -94,7 +96,7 @@ static gboolean _e2_bookmark_tree_selection_check_cb
 	{
 		gchar *label;
 		gtk_tree_model_get (set->ex.tree.model, &iter, 0, &label, -1);
-		gboolean result = !g_str_equal (label, _C(41));
+		gboolean result = strcmp (label, _C(41));
 		g_free (label);
 		GtkTreeView *view = gtk_tree_selection_get_tree_view (selection);
 		e2_option_tree_adjust_buttons (view, result);
@@ -118,7 +120,7 @@ static gboolean _e2_bookmark_visible_check_cb (GtkTreeModel *model,
 {
 	gchar *label;
 	gtk_tree_model_get (model, iter, 0, &label, -1);
-	gboolean result = !g_str_equal (label, _C(41));
+	gboolean result = strcmp (label, _C(41));
 	g_free (label);
 	return result;
 }
@@ -142,7 +144,7 @@ static gboolean _e2_bookmark_tree_draggable_check_cb
 	{
 		gchar *label;
 		gtk_tree_model_get (model, &iter, 0, &label, -1);
-		gboolean result = !g_str_equal (label, _C(41));
+		gboolean result = strcmp (label, _C(41));
 		g_free (label);
 		return result;
 	}
@@ -166,54 +168,73 @@ static void _e2_bookmark_confirm_response_cb (GtkDialog *dialog,
 			if (path != NULL)
 			{
 				GtkTreeIter iter;
-				if (gtk_tree_model_get_iter (set->ex.tree.model, &iter, path))
+				if (gtk_tree_model_get_iter (bookmarks_set->ex.tree.model, &iter, path))
 					_e2_bookmark_delete_mark (&iter);
+				gtk_tree_path_free (path);
 			}
 		}
 	}
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 /**
-@brief add an item (directory) to the bookmarks list
+@brief callback to add an item (directory) to the bookmarks list
 
-This is a callback. The activated item @a widget may be a menu
-item (eg in the panebar bookmarks sub-menu) or a context menu
-item (file pane or bookmarks widget).
-In the former case, it will have been processed as an action, and will
-have a string argument @a arg, with "pane1" or "pane2", and
-possibly including one of "top", "child" (othewise the add is treated
-as "after").
+The activated item @a widget may be a menu item (eg in the panebar bookmarks
+sub-menu) or a context menu item (file pane or bookmarks widget). In the former
+case, it will have been processed as an action, and will have a string argument
+@a arg, with "pane1" or "pane2" or a specific UTF-8 path string, and possibly
+including one of "top", "child" (othewise the add is treated as "after").
 When @a widget is a context menu item, @a arg will be NULL.
 This function needs to know which pane to use for the path to add to the list.
 For menu callbacks, @a arg needs to include "1" or "2" as appropriate. For
 context-menu callbacks, @a widget will have associated data "bookmark-pane" =
 0 (active), 1 or 2.
 
-@param widget NULL if a bookmarks menu item widget was selected, else a context menu item widget
+@param widget NULL if a bookmarks menu item widget was selected or running an
+ action directly, else a context menu item widget
 @param arg string which indicates where to put the new bookmark, or NULL
 @return
 */
-static void _e2_bookmark_add_cb (GtkWidget *widget, const gchar *arg)
+static void _e2_bookmark_add_cb (GtkMenuItem *widget, const gchar *arg)
 {
+	guint pane;
 	gboolean before = FALSE;
 	gboolean parent = FALSE;
-	guint pane = E2PANECUR;	//default to active pane
+	gchar *s = NULL;
+
 	if (arg != NULL && *arg != '\0')
 	{
-		if (strstr (arg, "1") != NULL)
-			pane = E2PANE1;
-		else if (strstr (arg, "2") != NULL)
-			pane = E2PANE2;
-		if (strstr (arg, _A(118)) != NULL)  //_(top
+		gchar *n;
+		if (strstr (arg, _A(119)) != NULL)  //_(top
 			before = TRUE;
-		if (strstr (arg, _A(108)) != NULL) //_(child
+		if (strstr (arg, _A(109)) != NULL) //_(child
 		{
-			parent = TRUE;
 			before = TRUE;
+			parent = TRUE;
+		}
+
+		if (strchr (arg, G_DIR_SEPARATOR) != NULL)
+			pane = 9999;	//arg has a path string
+		else if ((s = e2_utils_bare_strchr ((gchar *)arg, '1')) != NULL)
+		{
+			n = e2_utils_pass_whitespace (s+1);
+			pane = (n == NULL || *n == ',') ? E2PANE1 : 9999;
+		}
+		else if ((s = e2_utils_bare_strchr ((gchar *)arg, '2')) != NULL)
+		{
+			n = e2_utils_pass_whitespace (s+1);
+			pane = (n == NULL || *n == ',') ? E2PANE2 : 9999;
+		}
+		else
+		{
+			n = e2_utils_pass_whitespace ((gchar *)arg);
+			pane = (n == NULL) ? E2PANECUR : 9999;
 		}
 	}
 	else if (widget != NULL)
 		pane = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (widget), "bookmark-pane"));
+	else
+		pane = E2PANECUR;	//default to active pane
 
 	//get the relevant dir to add to the marks list
 	gchar *dirname;
@@ -223,10 +244,27 @@ static void _e2_bookmark_add_cb (GtkWidget *widget, const gchar *arg)
 	//FIXME dirs when not mounted local
 #else
 		case E2PANE1:
-			dirname = app.pane1_view.dir;
+			dirname = app.pane1.view.dir;
 			break;
 		case E2PANE2:
-			dirname = app.pane2_view.dir;
+			dirname = app.pane2.view.dir;
+			break;
+		case 9999:
+			if (before)
+			{
+				//find and handle extra parameter and maybe ','
+				//FIXME this is not robust enough
+			}
+			dirname = e2_utils_unquote_string (arg);
+			if (dirname == NULL)
+				return;
+			g_strchug (dirname); //path may validly have trailing whitespace
+			if (!g_path_is_absolute (dirname))
+			{
+				s = dirname;
+				dirname = e2_utils_strcat (curr_view->dir, s);
+				g_free (s);
+			}
 			break;
 		default:
 			dirname = curr_view->dir;
@@ -246,30 +284,38 @@ static void _e2_bookmark_add_cb (GtkWidget *widget, const gchar *arg)
 	options[0] = basename;	//menu label
 	options[1] = "";  //no icon
 	//strip the trailing / for paths other than root
-	gchar *s = NULL;
 	gint len = strlen (dirname);
 	if (len > 1)
 	{
-		s = g_strdup (dirname);	//non-NULL = freeme later
-		*(s+len-1) = '\0';
+		if (pane != 9999)
+		{
+			s =  g_strdup (dirname); //non-NULL = freeme later
+			*(s+len-1) = '\0';
+		}
+		else
+			s = dirname;
 		options[2] = s;
 	}
 	else
+	{
+		s = (pane != 9999) ? NULL : dirname; //non-NULL = freeme later
 		options[2] = dirname;  //tip = path
+	}
 	options[3] = options[2]; //path
 //	E2_OptionSet *set = e2_option_get ("bookmarks");  //internal name USE THE GLOBAL VALUE
 	GtkTreeIter iter;
 	GtkTreeIter iter2;
-	GtkTreePath *path = NULL;
-	if (widget != NULL)
-		path = g_object_get_data (G_OBJECT (widget), "parent-mark");
-	if ((path != NULL) && gtk_tree_model_get_iter (set->ex.tree.model, &iter2, path))
-		e2_option_tree_add (set, &iter, &iter2, !parent, before, 4, options);
+	GtkTreePath *path;
+	path = (widget != NULL) ?
+		g_object_get_data (G_OBJECT (widget), "parent-mark") : NULL;
+	if ((path != NULL) && gtk_tree_model_get_iter (bookmarks_set->ex.tree.model,
+			&iter2, path))
+		e2_option_tree_add (bookmarks_set, &iter, &iter2, !parent, before, 4, options);
 	else
 		//with no parent, the behaviour of "before" is vice versa
-		e2_option_tree_add (set, &iter, NULL, FALSE, !before, 4, options);
+		e2_option_tree_add (bookmarks_set, &iter, NULL, FALSE, !before, 4, options);
 
-	_e2_bookmark_recreate_toolbars ();
+	_e2_bookmark_recreate_toolbars (); //kludgy way to update bookmarks data in bars
 
 	if (s != NULL)
 		g_free (s);
@@ -281,18 +327,18 @@ static void _e2_bookmark_add_cb (GtkWidget *widget, const gchar *arg)
 @param path treestore path of bookmark to be deleted
 @return
 */
-static void _e2_bookmark_delete_cb (GtkWidget *widget, GtkTreePath *path)
+static void _e2_bookmark_delete_cb (GtkMenuItem *widget, GtkTreePath *path)
 {
 	GtkTreeIter iter;
-	if (gtk_tree_model_get_iter (set->ex.tree.model, &iter, path))
+	if (gtk_tree_model_get_iter (bookmarks_set->ex.tree.model, &iter, path))
 	{
-		gint children = gtk_tree_model_iter_n_children (set->ex.tree.model, &iter);
+		gint children = gtk_tree_model_iter_n_children (bookmarks_set->ex.tree.model, &iter);
 		if (e2_option_bool_get ("bookmarks-confirm-delete")
 			|| ((children > 0) && (e2_option_bool_get ("bookmarks-confirm-multi-delete")))
 		)
 		{
 			gchar *name;
-			gtk_tree_model_get (set->ex.tree.model, &iter, 0, &name, -1);
+			gtk_tree_model_get (bookmarks_set->ex.tree.model, &iter, 0, &name, -1);
 
 			gchar *question;
 			gchar *prompt = _("Are you sure that you want to delete the bookmark");
@@ -304,8 +350,9 @@ static void _e2_bookmark_delete_cb (GtkWidget *widget, GtkTreePath *path)
 				question = g_strdup_printf ("%s '<b>%s</b>' ?", prompt, name);
 
 			GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION, question,
-				_("confirm bookmark delete"), _e2_bookmark_confirm_response_cb,
-				gtk_tree_row_reference_new (set->ex.tree.model, path));
+				_("confirm bookmark delete"),
+				(ResponseFunc)_e2_bookmark_confirm_response_cb,
+				gtk_tree_row_reference_new (bookmarks_set->ex.tree.model, path));
 			e2_dialog_set_negative_response (dialog, GTK_RESPONSE_NO);
 
 			E2_Button no_btn;
@@ -323,12 +370,12 @@ static void _e2_bookmark_delete_cb (GtkWidget *widget, GtkTreePath *path)
 	}
 }
 /**
-@brief button-press-signal callback
+@brief "button-press-event" signal callback
 
-Middle button will open mark in other pane, and make that pane active, if the
-relevant options are in force.
-Right button will create and pop up a context menu, if not blocked (blocks will
-be in place for toolbar buttons, but not for (sub)menu items)
+Unmodified middle button will open mark in other pane, and make that pane active,
+if the relevant options are in force.
+Unmodified right button will create and pop up a destroyable context menu, if not
+blocked (blocks will be in place for toolbar buttons, but not for (sub)menu items)
 Context menu items 'add' and 'add-child' need to know which pane (directory) to
 use for the mark
 
@@ -379,8 +426,7 @@ gboolean e2_bookmark_button_press_cb (GtkWidget *widget, GdkEventButton *event,
 				return FALSE;
 			printd (DEBUG, "bookmark menu");
 			GtkWidget *menu = gtk_menu_new ();
-			GtkWidget *menuitem;
-			menuitem = e2_menu_add (menu, _("_Add after"), GTK_STOCK_ADD,
+			GtkWidget *menuitem = e2_menu_add (menu, _("_Add after"), GTK_STOCK_ADD,
 				_("Bookmark the current directory after the selected bookmark"),
 				_e2_bookmark_add_cb, NULL);
 			g_object_set_data (G_OBJECT (menuitem), "parent-mark", path);
@@ -389,7 +435,7 @@ gboolean e2_bookmark_button_press_cb (GtkWidget *widget, GdkEventButton *event,
 
 			menuitem = e2_menu_add (menu, _("Add as _child"), GTK_STOCK_INDENT,
 				_("Bookmark the current directory a a child of the selected bookmark"),
-				_e2_bookmark_add_cb, _A(108));	//_(child
+				_e2_bookmark_add_cb, _A(109));	//_(child
 			g_object_set_data (G_OBJECT (menuitem), "parent-mark", path);
 			//pass through the pane no. (or NULL for default) to use when adding an item
 			g_object_set_data (G_OBJECT (menuitem), "bookmark-pane", pane);
@@ -434,7 +480,40 @@ static gboolean _e2_bookmark_add (gpointer from, E2_ActionRuntime *art)
 	return TRUE;
 }
 /**
-@brief show bookmarks menu in specified pane
+@brief remove item from bookmarks menu
+@param from pointer to widget activated to initiate the action
+@param art pointer to runtime data for the action
+@return TRUE if the item was removed
+*/
+static gboolean _e2_bookmark_delete (gpointer from, E2_ActionRuntime *art)
+{
+	gchar *arg = (gchar *) art->data;
+//	if (arg == NULL) arg = curr_viewe->dir without trailing /
+	if (arg != NULL) //&& *arg != '\0')
+	{
+		//CHECKME handle "1" or "2" for pane dir ?
+		gboolean retval = FALSE;
+		gchar *clean = e2_utils_unquote_string (arg);
+		if (clean == NULL)
+			clean = arg;
+		GtkTreeIter iter;
+		while (e2_tree_find_iter_from_str (bookmarks_set->ex.tree.model, 3,
+				clean, &iter, FALSE))
+		{
+			GtkTreePath *tpath;
+			tpath = gtk_tree_model_get_path (bookmarks_set->ex.tree.model, &iter);
+			_e2_bookmark_delete_cb (NULL, tpath);
+			gtk_tree_path_free (tpath);
+			retval = TRUE;
+		}
+		if (clean != arg)
+			g_free (clean);
+		return retval;
+	}
+	return FALSE;
+}
+/**
+@brief construct and pop up destroyable bookmarks-menu, in specified pane
 @param from pointer to widget activated to initiate the action
 @param art pointer to runtime data for the action
 @return TRUE if menu was created
@@ -458,11 +537,13 @@ static gboolean _e2_bookmark_show (gpointer from, E2_ActionRuntime *art)
 		E2_Action *action = e2_action_get (action_name);
 		g_free (action_name);
 		GtkWidget *menu = gtk_menu_new ();
-		e2_menu_create_bookmarks_menu (action, pane, set, menu, &iter);
+		e2_menu_add_bookmark_items (menu, TRUE, action, pane, set->ex.tree.model, &iter);
+		g_signal_connect (G_OBJECT(menu), "selection-done",
+			G_CALLBACK (e2_menu_selection_done_cb), NULL);
 		guint32 time = gtk_get_current_event_time ();
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
 			(GtkMenuPositionFunc) e2_fileview_set_menu_position,
-			rt->view->treeview, 0, time);
+			rt->view.treeview, 0, time);
 		return TRUE;
 	}
 	return FALSE;
@@ -474,26 +555,30 @@ Action data includes path of directory to open, utf8 string
 Downstream functions expect BGL to be closed
 @param from pointer to widget activated to initiate the action
 @param art pointer to runtime data for the action
-@return TRUE always
+@return TRUE if the cd is initiated
 */
-gboolean e2_bookmark_open (gpointer from, E2_ActionRuntime *art)
+static gboolean _e2_bookmark_open (gpointer from, E2_ActionRuntime *art)
 {
 	const gchar *newpath = NULL;
 	E2_PaneRuntime *rt = e2_pane_get_runtime (from, art->data, &newpath);
 	if (rt == NULL || newpath == NULL)
 		return FALSE;
-	printd (DEBUG, "e2_bookmark_open (num:,path:%s)", newpath);
+	printd (DEBUG, "_e2_bookmark_open (path:%s)", newpath);
+	//bookmark may be bad now ...
+	if (e2_fs_cd_isok ((gchar *)newpath E2_ERR_NONE()))
+	{
 #ifdef E2_VFSTMP
-	//FIXME handle case where pane is non-mounted
-	if (0)
-		e2_pane_change_space_byuri (rt, "someplace");
+		//FIXME handle case where pane is non-mounted
+		if (0)
+			e2_pane_change_space_byuri (rt, "someplace");
 #endif
-	e2_pane_change_dir (rt, newpath);
-	if (rt != curr_pane && e2_option_bool_get ("bookmarks-focus-after-open"))
-		e2_pane_activate_other ();
-	return TRUE;
+		e2_pane_change_dir (rt, newpath);
+		if (rt != curr_pane && e2_option_bool_get ("bookmarks-focus-after-open"))
+			e2_pane_activate_other ();
+		return TRUE;
+	}
+	return FALSE;
 }
-
 /**
 @brief open a dialog showing bookmarks config data
 @param from pointer to widget activated to initiate the action
@@ -522,18 +607,17 @@ void e2_bookmark_clean ()
 */
 void e2_bookmark_actions_register (void)
 {
-	gchar *action_name = g_strconcat(_A(13),".",_A(24),NULL); //pane.<bookmarks>
-		e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
-		_e2_bookmark_show, NULL, TRUE);
-	action_name = g_strconcat(_A(0),".",_A(30),NULL); //bookmark.add
-		e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
-		_e2_bookmark_add, NULL, TRUE);
-	action_name = g_strconcat (_A(0),".",_A(27),NULL); //bookmark.<menu>
-	e2_action_register (action_name, E2_ACTION_TYPE_BOOKMARKS,
-		e2_bookmark_open, NULL, TRUE, E2_ACTION_EXCLUDE_ACCEL, NULL);
-	action_name = g_strconcat(_A(3),".",_C(1),NULL);
-	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
-		_e2_bookmarks_configure, NULL, FALSE);
+	E2_Action actions[] =
+	{
+	{g_strconcat(_A(0),".",_A(27),NULL), _e2_bookmark_open,      TRUE, E2_ACTION_TYPE_BOOKMARKS, E2_ACTION_EXCLUDE_ACCEL, NULL, NULL},
+	{g_strconcat(_A(0),".",_A(30),NULL), _e2_bookmark_add,       TRUE, E2_ACTION_TYPE_ITEM, 0, NULL, NULL},
+	{g_strconcat(_A(0),".",_A(42),NULL), _e2_bookmark_delete,    TRUE, E2_ACTION_TYPE_ITEM, 0, NULL, NULL},
+	{g_strconcat(_A(13),".",_A(24),NULL),_e2_bookmark_show,      TRUE, E2_ACTION_TYPE_ITEM, 0, NULL, NULL},
+	{g_strconcat(_A(3),".",_C(1),NULL),  _e2_bookmarks_configure,FALSE,E2_ACTION_TYPE_ITEM, 0, NULL, NULL},
+	};
+	guint i, count = sizeof (actions)/sizeof (E2_Action);
+	for (i = 0; i < count; i++)
+		e2_action_register (&actions[i]);
 }
 /**
 @brief install default tree options for boookmarks
@@ -572,21 +656,21 @@ void e2_bookmark_options_register (void)
 {
 //no screen rebuilds needed after any change to these options
 //	E2_OptionSet * SET THE GLOBAL VALUE
-	set = e2_option_tree_register ("bookmarks", _C(1), _C(1),  //_("bookmarks"
+	bookmarks_set = e2_option_tree_register ("bookmarks", _C(1), _C(1),  //_("bookmarks"
 		NULL, _e2_bookmark_tree_selection_check_cb, _e2_bookmark_tree_draggable_check_cb,
 		E2_OPTION_TREE_UP_DOWN | E2_OPTION_TREE_ADD_DEL,
-		E2_OPTION_FLAG_BASIC);
-	e2_option_tree_add_column (set, _("Label"), E2_OPTION_TREE_TYPE_STR, 0, G_DIR_SEPARATOR_S,
+		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_BUILDBARS); //toolbar(s) typically include static marks-related items
+	e2_option_tree_add_column (bookmarks_set, _("Label"), E2_OPTION_TREE_TYPE_STR, 0, G_DIR_SEPARATOR_S,
 		0, NULL, NULL);
-	e2_option_tree_add_column (set, _("Icon"), E2_OPTION_TREE_TYPE_ICON, 0, "",
+	e2_option_tree_add_column (bookmarks_set, _("Icon"), E2_OPTION_TREE_TYPE_ICON, 0, "",
 		0, _e2_bookmark_visible_check_cb, NULL);
-	e2_option_tree_add_column (set, _("Tooltip"), E2_OPTION_TREE_TYPE_STR, 0, "",
+	e2_option_tree_add_column (bookmarks_set, _("Tooltip"), E2_OPTION_TREE_TYPE_STR, 0, "",
 		0, _e2_bookmark_visible_check_cb, NULL);
-	e2_option_tree_add_column (set, _("Path"), E2_OPTION_TREE_TYPE_STR, 0, G_DIR_SEPARATOR_S,
+	e2_option_tree_add_column (bookmarks_set, _("Path"), E2_OPTION_TREE_TYPE_STR, 0, G_DIR_SEPARATOR_S,
 		0, _e2_bookmark_visible_check_cb, NULL);
-	e2_option_tree_create_store (set);
+	e2_option_tree_create_store (bookmarks_set);
 
-	e2_option_tree_prepare_defaults (set, _e2_bookmark_tree_defaults);
+	e2_option_tree_prepare_defaults (bookmarks_set, _e2_bookmark_tree_defaults);
 
 	gchar* group_name = g_strconcat(_C(1),".",_C(27),":",_C(26),NULL);  //_("bookmarks.options:miscellaneous"
 	e2_option_bool_register ("bookmarks-button2-other",
