@@ -1,6 +1,6 @@
-/* $Id: e2_keybinding.c 1421 2009-02-09 22:25:29Z tpgww $
+/* $Id: e2_keybinding.c 2033 2009-12-21 02:36:51Z tpgww $
 
-Copyright (C) 2004-2009 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -133,6 +133,99 @@ GNode *keybindings = NULL;
  /***** utils *****/
 /*****************/
 
+/**
+@brief translate any alphabetic keycode in the string @a original
+app.keyslocal need not be pre-created, but no checking here for NULL or empty
+@a original, faster to do that upstream
+@param original keystring in format that gtk understands, like <Control>a
+@return newly allocated replacement-keystring, or NULL if no change is needed
+*/
+static gchar *_e2_keybinding_translate_keycode (const gchar *original)
+{
+	const gchar *keystring, *translated;
+	gchar *replace = NULL;
+
+	if (app.keyslocal == NULL)
+		e2_utils_translate_keys (FALSE, TRUE);
+
+	keystring = strrchr (original, '>');
+	if (keystring == NULL)
+	{
+		if (*(original+1) == '\0') //only interested in single-letter key-names
+		{
+			translated = (const gchar *)g_hash_table_lookup (app.keyslocal, original);
+			if (!(translated == NULL || *translated == '\0'
+				 || !strcmp (translated, original)))
+				replace = g_strdup (translated);
+		}
+	}
+	else
+	{
+		keystring++;
+		if (*(keystring+1) == '\0') //only interested in single-letter key-names
+		{
+			translated = (const gchar *)g_hash_table_lookup (app.keyslocal, keystring);
+			if (!(translated == NULL || *translated == '\0'
+				|| !strcmp (translated, keystring)))
+			{
+				gchar *prefix;
+				prefix = g_strndup (original, keystring - original);
+				replace = g_strconcat (prefix, translated, NULL);
+				g_free (prefix);
+			}
+		}
+	}
+	return replace;
+}
+/**
+@brief recursively walk all or part of the keybindings store to localise all keycodes
+@param mdl pointer to bindings store treemodel
+@param iter pointer to iter to use for interrogating @a mdl
+
+@return
+*/
+static void _e2_keystring_walk (GtkTreeModel *mdl, GtkTreeIter *iter)
+{
+	gchar *keystring;
+	GtkTreeIter child_iter;
+	gtk_tree_model_get (mdl, iter, 1, &keystring, -1);
+	if (keystring != NULL && *keystring != '\0')
+	{
+		gchar *replace = _e2_keybinding_translate_keycode (keystring);
+		if (replace != NULL)
+		{
+			gtk_list_store_set (GTK_LIST_STORE(mdl), iter, 1, replace, -1);
+			g_free (replace);
+		}
+	}
+	g_free (keystring);
+	if (gtk_tree_model_iter_children (mdl, &child_iter, iter))
+	{
+		do
+		{
+			_e2_keystring_walk (mdl, &child_iter);
+		} while (gtk_tree_model_iter_next (mdl, &child_iter));
+	}
+}
+/**
+@brief convert all (if any) stored alphabetic key-bindings to localised form
+
+@param mdl pointer to keybindings store treemodel
+@param iter pointer to a toplevel iter, to use for interrogating @a mdl
+
+@return
+*/
+void e2_keybinding_localise (GtkTreeModel *mdl, GtkTreeIter *iter)
+{
+	printd (DEBUG, "localise all key bindings or for a category");
+	if (gtk_tree_model_get_iter_first (mdl, iter))
+	{
+		do
+		{
+			_e2_keystring_walk (mdl, iter);
+		} while (gtk_tree_model_iter_next(mdl, iter));
+	}
+}
 /**
 @brief cleanup bound-keys data in list @a keys
 Recurses to clean chained keys, if any
@@ -173,7 +266,8 @@ static void _e2_keybinding_free_category_keys (E2_KeyCatRuntime *rt)
 }
 /**
 @brief recursively translate a keybinding treestore branch into a listed struct
-Some cute stuff is done to conform keys-cell-renderer behaviour with gtk's
+Stored mod-letter-keycodes are translated back to asciified equivalents,
+if appropriate, to conform to the same sort of translation by UI widgets.
 @param model model for the bindings treestore
 @param iter pointer to iter to be used to interrogage @a model
 @param keys pointer to initialised list to which the created struct(s) will be appended
@@ -200,9 +294,12 @@ static void _e2_keybinding_sync_one (GtkTreeModel *model, GtkTreeIter *iter, GSL
 	if (valid)
 	{
 		k = ALLOCATE (E2_KeyRuntime);
+#if (CHECKALLOCATEDWARN)
 		CHECKALLOCATEDWARN (k, valid = FALSE;)
+#else
 		if (k == NULL)
 			valid = FALSE;
+#endif
 	}
 	else
 		k = NULL; //warning prevention
@@ -220,19 +317,28 @@ static void _e2_keybinding_sync_one (GtkTreeModel *model, GtkTreeIter *iter, GSL
 	gtk_accelerator_parse (key, &keyval, &mask);
 	if (keyval != 0)
 	{
-		/*for letters, when a <Shift> is pressed, or after <Lock>,
-		gtk actually returns the upper-case code, = lower-case - 0x20
-		if <Shift> , there will also be GDK_SHIFT_MASK
-		but e2's config widget always provides the <Shift>lower-case pair FIXED
-		we need to work around this mismatch */
 //		printd (DEBUG, "key - %s - %d - %d", key, keyval, mask);
-//		if (keyval >= GDK_a && keyval <= GDK_z && mask == GDK_SHIFT_MASK)
-//			keyval -= 0x20;
-		if (mask == 0 && (keyval < 0xF000 || keyval > 0xFFFF))
+		if (keyval < 0xF000 || keyval > 0xFFFF)
 		{
-			gchar *realkey = gdk_keyval_name (keyval);
-			if (!g_str_equal (key, realkey))
-				keyval = gdk_keyval_to_upper (keyval);
+			if (mask == 0)
+			{
+				/*for letters, gtk event-data includes the upper-case keycode
+				when relevant, but the conversion above got a lower-case one.
+				We need to fix that */
+				const gchar *realkey = gdk_keyval_name (keyval);
+				if (strcmp (key, realkey))
+					keyval = gdk_keyval_to_upper (keyval);
+			}
+			else	//modified
+			{
+				/* via e2_utils_translate_key_event(), UI widgets do translation
+				from 'locale'-specific mod-letter-codes to ascii-equivalent,
+				before processing. We need to conform the bindings to that */
+				guint asciicode = GPOINTER_TO_UINT (g_hash_table_lookup (app.keysnative,
+					GUINT_TO_POINTER (keyval)));
+				if (asciicode != 0)
+					keyval = asciicode;
+			}
 		}
 		k->keyval = keyval;
 		k->mask = mask;
@@ -424,19 +530,34 @@ This func is mainloop idle callback CHECKME why sync that way?
 */
 static gboolean _e2_keybinding_idle_sync_cb (E2_KeyCatRuntime *data)
 {
+//	GtkTreeIter iter;
+
+//	E2_OptionSet *set = e2_option_get ("keybindings");
+//	GtkTreeModel *mdl = set->ex.tree.model;
+
 	if (data != NULL)
-		//sync a single KeyCatRuntime
+	{
+/*		//sync a single KeyCatRuntime
+		if (app.keytrans)
+		{
+			//translate this part of store before any usage
+			GtkTreePath *tp = gtk_tree_row_reference_get_path (data->ref);
+			if (tp != NULL)
+			{
+				gtk_tree_model_get_iter (mdl, &iter, tp);
+				gtk_tree_path_free (tp);
+				e2_keybinding_localise (mdl, &iter);
+			}
+		}
+*/
 		_e2_keybinding_sync_category (data);
+	}
 	else
 	{
-/*		if (binding_cur == NULL)	//after all bindings (or none) have been synced ...
-			binding_cur = keybindings;	//start syncing from beginning of list
-		if (binding_cur != NULL)
+/*		if (app.keytrans)
 		{
-			//sync a single KeybindingRuntime
-			e2_keybinding_sync_category ((E2_KeyCatRuntime *) binding_cur->data);
-			//setup pointer for next idle callback
-			binding_cur = binding_cur->next;
+			gtk_tree_model_get_iter_first (mdl, &iter);
+			e2_keybinding_localise (mdl, &iter);	//translate all store before any usage
 		}
 */
 		g_node_traverse (keybindings, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
@@ -559,6 +680,8 @@ void _e2_keybinding_destroy_chain_idle (gpointer timeout_id)
 
 /**
 @brief find and run the action that's bound to a key
+Due to the order of callback connection, the key-data in @a event will not yet
+have been de-localised by e2_utils_translate_key_event() if that's relevant.
 @param widget a widget bound to the category represented by @a rt
 @param event data struct for the event
 @param user_data UNUSED data specified when cb was connected
@@ -575,7 +698,7 @@ static gboolean _e2_keybinding_key_press_cb (GtkWidget *widget,
 	if (e2_utils_key_is_modifier (event))
 #endif
 		return FALSE;
-//	printd (DEBUG,"key binding cb begins");
+//	printd (DEBUG,"bindings key-press cb begins, key %u", event->keyval);
 	gboolean retval = FALSE;
 	guint modifiers = gtk_accelerator_get_default_mod_mask ();
 	modifiers = event->state & modifiers;
@@ -687,7 +810,14 @@ static gboolean _e2_keybinding_key_press_cb (GtkWidget *widget,
 			}
 		}
 	}
-//	printd (DEBUG,"key binding cb ends, returned value = %s", (retval) ? "TRUE":"FALSE");
+/*	if (retval)
+	{
+CHECKME	any normal press-callback won't happen i.e. no delocalisation of the event's
+		key data, so what happens when a corresponding release-callback does a delocalisation ?
+??		e2_utils_translate_key_event (event);
+	}
+*/
+//	printd (DEBUG,"bindings key-press cb ends, returned value = %s", (retval) ? "TRUE":"FALSE");
 	return retval;
 }
 
@@ -702,13 +832,16 @@ static gboolean _e2_keybinding_key_press_cb (GtkWidget *widget,
 */
 void e2_keybinding_register_all (void)
 {
-	e2_keybinding_enrol (NULL, _C(17), NULL);
+	//there's no generic press-handler for the whole window, so no need to muck
+	//about with the order of callback connections
+	//CHECKME specific binding cb for app.main_window ?
+	e2_keybinding_enrol (app.main_window, _C(17), (void(*)(E2_OptionSet*))NULL);
 	gchar *category = g_strconcat(_C(17),".",_C(23),NULL);  //_("general.main
-	e2_keybinding_enrol (NULL, category, NULL);
+	e2_keybinding_enrol (app.main_window, category, (void(*)(E2_OptionSet*))NULL);
 	g_free (category);
 
-	e2_fileview_register_keybindings (app.pane1_view.treeview, &app.pane1_view);
-	e2_fileview_register_keybindings (app.pane2_view.treeview, &app.pane2_view);
+	e2_fileview_register_keybindings (app.pane1.view.treeview, &app.pane1.view);
+	e2_fileview_register_keybindings (app.pane2.view.treeview, &app.pane2.view);
 
 	GList *member;
 	for (member = app.command_lines; member != NULL; member = g_list_next (member))
@@ -716,7 +849,7 @@ void e2_keybinding_register_all (void)
 		E2_CommandLineRuntime *rt = (E2_CommandLineRuntime *) member->data;
 		e2_command_line_register_keybindings (rt);
 	}
-
+	//tab bindings are also set when the tab is created - so that new ones are covered
 	for (member = app.tabslist; member != NULL ; member = g_list_next (member))
 	{
 		GtkTextView *tvw = ((E2_OutputTabRuntime *)member->data)->text;
@@ -766,11 +899,15 @@ transient item that uses the category)
 @return
 */
 void e2_keybinding_enrol (GtkWidget *widget, const gchar *category,
-	gpointer defaults_func)
+	void (*defaults_func)(E2_OptionSet*))
 {
 	GNode *member = _e2_keybinding_get_category (category);
 	if (member == NULL)
+	{
+		printd (WARN, "e2_keybinding_enrol: category %s not found", category);
 		return;
+	}
+
 	E2_KeyCatRuntime *rt = (E2_KeyCatRuntime *)member->data;
 	gboolean first = (rt->ref == NULL);	//first-time usage of category
 	if (first)
@@ -783,19 +920,17 @@ void e2_keybinding_enrol (GtkWidget *widget, const gchar *category,
 #endif
 		e2_tree_get_lowest_iter_for_str (set->ex.tree.model, 0, &iter, category);
 		//the model may change between registrations, so remember where this cat starts
-		GtkTreePath *path = gtk_tree_model_get_path (set->ex.tree.model, &iter);
-		rt->ref = gtk_tree_row_reference_new (set->ex.tree.model, path);
-		gtk_tree_path_free (path);
+		GtkTreePath *tp = gtk_tree_model_get_path (set->ex.tree.model, &iter);
+		rt->ref = gtk_tree_row_reference_new (set->ex.tree.model, tp);
 #ifdef E2_TRANSIENTBINDINGS
-		if (defaults_func != NULL && !old)
+		if (defaults_func != (void(*)(E2_OptionSet*))NULL && !old)
 		{
 			//run the defaults-install function, which in turn calls
 			//e2_option_tree_setup_defaults() which sets up a pointer array at
-			//dummy_set.ex.tree.def
-			void (*install_func) (E2_OptionSet *) = defaults_func;
+			//dummy_set.ex.tree.def.tree_strings
 			E2_OptionSet dummyset;
-			(*install_func) (&dummyset);
-			gchar **split = (gchar **) dummyset.ex.tree.def;
+			(*defaults_func) (&dummyset);
+			gchar **split = dummyset.ex.tree.def.tree_strings;
 
 			//adjust start-index of parsed array
 			const gchar *p = strrchr (category, '.');
@@ -815,6 +950,12 @@ void e2_keybinding_enrol (GtkWidget *widget, const gchar *category,
 		printd (DEBUG, "arranged keybinding idle-sync for %s", category);
 #else
 		//translate the option tree data to rt form, now
+		//update store translation before any usage
+		gtk_tree_model_get_iter (set->ex.tree.model, &iter, tp); //back to where we started
+		gtk_tree_path_free (tp);
+/*		if (app.keytrans)
+			e2_keybinding_localise (set->ex.tree.model, &iter);
+*/
 		_e2_keybinding_sync_category (rt);
 #endif
 	}
@@ -827,6 +968,7 @@ void e2_keybinding_enrol (GtkWidget *widget, const gchar *category,
 		if (boundto == NULL)	//this is the first time widget is enrolled
 		{
 			//the model is: just one cb for all bindings
+			//CHECKME how is main-window handled ?
 			g_signal_connect (G_OBJECT (widget), "key-press-event",
 				G_CALLBACK (_e2_keybinding_key_press_cb), NULL);
 
@@ -886,16 +1028,21 @@ static gboolean _e2_keybinding_walk_unregister (GNode *node, E2_NodeData *data)
 		{
 			if (data->widget == widgets->data || data->widget == NULL)
 			{
-	printd (DEBUG, "UNregister widget");
-				//in case widget isn't to be destroyed
-				g_signal_handlers_disconnect_matched (widgets->data,
-					G_SIGNAL_MATCH_FUNC, 0, 0, NULL, _e2_keybinding_key_press_cb, NULL);
-				gpointer boundto = g_object_get_data (G_OBJECT (widgets->data),
-						KEY_BINDINGS_KEY);
-				if (boundto != NULL)
+				if (GTK_IS_WIDGET(widgets->data))
 				{
-					while (g_ptr_array_remove_fast ((GPtrArray *)boundto, rt)); //probably only one
+					//in case widget isn't to be destroyed
+					g_signal_handlers_disconnect_matched (widgets->data,
+						G_SIGNAL_MATCH_FUNC, 0, 0, NULL, _e2_keybinding_key_press_cb, NULL);
+					printd (DEBUG, "UNregister keybinding for widget");
+					gpointer boundto = g_object_get_data (G_OBJECT (widgets->data),
+							KEY_BINDINGS_KEY);
+					if (boundto != NULL)
+					{
+						while (g_ptr_array_remove_fast ((GPtrArray *)boundto, rt)); //probably only one
+					}
 				}
+				else
+					printd (DEBUG, "No UNregister key-binding for destroyed widget");
 				widgets->data = NULL;	//don't remove, list pointer cannot change yet
 			}
 		}
@@ -1322,9 +1469,9 @@ static gboolean _e2_keybinding_issue (gpointer from, E2_ActionRuntime *art)
 */
 void e2_keybinding_actions_register (void)
 {
-	gchar *action_name = g_strconcat(_A(123),".",_A(124),NULL);	//_("key.fake"
-	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
-		_e2_keybinding_issue, NULL, TRUE);
+	E2_Action action =
+	{g_strconcat(_A(124),".",_A(125),NULL),_e2_keybinding_issue,TRUE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+	e2_action_register (&action);
 }
 #endif	//def WITH_KEYFAKE
 /**
@@ -1352,8 +1499,8 @@ static void _e2_keybinding_tree_defaults (E2_OptionSet *set)
 	g_strconcat("\t\t|<Control>F1||",_A(15),".",_A(58),"|",NULL), //"pending.list
 	g_strconcat("\t\t|<Alt>F1||",_A(2),".",_A(58),"|",NULL), //"children.list
 	g_strconcat("\t\t|F2||",_A(6),".",_A(76),"|",NULL), //"rename"
-	g_strconcat("\t\t|F3||",_A(6),".",_A(105),"|",NULL), //"view"
-	g_strconcat("\t\t|<Shift>F3||",_A(6),".",_A(106),"|",NULL), //"view_again"
+	g_strconcat("\t\t|F3||",_A(6),".",_A(106),"|",NULL), //"view"
+	g_strconcat("\t\t|<Shift>F3||",_A(6),".",_A(107),"|",NULL), //"view_again"
 	g_strconcat("\t\t|<Control>F3||",_A(6),".",_A(46),"|",NULL), //"find" (was F4)
 	g_strconcat("\t\t|F4||",_A(6),".",_A(43),"|",NULL), //"edit"
 	g_strconcat("\t\t|<Shift>F4||",_A(6),".",_A(44),"|",NULL), //"edit_again"
@@ -1369,15 +1516,15 @@ static void _e2_keybinding_tree_defaults (E2_OptionSet *set)
 	g_strconcat("\t\t|<Shift><Control>F8||",_A(6),".",_A(42),"|",NULL), //"delete"
 	g_strconcat("\t\t|F9||",_A(6),".",_A(55),"|",NULL), //"file.info"
 #ifdef E2_TREEDIALOG
-	g_strconcat("\t\t|<Shift>F9||",_A(13),".",_A(102),"|",NULL), //"pane_active.tree" popup directories tree dialog
+	g_strconcat("\t\t|<Shift>F9||",_A(13),".",_A(103),"|",NULL), //"pane_active.tree" popup directories tree dialog
 #endif
 #ifdef E2_VFS
-	g_strconcat("\t\t|<Control>F9||",_A(4),".",_A(121),"|",NULL), //"dialog.namespace"
-	g_strconcat("\t\t|<Alt>F9||",_A(13),".",_A(119),"|",NULL), //"pane.<spacemenu>"
+	g_strconcat("\t\t|<Control>F9||",_A(4),".",_A(122),"|",NULL), //"dialog.namespace"
+	g_strconcat("\t\t|<Alt>F9||",_A(13),".",_A(120),"|",NULL), //"pane.<spacemenu>"
 #endif
 	g_strconcat("\t\t|<Control>F11||",_A(10),".",_A(30),"|",NULL), //"output.add
 	g_strconcat("\t\t|<Shift><Control>F11||",_A(10),".",_A(42),"|",NULL), //"output.delete
-	g_strconcat("\t\t|<Shift>F12||",_A(14),".",_A(98),"|",NULL), //"panes.toggle direction"
+	g_strconcat("\t\t|<Shift>F12||",_A(14),".",_A(99),"|",NULL), //"panes.toggle direction"
 //	_I( single letters in the following not worth translating ?
 	//conform this help string to the following 'jump-keys'
 	g_strconcat("\t\t|<Control>g||",_A(10),".",_A(71),"|",_("Now press one of h,m,d\\n"),NULL), //"print"
@@ -1385,19 +1532,19 @@ static void _e2_keybinding_tree_defaults (E2_OptionSet *set)
 	g_strconcat("\t\t\t|h||cd|$HOME",NULL), //_A(20)
 	g_strconcat("\t\t\t|m||cd|/mnt",NULL),  //_A(20)
 	g_strconcat("\t\t\t|d||cd|$HOME/downloads",NULL),  //_A(20)
-	g_strconcat("\t\t|<Control>w||",_A(10),".",_A(31),"|*,1",NULL),  //"adjust ratio" no arg translation
-	g_strconcat("\t\t|<Control>e||",_A(10),".",_A(31),"|*,0",NULL),    //"adjust ratio" no arg translation
-	g_strconcat("\t\t|<Control>z||",_A(1),".",_A(99),"|",NULL), //"focus toggle"
+	g_strconcat("\t\t|<Control>w||!",_A(10),".",_A(31),"|1,*",NULL),  //"adjust_ratio" no arg translation NOTE blocked * expansion
+	g_strconcat("\t\t|<Control>e||!",_A(10),".",_A(31),"|0,*",NULL),  //"adjust_ratio" no arg translation ditto
+	g_strconcat("\t\t|<Control>z||",_A(1),".",_A(100),"|",NULL), //"focus toggle"
 	g_strconcat("\t\t|<Control>r||",_A(14),".",_A(73),"|",NULL), //"refresh"
-	g_strconcat("\t\t|<Control>1||",_A(5),".",_A(99),"|1",NULL), //"focus toggle" no arg translation
-	g_strconcat("\t\t|<Control>2||",_A(5),".",_A(99),"|2",NULL), //"focus toggle"no arg translation
+	g_strconcat("\t\t|<Control>1||",_A(5),".",_A(100),"|1",NULL), //"focus toggle" no arg translation
+	g_strconcat("\t\t|<Control>2||",_A(5),".",_A(100),"|2",NULL), //"focus toggle"no arg translation
 	g_strconcat("\t\t|<Control>Insert||",_A(1),".",_A(56),"|",NULL), //"insert selection"
-	g_strconcat("\t\t|<Alt>Insert||",_A(1),".",_A(56),"|",_A(116),NULL), //"insert selection, quote"
+	g_strconcat("\t\t|<Alt>Insert||",_A(1),".",_A(56),"|",_A(117),NULL), //"insert selection, quote"
 	g_strconcat("\t\t",_C(33),"||||",NULL),  //_("panes"
 	//note "Space" not recognised, must be "space"
-//	g_strconcat("\t\t\t|space||",_A(14),".",_A(94),"|",NULL), //"switch"  something else makes this 'open' the selected item
-	g_strconcat("\t\t\t|space||",_A(7),".",_A(101),"|",NULL), //"toggle selected"
-	g_strconcat("\t\t\t|Tab||",_A(14),".",_A(94),"|",NULL), //"switch"
+//	g_strconcat("\t\t\t|space||",_A(14),".",_A(95),"|",NULL), //"switch"  something else makes this 'open' the selected item
+	g_strconcat("\t\t\t|space||",_A(7),".",_A(102),"|",NULL), //"toggle selected"
+	g_strconcat("\t\t\t|Tab||",_A(14),".",_A(95),"|",NULL), //"switch"
 	g_strconcat("\t\t\t|Return||",_A(6),".",_A(64),"|",NULL), //"open"
 	g_strconcat("\t\t\t|<Shift>Return||",_A(6),".",_A(65),"|",NULL), //"open in other"
 	g_strconcat("\t\t\t|<Control>Return|||",NULL), //nothing
@@ -1411,31 +1558,32 @@ static void _e2_keybinding_tree_defaults (E2_OptionSet *set)
 	g_strconcat("\t\t\t|Delete||",_A(6),".",_A(42),"|",NULL), //"delete"
 	g_strconcat("\t\t\t|<Shift>Delete||",_A(18),".",_A(42),"|",NULL), //"trash.delete"
 //	g_strconcat("\t\t\t|Home||cd|$HOME",NULL),   //_A(20)
-	g_strconcat("\t\t\t|<Control>a||",_A(7),".",_A(100),"|",NULL), //"toggle select all"
+	g_strconcat("\t\t\t|<Control>a||",_A(7),".",_A(101),"|",NULL), //"toggle select all"
 	g_strconcat("\t\t\t|<Control>i||",_A(7),".",_A(57),"|",NULL), //"invert selection"
-	g_strconcat("\t\t\t|<Control>d||",_A(7),".",_A(126),"|",NULL), //"select_type"
+	g_strconcat("\t\t\t|<Control>d||",_A(7),".",_A(80),"|",NULL), //"select_type"
 	g_strconcat("\t\t\t|<Control>f||",_A(6),".",_A(46),"|",NULL), //"find"
-	g_strconcat("\t\t\t|<Control>x||",_A(14),".",_A(97),"|",NULL), //"sync"
-	g_strconcat("\t\t\t|<Control>h||",_A(13),".",_A(83),"|",NULL),  //"toggle hidden"
+	g_strconcat("\t\t\t|<Control>x||",_A(14),".",_A(98),"|",NULL), //"sync"
+	g_strconcat("\t\t\t|<Control>h||",_A(13),".",_A(84),"|",NULL),  //"toggle hidden"
 	g_strconcat("\t\t\t|<Control>m||",_A(13),".",_A(24),"|",NULL), //"<bookmarks>"
 	g_strconcat("\t\t\t|<Control>t||",_A(13),".",_A(25),"|",NULL), //"<filters>"
 //	g_strconcat("\t\t\t|/||",_A(6),".",_A(46),"|",NULL),  //"find"
-	g_strconcat("\t\t\t|<Control>Left||",_A(14),".",_A(31),"|","*,1",NULL), //"adjust ratio" no arg translation
-	g_strconcat("\t\t\t|<Control>Right||",_A(14),".",_A(31),"|","*,0",NULL), //"adjust ratio" no arg translation
-	g_strconcat("\t\t\t|<Control>p||",_A(13),".",_A(84),"|",NULL),  //"show menu"
-	g_strconcat("\t\t\t|<Shift><Control>p||",_A(13),".",_A(84),"|",_A(117),NULL), //"show menu, shift"
-	g_strconcat("\t\t\t|<Control><Alt>p||",_A(13),".",_A(84),"|",_A(109),NULL),  //"show menu, ctrl"
-	g_strconcat("\t\t\t|<Control><Alt>n||",_A(7),".",_A(90),"|",NULL),  //"filelist.sortname"
-	g_strconcat("\t\t\t|<Control><Alt>m||",_A(7),".",_A(89),"|",NULL),  //"filelist.sortmodified"
-	g_strconcat("\t\t\t|<Control><Alt>s||",_A(7),".",_A(92),"|",NULL),  //"filelist.sortsize"
+	g_strconcat("\t\t\t|<Control>Left||!",_A(14),".",_A(31),"|0,*",NULL), //"adjust ratio" no arg translation NOTE blocked * expansion
+	g_strconcat("\t\t\t|<Control>Right||!",_A(14),".",_A(31),"|1,*",NULL), //"adjust ratio" no arg translation ditto
+	g_strconcat("\t\t\t|<Control>p||",_A(13),".",_A(85),"|",NULL),  //"show menu"
+	g_strconcat("\t\t\t|<Shift><Control>p||",_A(13),".",_A(85),"|",_A(118),NULL), //"show menu, shift"
+	g_strconcat("\t\t\t|<Control><Alt>p||",_A(13),".",_A(85),"|",_A(110),NULL),  //"show menu, ctrl"
+	g_strconcat("\t\t\t|<Control><Alt>n||",_A(7),".",_A(91),"|",NULL),  //"filelist.sortname"
+	g_strconcat("\t\t\t|<Control><Alt>m||",_A(7),".",_A(90),"|",NULL),  //"filelist.sortmodified"
+	g_strconcat("\t\t\t|<Control><Alt>s||",_A(7),".",_A(93),"|",NULL),  //"filelist.sortsize"
 	g_strconcat("\t\t",_C(12),"||||",NULL),  //_"directory line"
-	g_strconcat("\t\t\t|Tab||",_A(1),".",_A(35),"|",_A(110),NULL), //"complete dirs"
+	g_strconcat("\t\t\t|Tab||",_A(1),".",_A(35),"|",_A(111),NULL), //"complete dirs"
 	g_strconcat("\t\t\t|<Control>Tab||",_A(13),".",_A(64),"|",NULL), //"open"
 	g_strconcat("\t\t\t|<Control>Delete||",_A(1),".",_A(33),"|",NULL),  //"clear"
+	g_strconcat("\t\t\t|<Control>k||",_A(5),".",_A(34),"|",NULL),  //"unlog_current"
 	g_strconcat("\t\t\t|<Alt>Delete||",_A(1),".",_A(34),"|",NULL),  //"clear_history"
 	g_strconcat("\t\t",_C(5),"||||",NULL),  //_("command line"
 	g_strconcat("\t\t\t|Tab||",_A(1),".",_A(35),"|",NULL), //"complete"
-	g_strconcat("\t\t\t|<Shift>Return||",_A(1),".",_A(80),"|",NULL), //"command.send"
+	g_strconcat("\t\t\t|<Shift>Return||",_A(1),".",_A(81),"|",NULL), //"command.send"
 	g_strconcat("\t\t\t|<Control>Delete||",_A(1),".",_A(33),"|",NULL), //"clear"
 	g_strconcat("\t\t\t|<Alt>Delete||",_A(1),".",_A(34),"|",NULL),  //"clear_history"
 	g_strconcat("\t\t\t|<Alt>Insert||",_A(2),".",_A(27),"|",NULL), //"children<menu>"
