@@ -1,6 +1,6 @@
-/* $Id: e2_pane.h 1412 2009-02-01 21:51:22Z tpgww $
+/* $Id: e2_pane.h 2023 2009-12-04 23:29:09Z tpgww $
 
-Copyright (C) 2004-2009 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -30,31 +30,31 @@ enum { E2PANECUR, E2PANE1, E2PANE2, E2PANENATIVE, E2PANECONF } ;
 
 typedef struct _E2_PaneRuntime
 {
+	ViewInfo view;	//at start of struct to support casting between pane and view
+	gchar *path;	//UTF-8 string, generally = view->dir, for use when allocated version is needed
 	gchar *name;	//"pane1" or "pane2", not translated, for constructing set names
 	GtkWidget *outer_box;
 	GtkWidget *inner_box;
 	GtkWidget *pane_sw;
 //	GtkWidget *focus_widget;
+	E2_ToolbarRuntime toolbar;
 #ifdef E2_FAM
 	gint FAMreq;	//FAM request id used for this pane
 #endif
-	ViewInfo *view;
-	gchar *path;	//utf-8 string, generally = view->dir, but on heap and not length-constrained
+	E2_OptionSet *opt_transparent;	//for quick checks whether to interpret relative path strings
 	GList *opendirs;	/*list of paths of dirs previously opened (utf-8,
 						absolute for current namespace, with trailer). Used for
 						"goto" buttons menu. List was cached, and then, data items
 						were heaped copies of path strings. Now not cached, and
-						shares data with pane->view->dir_history. Unlike the latter,
-						this list may have multiple entries for the same place */
+						shares data with app.dir_history. Unlike the latter, this
+						list may have multiple entries for the same dir */
 	guint opendir_cur;	//0-based index of current position in opendirs list
-	E2_ToolbarRuntime toolbar;
-	E2_OptionSet *opt_transparent;	//for quick checks whether to interpret relative path strings
-	GHookList hook_change_dir;	//data for functions to run at end of change-dir function, for the pane
+	GHookList hook_change_dir;	//data for functions to run during change-dir function,
+								//for the pane
 } E2_PaneRuntime;
 
 gboolean e2_pane_choose_new_dir (E2_PaneRuntime *rt, GtkWidget *entry);
 gboolean e2_pane_cd_checks (gchar *path);
-gboolean e2_pane_activate_other_action (gpointer from, E2_ActionRuntime *art);
 gboolean e2_pane_change_dir_action (gpointer from, E2_ActionRuntime *art);
 #ifdef E2_VFS
 gboolean e2_pane_change_space (E2_PaneRuntime *rt, VPATH *utfpath);
@@ -73,6 +73,7 @@ void e2_pane_create_option_data (E2_PaneRuntime *rt);
 // initialisation things
 void e2_pane_flag_active (void);
 //void e2_pane_flag_inactive (void);
+//void e2_pane_flag_history (E2_PaneRuntime *rt, gboolean value);
 void e2_pane_actions_register (void);
 void e2_pane_options_register (gint num);
 
