@@ -1,6 +1,6 @@
-/* $Id: e2_plugins.c 1291 2008-09-30 06:28:50Z tpgww $
+/* $Id: e2_plugins.c 2005 2009-11-11 22:11:02Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -71,13 +71,15 @@ gboolean init_plugin (Plugin *p)
 		  of the name must be _A(6) which is (in english) "file", in order to make
 		  the plugins context-menu work properly. And the converse, do not use
 		  _A(6) unless that condition applies * /
-		gchar *action_name = g_strconcat (_A(14),".",aname,NULL);
-		p->action = e2_plugins_action_register (action_name, E2_ACTION_TYPE_ITEM,
-			_e2p_hello_world, NULL, FALSE, 0, NULL);
-
-		//other initialization stuff, as appropriate ...
-
-		return TRUE; //or FALSE, if the initialisation failed
+		E2_Action plugact =
+		{g_strconcat (_A(14),".",aname,NULL),_e2p_hello_world,FALSE,0,NULL,NULL};
+		p->action = e2_plugins_action_register (&plugact);
+		if (G_LIKELY(p->action != NULL))
+		{
+			//other initialization stuff, as appropriate ...
+			return TRUE;
+		}
+		g_free (plugact.name);
 	}
 	return FALSE;
 }
@@ -146,7 +148,7 @@ static GList *_e2_plugins_find_ref (GList *list, gchar *name)
 	for (member = list; member != NULL; member = member->next)
 	{
 		E2P_Ref *data = member->data;
-		if (g_str_equal (data->name, name))
+		if (!strcmp (data->name, name))
 			break;
 	}
 	return member;
@@ -167,15 +169,20 @@ static guint _e2_plugins_ref (GList **list, gchar *name)
 	if (member == NULL)
 	{
 		data = MALLOCATE (E2P_Ref);	//too small for slice
+#if (CHECKALLOCATEDWARN)
 		CHECKALLOCATEDWARN (data, return 0;)
+#else
 		if (data != NULL)
 		{
+#endif
 			*list = g_list_append (*list, data);
 			data->name = g_strdup (name);
 			data->refcount = 1;
 			return 1;
+#if !(CHECKALLOCATEDWARN)
 		}
 		return 0;
+#endif
 	}
 	else
 	{
@@ -233,7 +240,7 @@ BUT coverage is too limited
 	//these are in the same order as the enerator
 	//FIXME add other relevant fns
 	gpointer interface [E2API_POINTER_COUNT] = {
-		e2_action_register_simple,
+		e2_action_register,
 		e2_action_unregister,
 		e2_action_get,
 		e2_option_get,
@@ -421,7 +428,7 @@ This does not affect app.plugins
 
 @return
 */
-void e2_plugins_store_child_data (GtkTreeModel *model, GtkTreeIter *iter, Plugin *p)
+static void _e2_plugins_store_child_data (GtkTreeModel *model, GtkTreeIter *iter, Plugin *p)
 {
 	if (p->child_list != NULL)
 	{
@@ -442,6 +449,99 @@ void e2_plugins_store_child_data (GtkTreeModel *model, GtkTreeIter *iter, Plugin
 	}
 }
 /**
+@brief process a plugin's UI data into config store
+Any child-plugin data are also handled. This does not affect app.plugins.
+@param model model for plugins config treestore
+@param iter pointer to treeiter to use for updating @a model
+@param p pointer to data struct for plugin
+@param inmenu setting for the plugin's loaded/in-menu data
+@param localpath localized absolute path-string for the plugin file
+
+@return
+*/
+void e2_plugins_store_data (GtkTreeModel *model, GtkTreeIter *iter,
+	Plugin *p, gboolean inmenu, gchar *localpath)
+{
+	p->action = GINT_TO_POINTER (1); //non-NULL prevents full initialisation
+	p->plugin_init (p);
+	const gchar *menu_name = (p->menu_name != NULL) ? p->menu_name : "";
+	const gchar *icon = (p->icon != NULL) ? p->icon : "";
+	const gchar *description = (p->description != NULL) ? p->description : NULL;
+	gchar *local = strrchr (localpath, G_DIR_SEPARATOR);	//should never fail
+	*local = '\0';	//truncate plocal at trailing /
+	gchar *utf = F_FILENAME_FROM_LOCALE (local + 1);
+	gchar *utfp;
+	if (!strcmp (localpath, PLUGINS_DIR))	//from Makefile this is localised, no trailer
+		utfp = "";
+	else
+		utfp = F_FILENAME_FROM_LOCALE (localpath);
+
+	gtk_tree_store_set (GTK_TREE_STORE (model),	iter,
+		0, inmenu, 1, inmenu, 2, menu_name, 3, icon, 4, description, 5, utf, 6, utfp, -1);
+	p->show_in_menu = inmenu;	//any child-plugin uses this
+	//process any child-plugin data too
+	_e2_plugins_store_child_data (model, iter, p);
+
+	*local = G_DIR_SEPARATOR;
+	if (*utfp != '\0')
+		F_FREE (utfp, localpath);
+	F_FREE (utf, local + 1);
+}
+/* *
+@brief reconcile on-disk plugin files with plugins-config-data
+
+@return
+*/
+/*void e2_plugins_freshen_data (void)
+{
+	gboolean TODO;
+	GList *member, *loaded = NULL;
+	GtkTreeIter iter;
+	E2_OptionSet *set = e2_option_get ("plugins");
+	GtkTreeModel *mdl = set->ex.tree.model;
+
+	if (gtk_tree_model_get_iter_first (mdl, &iter))
+	{
+		gchar *utfpath, *utfname;
+		do
+		{
+			gtk_tree_model_get (model, &iter, 5, &utfname, 6, &utfpath, -1);
+			if (*utfpath == '\0')
+			{
+				construct localpath
+				if stat() fails
+				{
+					remove iter from store
+					g_free (localpath);
+				}
+				else
+					loaded = g_list_prepend (loaded, localpath);
+			}
+			g_free (utfname);
+			g_free (utfpath);
+		} while gtk_tree_model_iter_next (mdl, &iter));
+	}
+
+/ *	foreach file e2p_*.so in PLUGINS_DIR
+	{
+		check if it's in loaded list
+		if not
+		{
+			construct full path path;
+			open Plugin *p;
+			if (p)
+			{
+				append iter to model
+				e2_plugins_store_data (mdl, &iter, p, FALSE, path);
+				g_module_close (p->module);
+				DEALLOCATE (Plugin, p);
+			}
+		}
+	}
+* /
+}
+*/
+/**
 @brief check if plugin data @a child has signature @a signature
 
 @param child pointer to child plugin data struct
@@ -451,7 +551,7 @@ void e2_plugins_store_child_data (GtkTreeModel *model, GtkTreeIter *iter, Plugin
 */
 static gint _e2_plugins_match_child (Plugin *child, gchar *signature)
 {
-	return (!g_str_equal (child->signature, signature));
+	return (strcmp (child->signature, signature));
 }
 /**
 @brief process "child plugin" data when a plugin has > 1 action
@@ -638,7 +738,7 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 	{
 		gchar *file_name = F_FILENAME_TO_LOCALE (utf_name);
 		localpath = g_build_filename (localdir, file_name, NULL);
-		F_FREE (file_name);
+		F_FREE (file_name, utf_name);
 
 		if (load_this || in_menu //this is one we want to keep
 			|| *label == '\0')	//no data logged CHECKME is this test ok ?
@@ -657,7 +757,7 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 						if (*label == '\0' && p->menu_name != NULL)
 							gtk_tree_store_set (GTK_TREE_STORE (model), iter,
 								2, p->menu_name, -1);
-						else if (p->menu_name == NULL || !g_str_equal (p->menu_name, label))
+						else if (p->menu_name == NULL || strcmp (p->menu_name, label))
 						{
 							if (p->menu_name != NULL && (p->cleanflags & E2P_CLEANLABEL))
 								g_free (p->menu_name);
@@ -667,7 +767,7 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 						if (*icon == '\0' && p->icon != NULL)
 							gtk_tree_store_set (GTK_TREE_STORE (model), iter,
 								3, p->icon, -1);
-						else if (p->icon == NULL || !g_str_equal (p->icon, icon))
+						else if (p->icon == NULL || strcmp (p->icon, icon))
 						{
 							if (p->icon != NULL && (p->cleanflags & E2P_CLEANICON))
 								g_free (p->icon);
@@ -677,7 +777,7 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 						if (*tip == '\0' && p->description != NULL)
 							gtk_tree_store_set (GTK_TREE_STORE (model), iter,
 								4, p->description, -1);
-						else if (p->description == NULL || !g_str_equal (p->description, tip))
+						else if (p->description == NULL || strcmp (p->description, tip))
 						{
 							if (p->description != NULL && (p->cleanflags & E2P_CLEANTIP))
 								g_free (p->description);
@@ -685,35 +785,55 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 							p->cleanflags |= E2P_CLEANTIP;
 						}
 
-						app.plugins = g_list_append (app.plugins, p); //parent listed before any children
+						if (e2_plugins_check_installed (p->signature) == NULL)
+						{
+							app.plugins = g_list_append (app.plugins, p); //parent listed before any children
 
-						if (p->child_list == NULL)
-						{	//no child plugin(s)
-							if (*tip == '\0' && p->description != NULL)
-								gtk_tree_store_set (GTK_TREE_STORE (model), iter,
-									4, p->description, -1);
-							else if (p->description == NULL || !g_str_equal (p->description, tip))
-								p->description = g_strdup (tip);
-							//FIXME if there were children before, make sure gone
-							//from store too
+							if (p->child_list == NULL)
+							{	//no child plugin(s)
+								if (*tip == '\0' && p->description != NULL)
+									gtk_tree_store_set (GTK_TREE_STORE (model), iter,
+										4, p->description, -1);
+								else if (p->description == NULL || strcmp (p->description, tip))
+									p->description = g_strdup (tip);
+								//FIXME if there were children before, make sure gone
+								//from store too
+							}
+							else
+							{
+								//when child plugin(s) exist, parent tip won't show in menu
+								 //so make submenu item's tip blank
+								gtk_tree_store_set (GTK_TREE_STORE (model), iter, 4, "", -1);
+								//setup children data
+								_e2_plugins_record_children (model, iter, p);
+							}
 						}
 						else
 						{
-							//when child plugin(s) exist, parent tip won't show in menu
-							 //so make submenu item's tip blank
-							gtk_tree_store_set (GTK_TREE_STORE (model), iter, 4, "", -1);
-							//setup children data
-							_e2_plugins_record_children (model, iter, p);
+							printd (WARN, "aborted attempt to load 2nd copy of plugin: %s", p->signature);
+						    //CHECKME does this also remove any children from store
+						    gtk_tree_store_remove (GTK_TREE_STORE (model), iter); //CHECKME effect on store iteration
+							goto killit;
 						}
 					}
 					else	//init function failed
 					{
 						printd (WARN, "can't initialize plugin: %s", utf_name);
-						//gboolean QQQQcheckmestore;
-						//CHECKME does this also remove any children from store
-						gtk_tree_store_remove (GTK_TREE_STORE (model), iter);
+						//mark it as un-available (i.e. don't completely remove from store)
+						gtk_tree_store_set (GTK_TREE_STORE (model), iter,
+							0, FALSE, 1, FALSE, -1);
+						GtkTreeIter iter2;
+						if (gtk_tree_model_iter_children (model, &iter2, iter))
+						{
+							//no need for deeper recursion
+							do
+							{
+								gtk_tree_store_set (GTK_TREE_STORE (model), &iter2,
+									0, FALSE, 1, FALSE, -1);
+							} while (gtk_tree_model_iter_next (model, &iter2));
+						}
+killit:
 						if (g_module_symbol (p->module, "clean_plugin", (gpointer) &unload))
-						//CHECKME handle case where want to retain ??
 							unload (p);
 						g_module_close (p->module);
 						DEALLOCATE (Plugin, p);
@@ -741,7 +861,7 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 						}
 						//get store and list data for children, if any
 						p->show_in_menu = FALSE;	//set flags in store
-						e2_plugins_store_child_data (model, iter, p);
+						_e2_plugins_store_child_data (model, iter, p);
 						//junk it again, now, if not otherwise wanted
 						//e2_plugins_unload1 (p, FALSE);
 						g_module_close (p->module);
@@ -756,8 +876,8 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 			}
 			else	//plugin file not loaded
 			{
-				printd (WARN, "plugin doesn't exist: %s", utf_name);
-				gtk_tree_store_remove (GTK_TREE_STORE (model), iter);
+				printd (WARN, "plugin %s doesn't exist or won't load", utf_name);
+				gtk_tree_store_remove (GTK_TREE_STORE (model), iter); //CHECKME effect on store iteration
 			}
 		}
 		g_free (localpath);
@@ -765,10 +885,10 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 	else //no valid dir for the plugin
 	{
 		printd (WARN, "plugin dir doesn't exist: %s", utf_path);
-		gtk_tree_store_remove (GTK_TREE_STORE (model), iter);
+		gtk_tree_store_remove (GTK_TREE_STORE (model), iter); //CHECKME effect on store iteration
 	}
 	if (*utf_path != '\0')
-		F_FREE (localdir);
+		F_FREE (localdir, utf_path);
 	g_free (label);
 	g_free (icon);
 	g_free (tip);
@@ -845,7 +965,7 @@ void e2_plugins_update (void)
 			{
 				for (member = oldlist; member != NULL; member = member->next)
 				{
-					if (g_str_equal (name, ((Plugin *)member->data)->menu_name))
+					if (!strcmp (name, ((Plugin *)member->data)->menu_name))
 						break;
 				}
 			}
@@ -966,7 +1086,7 @@ Plugin *e2_plugins_check_installed (const gchar *signature)
 	for (tmp = app.plugins; tmp != NULL; tmp = tmp->next)
 	{
 		p = tmp->data;
-		if (g_str_equal (p->signature, signature))
+		if (!strcmp (p->signature, signature))
 			return p;
 	}
 	return NULL;
@@ -1003,10 +1123,10 @@ Plugin *e2_plugins_load_plugin (const gchar *signature)
 				if (ppath != NULL && *ppath != '\0')
 				{	//plugin is recorded in config data
 					s = ppath;
-					local = F_FILENAME_TO_LOCALE (ppath);
+					local = F_FILENAME_TO_LOCALE (s);
 					ppath = g_build_filename (local, pname, NULL); //create path from config data
 					g_free (s);
-					F_FREE (local);
+					F_FREE (local, s);
 				}
 				else if (ppath != NULL)
 				{
@@ -1048,7 +1168,7 @@ Plugin *e2_plugins_load_plugin (const gchar *signature)
 					//if path is not already in config, then it must be default
 					0, TRUE, 1, FALSE, 2, label, 3, icon, 4, tip, 5, name, 6, "", -1);
 
-				F_FREE (name);
+				F_FREE (name, pname);
 			}
 		}
 		else
@@ -1118,29 +1238,23 @@ gboolean e2_plugins_configure (gpointer from, E2_ActionRuntime *art)
 @brief register plugin action
 
 This is a wrapper for the standard action registration function
-It also manages a refcount for the action
-
-@param name freeable string with possible '.' separating 'parent/ and 'child' parts of the action
-@param type flags for one of: item (?) submenu, file_actions, custom_command, plugins, separator, tear_off_menu, toggle, command_line, bookmarks, dummy
-@param func the function to be called when the action is run
-@param data pointer to data to be supplied to @a func, or NULL
-@param has_arg boolean flag signalling whether ??WHAT has argument(s)
-@param exclude flags indicating whether to exclude this action from toolbars, menus &| keybindings
-@param data2 additional data needed by the action, or NULL
+It also manages a refcount for the action, and may free the name of @a newaction
+@param newaction pointer to original action data to be copied and heaped
 
 @return registered action E2_Action
 */
-E2_Action *e2_plugins_action_register (gchar *name, E2_ACTION_TYPE type,
-	void *func, gpointer data, gboolean has_arg, E2_ACTION_EXCLUDE exclude,
-	gpointer data2)
+E2_Action *e2_plugins_action_register (const E2_Action *newaction)
 {
+	guint refcount = _e2_plugins_ref (&action_refs, newaction->name);
 	E2_Action *action;
-	guint refcount = _e2_plugins_ref (&action_refs, name);
-	if (refcount == 1)
-		action = e2_action_register (name, type, func, data, has_arg,
-			exclude, data2);
+ 	if (refcount == 1)
+		action = e2_action_register (newaction);
 	else
-		action = e2_action_get (name);
+	{
+ 		action = e2_action_get (newaction->name);
+		if (G_LIKELY(action != NULL))	//prevent double-free upstream
+			g_free (newaction->name);
+	}
 	return action;
 }
 /**
@@ -1160,22 +1274,6 @@ gboolean e2_plugins_action_unregister (gchar *name)
 		return FALSE;
 }
 /**
-@brief perform a plugin's action
-This is the callback for all plugins-menu items
-@param menu_item the activated item
-@param action plugin-action data struct
-
-@return
-*/
-void e2_plugins_do_action (GtkWidget *menu_item, E2_Action *action)
-{
-	//pdata this will generally be NULL, tho' action->data may be relevant
-	gpointer pdata = g_object_get_data (G_OBJECT (menu_item), "e2-action-data");
-	E2_ActionRuntime *rt = e2_action_pack_runtime (action, pdata, NULL);
-	e2_action_run (menu_item, rt);	//completion code ignored
-	e2_action_free_runtime (rt);
-}
-/**
 @brief register a plugin config option
 This is a wrapper for the standard option registration function
 It also manages a refcount for the option
@@ -1186,17 +1284,48 @@ It also manages a refcount for the option
 @param desc textual description of the option used in config dialog, a r-t _() string FREEME ?
 @param tip tooltip used when displaying the config dialog, a _() string, or NULL
 @param depends name of another option this one depends on, or NULL
+@param ex pointer to type-specific data for initialisation
 @param flags bitflags determining how the option data will be handled
 
 @return the option data struct
 */
 E2_OptionSet *e2_plugins_option_register (E2_OptionType type, gchar *name,
-	gchar *group, gchar *desc, gchar *tip, gchar *depends, E2_OptionFlags flags)
+	gchar *group, gchar *desc, gchar *tip, gchar *depends,
+	E2_OptionSetupExtra *ex, E2_OptionFlags flags)
 {
 	E2_OptionSet *set;
 	guint refcount = _e2_plugins_ref (&option_refs, name);
 	if (refcount == 1)
-		set = e2_option_register (type, name, group, desc, tip, depends, flags);
+	{
+		switch (type)
+		{
+			case E2_OPTION_TYPE_BOOL:
+				set = e2_option_bool_register (name, group, desc, tip, depends,
+					ex->exbool, flags);
+				break;
+			case E2_OPTION_TYPE_INT:
+				set = e2_option_int_register (name, group, desc, tip, depends,
+					ex->exint.def, ex->exint.min, ex->exint.max, flags);
+				break;
+			case E2_OPTION_TYPE_SEL:
+				set = e2_option_sel_register (name, group, desc, tip, depends,
+				ex->exsel.def, ex->exsel.values, flags);
+				break;
+//			case E2_OPTION_TYPE_STR:
+//				break;
+//			case E2_OPTION_TYPE_FONT:
+//				break;
+//			case E2_OPTION_TYPE_COLOR:
+//				break;
+//			case E2_OPTION_TYPE_ICON:
+//				break;
+//			case E2_OPTION_TYPE_TREE:
+//				break;
+			default:
+				printd (ERROR, "BAD attempt to register unsupported plugin option");
+				return NULL;
+		}
+	}
 	else
 		set = e2_option_get (name);
 	return set;
@@ -1214,7 +1343,7 @@ gboolean e2_plugins_option_unregister (gchar *name)
 {
 	guint refcount = _e2_plugins_unref (&option_refs, name);
 	if (refcount == 0)
-		return (e2_option_unregister (name));
+		return (e2_option_backup (name));
 	else
 		return FALSE;
 }
@@ -1233,21 +1362,35 @@ static void _e2_plugins_tree_defaults (E2_OptionSet *set)
 	e2_option_tree_setup_defaults (set,
 	g_strdup("plugins=<"), //internal name
 	g_strdup ("true|true||||e2p_glob.so|"),
-	g_strdup ("true|true||||e2p_rename.so|"),
+	g_strdup ("false|false||||e2p_selmatch.so|"),
+	g_strdup ("false|false||||e2p_tag.so|"),
 	g_strdup ("true|true||||e2p_for_each.so|"),
-	g_strdup ("true|true||||e2p_pack.so|"),
-	g_strdup ("true|true||||e2p_names_clip.so|"),
-	g_strdup ("false|false||||e2p_sort_by_ext.so|"),
-	g_strdup ("true|true||||e2p_du.so|"),
-	g_strdup ("true|false||||e2p_unpack.so|"),
+	g_strdup ("true|true||||e2p_rename.so|"),
 	g_strdup ("true|true||||e2p_find.so|"),
+#ifdef E2_TRACKER
+	g_strdup ("false|false||||e2p_track.so|"),
+#endif
+#ifdef E2_THUMBNAILS
+	g_strdup ("true|true||||e2p_thumbs.so|"),
+#endif
+	g_strdup ("true|true||||e2p_pack.so|"),
+	g_strdup ("true|false||||e2p_unpack.so|"),
+	g_strdup ("true|true||||e2p_names_clip.so|"),
 	g_strdup ("false|false||||e2p_cpbar.so|"),
 	g_strdup ("false|false||||e2p_mvbar.so|"),
 	g_strdup ("false|false||||e2p_times.so|"),
 	g_strdup ("false|false||||e2p_dircmp.so|"),
+#ifdef E2_ACL
+	g_strdup ("false|false||||e2p_acl.so|"),
+#endif
 	g_strdup ("false|false||||e2p_config.so|"),
+//#ifdef E2_POLKIT
+//	g_strdup ("false|false||||e2p_privilege.so|"),
+//#endif
 	g_strdup ("false|false||||e2p_clone.so|"),
 	g_strdup ("false|false||||e2p_view.so|"),
+	g_strdup ("false|false||||e2p_sort_by_ext.so|"),
+	g_strdup ("true|true||||e2p_du.so|"),
 #ifdef E2_VFS
 	g_strdup ("false|false||||e2p_vfs.so|"),
 	g_strdup ("false|false||||e2p_gvfs.so|"),
