@@ -1,6 +1,6 @@
-/* $Id: e2_combobox.c 1093 2008-08-13 04:00:57Z tpgww $
+/* $Id: e2_combobox.c 2034 2009-12-21 02:37:51Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -59,19 +59,22 @@ static gint _e2_combobox_last_index (GtkComboBox *combo)
 /*******************/
 
 /**
-@brief "key-press-event" signal callback to handle accessing combobox history items
-This applies to all created comboboxes whether or not they have a text-entry
+@brief "key-press-event" signal callback to handle accessing combobox history items and translate relevant keycodes
+This applies to _all_ created comboboxes whether or not they have a text-entry
 @param combo a combobox widget where the keypress occurred
 @param event pointer to key event data struct
 @param data pointerised gboolean, TRUE to cycle to other end of history when relevant
 
-@return TRUE if pressed key was <Up> or <Down>
+@return TRUE if pressed key was <Up> or <Down> and cycling not-necessary or allowed
 */
 static gboolean _e2_combobox_key_press_cb (GtkWidget *combo, GdkEventKey *event,
 	gpointer data)
 {
 	if (!(event->keyval == GDK_Up || event->keyval == GDK_Down))
+	{
+		e2_utils_translate_key_event (event);
 		return FALSE;
+	}
 
 	if (event->keyval == GDK_Down &&
 		(event->state & gtk_accelerator_get_default_mod_mask ()) == GDK_CONTROL_MASK)
@@ -101,7 +104,12 @@ static gboolean _e2_combobox_key_press_cb (GtkWidget *combo, GdkEventKey *event,
 	e2_combobox_set_active (combo, pos);
 	if (GTK_IS_COMBO_BOX_ENTRY (combo))
 	{
-		GtkWidget *entry = GTK_BIN (combo)->child;
+		GtkWidget *entry =
+#ifdef USE_GTK2_14
+			gtk_bin_get_child (GTK_BIN (combo));
+#else
+			GTK_BIN (combo)->child;
+#endif
 		if (pos == -1)
 			gtk_entry_set_text (GTK_ENTRY (entry), "");
 		else
@@ -127,17 +135,19 @@ Otherwise, <Shift>Delete clears text from cursor to end
 static gboolean _e2_combobox_key_press_cb2 (GtkWidget *entry, GdkEventKey *event,
 	GList **history)
 {
-	gint start;
-	GtkComboBox *combo;
-	GtkTreeModel *model;
+	//prevent sporadic bad latency in UI updates
+	gtk_widget_queue_draw (entry);
 
 	if (event->keyval == GDK_Delete)
 	{
+		gint start;
 		guint modifiers = gtk_accelerator_get_default_mod_mask ();
 		if ((event->state & modifiers) == GDK_SHIFT_MASK)
 		{	//there is no action for clearing "the rest" of an entry
 			start = gtk_editable_get_position (GTK_EDITABLE (entry));
 			gtk_editable_delete_text (GTK_EDITABLE (entry), start, -1);
+			if (g_object_get_data (G_OBJECT (entry), "e2-dir-line") != NULL)
+				e2_command_line_highlight (entry, FALSE);
 			return TRUE;
 		}
 		else if ((event->state & modifiers) == GDK_MOD1_MASK)
@@ -145,40 +155,23 @@ static gboolean _e2_combobox_key_press_cb2 (GtkWidget *entry, GdkEventKey *event
 			const gchar *this = gtk_entry_get_text (GTK_ENTRY (entry));
 			if (this != NULL)	//cannot happen ?
 			{
-				combo = GTK_COMBO_BOX (
+				e2_combobox_clear_value (
 #ifdef USE_GTK2_14
-					gtk_widget_get_parent (entry)
+					gtk_widget_get_parent (entry),
 #else
-					entry->parent
+					entry->parent,
 #endif
-				);
-				model = gtk_combo_box_get_model (combo);
-				GtkTreeIter iter;
-
-				if (gtk_tree_model_get_iter_first (model, &iter))
-				{
-					while (e2_tree_find_iter_from_str (model, 0, this, &iter, FALSE)
-						&& gtk_list_store_remove (GTK_LIST_STORE (model), &iter)) {}
-				}
-
-				if (history != NULL && *history != NULL)
-				{
-					GList *tmp;
-					while ((tmp = g_list_find_custom (*history, this,
-							(GCompareFunc) e2_list_strcmp)) != NULL)
-					{
-						g_free (tmp->data);
-						*history = g_list_delete_link (*history, tmp);
-					}
-				}
+					this, TRUE);
 			}
-			gtk_entry_set_text (GTK_ENTRY (entry), "");
 			return TRUE;
 		}
 		else if ((event->state & modifiers) == (GDK_SHIFT_MASK | GDK_MOD1_MASK))
 		{	//hardcode history clearance for most comboboxes
 			//a keybinding probably applies to dirlines and commandline but
 			//in that case they don't use this function
+			GtkComboBox *combo;
+			GtkTreeModel *model;
+
 			gtk_entry_set_text (GTK_ENTRY (entry), "");
 			combo = GTK_COMBO_BOX (
 #ifdef USE_GTK2_14
@@ -217,18 +210,31 @@ static void _e2_combobox_changed_cb (GtkWidget *combo, gpointer data)
 		return;
 	}
 	//active item will be -1 whenever an item is edited
-	if ((GTK_WIDGET_VISIBLE (combo)) &&
-		(gtk_combo_box_get_active (GTK_COMBO_BOX (combo)) != -1))
+	if (
+#ifdef USE_GTK2_18
+		gtk_widget_get_visible (combo)
+#else
+		GTK_WIDGET_VISIBLE (combo)
+#endif
+		&& gtk_combo_box_get_active (GTK_COMBO_BOX (combo)) != -1)
 	{
 		gboolean focus = GPOINTER_TO_INT (data);
-		GtkWidget *entry = GTK_BIN (combo)->child;
+		GtkWidget *entry =
+#ifdef USE_GTK2_14
+			gtk_bin_get_child (GTK_BIN (combo));
+#else
+			GTK_BIN (combo)->child;
+#endif
 		if (focus)
 		{
 			gtk_widget_grab_focus (entry);
 			gtk_editable_set_position (GTK_EDITABLE (entry), -1);
 		}
 		else
+		{
+			printd (DEBUG, "issue activate signal for combobox entry");
 			g_signal_emit_by_name (G_OBJECT (entry), "activate");
+		}
 	}
 }
 
@@ -252,9 +258,10 @@ history list.
 */
 void e2_combobox_activated_cb (GtkWidget *entry, gpointer data)
 {
-	printd (NOTICE, "e2_combobox_activated_cb");
+	printd (DEBUG, "e2_combobox_activated_cb");
 	gchar *text = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
-	g_strstrip (text);
+	//a directory path may properly have trailing whitespace, so no strip
+	g_strchug (text);
 	if (*text != '\0')
 	{
 		GtkComboBox *combo = GTK_COMBO_BOX (
@@ -264,14 +271,12 @@ void e2_combobox_activated_cb (GtkWidget *entry, gpointer data)
 			entry->parent
 #endif
 		);
-		gtk_combo_box_prepend_text (combo, text);
-		e2_combobox_set_active (GTK_WIDGET (combo), 0);
+
 		if (data == NULL)	// !GPOINTER_TO_INT (data))
-		{	//remove any existing item that's the same
-			GtkTreeModel *model = gtk_combo_box_get_model (combo);
+		{
 			GtkTreeIter iter;
-			gtk_tree_model_get_iter_first (model, &iter);	//this is the one just prepended
-			if (gtk_tree_model_iter_next (model, &iter))
+			GtkTreeModel *model = gtk_combo_box_get_model (combo);
+			if (gtk_tree_model_get_iter_first (model, &iter))
 			{
 				//actually, there should only ever be 1 match at most
 				while (e2_tree_find_iter_from_str_simple (model, 0, text, &iter, FALSE))
@@ -285,6 +290,9 @@ void e2_combobox_activated_cb (GtkWidget *entry, gpointer data)
 				}
 			}
 		}
+
+		gtk_combo_box_prepend_text (combo, text);
+		e2_combobox_set_active (GTK_WIDGET (combo), 0);
 	}
 	g_free (text);
 }
@@ -309,7 +317,12 @@ entry, and moves the cursor to the end
 */
 /*void e2_combobox_select_last (GtkWidget *combo)
 {
-	GtkWidget *entry = GTK_BIN (combo)->child;
+	GtkWidget *entry =
+#ifdef USE_GTK2_14
+		gtk_bin_get_child (GTK_BIN (combo));
+#else
+		GTK_BIN (combo)->child;
+#endif
 //	GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
 	gint count = _e2_combobox_last_index ((GTK_COMBO_BOX (combo));
 	gchar *text = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
@@ -381,22 +394,57 @@ At this time, this func is only used for GtkComboBoxEntry's
 gchar *e2_combobox_get_active_text (GtkWidget *combo)
 {
 	if (gtk_combo_box_get_active (GTK_COMBO_BOX (combo)) != -1)
-	{
-		gchar *retval;
-//		if (GTK_IS_COMBO_BOX_ENTRY (combo))
-			retval = gtk_combo_box_get_active_text (GTK_COMBO_BOX (combo));
-/*		else
-		{
-			GtkTreeIter iter;
-			gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo), &iter)
-			GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
-			gtk_tree_model_get (model, &iter, 0, &retval, -1);
-		}
-*/
-		return retval;
-	}
+		return (gtk_combo_box_get_active_text (GTK_COMBO_BOX (combo)));
 	return NULL;
 }
+/**
+@brief clear any history item of @a combo corresponding to @a value, and if relevant, the displayed value
+@param combo a combo widget with a text entry
+@param value the string to be cleared
+@param with_entry TRUE to also clear the displayed value
+@return
+*/
+void e2_combobox_clear_value (GtkWidget *combo, const gchar *value, gboolean with_entry)
+{
+	GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
+	GtkTreeIter iter;
+
+	if (gtk_tree_model_get_iter_first (model, &iter))
+	{
+		while (e2_tree_find_iter_from_str (model, 0, value, &iter, FALSE))
+			gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+	}
+
+	GList **history = (GList**) g_object_get_data (G_OBJECT(combo), "e2-combo-history");
+	if (history != NULL && *history != NULL)
+	{
+		GList *tmp;
+		while ((tmp = g_list_find_custom (*history, value,
+				(GCompareFunc) e2_list_strcmp)) != NULL)
+		{
+			g_free (tmp->data);
+			*history = g_list_delete_link (*history, tmp);
+		}
+	}
+
+	if (with_entry)
+	{
+		//now it's safe to change entry content (and therefore, possibly value)
+		GtkWidget *entry =
+#ifdef USE_GTK2_14
+			gtk_bin_get_child (GTK_BIN (combo));
+#else
+			GTK_BIN(combo)->child;
+#endif
+		const gchar *now = gtk_entry_get_text (GTK_ENTRY(entry));
+		if (!strcmp (now, value))
+		{
+			gtk_entry_set_text (GTK_ENTRY(entry), "");
+			if (g_object_get_data (G_OBJECT (entry), "e2-dir-line") != NULL)
+				e2_command_line_highlight (entry, FALSE);
+		}
+	}
+}
 /* *
 @brief backup all of the history data of @a combo into @a history
 
@@ -483,68 +531,82 @@ command line, or generally NULL for an ad-hoc combobox
 @a history size figures prominently in any (downstream) clearing of the combobox
 model, so it should not be static if there's a prospect of multiple widgets
 clearing the history
-@param activated_cb "activate" callback for the combobox entry, or NULL
-@param activated_data data to provide to the activate callback
+@param activate_cb "activate" callback for the combobox entry, or NULL
+@param activate_data data to provide to the activate callback
 @param history history-list pointer, or NULL
 @param flags flags to influence the usage of the GtkComboBox
 
 @return the GtkComboBox that was created and packed
 */
-GtkWidget *e2_combobox_get (gpointer activated_cb, gpointer activated_data,
-	GList **history, E2_ComboBoxFlags flags)
+GtkWidget *e2_combobox_get (void (*activate_cb)(GtkEntry*,gpointer),
+	gpointer activate_data, GList **history, E2_ComboBoxFlags flags)
 {
-	GtkWidget *combo;
-	if (flags & E2_COMBOBOX_HAS_ENTRY)
-	{
-		combo = gtk_combo_box_entry_new_text ();
-		//enable focusing etc
-		g_signal_connect (G_OBJECT (combo), "changed",
-			G_CALLBACK (_e2_combobox_changed_cb),
-			GINT_TO_POINTER (flags & E2_COMBOBOX_FOCUS_ON_CHANGE));
-		//enable deletions
-		g_signal_connect (G_OBJECT (GTK_BIN (combo)->child), "key-press-event",
-			G_CALLBACK (_e2_combobox_key_press_cb2), history);
+	GtkWidget *combo = (flags & E2_COMBOBOX_HAS_ENTRY) ?
+		gtk_combo_box_entry_new_text () : gtk_combo_box_new_text ();
 
-		if (!(flags & E2_COMBOBOX_NO_AUTO_HISTORY))
-			g_signal_connect (G_OBJECT (GTK_BIN (combo)->child), "activate",
-				G_CALLBACK (e2_combobox_activated_cb),
-				GINT_TO_POINTER (flags & E2_COMBOBOX_ALLOW_DOUBLE));
-		//connect caller-specified callback for "activate" signal to combobox's entry
-		if (activated_cb != NULL)
-			g_signal_connect (G_OBJECT (GTK_BIN (combo)->child), "activate",
-				G_CALLBACK (activated_cb), activated_data);
-	}
-	else
+	if (!(flags & E2_COMBOBOX_MENU_STYLE))
 	{
-		combo = gtk_combo_box_new_text ();
+		static gboolean style_parsed = FALSE;
+		if (!style_parsed)
+		{
+			gtk_rc_parse_string (
+			"style \"list-style-style\" { GtkComboBox::appears-as-list = 1 } "
+			"widget \"*.list-style\" style \"list-style-style\"");
+			style_parsed = TRUE;
+		}
+		gtk_widget_set_name (combo, "list-style");
 	}
 
-	g_signal_connect (G_OBJECT ((combo)), "key-press-event",
-		G_CALLBACK (_e2_combobox_key_press_cb),
-		GUINT_TO_POINTER (flags & E2_COMBOBOX_CYCLE_HISTORY));
-
 	if (history != NULL)
 		e2_combobox_append_history (combo, *history);
 
-	if (!(flags & E2_COMBOBOX_MENU_STYLE))
+	if (flags & E2_COMBOBOX_HAS_ENTRY)
 	{
-/*		gchar *name = "list-style";
-		//work around gtk silliness, the 'appears-as-list'
-		//is a style property instead of a widget property
-		gchar *str = g_strconcat ("style \"", name,
-		"-style\" { GtkComboBox::appears-as-list = 1 } widget \"*.", name,
-		"\" style \"", name, "-style\"", NULL);
-		gtk_rc_parse_string (str);
-		g_free (str);
-		gtk_widget_set_name (combo, name);
-*/
-		gtk_rc_parse_string (
-		"style \"list-style-style\" { GtkComboBox::appears-as-list = 1 } "
-		"widget \"*.list-style\" style \"list-style-style\"");
-		gtk_widget_set_name (combo, "list-style");
+#ifdef USE_GTK2_14
+		GtkWidget *child = gtk_bin_get_child (GTK_BIN (combo));
+#endif
+		//enable prompt UI updates and deletions
+		g_signal_connect (G_OBJECT
+#ifdef USE_GTK2_14
+			(child),
+#else
+			(GTK_BIN (combo)->child),
+#endif
+			"key-press-event",
+			G_CALLBACK (_e2_combobox_key_press_cb2), history); //for <Delete>, no key translation needed
+		g_object_set_data (G_OBJECT(combo), "e2-combo-history", history);
+		if (!(flags & E2_COMBOBOX_NO_AUTO_HISTORY))
+			g_signal_connect (G_OBJECT
+#ifdef USE_GTK2_14
+				(child),
+#else
+				(GTK_BIN (combo)->child),
+#endif
+				"activate",
+				G_CALLBACK (e2_combobox_activated_cb),
+				GINT_TO_POINTER (flags & E2_COMBOBOX_ALLOW_DOUBLE));
+		//connect caller-specified callback for "activate" signal to combobox's entry
+		if (activate_cb != NULL)
+			g_signal_connect (G_OBJECT
+#ifdef USE_GTK2_14
+				(child),
+#else
+				(GTK_BIN (combo)->child),
+#endif
+				"activate",
+				G_CALLBACK (activate_cb), activate_data);
+		//enable focusing etc
+		g_signal_connect (G_OBJECT (combo), "changed",
+			G_CALLBACK (_e2_combobox_changed_cb),
+			GINT_TO_POINTER (flags & E2_COMBOBOX_FOCUS_ON_CHANGE));
 	}
 
-	gtk_widget_show (combo);
+	g_signal_connect (G_OBJECT ((combo)), "key-press-event",
+		G_CALLBACK (_e2_combobox_key_press_cb), //includes translation
+		GUINT_TO_POINTER (flags & E2_COMBOBOX_CYCLE_HISTORY));
+	g_signal_connect (G_OBJECT (combo), "key-release-event",
+		G_CALLBACK (e2_utils_key_translate_cb), NULL);
+
 	return combo;
 }
 /**
@@ -556,17 +618,18 @@ it into @a box and connects @a func to the "activate" signal.
 @param box parent box to add the GtkComboBox to
 @param expand packing property
 @param padding packing property
-@param activated_cb entry activate-signal callback, or NULL
-@param activated_data callback data
+@param activate_cb entry activate-signal callback, or NULL
+@param activate_data callback data
 @param history combo history list, or NULL
 @param flags flags to influence the usage of the GtkComboBox
 
 @return the GtkComboBox that was created and packed
 */
 GtkWidget *e2_combobox_add (GtkWidget *box, gboolean expand, guint padding,
-	gpointer activated_cb, gpointer activated_data, GList **history, E2_ComboBoxFlags flags)
+	void (*activate_cb)(GtkEntry*,gpointer), gpointer activate_data,
+	GList **history, E2_ComboBoxFlags flags)
 {
-	GtkWidget *combo = e2_combobox_get (activated_cb, activated_data, history, flags);
+	GtkWidget *combo = e2_combobox_get (activate_cb, activate_data, history, flags);
 
 	//add padding to the left & right side of the combobox
 	GtkWidget *pad_box;
@@ -579,6 +642,5 @@ GtkWidget *e2_combobox_add (GtkWidget *box, gboolean expand, guint padding,
 	gtk_box_pack_start (GTK_BOX (pad_box), combo, TRUE, TRUE, padding);
 	//add padding container to parent container
 	gtk_box_pack_start (GTK_BOX (box), pad_box, expand, expand, padding);
-	gtk_widget_show (pad_box);
 	return combo;
 }
