@@ -1,6 +1,6 @@
-/* $Id: e2_menu.c 1360 2008-12-12 19:00:56Z tpgww $
+/* $Id: e2_menu.c 2017 2009-11-19 20:36:36Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -43,18 +43,103 @@ extern pthread_mutex_t task_mutex;
 /*******************/
 
 /**
-@brief callback to destroy widget @a menu
+@brief execute action corresponding to item selected from a destroyable menu
+We do this outside the manu-item "activated" callback to avoid disrupting gtk's
+menu-activation process
+@param art pointer to action runtime data
 
-@param menu the menu widget to destroy
+@return FALSE to remove the source
+*/
+static gboolean _e2_menu_action_deferred_cb (E2_ActionRuntime *art)
+{
+	printd (DEBUG, "timer callback: _e2_menu_action_deferred_cb");
+	gdk_threads_enter ();
+	e2_action_run (app.main_window, art); //FIXME get better widget
+	gdk_threads_leave ();
+	e2_action_free_runtime (art);
+	e2_utils_fake_event (); //CHECKME does this actually help?
+	return FALSE;
+}
+/**
+@brief setup to execute action corresponding to item selected from a destroyable menu
+This is the "activate" signal callback for action-initiators in a menu that's to
+be destroyed after usage.
+To avoid disrupting gtk's menu-activation process, we do not run the action here.
+Note: depending on the menu-destruction process and whether the action is
+synchronous, menu_item may not persist until the action is finished running
+i.e. possible race
+@param menuitem the selected menu item widget
+@param data pointer to action runtime data
+
+@return
+*/
+void e2_menu_action_activated_cb (GtkMenuItem *menu_item, gpointer data)
+{
+	printd (DEBUG, "e2_menu_action_activated_cb");
+	//don't want this action-data to be destroyed with the menu
+	E2_ActionRuntime *art = (E2_ActionRuntime *)g_object_steal_data
+		(G_OBJECT (menu_item), "e2-free-actruntime");
+	//an idle-callback is no good !?
+	g_timeout_add (100, (GSourceFunc)_e2_menu_action_deferred_cb, art);
+}
+
+/**
+@brief like _e2_menu_action_deferred_cb, but for menus which are not destroyed
+@param art pointer to action runtime data
+
+@return FALSE to remove the source
+*/
+static gboolean _e2_menu_action_deferred_cb2 (E2_ActionRuntime *art)
+{
+	printd (DEBUG, "timer callback: _e2_menu_action_deferred_cb2");
+	gdk_threads_enter ();
+	e2_action_run (app.main_window, art); //FIXME get better widget
+	gdk_threads_leave ();
+	e2_utils_fake_event (); //CHECKME does this actually help?
+	return FALSE;
+}
+/**
+@brief like e2_menu_action_activated_cb, but for menus which are not destroyed
+@param menuitem the selected menu item widget
+@param data pointer to action runtime data
+*/
+void e2_menu_action_activated_cb2 (GtkMenuItem *menu_item, gpointer data)
+{
+	printd (DEBUG, "e2_menu_action_activated_cb2");
+	//an idle-callback is no good !?
+	g_timeout_add (100, (GSourceFunc)_e2_menu_action_deferred_cb2,
+		(E2_ActionRuntime *)data);
+}
+/**
+@brief timer callback to cleanup a redundant menu widget
+Assumes menu is hidden so that BGL is irrelevant
+@param menu the widget to be cleared
+@return FALSE to stop the timer
+*/
+gboolean e2_menu_destroy (GtkWidget *menu)
+{
+	printd (DEBUG, "timer callback: e2_menu_destroy, menu: %x", menu);
+	gtk_widget_destroy (menu);
+	return FALSE;
+}
+/**
+@brief "selection-done" signal callback for @a menu, initiates its destruction
+
+@param menu the menu widget
 @param data UNUSED data specified when the callback was connected
 
 @return
 */
-void e2_menu_destroy_cb (GtkWidget *menu, gpointer data)
+void e2_menu_selection_done_cb (GtkWidget *menu, gpointer data)
 {
-	//CHECKME is this not done automatically ?
-	gtk_container_foreach (GTK_CONTAINER (menu), (GtkCallback)gtk_widget_destroy, NULL);
-	gtk_widget_destroy (menu);
+	printd (DEBUG, "e2_menu_selection_done_cb, menu: %x", menu);
+	//must not destroy the menu immediately, gtk upstream has not necessarily finished with it
+#ifdef USE_GLIB2_14
+	g_timeout_add_seconds (10,
+#else
+	g_timeout_add (10000,
+#endif
+		(GSourceFunc)e2_menu_destroy, menu);
 }
 /**
 @brief callback to
@@ -64,7 +149,7 @@ void e2_menu_destroy_cb (GtkWidget *menu, gpointer data)
 
 @return
 */
-void e2_menu_control_cb (GtkWidget *menu_item, gpointer widget)
+void e2_menu_control_cb (GtkMenuItem *menu_item, gpointer widget)
 {
 	e2_option_connect (widget, gtk_check_menu_item_get_active
 		(GTK_CHECK_MENU_ITEM (menu_item)));
@@ -92,7 +177,7 @@ This is a hook-function callack upon change of set data associated with @a menu_
 
 @return TRUE always
 */
-static gboolean _e2_menu_check_change_value (gpointer state, GtkWidget *menu_item)
+static gboolean _e2_menu_check_change_value_hook (gpointer state, GtkWidget *menu_item)
 {
 	GtkWidget *controller = g_object_get_data (G_OBJECT (menu_item),
 		"e2-controller-widget");
@@ -113,13 +198,13 @@ static gboolean _e2_menu_check_change_value (gpointer state, GtkWidget *menu_ite
 @param menu menu widget
 @param set pointer to data for set whose value is tied to the check button
 @param controller widget which may have "e2-controller-blocked" data to prevent set value being updated when item is activated, can be NULL
-@param func void* callback function for handling menu item selection, or NULL
+@param func callback function for handling menu item toggle, or NULL
 @param data pointer to data to send to the callback
 
 @return the menu item widget
 */
 static GtkWidget *_e2_menu_add_tied_check (GtkWidget *menu, E2_OptionSet *set,
-	GtkWidget *controller, gpointer func, gpointer data)
+	GtkWidget *controller, void (*func)(GtkCheckMenuItem*,gpointer), gpointer data)
 {
 	GtkWidget *menu_item = e2_menu_add_check (menu, set->desc,
 		e2_option_bool_get_direct (set), func, data);
@@ -129,13 +214,10 @@ static GtkWidget *_e2_menu_add_tied_check (GtkWidget *menu, E2_OptionSet *set,
 		G_CALLBACK (_e2_menu_check_value_changed_cb), set);
 	//this will update the check menu item when the config data value changes
 	e2_option_attach_value_changed_simple (set, menu_item,
-		_e2_menu_check_change_value, menu_item);
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		menu_item, set->tip);
+		(HookFunc)_e2_menu_check_change_value_hook, menu_item);
+
+	e2_widget_set_safetip (menu_item, set->tip);
+
 	//conform to dependent set if any
 	e2_widget_handle_depends (menu_item, set);
 
@@ -148,80 +230,102 @@ static GtkWidget *_e2_menu_add_tied_check (GtkWidget *menu, E2_OptionSet *set,
 
 /**
 @brief add item to @a menu
-
+Args of @a func should be (GtkMenuItem*,some pointer)
 @param menu menu widget
 @param label menu item text, optionally with mnemonic
 @param icon custom iconfile path, or stock-icon identifier
 @param tip tooltip string
-@param func void* callback function for handling menu item selection, or NULL
-@param data pointer to data to send to the callback
+@param activate_cb callback function for handling menu item activation, or NULL
+@param data pointer to data to send to @a activate_cb
 
 @return the menu item widget
 */
-GtkWidget *e2_menu_add (GtkWidget *menu, gchar *label, gchar *icon,
-	gchar *tip, void *func, gpointer data)
+GtkWidget *e2_menu_add (GtkWidget *menu, gchar *labeltext, gchar *icon,
+	gchar *tip, void (*activate_cb)(), gpointer data)
 {
 	GtkWidget *menu_item;
-	if ((icon != NULL) && (e2_option_bool_get ("menu-show-icons")))
+	gint choice = e2_option_sel_get ("menu-show-icons");
+	if (choice == 0 && icon != NULL)	//theme
 	{
-		menu_item = gtk_image_menu_item_new_with_mnemonic
-			(label != NULL ? label : "");
-		GtkWidget *image = e2_widget_get_icon (icon,
-			e2_option_int_get ("menu-isize") + 1);
+#ifdef USE_GTK2_18
+		gboolean show;
+		GtkSettings* defs = gtk_settings_get_default ();	//assume this always exists
+		g_object_get (G_OBJECT (defs), "gtk-menu-images", &show, NULL);
+		if (show)
+#endif
+			choice = 1;
+	}
+	if (choice == 1 && icon != NULL)
+	{
+		GtkWidget *image = e2_widget_get_icon (icon, GTK_ICON_SIZE_MENU);	//e2_option_int_get ("menu-isize") + 1);
 		if (image != NULL)
+		{
+			menu_item = gtk_image_menu_item_new_with_mnemonic
+				(labeltext != NULL ? labeltext : "");
 			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), image);
+		}
+		else
+			menu_item = gtk_menu_item_new_with_mnemonic
+				(labeltext != NULL ? labeltext : _("Missing _image"));
 	}
 	else
-		menu_item = gtk_menu_item_new_with_mnemonic (label);
+		menu_item = gtk_menu_item_new_with_mnemonic (labeltext);
 
-	//turn on markup usage for the menu item
-	GList *children = gtk_container_get_children (GTK_CONTAINER (menu_item));
-	GtkWidget *lab = children->data;	//this is a GtkAccelLabel;
-	gtk_label_set_use_markup (GTK_LABEL (lab), TRUE);
+	//turn on markup for the label
+	GList *member, *children = gtk_container_get_children (GTK_CONTAINER (menu_item));
+	for  (member = children; member != NULL; member = member->next)
+	{
+		GtkWidget *child = (GtkWidget *)member->data;
+		if (GTK_IS_LABEL (child))
+		{
+			gtk_label_set_use_markup (GTK_LABEL (child), TRUE);
+			break;
+		}
+	}
+	g_list_free (children);
 
-	if (func != NULL)
+	if (activate_cb != G_CALLBACK(NULL))
 		g_signal_connect (G_OBJECT (menu_item), "activate",
-			G_CALLBACK (func), data);
+			G_CALLBACK (activate_cb), data);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
 	gtk_widget_show_all (menu_item);
 
 	if (tip != NULL && *tip != '\0')
-#ifdef USE_GTK2_12TIPS
-		gtk_widget_set_tooltip_text (
-#else
-		e2_widget_set_tooltip (NULL,
-#endif
-			menu_item, tip);
+		e2_widget_set_safetip (menu_item, tip);
 	return menu_item;
 }
 /**
-@brief add action item to @a menu
+@brief add action item to destroyable @a menu
 
 @param menu menu widget
 @param label menu item text, optionally with mnemonic
 @param icon custom iconfile path, or stock-icon identifier
 @param tip tooltip string
 @param action_name identifier of the form "_A(n)._A(m)", or external command
-@param arg argument(s) string for the action
-@param freefunc pointer to function that frees @a arg when the action is finished
+@param arg argument(s) string for the action, or NULL
 
-@return the menu item widget
+@return the menu item widget, or NULL
 */
 GtkWidget *e2_menu_add_action (GtkWidget *menu, gchar *label, gchar *icon,
-	gchar *tip, gchar *action_name, gchar *arg, gpointer freefunc)
+	gchar *tip, gchar *action_name, gchar *arg)
 {
 	gchar *real_arg;
 	E2_Action *action = e2_action_get_with_custom (action_name, arg, &real_arg);
-	E2_ActionRuntime *a_rt = e2_action_pack_runtime (action, real_arg, freefunc);
-	GtkWidget *menu_item = e2_menu_add (menu, label, icon, tip, e2_action_run,
-		a_rt);
-	g_object_set_data_full (G_OBJECT (menu_item), "free-callback-data",
-		a_rt, (GDestroyNotify) e2_action_free_runtime);
-	return menu_item;
+	E2_ActionRuntime *art = e2_action_pack_runtime (action, real_arg, g_free);
+	if (art != NULL)
+	{
+		//note: callback relates to destroyable menu
+		GtkWidget *item = e2_menu_add (menu, label, icon, tip,
+			e2_menu_action_activated_cb, art);
+		g_object_set_data_full (G_OBJECT(item), "e2-free-actruntime",
+			art, (GDestroyNotify)e2_action_free_runtime);
+		return item;
+	}
+	return NULL;
 }
 /**
 @brief add check item to @a menu
-
+Args of @a func should be (GtkCheckMenuItem*,some pointer)
 @param menu menu widget
 @param label menu item text, optionally with mnemonic
 @param state initial state of the created item, T/F
@@ -230,8 +334,8 @@ GtkWidget *e2_menu_add_action (GtkWidget *menu, gchar *label, gchar *icon,
 
 @return the menu item widget
 */
-GtkWidget *e2_menu_add_check (GtkWidget *menu,  gchar *label,
-	gboolean state, void *func, gpointer data)
+GtkWidget *e2_menu_add_check (GtkWidget *menu,  gchar *label, gboolean state,
+	void (*func)(), gpointer data)
 {
 	GtkWidget *check = gtk_check_menu_item_new_with_mnemonic (label);
 	if (menu != NULL)
@@ -244,18 +348,18 @@ GtkWidget *e2_menu_add_check (GtkWidget *menu,  gchar *label,
 }
 /**
 @brief add radio item to @a menu
-
+Args of @a func should be (GtkMenuItem*,some pointer)
 @param menu menu widget, or NULL
 @param group pointer to gslist of radio group
 @param label menu item text, optionally with mnemonic
 @param state initial state of the created item, T/F
-@param func void* callback function for handling menu-item selection, or NULL
+@param func callback function for handling menu-item activation, or NULL
 @param data pointer to data to send to the callback
 
 @return the menu item widget
 */
 GtkWidget *e2_menu_add_radio (GtkWidget *menu, GSList **group, gchar *label,
-	gboolean state, void *func, gpointer data)
+	gboolean state, void (*func)(), gpointer data)
 {
 	GtkWidget *radio = gtk_radio_menu_item_new_with_mnemonic (*group, label);
 	*group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (radio));
@@ -314,20 +418,21 @@ GtkWidget *e2_menu_add_submenu (GtkWidget *menu, gchar *label, gchar *icon)
 	return submenu;
 }
 /**
-@brief create toggle menu item and add it to @a menu
+@brief create toggle menu item and add it to destroyable @a menu
 This expects to be called twice in succession for each toggle-item.
 None of the strings may be NULL
 @param menu the menu to which the item will be added
 @param label menu label string
+@param destroy TRUE if @a menu will be destroyed after use
 @param icon menu icon name
 @param tip menu tip string
 @param type string "toggle.on" or "toggle.off"
 @param cmd toggle action, for creating a hash key
 
-@return NULL for the first of the pair, the menu item after the second
+@return after the first of the pair, NULL: after the second, the menu item or NULL upon error
 */
-GtkWidget *e2_menu_add_toggle (GtkWidget *menu, gchar *label, gchar *icon,
-	gchar *tip, gchar *type, gchar *cmd)
+GtkWidget *e2_menu_add_toggle (GtkWidget *menu, gboolean destroy,
+	gchar *label, gchar *icon, gchar *tip, gchar *type, gchar *cmd)
 {
 	static gboolean first = TRUE;
 	static gboolean firststate;
@@ -343,7 +448,7 @@ GtkWidget *e2_menu_add_toggle (GtkWidget *menu, gchar *label, gchar *icon,
 	{	//process 1st of a pair
 		first = FALSE;
 		//park until we get the paired name, so we can make hash key
-		firststate = g_str_has_suffix (type, _A(115));	//_(on
+		firststate = g_str_has_suffix (type, _A(116));	//_(on
 		firstlabel = g_strdup (label);
 		firsticon = g_strdup (icon);
 		firsttip = g_strdup (tip);
@@ -390,11 +495,18 @@ GtkWidget *e2_menu_add_toggle (GtkWidget *menu, gchar *label, gchar *icon,
 	}
 	gchar *realarg;
 	E2_Action *action = e2_action_get_with_custom (type, cmd, &realarg);
-	E2_ActionRuntime *actionrt = e2_action_pack_runtime (action, hashkey, g_free);
-	GtkWidget *item = e2_menu_add (menu, uselabel, useicon, usetip,
-				e2_action_run, actionrt);
-	g_object_set_data_full (G_OBJECT (item), "free-callback-data", actionrt,
-		(GDestroyNotify) e2_action_free_runtime);
+	E2_ActionRuntime *art = e2_action_pack_runtime (action, hashkey, g_free); //CHECKME realarg instead ?
+	GtkWidget *item;
+	if (art != NULL)
+	{
+		item = e2_menu_add (menu, uselabel, useicon, usetip,
+		(destroy) ? e2_menu_action_activated_cb:e2_menu_action_activated_cb2,
+		art);
+		g_object_set_data_full (G_OBJECT(item), "e2-free-actruntime",
+			art, (GDestroyNotify)e2_action_free_runtime);
+	}
+	else
+		item = NULL;
 
 	g_free (firstlabel);
 	g_free (firsticon);
@@ -404,7 +516,7 @@ GtkWidget *e2_menu_add_toggle (GtkWidget *menu, gchar *label, gchar *icon,
 	return item;
 }
 /**
-@brief popup menu @a menu
+@brief popup detroyable menu @a menu
 
 @param menu menu widget
 @param button event button
@@ -415,7 +527,7 @@ GtkWidget *e2_menu_add_toggle (GtkWidget *menu, gchar *label, gchar *icon,
 void e2_menu_popup (GtkWidget *menu, gint button, guint32 time)
 {
 	g_signal_connect (G_OBJECT (menu), "selection-done",
-		G_CALLBACK (e2_menu_destroy_cb), NULL);
+		G_CALLBACK (e2_menu_selection_done_cb), NULL);
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, button, time);
 }
 /* *
@@ -433,7 +545,7 @@ void e2_menu_popup (GtkWidget *menu, gint button, guint32 time)
 void _e2_menu_popup (GtkWidget *menu, gint x, gint y, gint button, guint32 time)
 {
 	g_signal_connect (G_OBJECT (menu), "selection-done",
-		G_CALLBACK (e2_menu_destroy_cb), NULL);
+		G_CALLBACK (e2_menu_selection_done_cb), NULL);
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, button, time);
 }
 */
@@ -444,7 +556,7 @@ void _e2_menu_popup (GtkWidget *menu, gint x, gint y, gint button, guint32 time)
 @param menu menu widget to populate, or NULL to create a new one
 @param set first of a NULL-terminated series of option data structs
 
-@return the menu widget
+@return @a menu or a new menu widget
 */
 GtkWidget *e2_menu_create_options_menu (GtkWidget *controller, GtkWidget *menu,
 	E2_OptionSet *set, ...)
@@ -455,7 +567,7 @@ GtkWidget *e2_menu_create_options_menu (GtkWidget *controller, GtkWidget *menu,
 	va_start (args, set);
 	while (set != NULL)
 	{
-		gpointer func = va_arg (args, gpointer);
+		void (*func)(GtkCheckMenuItem*,gpointer) = va_arg (args, gpointer);
 		gpointer data = va_arg (args, gpointer);
 		switch (set->type)
 		{
@@ -477,58 +589,58 @@ GtkWidget *e2_menu_create_options_menu (GtkWidget *controller, GtkWidget *menu,
 /**
 @brief add items to bookmarks menu, recursively if children exist
 
+@param menu widget to which the items will be added
+@param onetime TRUE if @a menu is destroyed after use
 @param action pointer to data for the pseudo-action being processed
 @param panenum the pane whose path will be used when adding a mark (E2PANECUR, E2PANE1 or E2PANE2)
-@param set pointer to bookmarks option data
-@param menu widget to which the items will be added
-@param iter pointer to iter to be used for interrogating the treemodel for @a set
+@param mdl pointer to bookmarks config data tree model
+@param iter pointer to iter to be used for interrogating @a mdl
 
 @return
 */
-void e2_menu_create_bookmarks_menu (E2_Action *action, gint panenum,
-	E2_OptionSet *set, GtkWidget *menu, GtkTreeIter *iter)
+void e2_menu_add_bookmark_items (GtkWidget *menu, gboolean onetime,
+	E2_Action *action, gint panenum, GtkTreeModel *mdl, GtkTreeIter *iter)
 {
 	do
 	{
 		gchar *name, *icon, *_tip, *_path;
 		//don't support variables in label
 //		gchar *_name, *icon, *_tip, *_path;
-		gtk_tree_model_get (set->ex.tree.model, iter, 0, &name, 1, &icon, 2, &_tip, 3, &_path, -1);
-//		gchar *name = e2_utils_replace_vars (_name);
-		gchar *tip = e2_utils_replace_vars (_tip);
-		gchar *path = e2_utils_replace_vars (_path);
+		gtk_tree_model_get (mdl, iter, 0, &name, 1, &icon, 2, &_tip, 3, &_path, -1);
+//		gchar *name = e2_utils_replace_vars (_name, FALSE);
+		gchar *tip = e2_utils_replace_vars (_tip, FALSE);
+		gchar *path = e2_utils_replace_vars (_path, TRUE);
 //		g_free (_name);
 		g_free (_tip);
 		g_free (_path);
 
-		GtkWidget *item;
-		gboolean child = gtk_tree_model_iter_has_child (set->ex.tree.model, iter);
-		if (child)
+		if (gtk_tree_model_iter_has_child (mdl, iter))
 		{
-			//sub-menu items can't show tip, context menu etc
-			GtkWidget *menu2 = gtk_menu_new ();
 			GtkTreeIter iter2;
-			gtk_tree_model_iter_children (set->ex.tree.model, &iter2, iter);
-			e2_menu_create_bookmarks_menu (action, panenum, set, menu2, &iter2);	//recurse
-			item = e2_menu_add (menu, name, icon, tip, NULL, NULL);
-			gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu2);
+			gtk_tree_model_iter_children (mdl, &iter2, iter);
+			//sub-menu items can't show tip, context menu etc
+			GtkWidget *submenu = e2_menu_add_submenu (menu, name, icon);
+			e2_menu_add_bookmark_items (submenu, onetime, action, panenum, mdl, &iter2);	//recurse
 			g_free (path);
 		}
 		else
 		{
-			item = e2_menu_add (menu, name, icon, tip, NULL, NULL);
+			GtkWidget *item = e2_menu_add (menu, name, icon, tip, NULL, NULL);
 			g_signal_connect_data (G_OBJECT (item), "button-press-event",
 				G_CALLBACK (e2_bookmark_button_press_cb),
-				gtk_tree_model_get_path (set->ex.tree.model, iter),
+				gtk_tree_model_get_path (mdl, iter),
 				(GClosureNotify) gtk_tree_path_free, 0);
-			//gtk sillyness fix, if we don't use connect_after for the activate
-			//signal of the menuitem, the button-press-event will never occur
-			//path is cleaned when art is destroyed
+			void (*activatecb)() = (onetime) ?
+				e2_menu_action_activated_cb : e2_menu_action_activated_cb2;
 			E2_ActionRuntime *art = e2_action_pack_runtime (action,
 				g_strdup_printf ("%d %s", panenum, path), g_free);
-			g_signal_connect_after (item, "activate", G_CALLBACK (e2_action_run), art);
-			g_object_set_data_full (G_OBJECT (item), "free-callback-data-bookmark", art,
+			//must connect_after to the activate signal, or else the
+			//button-press-event will never occur
+			g_signal_connect_after (G_OBJECT (item), "activate",
+				G_CALLBACK (activatecb), art);
+			g_object_set_data_full (G_OBJECT (item), "e2-free-actruntime", art,
 				(GDestroyNotify) e2_action_free_runtime);
+			//path to be cleaned when item is destroyed
 			g_object_set_data_full (G_OBJECT (item), "bookmark-path", path, g_free);
 			//signal which pane (or default) to use when adding a mark or opening a dir
 			g_object_set_data (G_OBJECT (item), "bookmark-pane", GINT_TO_POINTER (panenum));
@@ -536,18 +648,19 @@ void e2_menu_create_bookmarks_menu (E2_Action *action, gint panenum,
 		g_free (name);
 		g_free (icon);
 		g_free (tip);
-	} while (gtk_tree_model_iter_next (set->ex.tree.model, iter));
+	} while (gtk_tree_model_iter_next (mdl, iter));
 }
 /**
-@brief add items for all relevant menu-enabled plugins, to @a menu
+@brief add items for all relevant menu-enabled plugins, to destroyable @a menu
 Assumes app.plugins is in menu-order, for "non-child" plugins at least, and
 for any "parent" plugin, its embedded list of children is in sub-menu order
 @param menu widget to which items are added
+@param onetime TRUE if @a menu is destroyed after usage
 @param is_selection TRUE if one or more items are selected in active pane
 
 @return
 */
-void e2_menu_create_plugins_menu (GtkWidget *menu, gboolean is_selection)
+void e2_menu_create_plugins_menu (GtkWidget *menu, gboolean onetime, gboolean is_selection)
 {
 	gchar *prefix = (is_selection) ? NULL : g_strconcat (_A(6), ".", NULL);
 	GList *tmp;
@@ -556,6 +669,13 @@ void e2_menu_create_plugins_menu (GtkWidget *menu, gboolean is_selection)
 		Plugin *p = tmp->data;
 		if (p->show_in_menu && (is_selection || !g_str_has_prefix (p->action->name, prefix)))
 		{
+			E2_ActionRuntime *art;
+			GtkWidget *item;
+			//FIXME plugin action runtime data from ? ATM plugins expect no data,
+			// tho' action->data may be relevant
+//			gpointer pdata = NULL;
+//			gpointer pclean = NULL;
+			void (*activatecb)() = (onetime) ? e2_menu_action_activated_cb:e2_menu_action_activated_cb2;
 			if (p->child_list != NULL)
 			{
 				//create item and child menu
@@ -566,14 +686,31 @@ void e2_menu_create_plugins_menu (GtkWidget *menu, gboolean is_selection)
 				{
 					Plugin *pc = (Plugin *)member->data;
 					if (pc->show_in_menu)
-						e2_menu_add (menu2, (gchar *)pc->menu_name, (gchar *)pc->icon,
-							(gchar *)pc->description, e2_plugins_do_action, pc->action);
+					{
+						art = e2_action_pack_runtime (pc->action, NULL, NULL);
+						if (art != NULL)
+						{
+							item = e2_menu_add (menu2, (gchar *)pc->menu_name,
+								(gchar *)pc->icon, (gchar *)pc->description, activatecb, art);
+							g_object_set_data_full (G_OBJECT(item), "e2-free-actruntime",
+								art, (GDestroyNotify)e2_action_free_runtime);
+						}
+					}
 				}
 			}
 			else //just 1 action in the plugin, or it's a child
-				if (p->module != NULL)	//it's not a child
-					e2_menu_add (menu, (gchar *)p->menu_name, (gchar *)p->icon,
-						(gchar *)p->description, e2_plugins_do_action, p->action);
+				if (p->module != NULL	//it's not a child
+					&& p->show_in_menu)
+			{
+				art = e2_action_pack_runtime (p->action, NULL, NULL);
+				if (art != NULL)
+				{
+					item = e2_menu_add (menu, (gchar *)p->menu_name, (gchar *)p->icon,
+						(gchar *)p->description, activatecb, art);
+					g_object_set_data_full (G_OBJECT(item), "e2-free-actruntime",
+						art, (GDestroyNotify)e2_action_free_runtime);
+				}
+			}
 		}
 	}
 	if (!is_selection)
@@ -589,7 +726,7 @@ This is the callback for 'activated' signal for the menu item
 
 @return
 */
-static void _e2_menu_custom_cb (GtkWidget *item, gpointer data)
+static void _e2_menu_custom_cb (GtkMenuItem *item, gpointer data)
 {
 	//command may be slow, get this out of the way
 /*/	gtk_widget_hide (
@@ -623,23 +760,23 @@ static void _e2_menu_custom_cb (GtkWidget *item, gpointer data)
 static void _e2_menu_add_custom_items (GtkTreeModel *model, GtkTreeIter *iter, GtkWidget *menu)
 {
 	GtkTreeIter iter2;
-	GtkWidget *menu_item;
 	gchar *icon, *label, *tip, *command;
 	do
 	{
 		gtk_tree_model_get (model, iter, 1, &icon, 2, &label, 3, &tip, 4, &command, -1);
-		menu_item = e2_menu_add (menu, label, icon, tip, _e2_menu_custom_cb, NULL);
 		if (gtk_tree_model_iter_children (model, &iter2, iter))
 		{
-			GtkWidget *menu2 = gtk_menu_new ();
-			//recurse
-			_e2_menu_add_custom_items (model, &iter2, menu2);
-			gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu2);
+			GtkWidget *submenu = e2_menu_add_submenu (menu, label, icon);
+			_e2_menu_add_custom_items (model, &iter2, submenu);	//recurse
 			g_free (command);
 		}
 		else
-			g_object_set_data_full (G_OBJECT (menu_item),
-				"custom-command", command, (GDestroyNotify) g_free);
+		{
+			GtkWidget *menu_item = e2_menu_add (menu, label, icon, tip,
+					_e2_menu_custom_cb, NULL);
+			g_object_set_data_full (G_OBJECT (menu_item), "custom-command",
+				command, (GDestroyNotify) g_free);
+		}
 		//cleanup except command
 		g_free (icon);
 		g_free (label);
@@ -656,37 +793,37 @@ static void _e2_menu_add_custom_items (GtkTreeModel *model, GtkTreeIter *iter, G
 GtkWidget *e2_menu_create_custom_menu (gchar *name)
 {
 	GtkTreeIter iter, iter2;
-	GtkWidget *menu;
 	E2_OptionSet *set = e2_option_get ("custom-menus");
 	if (e2_tree_find_iter_from_str (set->ex.tree.model, 0, name, &iter, TRUE)
 		&& gtk_tree_model_iter_children (set->ex.tree.model, &iter2, &iter))
 	{
-		menu = gtk_menu_new ();
+		GtkWidget *menu = gtk_menu_new ();
 		_e2_menu_add_custom_items (set->ex.tree.model, &iter2, menu);
+		return menu;
 	}
-	else
-		menu = NULL;
-	return menu;
+	return NULL;
 }
 /**
-@brief construct a menu of running child processes, with items which callback to @a func
+@brief construct but not pop up a destroyable menu of running child processes,
+  with items which upon activation callback to @a func
+Args for @a func should be (GtkMenuItem*, some pointer)
 This func is usable for button-menu and output context menu
-We use toggle items (no mnemonics) to indicate which procesess are active
+Uses check-items (no mnemonics) to indicate which procesess are active
 @param type enumerator for which processes to gather
-@param func void* callback function for handling menu item selection
+@param activate_cb callback function for handling menu item activation
+
+The arguments for @a activate_cb must be (GtkMenuItem*,E2_TaskRuntime*).
 
 @return the menu widget, or NULL
 */
 #define CHILDMENUWIDTH 30
-GtkWidget *e2_menu_create_child_menu (E2_ChildMenuType type, gpointer func)
+GtkWidget *e2_menu_create_child_menu (E2_ChildMenuType type, void (*activate_cb)())
 {
 	gchar *label, *s;
 	gboolean active, empty = TRUE;
-	GtkWidget *menu = gtk_menu_new ();
-	GtkWidget *item;
-	GList *member;
+	GtkWidget *item, *menu = gtk_menu_new ();
 	pthread_mutex_lock (&task_mutex);
-	member = app.taskhistory;
+	GList *member = app.taskhistory;
 	pthread_mutex_unlock (&task_mutex);
 	while (member != NULL)
 	{
@@ -718,7 +855,7 @@ GtkWidget *e2_menu_create_child_menu (E2_ChildMenuType type, gpointer func)
 				else
 					subject = "";	//results in a redundant trailing space in the label
 				label = g_strdup_printf ("%s %s %s", atask->action->name, shortdir, subject);
-				F_FREE (srcdir);
+				F_FREE (srcdir, atask->currdir);
 				g_free (shortdir);
 			}
 			else if (!rt->action)
@@ -747,12 +884,10 @@ GtkWidget *e2_menu_create_child_menu (E2_ChildMenuType type, gpointer func)
 				{
 					item = gtk_menu_item_new_with_label (label);
 				}
-				if (func != NULL)
+				if (activate_cb != G_CALLBACK (NULL))
 					g_signal_connect (G_OBJECT (item), "activate",
-						G_CALLBACK (func), rt);
+						G_CALLBACK (activate_cb), rt);
 				gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-				gtk_widget_show_all (item);
-
 				g_free (label);
 			}
 		}
@@ -769,19 +904,21 @@ GtkWidget *e2_menu_create_child_menu (E2_ChildMenuType type, gpointer func)
 		}
 		else
 		{
-			label = _("no children");
-			e2_menu_add (menu, label, NULL, NULL, func, NULL);
+			e2_menu_add (menu, _("no children"), NULL, NULL, activate_cb, NULL);
 		}
 	}
 
 	if (menu != NULL)
+	{
+		gtk_widget_show_all (menu);
 		g_signal_connect (G_OBJECT (menu), "selection-done",
-			G_CALLBACK (e2_menu_destroy_cb), NULL);
+			G_CALLBACK (e2_menu_selection_done_cb), NULL);
+	}
 
 	return menu;
 }
 /**
-@brief create filters menu for showing in pane to which @a view belongs
+@brief create but not pop up a destroyable filters-menu for pane to which @a view belongs
 
 @param view data struct for view to be changed
 
@@ -809,7 +946,7 @@ GtkWidget *e2_menu_create_filter_menu (ViewInfo *view)
 	}
 
 	g_signal_connect (G_OBJECT (menu), "selection-done",
-		G_CALLBACK (e2_menu_destroy_cb), NULL);
+		G_CALLBACK (e2_menu_selection_done_cb), NULL);
 
 	return menu;
 }
@@ -823,13 +960,18 @@ shown in the menu, when clicked
 
 @return
 */
-static void _e2_menu_mount_cb (GtkWidget *item, gpointer from)
+static void _e2_menu_mount_cb (GtkCheckMenuItem *item, gpointer from)
 {
-	gboolean newstate = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (item));
+	gboolean newstate = gtk_check_menu_item_get_active (item);
 //	gint result;
 	gchar *elsewhere = NULL;
-	gchar *point = g_object_get_data (G_OBJECT (item), "_mountpoint_");
-#ifdef E2_HAL
+	gchar *point;
+//#ifdef USE_GTK2_16
+//	point = gtk_menu_item_get_label (GTK_MENU_ITEM (item));
+//#else
+	point = g_object_get_data (G_OBJECT (item), "_mountpoint_");
+//#endif
+#if defined (E2_HAL) || defined (E2_DEVKIT)
 	gboolean remove = e2_fs_mount_is_ejectable (point);	//check this before unmount
 #endif
 	//CHECKME synchronous umount command to mimimize risk of CWD error
@@ -882,7 +1024,7 @@ static void _e2_menu_mount_cb (GtkWidget *item, gpointer from)
 	}
 	else	//doing an unmount
 	{
-#ifdef E2_HAL
+#if defined (E2_HAL) || defined (E2_DEVKIT)
 		if (remove)
 		{
 			cmd = g_strdup_printf ("eject %s", qp);
@@ -924,17 +1066,53 @@ Can't use e2_menu_add_check() because we don't want mnemonics
 @return the menu item widget
 */
 static GtkWidget *_e2_menu_mount_add_check (GtkWidget *menu,  gchar *label,
-	gboolean state, gpointer func, gpointer data)
+	gboolean state, void (*func)(GtkCheckMenuItem*,gpointer), gpointer data)
 {
 	GtkWidget *check = gtk_check_menu_item_new_with_label (label);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), check);
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (check), state);
-	g_signal_connect (G_OBJECT (check), "toggled", G_CALLBACK (func), data);
-	gtk_widget_show (check);
+	if (func != NULL)
+		g_signal_connect (G_OBJECT (check), "toggled", G_CALLBACK (func), data);
 	return check;
 }
+
+static GtkWidget *selectedmount = NULL;
 /**
-@brief create and pop up a mountpoints menu
+@brief "select" signal callback to log which item in mounts-menu is current
+@param item the slected menu-item
+@param user_data UNUSED
+@return
+*/
+void _e2_menu_mount_select_cb (GtkItem *item, gpointer user_data)
+{
+	selectedmount = GTK_WIDGET(item);
+}
+/**
+@brief key press signal callback to insert into the commandline the path of the selected item in @a menu
+@param menu the focused menu-widget when the key was pressed
+@param event pointer to event data
+@param user_data UNUSED
+@return
+*/
+static gboolean _e2_menu_mount_keypress_cb (GtkWidget *menu,
+	GdkEventKey *event, gpointer user_data)
+{
+	printd (DEBUG, "_e2_menu_mount_keypress_cb");
+	if (event->keyval == GDK_Insert)
+	{
+		const gchar *path;
+//#ifdef USE_GTK2_16
+//		path = gtk_menu_item_get_label (GTK_MENU_ITEM (selectedmount));
+//#else
+		path = g_object_get_data (G_OBJECT(selectedmount), "_mountpoint_");
+//#endif
+		e2_command_line_insert (path);
+		return TRUE;
+	}
+	return FALSE;
+}
+/**
+@brief create and pop up a destroyable mountpoints menu
 
 @param from the button that was clicked to popup the menu
 @param art action runtime data
@@ -947,17 +1125,22 @@ gboolean e2_menu_create_mounts_menu (gpointer from, E2_ActionRuntime *art)
 	GList *mounts, *mountables, *member, *node;
 	GtkWidget *item;
 
-	art->state &= ~GDK_MOD2_MASK;
-	if (!ACTION_CLICK(art))
-		return FALSE;
+	if (GTK_IS_BUTTON (from))
+	{
+		art->state &= ~GDK_MOD2_MASK;
+		if (!ACTION_CLICK(art))
+			return FALSE;
+	}
 
 	GtkWidget *menu = gtk_menu_new ();
 
 	mounts = e2_fs_mount_get_mounts_list ();
 	for (member = mounts ; member != NULL; member = member->next)
 	{
-		item = _e2_menu_mount_add_check (menu,
-			(gchar *) member->data, TRUE, _e2_menu_mount_cb, from);
+		item = _e2_menu_mount_add_check (menu, (gchar *) member->data, TRUE,
+				_e2_menu_mount_cb, from);
+		g_signal_connect (G_OBJECT (item), "select",
+			G_CALLBACK (_e2_menu_mount_select_cb), NULL);
 		g_object_set_data_full (G_OBJECT (item), "_mountpoint_",
 			member->data, g_free);
 	}
@@ -969,32 +1152,38 @@ gboolean e2_menu_create_mounts_menu (gpointer from, E2_ActionRuntime *art)
 		gboolean mounted = FALSE;
 		for (node = mounts; node != NULL; node = node->next)
 		{
-			if (g_str_equal (point, (gchar *) node->data))
+			if (!strcmp (point, (gchar *) node->data))
 			{
 				mounted = TRUE;
 				break;
 			}
 		}
-		if (!mounted)
+		if (mounted)
+			g_free (member->data);
+		else
 		{
-			item = _e2_menu_mount_add_check (menu,
-				(gchar *) member->data, FALSE, _e2_menu_mount_cb, from);
+			item = _e2_menu_mount_add_check (menu, (gchar *) member->data,
+					FALSE, _e2_menu_mount_cb, from);
+			g_signal_connect (G_OBJECT (item), "select",
+				G_CALLBACK (_e2_menu_mount_select_cb), NULL);
 			g_object_set_data_full (G_OBJECT (item), "_mountpoint_",
-				g_strdup ((gchar *) member->data), g_free);
+				member->data, g_free);
 		}
 	}
 
 	g_list_free (mounts);	//its data are cleared with the menu
-	e2_list_free_with_data (&mountables);
+	g_list_free (mountables); //ditto
 
+	g_signal_connect (G_OBJECT (menu), "key-press-event",
+		G_CALLBACK(_e2_menu_mount_keypress_cb), NULL);
 	g_signal_connect (G_OBJECT (menu), "selection-done",
-		G_CALLBACK (e2_menu_destroy_cb), NULL);
+		G_CALLBACK (e2_menu_selection_done_cb), NULL);
 
+	gtk_widget_show_all (menu);
 	if (GTK_IS_BUTTON (from))
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
 			(GtkMenuPositionFunc) e2_toolbar_set_menu_position, from, 1, 0);
 	else
-		//FIXME
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 1, 0);
 
 	return TRUE;
